// ArboView.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "MuzipArc.h"
#include "MuzipDoc.h"
#include "ArboView.h"
#include "MuzipView.h"
#include "ApercuView.h"
#include "Interf.h"

#include <Pile.h>

#include "dlgConfirm.h"
#include "dlgDecomp.h"
#include "dlgProprietes.h"


IMPLEMENT_DYNCREATE(CArboView, CTreeView)
BEGIN_MESSAGE_MAP(CArboView, CTreeView)
	//{{AFX_MSG_MAP(CArboView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_NOTIFY_REFLECT   (NM_RCLICK,                 OnRClick)
	ON_NOTIFY_REFLECT   (TVN_GETDISPINFO,           OnGetDispInfo)
	ON_NOTIFY_REFLECT   (TVN_KEYDOWN,               OnKeyDown)
	ON_NOTIFY_REFLECT   (TVN_SELCHANGED,            OnSelChanged)
	ON_NOTIFY_REFLECT   (TVN_ITEMEXPANDING,         OnItemExpanding)
	ON_NOTIFY_REFLECT   (TVN_BEGINDRAG,             OnBeginDrag)
	ON_NOTIFY_REFLECT   (TVN_BEGINLABELEDIT,        OnBeginLabelEdit)
	ON_NOTIFY_REFLECT   (TVN_ENDLABELEDIT,          OnEndLabelEdit)
	ON_COMMAND          (ID_ARCHIVE_MOINS,          OnArchiveMoins)
	ON_COMMAND          (ID_ARCHIVE_DECOMP,         OnArchiveDecomp)
	ON_COMMAND          (ID_ARCHIVE_RENOMMER,       OnArchiveRenommer)
	ON_COMMAND          (ID_ARCHIVE_PROPRIETES,     OnArchiveProprietes)
	ON_COMMAND          (ID_ARCHIVE_RECOMP,         OnArchiveRecomp)
	ON_COMMAND          (ID_ARCHIVE_CACHER,         OnArchiveCacher)
	ON_COMMAND          (ID_ARCHIVE_NOUVEAUDOSSIER, OnArchiveNouveauDossier)
	ON_COMMAND          (ID_ARCHIVE_TESTER,         OnArchiveTester)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_DECOMP,         OnUpdateFautSel)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_MOINS,          OnUpdateFautSelModif)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_RENOMMER,       OnUpdateArchiveRenommer)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_RECOMP,         OnUpdateFautSelModif)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_CACHER,         OnUpdateFautSel)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_NOUVEAUDOSSIER, OnUpdateArchiveNouveauDossier)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


// Rcuprer le dossier slectionn (NULL si aucun)
CDossier* CArboView::GetSelDossier() const
{
	const HTREEITEM htiSel = m_ptrv->GetSelectedItem();
	
	return (htiSel) ? (CDossier*)m_ptrv->GetItemData(htiSel) : NULL;
}

// Mme chose que GetSelDossier, mais renvoie le dossier de l'archive
// s'il n'y a pas de slection
CDossier* CArboView::GetSelDossierEx() const
{
	CDossier *const pSelDossier = GetSelDossier();
	return (pSelDossier) ? pSelDossier : m_pArc;
}



// Cration
int CArboView::OnCreate(CREATESTRUCT* pcs)
{
	m_ptrv = &GetTreeCtrl();
	m_fRenommage = false;
	
	// Crer l'arborescence
	pcs->style |= TVS_HASBUTTONS | TVS_HASLINES | TVS_LINESATROOT |
		TVS_EDITLABELS | TVS_SHOWSELALWAYS;
	if (CTreeView::OnCreate(pcs)==-1)
		return -1;
	
	// Affecter la liste d'images systme  l'arborescence
	TreeView_SetImageList(*m_ptrv, oApp.m_himlSystem, TVSIL_NORMAL);
	
	// Enregistrer la cible de glisser-dplacer
	m_mdt.m_pDoc = GetDocument();
	m_mdt.Register(m_ptrv);
	
	// Passer la vue au document
	GetDocument()->m_ptrv = this;
	
	return 0;
}

// Destruction
void CArboView::OnDestroy()
{
	TreeView_SetImageList(*m_ptrv, NULL, TVSIL_NORMAL);
	
	CTreeView::OnDestroy();
	GetDocument()->m_ptrv = NULL;
}


// Mise  jour
void CArboView::OnUpdate(CView*, LPARAM, CObject*)
{
	CMuzipDoc *const pDoc = GetDocument();
	CMuzipArc *const pArc = pDoc->m_pArc;
	
	CDossier *const pCur = pDoc->m_pCur;
	
	// Vider l'arborescence et commencer  la remplir
	m_ptrv->DeleteAllItems();
	pArc->trvLister();
	
	// Ouvrir les items l'tant prcdemment,
	// grce au flag CDossier::m_fOuvert
	CPileCh<HTREEITEM> pilItem;
	pilItem.Push(m_ptrv->GetRootItem());
	while (pilItem.CanPop()) {
		HTREEITEM hti = pilItem.Pop();
		while (hti) {
			const CDossier *const pDossier = (CDossier*)m_ptrv->GetItemData(hti);
			
			// Si l'item tait ouvert, alors on l'ouvre !
			if (pDossier->m_fOuvert) {
				m_ptrv->Expand(hti, TVE_EXPAND);
				pilItem.Push(m_ptrv->GetChildItem(hti));
			}
			
			// L'item correspondant au dossier courant doit tre slectionn !
			if (pDossier==pCur)
				m_ptrv->SelectItem(hti);
			
			hti = m_ptrv->GetNextSiblingItem(hti);
		}
	}
}


// Demande d'informations
void CArboView::OnGetDispInfo(NMHDR* pNMHDR, LRESULT*)
{
	NMTVDISPINFO *const pNMDI = (NMTVDISPINFO*)pNMHDR;
	const CDossier *const pDossier = (const CDossier*)pNMDI->item.lParam;
	
	if (pNMDI->item.mask & TVIF_TEXT)
		lstrcpy(pNMDI->item.pszText, pDossier->m_pszNom);
	if (pNMDI->item.mask & TVIF_CHILDREN)
		pNMDI->item.cChildren = ToBool(pDossier->m_lstDossier.GetNb());
}

// Touche enfonce
void CArboView::OnKeyDown(NMHDR* pNMHDR, LRESULT*)
{
	if (((NMTVKEYDOWN*)pNMHDR)->wVKey==VK_DELETE)
		OnArchiveMoins();
}

// Slection modifie
void CArboView::OnSelChanged(NMHDR* pNMHDR, LRESULT*)
{
	CMuzipDoc *const pDoc = GetDocument();
	
	pDoc->m_pCur = (CDossier*)((NMTREEVIEW*)pNMHDR)->itemNew.lParam;
	pDoc->m_plst->OnUpdate(NULL,NULL,NULL);
	pDoc->m_pArc->m_pi->OnDossierChange();
}

// Expansion d'item
void CArboView::OnItemExpanding(NMHDR* pNMHDR, LRESULT* pResult)
{
	*pResult = FALSE;   // Ne pas annuler
	
	const NMTREEVIEW *const pNMTRV = (const NMTREEVIEW*)pNMHDR;
	CDossier *const pDossier = (CDossier*)pNMTRV->itemNew.lParam;
	
	pDossier->m_fOuvert = (pNMTRV->action==TVE_EXPAND);
	if (pDossier->m_fOuvert) {
		// Lister les sous-dossiers s'ils ne l'ont pas dj t
		if (!m_ptrv->GetChildItem(pNMTRV->itemNew.hItem))
			pDossier->trvLister(pNMTRV->itemNew.hItem);
	}
}


// Renommage - dbut
void CArboView::OnBeginLabelEdit(NMHDR* pNMHDR, LRESULT* pResult)
{
	m_fRenommage = (CDossier*)((const NMTVDISPINFO*)pNMHDR)->item.lParam!=m_pArc;
	*pResult = !m_fRenommage;
}

// Renommage - fin
void CArboView::OnEndLabelEdit(NMHDR* pNMHDR, LRESULT* pResult)
{
	m_fRenommage = false;
	
	const NMTVDISPINFO *const pNMDI = (const NMTVDISPINFO*)pNMHDR;
	CDossier *pDossier = (CDossier*)pNMDI->item.lParam;
	
	// Si on a annul le renommage, ne rien faire
	const LPTSTR pszNom = pNMDI->item.pszText;
	*pResult = (pszNom) ? pDossier->Renommer(pszNom) : FALSE;
	
	if (!*pResult) {
		if (!*pDossier->m_pszNom)
			// Cration de dossier annule: supprimer le dossier
			pDossier->Supprimer();
	}else{
		// Ajout OK: il faut retrier le contenu du dossier parent
		// Il faut POSTER le message pour qu'il soit excut
		// APRS l'ajout de l'item
		// C'est pour cela que la mthode SortChildren() n'est pas utilise
		pDossier = pDossier->m_pParent;
		m_ptrv->PostMessage(TVM_SORTCHILDREN, 0,
			(pDossier) ? (LPARAM)pDossier->m_hti : NULL);
	}
}


// Glisser-dplacer
void CArboView::OnBeginDrag(NMHDR* pNMHDR, LRESULT*)
{
	const TVITEM &itemNew = ((const NMTREEVIEW*)pNMHDR)->itemNew;
	
	// Construire la source de donnes OLE
	CMuzipDataSource mds;
	mds.m_pDossier = (CDossier*)itemNew.lParam;
	mds.DelayRenderData(CF_HDROP);
	mds.Init(m_pArc);
	
	// Effectuer le glisser-dplacer
	if (mds.DoDragDrop()==DROPEFFECT_MOVE &&
	    CMuzipDataSource::s_pmds) {
		const HWND hwndFocus = ::GetFocus();
		
		// Si CMuzipDataSource::s_pmds==NULL,
		// le dplacement a dj t effectu
		m_ptrv->SelectItem(itemNew.hItem);
		oApp.m_nConfirm--;
		OnArchiveMoins();
		oApp.m_nConfirm++;
		
		::SetFocus(hwndFocus);
	}
}


// Clic droit
void CArboView::OnRClick(NMHDR*, LRESULT*)
{
	HTREEITEM hti;
	
	// Rcuprer l'item cliqu
	CPoint pt;
	GetCursorPos(&pt);
	CPoint ptTemp = pt;
	ScreenToClient(&ptTemp);
	hti = m_ptrv->HitTest(ptTemp);
	if (!hti)
		return;
	m_ptrv->SelectItem(hti);
	
	// Crer le menu contextuel du dossier
	CMenu mnu;
	mnu.LoadMenu(IDM_CONTEXT);
	CMenu *const pMenu = mnu.GetSubMenu(1);
	pMenu->TrackPopupMenu(
		TPM_LEFTALIGN | TPM_RIGHTBUTTON,
		pt.x,pt.y, this);
}


// Supprimer
void CArboView::OnArchiveMoins()
{
	if (!CDlgConfirm::GetReponse(mcfArchiveMoins, DEM_ARCHIVEMOINS))
		return;
	
	CMuzipDoc *const pDoc = GetDocument();
	
	// Vider la liste de droite
	pDoc->m_plst->m_plst->DeleteAllItems();
	
	// Supprimer le dossier, et son parent s'il est vide
	CDossier *const pDossier = GetSelDossier();
	if (e_opt.m_fGeneralSupprDossierVide)
		pDossier->SupprimerEtParentSiVide();
	else
		pDossier->Supprimer();
	
	pDoc->m_pArc->FautReconstruire(rmoSupprimer);
	pDoc->m_plst->OnUpdate(NULL,NULL,NULL);
}

// Dcompresser
void CArboView::OnArchiveDecomp()
{
	CDlgDecomp dlgDecomp(true, m_pArc);
	if (dlgDecomp.DoModal()==IDCANCEL)
		return;
	
	// Dcompresser le dossier
	GetSelDossier()->DossierDecomp(dlgDecomp.m_sDest);
}


// Renommer
void CArboView::OnArchiveRenommer()
{
	m_ptrv->EditLabel(m_ptrv->GetSelectedItem());
}
void CArboView::OnUpdateArchiveRenommer(CCmdUI* pCmdUI)
{
	// Interdire le renommage du dossier racine
	CDossier *const pSelDossier = GetSelDossierEx();
	pCmdUI->Enable(pSelDossier!=pSelDossier->m_pArc &&
		!pSelDossier->m_pArc->IsLectureSeule());
}

// Proprits
void CArboView::OnArchiveProprietes()
{
	CDlgProprietes(GetSelDossier(), this).DoModal();
}

// Recompresser
void CArboView::OnArchiveRecomp()
{
	m_pArc->ArchiveRecomp(GetSelDossier());
}


// Mettre en cache
void CArboView::OnArchiveCacher()
{
	GetSelDossierEx()->CacheAddRef();
}


// Nouveau dossier
void CArboView::OnArchiveNouveauDossier()
{
	ArchiveNouveauDossier(GetSelDossier());
}
void CArboView::OnUpdateArchiveNouveauDossier(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(!m_fRenommage && GetSelDossier() &&
		(m_pArc->GetCap() & capArbo) && !m_pArc->IsLectureSeule());
}
void CArboView::ArchiveNouveauDossier(CDossier* pDossier)
{
	CDossier *const pnewDossier = new CDossier(pDossier, _T(""));
	pDossier->m_lstDossier.AddFin(pnewDossier);
	
	pnewDossier->trvLister();
	m_ptrv->EnsureVisible(pnewDossier->m_hti);
	m_ptrv->SelectItem   (pnewDossier->m_hti);
	OnArchiveRenommer();
}


// Tester
void CArboView::OnArchiveTester()
{
	CLstFichier lstFichier;
	GetSelDossierEx()->ListerFichier(lstFichier);
	GetDocument()->m_pArc->ArchiveTester(lstFichier);
}


// MAJ: un dossier doit tre slectionn
void CArboView::OnUpdateFautSel(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(!m_fRenommage && m_ptrv->GetSelectedItem());
}

// MAJ: un dossier doit tre slectionn,
// et l'archive doit tre modifiable
void CArboView::OnUpdateFautSelModif(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(!m_fRenommage && m_ptrv->GetSelectedItem() &&
		!GetDocument()->m_pArc->IsLectureSeule());
}


// Glisser-dplacer - Dbut de survol
DROPEFFECT CArboView::OnDragEnter(COleDataObject* pDataObject, DWORD dwKeyState, CPoint pt)
{
	return m_mdt.OnDragEnter(pDataObject, dwKeyState, pt);
}

// Glisser-dplacer - Survol
DROPEFFECT CArboView::OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint pt)
{
	// Rcuprer la cible ; s'il n'y en a pas, alors on prend le dossier racine
	HTREEITEM hti = m_ptrv->HitTest(pt);
	if (!hti)
		hti = GetDocument()->m_pArc->m_hti;
	m_ptrv->SelectDropTarget(hti);
	
	// Conserver le dossier cible
	GetDocument()->m_pDragD = (CDossier*)m_ptrv->GetItemData(hti);
	
	return m_mdt.OnDragOver(pDataObject, dwKeyState, pt);
}

// Glisser-dplacer - Sortie
void CArboView::OnDragLeave()
{
	m_ptrv->SelectDropTarget(NULL);
	GetDocument()->m_pDragD = NULL;
	m_mdt.OnDragLeave();
}


// Glisser-dplacer - Lcher
BOOL CArboView::OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint pt)
{
	// Rcuprer la cible ; s'il n'y en a pas, alors on prend le dossier racine
	HTREEITEM hti = m_ptrv->HitTest(pt);
	if (!hti)
		hti = GetDocument()->m_pArc->m_hti;
	
	// La cible devient le dossier courant
	m_ptrv->SelectItem(hti);
	
	const BOOL fRet = m_mdt.OnDrop(pDataObject, dropEffect, pt);
	
	m_ptrv->SelectDropTarget(NULL);
	GetDocument()->m_pDragD = NULL;
	
	GetParentFrame()->ActivateFrame();
	return fRet;
}


// Effectuer un dfilement
BOOL CArboView::OnScroll(UINT nScrollCode, UINT nPos, BOOL bDoScroll)
{
	if (bDoScroll)
		// Rellement dfiler
		// Le dfilement vertical est dans l'octet de poids fort
		// de nScrollCode
		SendMessage(WM_VSCROLL, MAKEWORD(HIBYTE(nScrollCode), nPos));
	
	return TRUE;
}
