// Dossier.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
#include "Muzip.h"
#include "Dossier.h"

#include "MuzipArc.h"
#include "Interf.h"
#include "ArboView.h"

#include "Decomp.h"

#include "IOFichier.h"
#include "Options.h"

#ifdef UNICODE
#include <GUnicode.h>
#endif


// Constructeur
CDossier::CDossier(CDossier* pParent, LPCTSTR pszNom)
{
	m_pParent = pParent;
	m_pArc    = (m_pParent) ? m_pParent->m_pArc : (CMuzipArc*)this;
	m_pszNom  = newDup(pszNom);
	m_pszPath = NULL;
	MAJPath();
	
	m_dtM.SetNow();
	
	m_fTemp   = false;
	
	m_fOuvert = false;
}

// Destructeur
CDossier::~CDossier()
{
	Clean();
	
	delete [] m_pszNom;
	delete [] m_pszPath;
}

// Supprimer les fichiers et les dossiers
void CDossier::Clean()
{
	m_lstFichier.DeleteAll();
	m_lstDossier.DeleteAll();
}



// Retourner le dossier d'un fichier, en le crant si ncessaire
// fTemp==true indique qu'il faut mettre  TRUE le drapeau fTemp
// des dossiers crs
// fDossier==true indique qu'on passe un nom de fichier et qu'il faut
// enlever la spcification de fichier
CDossier* CDossier::GetDossier(LPCTSTR psz, GETDOSSIER gd, bool fFichier)
{
	// Dupliquer le chemin d'accs pass
	// S'il s'agit d'un dossier, rajouter un antislash  la fin
	// Dans tous les cas enlever la spcification du fichier
	// Le dernier token (spar par des '\\' ou des '/' doit tre vide
	CString sPath = psz;
	sPath.Replace('/', '\\');
	sPath.GetBuffer(MAX_PATH);
	if (fFichier)
		PathRemoveFileSpec((STRING)sPath);
	PathRemoveBackslash((STRING)sPath);
	
	// - Isoler chaque partie du chemin d'accs
	//   et les rajouter  la liste si ncessaire
	// - Garder le sous-dossier de dernier niveau
	bool fExiste = true;
	CLstDossier *plstDossier = &m_lstDossier;
	CDossier *pDossierCur = this;
	TCHAR *pcPath = (STRING)sPath;
	CString sPartie;
	while (GetPathPartieSuiv(pcPath, sPartie)) {
		if (sPartie.IsEmpty())
			continue;
		
		if (fExiste) {
			// Vrifier que le sous-dossier existe bel et bien
			
			POS pos = plstDossier->GetPosDeb();
			while (pos) {
				CDossier *const pDossier = plstDossier->GetSuiv(pos);
				if (!lstrcmpi(sPartie, pDossier->m_pszNom)) {
					// Le dossier existe: OK
					pDossierCur = pDossier;
					goto SuitePartie;
				}
			}
		}
		
		
		// Le dossier n'existe pas: le crer
		// et imposer la cration pour les dossiers futurs
		if (gd==gdGet)
			return NULL;
		fExiste = false;
		
		// Crer et ajouter le dossier
		pDossierCur = new CDossier(pDossierCur, sPartie);
		plstDossier->AddFin(pDossierCur);
		
		// S'il s'agit d'un dossier temporaire, marquer le drapeau
		if (gd==gdCreerTemp)
			pDossierCur->m_fTemp = true;
		
		
		// Passer  la partie suivante
SuitePartie:
		plstDossier = &pDossierCur->m_lstDossier;
	}
	
	return pDossierCur;
}


// Compter les fichiers du dossier et de ses sous-dossiers
UINT CDossier::GetNbFichier() const
{
	UINT nbFichier = 0;
	POS pos;
	
	// Compter les fichiers non-ignors
	pos = m_lstFichier.GetPosDeb();
	while (pos)
		nbFichier += !m_lstFichier.GetSuiv(pos)->m_fIgnorer;
	
	// Poursuivre dans les sous-dossiers
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		nbFichier += m_lstDossier.GetSuiv(pos)->GetNbFichier();
	
	return nbFichier;
}

// Compter les sous-dossiers du dossier et ceux de ses sous-dossiers
UINT CDossier::GetNbDossier() const
{
	UINT nbDossier = m_lstDossier.GetNb();
	POS pos = m_lstDossier.GetPosDeb();
	while (pos)
		nbDossier += m_lstDossier.GetSuiv(pos)->GetNbDossier();
	return nbDossier;
}

// Lister les fichiers
void CDossier::ListerFichier(CLstFichier& lstFichier) const
{
	lstFichier.AddFin(m_lstFichier);
	POS pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->ListerFichier(lstFichier);
}

// Mettre  jour le chemin d'accs du dossier et de ses sous-dossiers
void CDossier::MAJPath()
{
	// Dtruire l'ancien chemin d'accs
	delete [] m_pszPath;
	
	if (m_pParent) {
		// Composer le nouveau
		
		size_t lenParent = lstrlen(m_pParent->m_pszPath);
		if (lenParent) {
			// Le parent n'est pas la racine
			
			m_pszPath = new TCHAR[lenParent + lstrlen(m_pszNom)+2];
			lstrcpy(m_pszPath, m_pParent->m_pszPath);
			m_pszPath[lenParent++] = '\\';
			lstrcpy(m_pszPath+lenParent, m_pszNom);
			
		}else
			// Le parent est la racine
			m_pszPath = newDup(m_pszNom);
		
	}else
		// Dossier racine: pas de chemin d'accs
		*(m_pszPath = new TCHAR[1]) = 0;
	
	// Poursuivre dans les sous-dossiers
	POS pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->MAJPath();
}

// Mettre tous les fichiers du dossier  "il ne faut pas compresser"
// Utilis pour viter que des fichiers soient ignors par OpenArchiveTemp()
// qui croirait que le fichier serait ajout plus tard par ArchivePlus()
// En effet "il faut compresser" revient  dire "ne PAS recopier tel quel
//  partir de l'ancienne archive"
void CDossier::SetFichierFautPasComp()
{
	POS pos;
	
	// Traiter les fichiers
	pos = m_lstFichier.GetPosDeb();
	while (pos)
		m_lstFichier.GetSuiv(pos)->m_fFautComp = false;
	
	// Traiter les sous-dossiers
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->SetFichierFautPasComp();
}

// Initialiser les fichiers et dossiers
// avant une recherche de requte de compression
void CDossier::AvantRecherche(bool fTemp)
{
	POS pos;
	
	// Traiter les fichiers
	pos = m_lstFichier.GetPosDeb();
	while (pos) {
		CFichier *const pFichier = m_lstFichier.GetSuiv(pos);
		pFichier->m_fTrouveAjout =
		pFichier->m_fFautComp    = false;
		pFichier->m_fSupprimer  |= fTemp;
	}
	
	// Traiter les sous-dossiers
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->AvantRecherche(fTemp);
	
	// Traiter le dossier lui-mme
	m_fTemp |= fTemp;
}


// Supprimer le dossier
void CDossier::Supprimer()
{
	m_pArc->m_pi->OnDossierSupprimer(this);
	
	// Supprimer fichiers et sous-dossiers
	m_lstFichier.DeleteAll();
	m_lstDossier.DeleteAll();
	
	if (m_pParent)
		// Enlever le dossier de la liste de son parent
		// et le dtruire en mme temps
		m_pParent->m_lstDossier.DeleteAt(
			m_pParent->m_lstDossier.Find(this));
}

// Supprimer tous les sous-dossiers vides
// puis le dossier lui-mme s'il est vide
// Retourner TRUE si le dossier a pu tre vid entirement et donc supprim
bool CDossier::SupprimerSiVide()
{
	// Traiter les sous-dossiers
	POS pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->SupprimerSiVide();
	
	// Il ne doit rester ni fichiers ni dossier
	VERIF(m_lstFichier.GetNb()+m_lstDossier.GetNb()==0);
	
	// Supprimer le dossier, et indiquer qu'il est vide en retournant TRUE
	Supprimer();
	return true;
}

// Supprimer le dossier, et ses parents s'ils sont vides
void CDossier::SupprimerEtParentSiVide()
{
	// Remonter l'arborescence tant que le parent
	// n'a aucun fichier mais un seul sous-dossier (le fils)
	CDossier *pDossier = this;
	CDossier *pParent;
	for (;;) {
		pParent = pDossier->m_pParent;
		if (!pParent ||
		     pParent->m_lstFichier.GetNb() ||
		     pParent->m_lstDossier.GetNb()>1)
			break;
		pDossier = pParent;
	}
	
	// Supprimer le dernier dossier trouv
	pDossier->Supprimer();
}

// Marquer comme devant tre supprims
// les fichiers non-trouvs du dossier et de ses sous-dossiers
void CDossier::SetNonTrouveFautSupprimer()
{
	POS pos;
	
	// Traiter les fichiers
	pos = m_lstFichier.GetPosDeb();
	while (pos) {
		CFichier *const pFichier= m_lstFichier.GetSuiv(pos);
		pFichier->m_fSupprimer |= !pFichier->m_fTrouveAjout;
	}
	
	// Traiter les sous-dossiers
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->SetNonTrouveFautSupprimer();
}

// Supprimer les fichiers et dossiers marqus comme tels
// Ne supprimer les dossiers que s'ils sont vides
void CDossier::SupprimerSiTempVide()
{
	POS pos;
	
	// Traiter les fichiers
	pos = m_lstFichier.GetPosDeb();
	while (pos) {
		const POS posSauve = pos;
		if (m_lstFichier.GetSuiv(pos)->m_fSupprimer)
			m_lstFichier.DeleteAt(posSauve);
	}
	
	// Traiter les sous-dossiers
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->SupprimerSiTempVide();
	
	// Traiter le dossier lui-mme
	if (m_fTemp)
		SupprimerSiVide();
}


// Renommer le dossier
bool CDossier::Renommer(LPTSTR pszNom)
{
	// S'assurer que le nom est correct
	VERIF(NomCorriger(pszNom));
	
	// Vrifier qu'il n'y a pas de doublons avec les dossiers
	// de mme rang dans l'arborescence
	CLstDossier &lstDossier = m_pParent->m_lstDossier;
	POS pos = lstDossier.GetPosDeb();
	while (pos) {
		CDossier *const pDossier = lstDossier.GetSuiv(pos);
		if (!lstrcmpi(pszNom, pDossier->m_pszNom)) {
			
			// Si le dossier se renomme en lui-mme, tout est OK (pas de doublon)
			if (pDossier==this)
				return true;
			
			// Doublon
			AfxFormatString2(e_sMsg, ERR_RENOMDOUBLOND, pszNom,pDossier->m_pszNom);
			AfxMessageBox(e_sMsg);
			return false;
		}
	}
	
	// Pas de doublons: on peut renommer
	delete [] m_pszNom;
	m_pszNom = newDup(pszNom);
	// Mettre  jour le chemin d'accs du dossier
	// et de ses sous-dossiers
	MAJPath();
	
	// Reconstruire l'archive
	bool fOK = false;
	if (!m_pArc->m_pi->IsModified() && (m_pArc->GetCap() & capRapideRenommer)) {
		// Dplacement rapide permis: l'effectuer pour chaque fichier
		if (m_pArc->WriteOpen()) {
			fOK = RenommerSuite();
			m_pArc->WriteClose();
		}
	}
	if (!fOK)
		m_pArc->FautReconstruire(rmoDeplacer);
	
	return true;
}

// Avertir les fichiers du renommage
bool CDossier::RenommerSuite()
{
	POS pos;
	
	// Traiter les fichiers
	pos = m_lstFichier.GetPosDeb();
	while (pos)
		VERIF(m_lstFichier.GetSuiv(pos)->OnRenommer());
	
	// Traiter les sous-dossiers
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		VERIF(m_lstDossier.GetSuiv(pos)->RenommerSuite());
	
	return true;
}


// Effectuer une recherche
void CDossier::Rechercher(const RECHOPTIONS& rch, PFN_ONFICHIERTROUVE pfnOnFichierTrouve)
{
	RECH rech;
	(RECHOPTIONS&)rech = rch;
	rech.pfnOnFichierTrouve = pfnOnFichierTrouve;
	rech.GetMskAttr(rech.mskAttrO, rech.mskAttrN);
	
	
	if (!rech.DO_sContenu.IsEmpty()) {
		// Prparer la recherche
		
		SPACE_DECOMP::e_lenRech = lstrlen(rech.DO_sContenu);
#ifdef UNICODE
		SPACE_DECOMP::e_pszRech = new char[SPACE_DECOMP::e_lenRech+1];
		strtoA(SPACE_DECOMP::e_pszRech, rech.DO_sContenu, SPACE_DECOMP::e_lenRech+1);
#else
		SPACE_DECOMP::e_pszRech = (STRING)rech.DO_sContenu;
#endif
		m_pArc->PrepareDecomp();
	}
	
	RechercherSuite(rech);
	
#ifdef UNICODE
	if (!rech.DO_sContenu.IsEmpty())
		// Nettoyer la recherche
		delete [] SPACE_DECOMP::e_pszRech;
#endif
	
	// Mettre  jour les rsultats de la recherche
	m_pArc->m_pi->MAJResRech();
}

// Effectuer la recherche (fonction interne et rcursive)
void CDossier::RechercherSuite(const RECH& rech)
{
	POS pos;
	
	const bool fdtM = ToBool(m_pArc->GetCap() & capdtM);
	
	pos = m_lstFichier.GetPosDeb();
	while (pos) {
		bool fTrouve = false;
		CFichier *const pFichier = m_lstFichier.GetSuiv(pos);
		
		// Spcification
		if (!PathMatchSpec(pFichier->m_pszNom, rech.DO_sFiltre))
			goto PasTrouve;
		
		// Attributs
		if (!(pFichier->m_dwAttr & rech.mskAttrO))  goto PasTrouve;
		if (  pFichier->m_dwAttr & rech.mskAttrN )  goto PasTrouve;
		
		// Taille
		if (rech.TA_nActive) {
			const DWORD lenAjuste = pFichier->m_dwTailleN>>(rech.TA_nUnite*10);
			if       (rech.TA_nOp==0)   {  // ==
				if (rech.TA_uTaille!=lenAjuste)  goto PasTrouve;
			}else if (rech.TA_nOp==1)   {  // >=
				if (rech.TA_uTaille>=lenAjuste)  goto PasTrouve;
			}else /* (rech.TA_nOp==2)*/ {  // <=
				if (rech.TA_uTaille<=lenAjuste)  goto PasTrouve;
			}
		}
		
		// Date
		if (rech.DA_rdM.fActive && fdtM &&
		   !rech.DA_rdM.Correspond(pFichier->m_dtM))   goto PasTrouve;
		
		// Contenu (en dernier car a prend du temps)
		if (!rech.DO_sContenu.IsEmpty()) {
			SPACE_DECOMP::e_dm      = dmRechercher;
			SPACE_DECOMP::e_fTrouve = false;
			SPACE_DECOMP::e_lenMem  = 0;
			pFichier->Decomp(NULL, &e_decEcraseToujours);
			SPACE_DECOMP::e_dm = dmNormal;
			fTrouve = SPACE_DECOMP::e_fTrouve;
		}else
			fTrouve = true;
		
PasTrouve:
		pFichier->m_fTrouveRech = fTrouve;
		if (fTrouve)
			rech.pfnOnFichierTrouve(pFichier);
	}
	
	// Poursuivre la recherche dans les sous-dossiers
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->RechercherSuite(rech);
}

// Poursuivre la recherche
void CDossier::RechercherSuivant(int nItem) const
{
	m_pArc->m_pi->RechercherSuivant(this, nItem);
}

// Lister le dossier dans l'arborescence
void CDossier::trvLister() const
{
	TVINSERTSTRUCT tvis;
	tvis.hParent             = (m_pParent) ? m_pParent->m_hti : NULL;
	tvis.hInsertAfter        = TVI_SORT;
	tvis.item.mask           = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_PARAM | TVIF_CHILDREN;
	tvis.item.iImage         = oApp.m_iIconRepClosed;
	tvis.item.iSelectedImage = oApp.m_iIconRepOpen;
	tvis.item.state          = 0;
	tvis.item.stateMask      = 0;
	tvis.item.cChildren      = I_CHILDRENCALLBACK;
	tvis.item.pszText        = m_pszNom;
	tvis.item.lParam         = (LPARAM)this;
	trvLister(m_pArc->m_pi->InsertItem(&tvis));
}

// Lister le contenu du dossier dans l'arborescence
// d'aprs son handle d'item
void CDossier::trvLister(HTREEITEM hti) const
{
	m_hti = hti;
	if (!m_fOuvert)
		return;
	
	// Lister les sous-dossiers
	POS pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->trvLister();
}


// Copier l'entre d'un fichier  un autre
bool CDossier::FichierCopier(CFileRead& fr)
{
	POS pos = m_lstFichier.GetPosDeb();
	while (pos)
		VERIF(m_lstFichier.GetSuiv(pos)->FichierCopier(fr));
	
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		VERIF(m_lstDossier.GetSuiv(pos)->FichierCopier(fr));
	
	return true;
}


// Rcuprer le nom du dossier dans le cache
void CDossier::GetCacheDir(CString& sDir) const
{
	PathCombine(sDir.GetBuffer(MAX_PATH),
		m_pArc->GetCachePath(), m_pszPath);
	sDir.ReleaseBuffer();
}

// Mettre le dossier en cache
// nTaille est la taille maximale totale (en octets)
// des fichiers qu'il est possible de mettre en cache
// Si fOK==false, les compteurs d'utilisation des fichiers
// seront incrments mais les fichiers ne seront pas mis en cache
#pragma warning (disable: 4706)
bool CDossier::CacheAddRef(DWORD* pdwTaille, CFichier* pFichierIgnorer)
{
	m_pArc->PrepareDecomp();
	bool fOK = true;
	
	POS pos;
	CString sFile;
	
	// Mettre les fichiers du dossier en cache
	pos = m_lstFichier.GetPosDeb();
	while (pos) {
		CFichier *const pFichier = m_lstFichier.GetSuiv(pos);
		
		// Essayer de mettre le fichier en cache
		// Si on y arrive, il reste moins de place dans le cache
		if (pFichier!=pFichierIgnorer &&
		    (fOK &= pFichier->CacheAddRefInterne(sFile,
		      (!pdwTaille || *pdwTaille > pFichier->m_dwTailleN))))
			if (pdwTaille)
				*pdwTaille -= pFichier->m_dwTailleN;
	}
	
	// Mettre les sous-dossiers en cache
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		fOK &= m_lstDossier.GetSuiv(pos)->CacheAddRef(pdwTaille);
	
	return fOK;
}
#pragma warning (default: 4706)

// Enlever du cache le dossier
void CDossier::CacheRelease()
{
	// Dcacher les fichiers du dossier
	m_pArc->CacheRelease(m_lstFichier);
	
	// Dcacher les sous-dossiers
	POS pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->CacheRelease();
}



// Commencer une dcompression de dossier entier
bool CDossier::DossierDecomp(LPCTSTR pszDest, const DECOMP* pdec)
{
	m_pArc->PrepareDecomp();
	
	TCHAR pszDestFull[MAX_PATH];
	MyGetFullPathName(pszDest, pszDestFull);
	
	// Paramtres par dfaut = paramtres courants
	if (!pdec)
		pdec = &oApp.m_dec;
	
	if (pdec->fArbo) {
		// Respecter l'arborescence
		
		// Rcuprer les fichiers
		CLstFichierDecomp lstFichierDecomp(true);
		CLstStr1 lstRep(true);
		DossierDecompSuite(lstFichierDecomp, pszDestFull, pdec, lstRep);
		
		// Les dcompresser
		((DECOMP*)pdec)->fArbo = false;
		const bool fOK = m_pArc->ArchiveDecomp(lstFichierDecomp, pdec);
		((DECOMP*)pdec)->fArbo = true;
		return fOK;
	}
	
	VERIF(DossierCreer(pszDestFull));
	
	// Dcompresser tous les fichiers au mme endroit
	CLstFichier lstFichier;
	ListerFichier(lstFichier);
	return m_pArc->ArchiveDecomp(lstFichier, pszDestFull, pdec);
}

// Lister le contenu du dossier entier en vue de dcompression
bool CDossier::DossierDecompSuite(CLstFichierDecomp& lstFichierDecomp, LPCTSTR pszDest, const DECOMP* pdec, CLstStr1& lstRep)
{
	while (!DossierCreer(pszDest, false)) {
		AfxFormatString1(e_sMsg, ERR_DOSSIERCREER, pszDest);
		const int ret = AfxMessageBox(e_sMsg, MB_ABORTRETRYIGNORE | MB_DEFBUTTON2);
		if (ret == IDABORT)
			return false;
		if (ret == IDIGNORE)
			return true;
	}
	
	// Composer le dossier d'extraction, avec un antislash final
	LPTSTR pszDestSuite = new TCHAR[MAX_PATH];
	lstrcpy(pszDestSuite, pszDest);
	TCHAR *const pcFin = PathAddBackslash(pszDestSuite);
	lstRep.AddDeb(pszDestSuite);
	
	// Lister les fichiers  dcompresser
	lstFichierDecomp.Ajouter(m_lstFichier, pdec, pszDestSuite, pcFin);
	
	// Dcompresser les sous-dossiers
	POS pos = m_lstDossier.GetPosDeb();
	while (pos) {
		CDossier *const pDossier = m_lstDossier.GetSuiv(pos);
		lstrcpy(pcFin, pDossier->m_pszNom);
		VERIF(pDossier->DossierDecompSuite(lstFichierDecomp, pszDestSuite, pdec, lstRep));
	}
	
	return true;
}


// Lister les macros
// La reconnaissance ne s'effectue que sur l'extension des fichiers,
// certaines macros n'en sont donc peut-tre pas tant qu'on n'a pas
// rellement ouvert la macro
void CDossier::ListMacro(CLstFichier& lstFichierMacro)
{
	POS pos;
	
	// Chercher les macros dans les fichiers
	pos = m_lstFichier.GetPosDeb();
	while (pos) {
		CFichier *const pFichier = m_lstFichier.GetSuiv(pos);
		
		if (PathMatchSpec(pFichier->m_pszNom, _T("*.mum")))
			// On a trouv un fichier qui pourrait convenir
			lstFichierMacro.AddFin(pFichier);
	}
	
	// Poursuivre la recherche dans les sous-dossiers
	pos = m_lstDossier.GetPosDeb();
	while (pos)
		m_lstDossier.GetSuiv(pos)->ListMacro(lstFichierMacro);
}
