// Fichier.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
#include "Muzip.h"
#include "Fichier.h"

#include "MuzipArc.h"
#include "Interf.h"
#include "PlusItem.h"

#include "Decomp.h"

#include "IOFichier.h"

#include "dlgConfirm.h"

#include <GUnicode.h>


UINT CFichier::s_nbFichier;
UINT CFichier::s_nFichier;


// Constructeur
CFichier::CFichier() : m_pszNom(NULL), m_pArc(NULL), m_fFautComp(false),
	m_fIgnorer(false), m_fSupprimer(false), m_nCache(0)
{
}

// Destructeur
CFichier::~CFichier()
{
	if (m_pArc)
		m_pArc->m_pi->OnFichierSupprimer(this);
	
	delete [] m_pszNom;
}


// Lister le fichier
void CFichier::lstLister(CListCtrl& lst, int nItem, bool fCallback)
{
	// Rcuprer l'index de l'icne et le type du fichier
	SHFILEINFO shfi;
	SHGetFileInfo(m_pszNom, 0, &shfi, sizeof(shfi),
		SHGFI_USEFILEATTRIBUTES |
		SHGFI_SYSICONINDEX | SHGFI_SMALLICON |
		SHGFI_TYPENAME);
	
	LVITEM lvi;
	TCHAR psz[MAX_PATH];
	LPTSTR pszCallback = (fCallback) ? LPSTR_TEXTCALLBACK : psz;
	
	// Ajouter l'item
	lvi.iItem     = nItem;
	lvi.mask      = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE;
	lvi.iSubItem  = 0;
	if (!fCallback)
		GetSubItemText(lvi.iSubItem, psz, nbArray(psz));
	lvi.pszText   = pszCallback;
	lvi.iImage    = shfi.iIcon;
	lvi.lParam    = (LPARAM)this;
	lvi.stateMask = LVIS_CUT;
	lvi.state     = (IsCrypte()) ? LVIS_CUT : 0;
	lvi.iItem     = lst.InsertItem(&lvi);
	
	// Sous-items
	lvi.mask      = LVIF_TEXT | LVIF_IMAGE;
	
	// Type
	lvi.iSubItem++;
	lvi.pszText = shfi.szTypeName;
	lst.SetItem(&lvi);
	
	// Autres items
	while (lvi.iSubItem<7) {
		lvi.iSubItem++;
		if (!fCallback)
			GetSubItemText(lvi.iSubItem, psz, nbArray(psz));
		lvi.pszText = pszCallback;
		lst.SetItem(&lvi);
	}
}


// Retourner le contenu du texte d'une colonne
// pour le fichier dans une liste
void CFichier::GetSubItemText(int iSubItem, LPTSTR psz, int buf) const
{
	LPCTSTR pszSource = NULL;
	
	switch (iSubItem) {
		
		// Nom
		case 0:
			pszSource = m_pszNom;
			break;
		
		// Taille
		case 2:
			StrFormatByteSize(m_dwTailleN, psz, buf);
			return;
		
		// Compress
		case 3:
			StrFormatByteSize(m_dwTailleC, psz, buf);
			return;
		
		// %
		case 4:
			wsprintf(psz, _T("%hd %%"), m_nPC);
			return;
		
		// Modifi
		case 5:
			{
				SYSTEMTIME st = m_dtM;
				*psz = 0;
				const int bufOut = GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &st, NULL, psz, buf);
				if (e_opt.m_fAffichageModifHeure && bufOut<buf) {
					// Rajouter l'heure  la date
					if (bufOut)
						psz[bufOut-1] = ' ';
					GetTimeFormat(LOCALE_USER_DEFAULT, 0, &st, NULL, &psz[bufOut], buf-bufOut);
				}
			}
			return;
		
		// Attributs
		case 6:
			pszSource = m_pszAttr;
			break;
		
		// Chemin
		case 7:
			pszSource = m_pDossier->m_pszPath;
			break;
	}
	
	lstrcpyn(psz, pszSource, buf);
}


// Retourner TRUE si le fichier est l'archive elle-mme
bool CFichier::CompresseArchive()
{
	TCHAR pszFichier[MAX_PATH];
	MyGetFullPathName(m_pszNomES, pszFichier);
	
	return !lstrcmpi(pszFichier, m_pArc->m_sPath);
}

// Initialiser le fichier
void CFichier::NewFichier(LPCTSTR pszFichier, const WIN32_FIND_DATA& wfd)
{
	m_pszNomES  = newDup(pszFichier);
	m_pszNom    = newDup(wfd.cFileName);
	m_dtM       = wfd.ftLastWriteTime;
	m_dwTailleC =
	m_dwTailleN = wfd.nFileSizeLow;
	m_dwAttr    = wfd.dwFileAttributes;
	MAJ();
}

// Dfinir la date de modification du fichier, entre au format Unix
void CFichier::SetUnixDateM(DWORD dtM)
{
	FILETIME ft;
	SYSTEMTIME st;
	DosDateTimeToFileTime(HIWORD(dtM), LOWORD(dtM), &ft);
	FileTimeToSystemTime(&ft, &st);
	m_dtM = st;
}

// Dfinir la date de modification du fichier au format Unix
DWORD CFichier::GetUnixDate(const CTempsDHS& dt)
{
	DWORD dtUnix;
	
	const SYSTEMTIME st = dt;
	FILETIME ft;
	SystemTimeToFileTime(&st, &ft);
	FileTimeToDosDateTime(&ft, ((WORD*)&dtUnix)+1, (WORD*)&dtUnix);
	
	return dtUnix;
}


// Supprimer le fichier de son dossier
void CFichier::Supprimer(bool fSupprimerParentSiVide)
{
	CDossier *const pDossier = m_pDossier;
	
	pDossier->m_lstFichier.DeleteAt(
		pDossier->m_lstFichier.Find(this));
	
	if (fSupprimerParentSiVide)
		pDossier->SupprimerSiVide();
}

// Renommer le fichier
bool CFichier::Renommer(LPTSTR pszNom)
{
	// S'assurer que le nom est correct
	VERIF(NomCorriger(pszNom));
	
	// Vrifier qu'il n'y a pas de doublons avec les fichiers
	// appartenant au mme dossier que le fichier
	CLstFichier &lstFichier = m_pDossier->m_lstFichier;
	POS pos = lstFichier.GetPosDeb();
	while (pos) {
		CFichier *const pFichier = lstFichier.GetSuiv(pos);
		if (!lstrcmpi(pszNom, pFichier->m_pszNom)) {
			
			// Si le fichier est renomm en lui-mme, alors c'est OK (pas de doublon)
			if (pFichier==this)
				return true;
			
			// Doublon
			AfxFormatString2(e_sMsg, ERR_RENOMDOUBLONF, pszNom,pFichier->m_pszNom);
			AfxMessageBox(e_sMsg);
			return false;
		}
	}
	
	// Renommer le fichier
	delete [] m_pszNom;
	m_pszNom = newDup(pszNom);
	
	// Reconstruire l'archive
	bool fOK = false;
	if (!m_pArc->m_pi->IsModified() && (m_pArc->GetCap() & capRapideRenommer)) {
		// Dplacement rapide permis: l'effectuer pour chaque fichier
		
		if (m_pArc->WriteOpen()) {
			fOK = OnRenommer();
			m_pArc->WriteClose();
		}
	}
	
	if (!fOK)
		m_pArc->FautReconstruire(rmoDeplacer);
	
	return true;
}


// Mettre  jour les donnes calcules du fichier:
// pourcentage de compression, attributs
void CFichier::MAJ()
{
	// Pourcentage de compression
	m_nPC = 100;
	if (m_dwTailleN)
		m_nPC -= m_dwTailleC*100/m_dwTailleN;
	
	// Attributs
	int i = 0;
	if (m_dwAttr & FILE_ATTRIBUTE_READONLY)  m_pszAttr[i++] = 'R';
	if (m_dwAttr & FILE_ATTRIBUTE_HIDDEN)    m_pszAttr[i++] = 'H';
	if (m_dwAttr & FILE_ATTRIBUTE_SYSTEM)    m_pszAttr[i++] = 'S';
	if (m_dwAttr & FILE_ATTRIBUTE_ARCHIVE)   m_pszAttr[i++] = 'A';
	m_pszAttr[i] = 0;
}


// Dcompresser le fichier
bool CFichier::ArchiveDecompSuite(LPCTSTR pszRep, LPTSTR* ppszDest, const DECOMP* pdec)
{
	*ppszDest = new TCHAR[MAX_PATH];
	PathCombine(*ppszDest, pszRep, m_pszNom);
	
	// Dcompresser le fichier
	if (!Decomp(*ppszDest, pdec)) {
		// Erreur
		delete [] *ppszDest;
		return false;
	}
	
	return true;
}

// Dcompresser le fichier
bool CFichier::Decomp(LPCTSTR pszDest, const DECOMP* pdec)
{
	ASSERT(SPACE_DECOMP::e_pfr);
	
	const bool fOK = DecompInterne(pszDest);
	
	// Supprimer le fichier s'il y a eu une erreur
	// et que l'utilisateur ne veut pas garder les fichiers abms
	if (pszDest && !fOK && !pdec->fGardeErreur)
		DeleteFile(pszDest);
	
	return fOK;
}


// Recompresser un fichier modifi
bool CFichier::RecompModif(LPCTSTR pszFile)
{
	CArchivePlus arPlus(m_pDossier);
	CPlusItem pi;
	ToPlusItem(pi);
	pi.DO_sFiltre = pszFile;
	arPlus.AddDeb(&pi);
	return m_pArc->ArchivePlus(arPlus);
}


// Excute un fichier et surveiller son excution
void CFichier::SurvExecuter(LPCTSTR pszFichier, LPCTSTR pszRep, LPCTSTR pszVerbe) const
{
	// Ouvrir le fichier
	SHELLEXECUTEINFO sei;
	ZeroMemory(&sei, sizeof(sei));
	sei.cbSize       = sizeof(sei);
	sei.fMask        = SEE_MASK_NOCLOSEPROCESS | SEE_MASK_FLAG_NO_UI;
	sei.lpVerb       = pszVerbe;
	sei.lpFile       = pszFichier;
	sei.lpDirectory  = pszRep;
	sei.nShow        = SW_SHOWDEFAULT;
	if (!ShellExecuteEx(&sei)) {
		// Le fichier n'a pas pu tre ouvert:
		// demander s'il faut essayer avec Notepad
		
		if (!CDlgConfirm::GetReponse(mcfOuvrirNotepad, DEM_OUVRIRNOTEPAD))
			goto Supprimer;
		
		sei.lpFile       = _T("Notepad.exe");
		sei.lpParameters = pszFichier;
		if (!ShellExecuteEx(&sei))
			goto Supprimer;
	}
	
	// Surveiller le fichier dcompress
	new CSurvDecomp(sei.hProcess, pszFichier, (CFichier*)this, m_pArc);
	return;
	
Supprimer:
	// Erreur d'ouverture: supprimer le fichier
	FichierSupprimer(pszFichier, false);
}


// Rcupre le nom du fichier dans le cache
void CFichier::GetCacheFile(CString& sFile) const
{
	m_pDossier->GetCacheDir(sFile);
	PathAppend(sFile.GetBuffer(MAX_PATH), m_pszNom);
	sFile.ReleaseBuffer();
}

// Dcompresser le fichier dans le cache
// sFile est fourni EN SORTIE
// La fonction est totalement autonome
// Pour dcompresser plusieurs fichiers, il ne faut pas l'appeler
// car on perdrait des performances  faire plein de fois la mme chose
bool CFichier::CacheAddRef(CString& sFile)
{
	m_pArc->PrepareDecomp();
	
	const bool fOK = CacheAddRefInterne(sFile, true);
	if (fOK && !(e_opt.m_fCachePrevAmovible && m_pArc->IsSurDisqueAmovible())) {
		DWORD dwTaille = e_opt.m_uCachePrevMax<<10;
		m_pDossier->CacheAddRef(&dwTaille, this);
	}
	
	return fOK;
}

// Mme chose que CacheAddRef() mais optimis pour les dcompressions multiples
bool CFichier::CacheAddRefInterne(CString& sFile, bool fOK)
{
	m_nCache++;
	VERIF(fOK);
	
	ASSERT(SPACE_DECOMP::e_pfr == &m_pArc->m_fr);
	
	GetCacheFile(sFile);
	
	// Si le fichier est dj dans le cache, pas besoin de le dcompresser
	if (PathFileExists(sFile))
		return true;
	
	// Crer le rpertoire
	CString sDir;
	m_pDossier->GetCacheDir(sDir);
	VERIF(DossierCreer(sDir));
	
	// Dcompresser le fichier
	return Decomp(sFile, &e_decEcraseToujours);
}

// Un fichier cach n'est plus requis
void CFichier::CacheRelease()
{
	if (CacheReleaseNoDelete()) {
		// Le fichier n'est pas absolument ncessaire
		// Regarder s'il faut le dcacher
		
		if (m_dwTailleN>>10 > e_opt.m_uCacheGarderMax &&
		    !(e_opt.m_fCacheGarderAmovible && m_pArc->IsSurDisqueAmovible())) {
			// Dcacher le fichier
			CString sFile;
			GetCacheFile(sFile);
			DeleteFile(sFile);
		}
	}
}

// Un fichier cach n'est plus requis, mais il faut le garder dans le cache
// Retourner TRUE si la dernire instance du cache a t libre
// (il faudrait donc supprimer le fichier du cache)
bool CFichier::CacheReleaseNoDelete()
{
	ASSERT(m_nCache > 0);
	
	return (--m_nCache) == 0;
}



// Remplir une classe CPlusItem avec les donnes permettant
// de recompresser le fichier  l'identique
void CFichier::ToPlusItem(CPlusItem& pi) const
{
	pi.SetDefautExt();
	
	pi.CO_nArbo  = arboNon;
	
	pi.OP_fSous  = FALSE;
	pi.OP_fCache = FALSE;
	
	// pi.CO_nNiveau:  faire dans les classes drives de CFichier
}


// Copier l'entre d'un fichier  un autre
// Ici on ne fait que mettre  jour la progression
bool CFichier::FichierCopier(CFileRead& fr)
{
	// m_fFautComp indique qu'il faut compresser le fichier
	// et non se contenter de le recopier  partir de l'ancienne archive.
	// Si m_fSupprimer == true, le fichier ne doit pas tre conserv
	// dans la nouvelle archive.
	if (m_fFautComp || m_fSupprimer)
		return true;
	
	if (s_nbFichier) {
		oApp.SetProgress(STA_COPIEVIEUX, m_pszNom, s_nFichier, s_nbFichier);
		oApp.SetStatusAvance(s_nFichier++);
		VERIF(oApp.UpdateProgressStatus() == prsOK);
	}
	
	return FichierCopierInterne(fr);
}

// Copier des donnes en mettant  jour la barre de progression principale
bool CFichier::FichierCopierHelper(CFileRead& fr, DWORD len, bool fAvance2)
{
	while (len) {
		
		// On prend CFileBuf::lenFileBuf+1
		// pour passer en mode direct (pas de buffer)
		const DWORD lenES = (CFileBuf::lenFileBuf + 1 > len)
			? len : CFileBuf::lenFileBuf+1;
		
		// crire
		VERIF(e_fw.WriteFrom(fr, lenES));
		
		// Mettre  jour
		if (!s_nbFichier)  oApp.AddStatusAvance(0, lenES);
		if (fAvance2)      oApp.AddStatusAvance(1, lenES);
		len -= lenES;
		
		VERIF(oApp.UpdateProgressStatus() == prsOK);
	}
	
	return true;
}


// Dfinir les proprits
void CFichier::SetProp(DWORD dwAttr, CTempsDHS& dtModif)
{
	if (m_dwAttr==dwAttr && m_dtM==dtModif)
		return;
	
	// Attributs et date de modification
	m_dwAttr = dwAttr;
	m_dtM    = dtModif;
	
	// Mettre  jour l'item
	MAJ();
	m_pArc->m_pi->UpdateFichier(this);
	
	// Reconstruire
	bool fOK = false;
	if (!m_pArc->m_pi->IsModified() && (m_pArc->GetCap() & capRapideModifProp)) {
		if (m_pArc->WriteOpen()) {
			fOK = OnModifProp();
			m_pArc->WriteClose();
		}
	}
	
	if (!fOK)
		m_pArc->FautReconstruire(rmoProprietes);
}
