// Fichier.h


#ifndef _FICHIER_H_
#define _FICHIER_H_
#if _MSC_VER>1000
#pragma once
#endif

#include "Global.h"
#include "Decomp.h"

class CMuzipArc;
class CPlusItem;
class CRechDate;

class CFileRead;
class CFileWrite;

struct DECOMP;


class CFichier
{
	friend class CMuzipArc;
	friend class CZipArc;
	
public:  // Fonctions
	
	CFichier();
	virtual ~CFichier();
	
	void lstLister(CListCtrl& lst, int nItem, bool fCallback=true);
	void GetSubItemText(int iSubItem, LPTSTR psz, int buf) const;
	
	bool CompresseArchive();
	bool FichierCopier(CFileRead& fr);
	
	void NewFichier(LPCTSTR pszFichier, const WIN32_FIND_DATA& wfd);
	void SetUnixDateM(DWORD dtM);
	static DWORD GetUnixDate(const CTempsDHS& dt);
	
	void Supprimer(bool fSupprimerParentSiVide);
	bool Renommer(LPTSTR pszNom);
	
	void MAJ();
	
	bool ArchiveDecompSuite(LPCTSTR pszRep, LPTSTR* ppszDest, const DECOMP* pdec);
	bool Decomp(LPCTSTR pszDest, const DECOMP* pdec);
	
	bool RecompModif(LPCTSTR pszFile);
	void SurvExecuter(LPCTSTR pszFichier, LPCTSTR pszRep, LPCTSTR pszVerbe=NULL) const;
	
	void GetCacheFile(CString& sFile) const;
	bool CacheAddRef       (CString& sFile);
	bool CacheAddRefInterne(CString& sFile, bool fOK);
	void CacheRelease();
	bool CacheReleaseNoDelete();
	
	
public:  // Hritage
	
	virtual DWORD GetOffSfx() const { return 0; }
	
	virtual bool IsCrypte() const { return false; }
	virtual void ToPlusItem(CPlusItem& pi) const;
	
	virtual void SetProp(DWORD dwAttr, CTempsDHS& dtModif);
	virtual bool OnRenommer()  { return false; }
	virtual bool OnModifProp() { return false; }
	
	
public:  // Donnes
	
	CMuzipArc *m_pArc;         // Document parent
	CDossier  *m_pDossier;     // Dossier parent
	
	LPTSTR     m_pszNom;       // Nom du fichier sans chemin d'accs
	LPTSTR     m_pszNomES;     // Nom pour les E/S
	
	DWORD      m_dwTailleC;    // Taille du fichier compress
	DWORD      m_dwTailleN;    // Taille relle du fichier
	DWORD      m_dwAttr;       // Attributs du fichier
	int        m_nPC;          // Pourcentage de compression
	CTempsDHS  m_dtM;          // Date de modification
	TCHAR      m_pszAttr[5];   // Attributs du fichier en chane
	
	bool       m_fTrouveRech;  // TRUE si trouv par la recherche
	
	
	// Pour la compression seulement:
	
	// m_fTrouveAjout: si gal  TRUE, le fichier a t trouv par la recherche
	// d'ajout, auquel cas il ne faudra pas le supprimer si l'option
	// "supprimer les fichiers non-trouvs par la recherche" est utilise
	bool       m_fTrouveAjout;
	
	// m_fFautComp: 2 significations possibles
	//
	// - si le fichier appartient  une archive:
	//   true:  il faut le recompresser, donc ne pas le recopier tel quel
	//          de l'ancienne  la nouvelle archive
	//   false: il faut le recopier tel quel
	//
	// - si le fichier appartiendra  la future archive:
	//   true:  il faut compresser le fichier
	//   false: il ne faut pas le compresser
	//
	bool       m_fFautComp;
	
	bool       m_fIgnorer;     // TRUE si le fichier a t ignor  la compression
	bool       m_fSupprimer;   // TRUE si le fichier doit tre supprim aprs la compression
	
	
public:  // Statique
	
	static UINT s_nbFichier;   // Nombre total de fichiers
	static UINT s_nFichier;    // Fichier en cours
	
	
protected:  // Interne
	
	UINT m_nCache;   // Compteur d'utilisation du cache
	
	
	virtual bool DecompInterne(LPCTSTR pszDest) = 0;
	
	virtual bool FichierCopierInterne(CFileRead& fr) { fr; return true; }
	static bool FichierCopierHelper(CFileRead& fr, DWORD len, bool fAvance2 = false);
};

#endif
