// FichierPlus.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "FichierPlus.h"

#include "Fichier.h"
#include "Dossier.h"
#include "PlusItem.h"
#include "MuzipArc.h"

#include "IOFichier.h"


// Constructeur
CFichierPlus::CFichierPlus(
	CFichier*  pFichierOld,
	POS        posFichierOld)
	: m_pFichier(NULL),
	  m_pFichierOld(pFichierOld),
	  m_posFichierOld(posFichierOld),
	  m_pszNomES(NULL), m_pfpGauche(NULL), m_pfpDroite(NULL)
{
}


// Enlever virtuellement le nud de l'arbre:
// il sera toujours l mais n'aura plus de contenu
// En fait on supprime l'objet CFichier et on ne garde
// que son nom (m_pszNomES)
void CFichierPlus::Supprimer()
{
	if (!m_pFichier)
		return;
	
	// Conserver  part le nom du fichier
	m_pszNomES = m_pFichier->m_pszNomES;
	
	// Supprimer le fichier
	m_pFichier->m_pszNomES = NULL;
	delete m_pFichier;
	m_pFichier = NULL;
}


// Traitement final
bool CFichierPlus::Finaliser()
{
	if (m_pszNomES) {
		// Il s'agit d'un faux nud
		
		delete [] m_pszNomES;
		m_pszNomES = NULL;
		return true;
	}
	
	VERIF(!m_pFichier->m_fIgnorer);
	
	// Supprimer l'ancien fichier et ajouter le nouveau
	CLstFichier &lstFichier = m_pFichier->m_pDossier->m_lstFichier;
	if (m_posFichierOld)
		lstFichier.DeleteAt(m_posFichierOld);
	lstFichier.AddFin(m_pFichier);
	
	const LPTSTR pszNomES = m_pFichier->m_pszNomES;
	
	// Options de compression:
	// - Supprimer le fichier
	if (m_ppi->CO_fActionDeplacer)
		FichierSupprimer(pszNomES, true);
	// - Enlever l'attribut archive
	else if (m_ppi->CO_fArchiveMoins && (m_pFichier->m_dwAttr & FILE_ATTRIBUTE_ARCHIVE))
		SetFileAttributes(pszNomES, m_pFichier->m_dwAttr & ~FILE_ATTRIBUTE_ARCHIVE);
	
	// La recherche est termine
	delete [] pszNomES;
	return true;
}

// Nettoyer les membres inutiliss et dtruire l'objet
// N'appeler cette fonction que si le fichier n'a pas t compress
// Attention ! Cette fonction n'assure pas l'intgrit de l'arbre
void CFichierPlus::Detruire()
{
	// Dtruire l'objet lui-mme
	if (m_pFichier) {
		delete [] m_pFichier->m_pszNomES;
		delete    m_pFichier;
	}
	
	delete [] m_pszNomES;
	
	delete this;
}


// Remplir une liste de CFichierPlus avec un arbre
void CFichierPlus::ArbreLister(CLstFichierPlus& lstFichierPlus, int& iLimite)
{
	if (!this)
		// On est arriv sur une feuille de l'arbre
		return;
	
	
	bool fOK = !m_pszNomES;
	
	if (fOK) {
		// S'occuper de la limite du nombre de fichiers ajouts
		// Plusieurs cas de figure:
		// - l'entre n'crase pas de fichiers: c'est un ajout
		// - l'entre crase un fichier  supprimer: c'est aussi un ajout
		// - l'entre crase un fichier conserv: ce n'est pas un ajout
		// - l'entre n'est pas une compression: ce n'est pas un ajout
		if (iLimite!=INT_MAX &&
		   (!m_pFichierOld || m_pFichierOld->m_fSupprimer) &&
		    m_pFichier && m_pFichier->m_fFautComp) {
			if (iLimite)
				iLimite--;
			else
				fOK = false;
		}
		
		// Ajouter le fichier  la liste
		if (fOK)
			lstFichierPlus.AddFin(this);
	}
	
	// Poursuivre dans les fils
	m_pfpGauche->ArbreLister(lstFichierPlus, iLimite);
	m_pfpDroite->ArbreLister(lstFichierPlus, iLimite);
	
	if (!fOK)
		Detruire();
}
