// GZip/GZipArc.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "../Muzip.h"
#include "../Muzip2.h"
#include "GZipArc.h"
#include "GZipFichier.h"
#include "GZip.h"
#include "../Interf.h"

#include "../CRC32.h"

#include "../FichierPlus.h"

#include "../Z/Fen.h"
#include "../Decomp.h"

#include "../Options.h"
#include "../Exceptions.h"

#include "dlgConfirm.h"
#include "dlgNomFichier.h"

#include <GUnicode.h>


// Constructeur
CGZipArc::CGZipArc(CInterf* pi) : CZArc(pi)
{
	m_pszCom = NULL;
}


// Crer une archive
CMuzipArc* CGZipArc::CreateArchive(CInterf* pi)
{
	return new CGZipArc(pi);
}


// Validation du type
bool CGZipArc::IsGoodType()
{
	GZIPHEADER gzh;
	VERIF(m_fr.Read(&gzh, sizeof(gzh)));
	
	return gzh.wID == gzipID && gzh.nMethode == zmDeflate;
}


// Ouverture
bool CGZipArc::OpenDocument(bool fSilent)
{
	// Lire l'en-tte
	GZIPHEADER gzh;
	VERIF(m_fr.Read(&gzh, sizeof(gzh)));
	
	// Vrifier l'ID, la mthode et les flags
	if (gzh.wID      != gzipID    ||
	    gzh.nMethode != zmDeflate ||
	  ((gzh.nFlags  & (gzfReserve | gzfMulti | gzfCrypte)) && !e_opt.m_nReparer)) {
		if (!fSilent)
			CMuzipException(ERR_FORMAT, m_fr.GetPath()).ReportError();
		return false;
	}
	
	BYTE b;
	DWORD off = sizeof(gzh);   // Dbut du premier fichier
	
	if (gzh.nFlags & gzfExtra) {
		// Sauter le champ extra
		
		// Lire la longueur du champ extra
		WORD len;
		VERIF(m_fr.Read(len));
		off += len;
		
		// Lire le champ extra
		VERIF(m_fr.Seek(len, FILE_CURRENT));
	}
	
	char  pszNom[_MAX_FNAME] = "";
	char  pszCom[1024] = "";
	int   lenNom = 0;
	int   lenCom = 0;
	char *pc;
	
	if (gzh.nFlags & gzfNom) {
		// Lire le nom original
		pc = pszNom;
		do{
			VERIF(m_fr.Read(b));
			*pc++ = (char)b;
			lenNom++;
		}while (b);
	}
	if (gzh.nFlags & gzfComment) {
		// Lire les commentaires
		pc = pszCom;
		do{
			VERIF(m_fr.Read(b));
			*pc++ = b;
			lenCom++;
		}while (b);
	}
	
	// Lire l'en-tte final
	DWORD adw[2];
	VERIF(m_fr.Seek(-(long)sizeof(adw), FILE_END));
	VERIF(m_fr.Read(adw, sizeof(adw)));
	
	off += lenNom + lenCom;
	
	// Garder les commentaires
	if (!lenCom)
		m_pszCom = NULL;
	else{
		m_pszCom = new TCHAR[lenCom+1];
		strfromA(m_pszCom, pszCom, lenCom);
	}
	
	// Rcuprer le nom du fichier
	TCHAR pszNomTemp[_MAX_FNAME];
	if (!lenNom) {
		// Demander le nom du fichier
		
		// Rechercher une ventuelle double-extension
		// Les extensions .tgz et .taz correspondent  .gz.tar
		lstrcpy(pszNomTemp, PathFindFileName(m_fr.GetPath()));
		TCHAR *pcExt = PathFindExtension(m_fr.GetPath());
		if (!lstrcmpi(pcExt, _T(".tgz")) ||
		    !lstrcmpi(pcExt, _T(".taz")))
			// Archive Tar compresse en GZip
			PathRenameExtension(pszNomTemp, _T(".tar"));
		
		else{
			PathRemoveExtension(pszNomTemp);
			pcExt = PathFindExtension(pszNomTemp);
			
			if (!*pcExt) {
				// Aucune extension spcifie: il faut renommer
				// Afficher la bote de dialogue de renommage
				lstrcpy(pcExt, _T("."));
				CDlgNomFichier(pszNomTemp).DoModal();
			}
		}
	}else
		strfromA(pszNomTemp, pszNom, _MAX_FNAME);
	
	// Crer l'unique fichier de l'archive
	CGZipFichier *const pgzf = new CGZipFichier;
	pgzf->m_pArc      = this;
	pgzf->m_pDossier  = this;
	pgzf->m_pszNom    = newDup(pszNomTemp);
	pgzf->m_dwTailleC = GetFileSize(m_fr, NULL) - off-sizeof(adw);
	pgzf->m_dwTailleN = adw[1];
	pgzf->m_dwAttr    = 0;
	pgzf->SetUnixDateM(gzh.dtM);
	pgzf->MAJ();
	pgzf->m_off       = off;
	pgzf->m_nCRC      = adw[0];
	m_lstFichier.AddDeb(pgzf);
	
	return OpenArchiveOK(fSilent, m_pszCom);
}


// Dfinir les commentaires
void CGZipArc::SetComment(LPCTSTR psz)
{
	// Recopier les commentaires
	delete [] m_pszCom;
	m_pszCom = (psz && *psz) ? newDup(psz) : NULL;
	
	// Mettre  jour
	FautReconstruire();
}

// Retourner les commentaires
LPCTSTR CGZipArc::GetComment() const
{
	return m_pszCom;
}


// Ajouter un fichier  la liste
CFichier* CGZipArc::NewFichier(LPCTSTR pszFichier, const WIN32_FIND_DATA& wfd)
{
	CGZipFichier *const pgzf = new CGZipFichier;
	pgzf->NewFichier(pszFichier, wfd);
	pgzf->m_pArc     = this;
	pgzf->m_pDossier = this;
	return pgzf;
}


// Ajouter des fichiers  l'archive
bool CGZipArc::ArchivePlusUn(CFichierPlus* pFichierPlus, CFileRead& fr)
{
	CGZipFichier *const pgzf = (CGZipFichier*)pFichierPlus->m_pFichier;
	
	// crire l'en-tte
	VERIF(pgzf->PutHeader());
	
	// Compresser et crire le fichier
	oApp.SetProgress(STA_ARCHIVEPLUS, pgzf->m_pszNomES);
	pgzf->m_dwTailleC = CZArc::Compresser(pgzf, fr,
		pFichierPlus->m_ppi->CO_nNiveau,NULL);
	pgzf->MAJ();
	pgzf->m_nCRC = s_nCRCIn;
	
	// crire l'en-tte final
	return pgzf->PutFooter();
}
