// InterfDoc.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "InterfDoc.h"

#include "MuzipDoc.h"
#include "MuzipArc.h"

#include "ArboView.h"
#include "MuzipView.h"
#include "ApercuView.h"

#include "Dossier.h"

#include "dlgResRech.h"


// Constructeur
CInterfDoc::CInterfDoc(CMuzipDoc* pDoc) : m_pDoc(pDoc)
{
}


// Flag de modification
void CInterfDoc::SetModifiedFlag(BOOL bModified)
{
	CInterf::SetModifiedFlag(bModified);
	m_pDoc->SetModifiedFlag(bModified);
}

// Mise  jour et vidage des vues
void CInterfDoc::UpdateView(bool fQueListe)
{
	m_pDoc->m_plst->OnUpdate(NULL,NULL,NULL);
	if (!fQueListe)
		m_pDoc->m_ptrv->OnUpdate(NULL,NULL,NULL);
}
void CInterfDoc::DeleteAll()
{
	m_pDoc->m_ptrv->m_ptrv->DeleteAllItems();
	m_pDoc->m_plst->m_plst->DeleteAllItems();
}

// Mettre  jour juste un fichier
void CInterfDoc::UpdateFichier(CFichier* pFichier)
{
	if (!m_pDoc->m_plst)
		return;
	
	CListCtrl &lst = *m_pDoc->m_plst->m_plst;
	
	// Rechercher l'item pour le supprimer
	// Conserver l'tat de l'item (slection, focus)
	// pour pouvoir le restaurer aprs
	LVFINDINFO lvfi;
	lvfi.flags  = LVFI_PARAM;
	lvfi.lParam = (LPARAM)pFichier;
	const int nItem = lst.FindItem(&lvfi);
	if (nItem<0)
		return;
	const UINT nState = lst.GetItemState(nItem, LVIS_DROPHILITED | LVIS_FOCUSED | LVIS_SELECTED);
	lst.DeleteItem(nItem);
	
	// Le rajouter  la liste
	pFichier->lstLister(lst, nItem);
	lst.SetItemState(nItem, nState, LVIS_DROPHILITED | LVIS_FOCUSED | LVIS_SELECTED);
}

// Fermer le document en le sauvegardant si ncessaire
void CInterfDoc::CloseDocument()
{
	m_pDoc->DoFileSave();
	m_pDoc->OnCloseDocument();
}


// Slectionner un fichier
void CInterfDoc::SelFichier(const CFichier* pFichier)
{
	m_pDoc->SelFichier(pFichier);
}


// Commencer  mettre  jour la liste des fichiers trouvs par la recherche
void CInterfDoc::MAJResRech()
{
	if (!e_pdlgResRech)
		return;
	
	e_pdlgResRech->AttachArc(m_pArc);
	
	CLstMuzip &lst = e_pdlgResRech->m_lst;
	lst.SetRedraw(FALSE);
	lst.DeleteAllItems();
	MAJResRechSuite(m_pArc);
	lst.Trier();
	lst.SetRedraw();
	e_pdlgResRech->MAJStatusBar();
}

// Mettre  jour la liste des fichiers trouvs par la recherche
void CInterfDoc::MAJResRechSuite(CDossier* pDossier)
{
	CListCtrl &lst = e_pdlgResRech->m_lst;
	
	POS pos = pDossier->m_lstFichier.GetPosDeb();
	while (pos) {
		CFichier *const pFichier = pDossier->m_lstFichier.GetSuiv(pos);
		if (pFichier->m_fTrouveRech)
			pFichier->lstLister(lst, lst.GetItemCount());
	}
	
	pos = pDossier->m_lstDossier.GetPosDeb();
	while (pos)
		MAJResRechSuite(pDossier->m_lstDossier.GetSuiv(pos));
}

// Recommencer la recherche
void CInterfDoc::Rechercher()
{
	m_pArc->Rechercher(oApp.m_rch, OnFichierTrouve);
}

// Un fichier est trouv par la recherche
void CInterfDoc::OnFichierTrouve(CFichier* pFichier)
{
	if (e_pdlgResRech)
		// Ajouter le fichier  la liste des fichiers trouvs
		pFichier->lstLister(e_pdlgResRech->m_lst, e_pdlgResRech->m_lst.GetItemCount());
}


// Poursuivre la recherche dans un dossier
void CInterfDoc::RechercherSuivant(const CDossier* pDossier, int nItem)
{
	if (!m_pDoc->m_plst || !m_pDoc->m_ptrv)
		return;
	
	CListCtrl &lst = *m_pDoc->m_plst->m_plst;
	CTreeCtrl &trv = *m_pDoc->m_ptrv->m_ptrv;
	LVITEM lvi;
	
	// Poursuivre la recherche
	POS pos;
	
	LVFINDINFO lvfi;
	lvfi.flags = LVFI_PARAM;
	
	const int nbItem = lst.GetItemCount();
	if (nItem>=0) {
		// Poursuivre la recherche dans la liste
		lvi.mask = LVIF_PARAM;
		while (++nItem<nbItem) {
			lvi.iItem = nItem;
			lst.GetItem(&lvi);
			if (((CFichier*)lvi.lParam)->m_fTrouveRech) {
				// Slectionner le fichier
				
SelectionnerFichier:
				// Tout dslectionner d'abord
				lvi.iItem     = nbItem;
				lvi.mask      = LVIF_STATE;
				lvi.stateMask = LVIS_SELECTED;
				lvi.state     = 0;
				while (--lvi.iItem>=0)
					lst.SetItem(&lvi);
				
				// Slectionner l'item
				lvi.iItem     = nItem;
				lvi.stateMask =
				lvi.state     = LVIS_SELECTED | LVIS_FOCUSED;
				lst.SetItem(&lvi);
				lst.SetSelectionMark(lvi.iItem);
				lst.EnsureVisible   (lvi.iItem, FALSE);
				lst.SetFocus();
				return;
			}
		}
	}else{
		pos = pDossier->m_lstFichier.GetPosDeb();
		while (pos) {
			const CFichier *const pFichier = pDossier->m_lstFichier.GetSuiv(pos);
			if (pFichier->m_fTrouveRech) {
				
				if (nItem<0) {
					// Slectionner le dossier une seule fois
					
					// Lister les dossiers  ouvrir
					CListe1<const CDossier*> lstDossierTemp;
					const CDossier *pDossierCur = pDossier;
					for (;;) {
						pDossierCur = pDossierCur->m_pParent;
						if (!pDossierCur)
							break;
						lstDossierTemp.AddFin(pDossierCur);
					}
					
					// Ouvrir les dossiers dans le bon ordre
					POS pos = lstDossierTemp.GetPosDeb();
					while (pos)
						trv.Expand(lstDossierTemp.GetSuiv(pos)->m_hti, TVE_EXPAND);
					
					// Slectionner le dossier
					trv.EnsureVisible(pDossier->m_hti);
					trv.SelectItem   (pDossier->m_hti);
				}
				
				// Rechercher le fichier,
				// et le garder s'il a une meilleure position
				lvfi.lParam = (LPARAM)pFichier;
				const int nTemp = lst.FindItem(&lvfi);
				if (nItem<0 || nTemp<nItem)
					nItem = nTemp;
			}
		}
		
		if (nItem>=0)
			goto SelectionnerFichier;
	}
	
	HTREEITEM htiSuiv = NULL;
	
	if (pDossier->m_lstDossier.GetNb()) {
		// Poursuivre la recherche dans les sous-dossiers
		
		htiSuiv = trv.GetChildItem(pDossier->m_hti);
		
		if (!htiSuiv) {
			// Lister les sous-dossiers
			pos = pDossier->m_lstDossier.GetPosDeb();
			while (pos)
				pDossier->m_lstDossier.GetSuiv(pos)->trvLister();
			// Rcuprer l'item
			htiSuiv = trv.GetChildItem(pDossier->m_hti);
		}
	}else{
		// Poursuivre la recherche dans le dossier frre suivant
		// S'il n'y en a pas, poursuivre l'arborescence
		
		const CDossier *pDossierCur = pDossier;
		do{
			htiSuiv = trv.GetNextSiblingItem(pDossierCur->m_hti);
			if (htiSuiv)
				// On a trouv un frre dans lequel poursuivre la recherche
				break;
			
			// Pas de frre: on remonte au parent
			pDossierCur = pDossierCur->m_pParent;
			
		}while (pDossierCur);
	}
	
	if (htiSuiv) {
		// Dossier suivant trouv
		RechercherSuivant((const CDossier*)trv.GetItemData(htiSuiv));
		return;
	}
	
	AfxMessageBox(ERR_RECHERCHEFIN);
}


// Ajouter un item  l'arborescence
HTREEITEM CInterfDoc::InsertItem(TVINSERTSTRUCT* ptvis)
{
	return m_pDoc->m_ptrv->m_ptrv->InsertItem(ptvis);
}


// Suppression de fichier
void CInterfDoc::OnFichierSupprimer(CFichier* pFichier)
{
	// Aperu du fichier
	CApercuView *const papr = m_pDoc->m_papr;
	if (papr && papr->m_pFichier==pFichier)
		papr->SetApercu(NULL);
	
	// Prsence du fichier dans la liste des fichiers trouvs
	if (pFichier->m_fTrouveRech &&
	    e_pdlgResRech &&
	    e_pdlgResRech->IsArc(m_pArc)) {
		// Supprimer le fichier de la liste des rsultats de la recherche
		CListCtrl &lst = e_pdlgResRech->m_lst;
		LVFINDINFO lvfi;
		lvfi.flags  = LVFI_PARAM;
		lvfi.lParam = (LPARAM)this;
		lst.DeleteItem(lst.FindItem(&lvfi));
	}
}

// Suppression de dossier
void CInterfDoc::OnDossierSupprimer(CDossier* pDossier)
{
	// Prendre le parent du dossier comme dossier courant
	// si le dossier  supprimer est le dossier actif
	if (m_pDoc->m_pCur==pDossier && pDossier->m_pParent) {
		m_pDoc->m_pCur = pDossier->m_pParent;
		m_pDoc->m_plst->m_plst->DeleteAllItems();
		m_pDoc->m_pArc->m_pi->OnDossierChange();
	}
	
	// Supprimer les items dans l'arborescence
	CTreeCtrl *const ptrv = m_pDoc->m_ptrv->m_ptrv;
	if (pDossier->m_pParent)
		// Sous-dossier
		
		ptrv->DeleteItem(pDossier->m_hti);
		
	else{
		// Dossier racine: insupprimable
		
		// Ne pas dtruire le dossier racine lui-mme
		// mais que les items fils
		HTREEITEM hti = ptrv->GetChildItem(pDossier->m_hti);
		while (hti) {
			const HTREEITEM htiSuiv = ptrv->GetNextSiblingItem(hti);
			ptrv->DeleteItem(hti);
			hti = htiSuiv;
		}
	}
}


// Changement du dossier courant
void CInterfDoc::OnDossierChange()
{
	if (m_pDoc->m_papr)
		m_pDoc->m_papr->SetApercu(NULL);
}
