// MuzipArc.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "Muzip2.h"
#include "MuzipDoc.h"
#include "MuzipArc.h"
#include "InterfDoc.h"

#include "PlusItem.h"
#include "FichierPlus.h"
#include "Recherche.h"

#include "Decomp.h"

#include "dlgConfirm.h"
#include "dlgEcrase.h"
#include "dlgDecomp.h"
#include "dlgPlusItem.h"
#include "dlgRecomp.h"

#include "Reg.h"
#include "IOFichier.h"
#include "Exceptions.h"

#include "lstMuzip.h"
#include "MuzipView.h"
#include "MuzipSfx/MuzipSfx.h"
#include "MuzipDll/MuzipDll.h"

#include "Z/UnZ.h"


CLstFichierPlus CMuzipArc::s_lstFichierPlus;

CLstMuzipArc e_lstArc;



// Constructeur
CMuzipArc::CMuzipArc(CInterf* pi)
	: CDossier(NULL, CStringRes(IDS_RACINE)), m_pi(pi),
	  m_lstRM(true), m_pmdt(NULL)
{
	m_nUse          = 1;
	m_fRechFiltre   = false;
	m_fRm           = false;
	m_fOuvert       = true;   // Dossier racine ouvert
	m_fLectureSeule = false;
	m_pi->SetArc(this);
	m_fr.SetPeutFermer(false);
	
	m_parcParent = m_parcFille = NULL;
	
	// Ajouter l'archive  la liste
	e_lstArc.AddDeb(this);
}

// Destructeur
CMuzipArc::~CMuzipArc()
{
	// Supprimer les fichiers et dossiers
	Clean();
	
	// Enlever l'archive de la liste
	e_lstArc.RemoveAt(e_lstArc.Find(this));
	
	delete m_pi;
}

// Instanciation
void CMuzipArc::AddRef(CMuzipDoc* pDoc)
{
	m_nUse++;
	if (pDoc && !m_pi->GetDoc()) {
		// Passer l'archive en mode document
		delete m_pi;
		m_pi = new CInterfDoc(pDoc);
		m_pi->SetArc(this);
	}
}

// Libration
void CMuzipArc::Release(bool fDoc)
{
	if (!--m_nUse) {
		// L'archive n'est plus rfrence nulle part, il faut:
		// 1) Demander de la reconstruire si ce n'est pas un document
		// 2) Fermer le document s'il y a un document
		//    (et cela se chargera de demander sa reconstruction)
		
		if (!fDoc)
			Reconstruire();
		CloseDocument();
		delete this;
		return;
	}
	
	if (fDoc) {
		// Passer l'archive en mode sans document
		
		delete m_pi;
		m_pi = new CInterf;
		m_pi->SetArc(this);
	}
}


// Rcuprer les capacits
int CMuzipArc::GetCap() const
{
	int cap = m_pmdt->GetCap();
	
	if (m_fLectureSeule)
		cap |= capLectureSeule;
	
	return cap;
}


// Ouvrir une archive
bool CMuzipArc::OpenDocument(LPCTSTR pszFichier, bool fSilent)
{
	// Ouvrir le fichier et ignorer les fichiers vides
	CloseDocument();
	m_pi->SetModifiedFlag(FALSE);
	oApp.DoEvents();
	m_fr.Close();
	
	// Rcuprer un dossier temporaire pour le cache de l'archive
	TCHAR pszCacheParent[MAX_PATH];
	e_opt.GetDossier(repTemp, pszCacheParent);
	GetDossierTemp(pszCacheParent, m_sCachePath.GetBuffer(MAX_PATH));
	m_sCachePath.ReleaseBuffer();
	
	m_fr.SetVerbeux(!fSilent);
	
	// Ouvrir l'archive
	VERIF(m_fr.Open(pszFichier, FILE_SHARE_READ));
	
	bool fRet = true;
	if (GetFileSize(m_fr, NULL)) {
		// Si elle a une taille nulle, ne pas aller plus loin
		
		// Effectuer l'ouverture en grant le sablier et la barre d'tat
		CWaitCursor wc;
		AfxFormatString1(e_sMsg, STA_ARCHIVEOPEN, pszFichier);
		oApp.SetMessageText(e_sMsg);
		fRet = OpenDocument(fSilent);
		oApp.SetMessageText(AFX_IDS_IDLEMESSAGE);
		
		m_fr.SetVerbeux();
	}
	
	if (fRet) {
		// Conserver le chemin d'accs
		lstrcpyn(m_sPath.GetBuffer(MAX_PATH), pszFichier, MAX_PATH);
		m_sPath.ReleaseBuffer();
	}
	
	m_fr.SetVerbeux();
	
	return fRet;
}


// Regarder si le fichier donn est une archive du type correspondant  la classe C*Arc
bool CMuzipArc::IsArchiveGoodType(LPCTSTR pszFichier)
{
	VERIF(m_pmdt->GetIdType() != IDR_SFXTYPE);
	
	m_fr.SetVerbeux(false);
	VERIF(m_fr.Open(pszFichier, FILE_SHARE_READ));
	const bool bGoodType = IsGoodType();
	m_fr.Close();
	
	return bGoodType;
}


// Fermer le document
void CMuzipArc::CloseDocument()
{
	m_fr.Close();
	
	if (m_parcFille) {
		// L'archive tait parent d'une autre archive:
		// fermer l'archive fille aussi
		
		m_parcFille->m_parcParent = NULL;
		m_parcFille->m_pi->CloseDocument();
		m_parcFille = NULL;
	}
	
	if (m_parcParent) {
		// L'archive tait fille d'une autre archive:
		// fermer l'archive parent aussi
		
		m_parcParent->m_parcFille = NULL;
		m_parcParent->m_pi->CloseDocument();
		m_parcParent = NULL;
	}
	
	// Nettoyer le cache, mais seulement si l'aperu est ferm
	if (!(m_pi->GetDoc() && m_pi->GetDoc()->m_papr) &&
	    !m_sCachePath.IsEmpty()) {
		DossierSupprimer(m_sCachePath);
		m_sCachePath.Empty();
	}
}


// Reconstruire l'archive
// On doit traiter deux cas distincts:
// 1) il y a des requtes de compression  faire
// 2) il n'y en a pas
bool CMuzipArc::Reconstruire()
{
	if (!m_pi->IsModified())
		// L'archive n'a pas t modifie: rien  faire, tout est OK
		return true;
	
	bool fOK;
	if (m_lstRM.GetNb())
		// Il reste des requtes d'ajout en attente
		// Cela se chargera au passage des autres modifications
		// (attributs, renommage, etc.)
		
		fOK = DoArchivePlus();
	
	else{
		// Pas de requtes d'ajout en attente
		
		fOK = ReadOpen();
		if (fOK) {
			CMuzipOp mop;
			fOK = OpenArchiveTemp() && CloseArchiveTemp();
		}
	}
	
	m_pi->SetModifiedFlag(FALSE);
	return fOK;
}

// Excut aprs une commande demandant reconstruction de l'archive
bool CMuzipArc::FautReconstruire(RM_OPTION_AC rmo)
{
	int i;
	
	
	// Positionner le flag de modification
	m_pi->SetModifiedFlag();
	
	// Recommencer la recherche
	m_pi->Rechercher();
	
	
	if (rmo==rmoReconstruire)
		goto ReconstruireMaintenant;
	
	// Si on est dj en mode reconstruction manuelle, y rester
	if (m_fRm)
		goto Manuel;
	
	
	// Examiner chaque condition une par une dans l'ordre
	for (i=0; i<nbRmOption; i++) {
		RM_OPTION &rrmo = e_opt.m_aRmOption[i];
		
		if (rrmo.fActive) {
			// Tester la condition
			
			switch (rrmo.rmo) {
				
				// L'archive est sur disque amovible ?
				case rmoAmovible:
					if (IsSurDisqueAmovible())
						goto Manuel;
					break;
				
				// L'archive est trop petite ?
				// (passage en mode reconstruction manuelle
				// uniquement si l'archive est suffisamment volumineuse)
				case rmoTaille:
					if ((GetFileSize(m_fr, NULL)) < e_opt.m_nRmTailleMin<<10)
						goto ReconstruireMaintenant;
					break;
				
				// Mode ajout ?
				case rmoAjouter:
					if (rmo==rmoAjouter)
						goto ReconstruireMaintenant;
					break;
				
				// Forcer la reconstruction
				case rmoReconstruire:
					goto ReconstruireMaintenant;
				
				// Autre cas: on passe en mode manuel
				default:
					goto Manuel;
			}
		}
		
		if (rrmo.rmo==rmo)
			goto Manuel;
	}
	
	
ReconstruireMaintenant:
	
	// Reconstruire directement
	return Reconstruire();
	
	
Manuel:
	m_fRm = true;
	return true;
}


// Ouvrir l'archive en mode lecture
bool CMuzipArc::ReadOpen()
{
	return (m_fr!=INVALID_HANDLE_VALUE || m_fr.Open(m_sPath, FILE_SHARE_READ));
}

// Fermer l'archive ouverte en mode lecture
void CMuzipArc::ReadClose()
{
	m_fr.Close();
}


// Ouvrir l'archive en mode criture
bool CMuzipArc::WriteOpen()
{
	ASSERT(e_fw==INVALID_HANDLE_VALUE);
	
	ReadClose();
	if (!e_fw.Open(m_sPath, 0, OPEN_EXISTING)) {
		// Erreur: on revient comme avant
		ReadOpen();
		return false;
	}
	
	return true;
}

// Fermer l'archive ouverte en mode criture
void CMuzipArc::WriteClose()
{
	e_fw.Close();
	ReadOpen();
}


// Retourner TRUE si l'archive est sur disque amovible
bool CMuzipArc::IsSurDisqueAmovible() const
{
	TCHAR pszDir[MAX_PATH];
	lstrcpy(pszDir, m_sPath);
	PathStripToRoot(pszDir);
	return (GetDriveType(pszDir)==DRIVE_REMOVABLE);
}


// Cacher une liste de fichiers
// Retourner TRUE si tous les fichiers ont t mis en cache avec succs
// Mme en cas d'erreur CacheRelease() devra tre appele
bool CMuzipArc::CacheAddRef(CLstFichier& lstFichier, bool fOK)
{
	PrepareDecomp();
	CMuzipOp mop(this, STA_ARCHIVEDECOMPT);
	CString sFile;
	POS pos;
	
	// Initialiser la progression
	DWORD dwTotal = 0;
	pos = lstFichier.GetPosDeb();
	while (pos)
		dwTotal += lstFichier.GetSuiv(pos)->m_dwTailleC;
	oApp.ShowStatusAvance(dwTotal);
	
	// Mettre les fichiers en cache
	UINT iFichier = 0;
	pos = lstFichier.GetPosDeb();
	while (pos && oApp.UpdateProgressStatus()!=prsAnnuler) {
		CFichier *const pFichier = lstFichier.GetSuiv(pos);
		oApp.SetProgress(STA_ARCHIVEDECOMP, pFichier->m_pszNom,
			iFichier++, lstFichier.GetNb());
		
		fOK &= pFichier->CacheAddRefInterne(sFile, fOK);
	}
	oApp.HideStatusAvance();
	oApp.m_fMsg = true;
	
	// Incrmenter le compteur d'utilisation des fichiers ignors
	while (pos)
		lstFichier.GetSuiv(pos)->CacheAddRefInterne(sFile, false);
	
	return fOK;
}

// Indiquer que certains fichiers du cache ne sont plus requis
void CMuzipArc::CacheRelease(CLstFichier& lstFichier)
{
	POS pos = lstFichier.GetPosDeb();
	while (pos)
		lstFichier.GetSuiv(pos)->CacheRelease();
}


// Ajouter des fichiers  l'archive
// Gre la mise en attente en cas de reconstruction manuelle
bool CMuzipArc::ArchivePlus(const CArchivePlus& arPlus)
{
	// Ajouter la requte  la liste
	m_pi->SetModifiedFlag();
	m_lstRM.AddFin(new CArchivePlus(arPlus));
	
	// Passer en mode reconstruction manuelle si demand
	if (arPlus.m_fAjoutRm)
		m_fRm = true;
	
	return FautReconstruire(rmoAjouter);
}


// Commencer l'ajout de fichiers  l'archive
// Les requtes de compression sont dans m_lstRM
bool CMuzipArc::DoArchivePlus()
{
	ASSERT(m_lstRM.GetNb()>0);
	
	m_pi->DeleteAll();
	m_pi->SetModifiedFlag(FALSE);
	
	POS pos;
	
	CMuzipOp mop(this, STA_ARCHIVEPLUST);
	
	// Compter les recherches  effectuer
	int nbRecherche = 0;
	pos = m_lstRM.GetPosDeb();
	while (pos)
		nbRecherche += m_lstRM.GetSuiv(pos)->GetNb();
	
	
	//*** Effectuer toutes les recherches
	//*** de toutes les requtes d'ajout en attente
	
	bool fOK = true;
	oApp.ShowStatusAvance(nbRecherche);
	CRecherche rech(this);
	
	pos = m_lstRM.GetPosDeb();
	while (pos) {
		CArchivePlus &arPlus = *m_lstRM.GetSuiv(pos);
		rech.SetArchivePlus(arPlus);
		
		// Prparer les infos de progression
		oApp.SetMessageText(STA_RECHERCHE);
		oApp.SetStatusLabel(IDCLBL_2, STA2_RECHERCHE);
		oApp.ShowStatusAvance2(arPlus.GetNb());
		
		// Initialiser les fichiers et dossiers
		// En mode "supprimer tous les fichiers", on considre
		// ds le dbut que tous les fichiers sont  supprimer
		AvantRecherche(arPlus.m_nSup==supTout);
		
		// Effectuer les recherches
		POS pos2 = arPlus.GetPosDeb();
		while (pos2) {
			CPlusItem *const ppi = arPlus.GetSuiv(pos2);
			
			if (!rech.Rechercher(ppi)) {
				// Annulation
				
				fOK = false;
				goto Finaliser;
			}
			
			oApp.AddStatusAvance(0);
			oApp.AddStatusAvance(1);
		}
		
		if (arPlus.m_nSup==supRecherche)
			// Supprimer tous les fichiers non-trouvs
			SetNonTrouveFautSupprimer();
	}
	
	
Finaliser:
	
	// On n'a plus besoin de la fentre de progression
	oApp.HideStatusAvance();
	
	// La finalisation consiste  rassembler tous les CFichier crs
	// dans une liste, au lieu d'un arbre
	rech.Finaliser(s_lstFichierPlus);
	
	
	// Annulation
	if (!fOK)
		goto Fin;
	
	
	// On traite sparment le cas o l'archive accepte plusieurs fichiers
	{
		const bool fMultiFichiers = !(GetCap() & capMaxUnFichier);
		
		if (fMultiFichiers) {
			
			// Ouvrir un fichier temporaire en criture
			fOK = OpenArchiveTemp();
			
			if (fOK) {
				
				// Effectuer la compression
				fOK = ArchivePlus();
				
				// Finaliser la compression
				oApp.m_fMsg = true;
				oApp.HideStatusAvance2();
				oApp.HideStatusAvance();
				FinaliserAjout();
				
				// Supprimer les fichiers non-compresss et dossiers temporaires
				// (dossiers ajouts lors de la recherche mais rests vides ensuite)
				SupprimerSiTempVide();
				
				// Si la compression n'a pas t entirement annule,
				// remplacer l'ancienne archive par la nouvelle
				// (succs ou arrt sans annulation)
				// Sinon dtruire l'archive temporaire
				if (fOK)
					CloseArchiveTemp();
				else{
					CString sTemp = e_fw.GetPath();
					WriteClose();
					FichierSupprimer(sTemp, false);
				}
			}
			
		}else{
			// L'archive n'accepte qu'un seul fichier:
			// tout est dans CMuzipApp::ArchivePlus()
			fOK = ArchivePlus();
			
			// Supprimer les fichiers non-compresss et dossiers temporaires
			// (dossiers ajouts lors de la recherche mais rests vides ensuite)
			SupprimerSiTempVide();
		}
	}
	
Fin:
	
	// Dtruire les fichiers restants (par exemple les fichiers ignors)
	pos = s_lstFichierPlus.GetPosDeb();
	while (pos)
		s_lstFichierPlus.GetSuiv(pos)->Detruire();
	s_lstFichierPlus.RemoveAll();
	
	// Remettre tous les CFichier::m_fFautComp  false
	// En effet, m_fFautComp nous a - entre autre - servi  ce que
	// OpenArchiveTemp() ne recopie pas les fichiers qui seraient
	// recompresss, mais OpenArchiveTemp() risque d'tre appele
	// plus tard, et cette fois pour recopier TOUS les fichiers...
	SetFichierFautPasComp();
	
	m_pi->Rechercher();
	m_pi->UpdateView();
	m_lstRM.DeleteAll();
	
	return fOK;
}


// Fonction appele lorsque l'ouverture de l'archive
// s'est droule correctement
// Les commentaires sont affichs si l'utilisateur l'a demand dans les options
bool CMuzipArc::OpenArchiveOK(bool fSilent, LPCTSTR pszComment)
{
	// Lorsqu'une archive limite  un seul fichier s'ouvre,
	// regarder si elle ne contient pas elle-mme une archive
	// Cette fonctionnalit est dsactive dans le cas
	// des ouvertures sans document (il faut rester silencieux)
	if (!fSilent && m_pi->GetDoc() && (m_pmdt->GetCap() & capMaxUnFichier)) {
		
		CFichier *const pFichier = m_lstFichier.GetDeb();
		
		const LPCTSTR pszExt = PathFindExtension(pFichier->m_pszNom);
		
		POS pos = oApp.m_lstAssoc.GetPosDeb();
		while (pos) {
			CAssoc *const pAssoc = oApp.m_lstAssoc.GetSuiv(pos);
			
			if (!lstrcmpi(pAssoc->m_pszExt, pszExt)) {
				// Le fichier est une archive !
				// On peut donc demander d'ouvrir le fichier comme archive
				//  la place de l'archive elle-mme (relation archive parent/fille)
				
				// Demander confirmation
				AfxFormatString1(e_sMsg, DEM_DECOMPAUTO, pFichier->m_pszNom);
				if (!CDlgConfirm::GetReponse(mcfDecompAuto))
					break;
				
				// Dcompresser la sous-archive
				CString sFile;
				if (!pFichier->CacheAddRef(sFile))
					break;
				
				// L'ouvrir
				m_parcFille = OpenArchive(sFile, m_pi->GetDoc());
				if (!m_parcFille) {
					pFichier->CacheRelease();
					break;
				}
				
				// L'archive n'est plus relie au document mais  l'archive fille
				AddRef();
				Release(true);
				
				// Complter les liens archive parent et archive fille
				m_parcFille->SetArchiveParent(this);
				return true;
			}
		}
	}
	
	
	// Effectuer la recherche
	m_pi->Rechercher();
	
	// S'occuper des commentaires
	if (!fSilent && e_opt.m_fGeneralCommentaire && pszComment && *pszComment)
		oApp.m_pMainWnd->PostMessage(WM_COMMAND, ID_ARCHIVE_COMMENT);
	
	return true;
}


// Ajouter des fichiers  une archive
// Par dfaut on suppose que l'archive ne peut contenir
// qu'un seul fichier car c'est la mthode par dfaut
// Pour grer plusieurs fichiers, la classe C*Arc doit implmenter
// cette mthode
// La suite est dans ArchivePlusUn() pour la compression proprement dite
bool CMuzipArc::ArchivePlus()
{
	// Rechercher le premier fichier  compresser
	CFichierPlus *pFichierPlus;
	POS pos = s_lstFichierPlus.GetPosDeb();
	do{
		if (!pos) {
			pFichierPlus = NULL;
			break;
		}
		pFichierPlus = s_lstFichierPlus.GetSuiv(pos);
	}while (!pFichierPlus->m_pFichier->m_fFautComp);
	VERIF(pFichierPlus);
	
	// Rechercher le suivant pour regarder
	// s'il y a plusieurs fichiers  compresser
	while (pos) {
		if (s_lstFichierPlus.GetSuiv(pos)->m_pFichier->m_fFautComp) {
			// S'il faut compresser plusieurs fichiers,
			// afficher un avertissement
			AfxFormatString1(e_sMsg, DEM_PLUSPLUSIEURS,
				pFichierPlus->m_pFichier->m_pszNomES);
			VERIF(CDlgConfirm::GetReponse(mcfPlusPlusieurs));
			break;
		}
	}
	
	CFichier *const pFichier = pFichierPlus->m_pFichier;
	bool fOK = false;
	
	if (!pFichier->CompresseArchive()) {
		oApp.ShowStatusAvance(pFichier->m_dwTailleN);
		
		CFileRead fr;
		
		// Ouvrir l'archive en mode criture
		if (!WriteOpen())
			goto Fin;
		
		// Ouvrir le fichier en lecture
		if (!fr.Open(pFichier->m_pszNomES, FILE_SHARE_ALL))
			goto Erreur;
		
		// Remplir la classe du fichier
		{
			CMyFindFile ff(pFichier->m_pszNomES);
			if (ff.IsError()) {
				CMuzipException(ERR_READ, pFichier->m_pszNomES).ReportError();
				goto Erreur;
			}
			
			// Rcuprer la taille et la date de modification
			pFichier->m_dwTailleN = ff.m_wfd.nFileSizeLow;
			pFichier->m_dtM       = ff.m_wfd.ftCreationTime;
		}
		
		if (!ArchivePlusUn(pFichierPlus, fr))
			goto Erreur;
		
		// Supprimer le fichier dj existant
		// sauf si c'est le mme que celui qu'on ajoute,
		// car CFichierPlus::Finaliser() le fait dj
		if (m_lstFichier.GetNb() && m_lstFichier.GetDeb()!=pFichierPlus->m_pFichierOld)
			m_lstFichier.DeleteAll();
		
		fOK = true;
		
Erreur:
		fr.Close();
		
		// Finaliser le CFichierPlus et le supprimer de la liste
		if (fOK) {
			pFichierPlus->Finaliser();
			s_lstFichierPlus.DeleteAt(s_lstFichierPlus.Find(pFichierPlus));
		}
		
Fin:
		oApp.SetStatusAvance(1);
		oApp.HideStatusAvance();
		WriteClose();
	}
	
	return fOK;   // Car on finalise manuellement
}

// Grer les erreurs  la compression
PRGSTATUS CMuzipArc::ErreurArchivePlus()
{
	const PRGSTATUS prs = oApp.GetProgressStatus();
	return (prs==prsOK) ? prsErreur : prs;
}

// Grer la progression de la compression
// (ignorance de fichier, annulation de compression, etc.)
// Retourner FALSE s'il faut stopper la compression
bool CMuzipArc::ProgressArchivePlus(bool& rfOK, CFichier* pFichier, PRGSTATUS prs, POS pos, DWORD offOld)
{
	if (prs==prsOK)
		// Pas d'erreur, pas d'annulation
		return true;
	
	
	bool fContinuer = true;
	
	// Regarder si l'utilisateur a annul
	// ou bien s'il y a eu une erreur
	const bool fPeutAnnuler = (prs!=prsErreur);
	if (prs==prsErreur && pos && !oApp.DemanderEncoreErreur())
		prs = prsAnnuler;
	
	if (prs==prsAnnuler) {
		// Il y a eu annulation
		
		const UINT ret = AfxMessageBox(
			(fPeutAnnuler) ? DEM_ANNULERCOMP1 : DEM_ANNULERCOMP2,
			(fPeutAnnuler)
				? MB_YESNOCANCEL | MB_ICONQUESTION
				: MB_YESNO       | MB_ICONQUESTION);
		if (ret==IDCANCEL)
			// Continuer sans rien stopper ni annuler
			return true;
		
		// Ignorer tous les fichiers restants (si on a rpondu Non pour
		// conserver les fichiers dj compresss)
		// ou bien la totalit des fichiers, y compris ceux dj compresss
		// (si on a rpondu Oui)
		if (ret==IDYES)
			pos = pos = s_lstFichierPlus.GetPosDeb();
		while (pos)
			s_lstFichierPlus.GetSuiv(pos)->m_pFichier->m_fIgnorer = true;
		
		if (ret==IDYES) {
			rfOK = false;
			return false;
		}
		
		fContinuer = false;
	}
	
	// Remettre l'tat de progression  zro
	// Si prs==prsIgnorer, il ne faut ignorer qu'un fichier
	// Si prs==prsAnnuler, il ne faut pas annuler la finalisation de l'archive
	// car on est en mode stop de compression (et non annulation de compression)
	oApp.ResetProgressStatus();
	
	// prs==prsIgnorer ou prs==prsAnnuler: ignorer le fichier courant
	IgnorerFichier(offOld, pFichier);
	
	return fContinuer;
}

// Finaliser l'ajout
void CMuzipArc::FinaliserAjout()
{
	// Finaliser les fichiers
	POS pos = s_lstFichierPlus.GetPosDeb();
	while (pos) {
		const POS posSauve = pos;
		if (s_lstFichierPlus.GetSuiv(pos)->Finaliser())
			s_lstFichierPlus.DeleteAt(posSauve);
	}
}

// Ignorer un fichier
void CMuzipArc::IgnorerFichier(DWORD offOld, CFichier* pFichier)
{
	// Retourner en arrire dans le fichier
	e_fw.Seek(offOld, FILE_BEGIN);
	SetEndOfFile(e_fw);
	
	// Marquer le fichier comme ignor
	// (ne pas le compter dans la finalisation de l'archive)
	pFichier->m_fIgnorer = true;
}


// Supprimer
void CMuzipArc::ArchiveMoins(CLstFichier& lstFichier)
{
	if (!lstFichier.GetNb())
		return;
	
	{
		CMuzipOp mop(this, STA_ARCHIVEMOINST);
		
		// Enlever les fichiers de la liste
		oApp.ShowStatusAvance(lstFichier.GetNb());
		UINT iFichier = 0;
		POS pos = lstFichier.GetPosDeb();
		while (pos) {
			CFichier *const pFichier = lstFichier.GetSuiv(pos);
			oApp.SetProgress(STA_ARCHIVEMOINS, pFichier->m_pszNom,
				iFichier++, lstFichier.GetNb());
			
			pFichier->Supprimer(false);
			oApp.SetStatusAvance(iFichier);
		}
		
		if (e_opt.m_fGeneralSupprDossierVide) {
			// Supprimer les dossiers vides
			oApp.SetMessageText(STA_SUPPRDOSSIERVIDE);
			SupprimerSiVide();
		}
		
		oApp.HideStatusAvance();
	}
	
	FautReconstruire(rmoSupprimer);
}


// Dcompresser
bool CMuzipArc::ArchiveDecomp(CLstFichierDecomp& lstFichierDecomp, const DECOMP* pdec)
{
	PrepareDecomp();
	
	POS pos;
	
	// Appliquer le filtre
	if (pdec && (pdec->nEcrase==ecrJamais || pdec->fEcraseDate)) {
		
		// Enlever de la liste les fichiers  ne pas dcompresser
		pos = lstFichierDecomp.GetPosDeb();
		while (pos) {
			const POS posSauve = pos;
			FICHIERDECOMP *const pFichierDecomp = lstFichierDecomp.GetSuiv(pos);
			
			// Composer le nom du fichier aprs dcompression
			lstrcpy(pFichierDecomp->pcFin, pFichierDecomp->pFichier->m_pszNom);
			
			// Rcuprer des informations sur le fichier
			CMyFindFile ff(pFichierDecomp->pszDest);
			if (!ff.IsError()) {
				// Le fichier existe: il faut grer l'crasement
				
				// Deux possibilits pour ne pas dcompresser un fichier
				// qui existe dj, et donc pour empcher un crasement:
				// - si on interdit l'crasement
				// - si l'option "n'craser que les fichiers plus anciens"
				//   est utilise et que la date est valide et trop rcente
				if (pdec->nEcrase==ecrJamais ||
				    (pdec->fEcraseDate &&
				    (ff.m_wfd.ftLastWriteTime.dwLowDateTime ||
				     ff.m_wfd.ftLastWriteTime.dwHighDateTime) &&
				    (CTempsDHS(ff.m_wfd.ftLastWriteTime) > pFichierDecomp->pFichier->m_dtM)))
					lstFichierDecomp.DeleteAt(posSauve);
			}
		}
	}
	
	// Il doit rester au moins un fichier
	VERIF(lstFichierDecomp.GetNb()>0);
	
	
	CMuzipOp mop(this, STA_ARCHIVEDECOMPT);
	
	// nEcrase est susceptible de changer si l'utilisateur
	// utilise l'option "ne pas redemander la prochaine fois"
	int nEcrase = pdec->nEcrase;
	
	// Calculer la taille totale des fichiers en lecture
	DWORD dwTotal = 0;
	pos = lstFichierDecomp.GetPosDeb();
	while (pos)
		dwTotal += lstFichierDecomp.GetSuiv(pos)->pFichier->m_dwTailleC;
	oApp.ShowStatusAvance(dwTotal);
	
	// Dcompresser chaque fichier
	const bool fArbo = (pdec && pdec->fArbo);
	UINT iFichier = 0;
	pos = lstFichierDecomp.GetPosDeb();
	while (pos && oApp.UpdateProgressStatus() != prsAnnuler) {
		FICHIERDECOMP *const pFichierDecomp = lstFichierDecomp.GetSuiv(pos);
		oApp.SetProgress(STA_ARCHIVEDECOMP, pFichierDecomp->pFichier->m_pszNom,
			iFichier++, lstFichierDecomp.GetNb());
		
		
		if (fArbo) {
			// Respecter l'arborescence
			
			const LPCTSTR pszPathFichier =
				pFichierDecomp->pFichier->m_pDossier->m_pszPath;
			
			// Crer le sous-dossier
			TCHAR pszDossier[MAX_PATH];
			*pFichierDecomp->pcFin = 0;
			PathCombine(pszDossier, pFichierDecomp->pszDest, pszPathFichier);
			while (!DossierCreer(pszDossier, false)) {
				AfxFormatString1(e_sMsg, ERR_DOSSIERCREER, pszDossier);
				const int ret = AfxMessageBox(e_sMsg, MB_ABORTRETRYIGNORE | MB_DEFBUTTON2);
				if (ret == IDABORT)
					goto End;
				if (ret == IDIGNORE)
					goto Next;
			}
			
			// Composer le chemin d'accs final du fichier
			PathCombine(pFichierDecomp->pcFin,
				pszPathFichier, pFichierDecomp->pFichier->m_pszNom);
			
			
		}else
			// Tous les fichiers dans le mme dossier
			
			lstrcpy(pFichierDecomp->pcFin, pFichierDecomp->pFichier->m_pszNom);
		
		
		if (PathFileExists(pFichierDecomp->pszDest)) {
			// Grer l'crasement
			
			int nEcraseTemp = nEcrase;
			
			if (nEcrase == ecrDemander) {
				// Demander  l'utilisateur ce qu'il faut faire
				
				// Afficher la bote de dialogue
				AfxFormatString1(e_sMsg, DEM_ECRASER, pFichierDecomp->pszDest);
				CDlgEcrase dlgEcrase;
				nEcraseTemp = dlgEcrase.DoModal();
				
				// Mmoriser le choix si on a coch "ne pas redemander"
				if (dlgEcrase.m_fPasConfirm)
					nEcrase = nEcraseTemp;
				
				// Annuler = on arrte tout
				if (nEcraseTemp == ecrAnnuler)
					break;
			}
			
			
			ASSERT(nEcraseTemp != ecrDemander && nEcraseTemp != ecrAnnuler);
			
			
			// Jamais = on ignore le fichier
			if (nEcraseTemp == ecrJamais)
				continue;
			
			// Renommer
			if (nEcraseTemp==ecrRenommer) {
				// On ajoute un numro au nom de l'ancien fichier
				// au lieu de l'craser
				
				for (UINT uNumero=2;; uNumero++) {
					
					// Rcuprer le nom sans extension
					CString sOld = pFichierDecomp->pszDest;
					PathRemoveExtension(sOld.GetBuffer(MAX_PATH));
					sOld.ReleaseBuffer();
					
					// Ajouter le numro
					CString sNumero;
					sNumero.Format(_T(" %u"), uNumero);
					sOld += sNumero;
					
					// Remettre l'extension
					sOld += PathFindExtension(pFichierDecomp->pszDest);
					
					if (!PathFileExists(sOld)) {
						// On a trouv un nom inutilis: c'est termin
						
						MoveFile(pFichierDecomp->pszDest, sOld);
						break;
					}
				}
			}
		}
		
		
		// Dcompresser le fichier
		pFichierDecomp->pFichier->Decomp(pFichierDecomp->pszDest, pdec);
Next:
		;
	}
End:
	oApp.HideStatusAvance();
	
	return true;
}

// Dcompresser une liste de fichiers
// On n'autorise la modification des options par dfaut (oApp.m_dec)
// que si celles-ci sont passes en paramtre  la fonction
bool CMuzipArc::ArchiveDecomp(CLstFichier& lstFichier, LPCTSTR pszRep, const DECOMP* pdec)
{
	// N'autoriser la modification des options par dfaut
	// que si celles-ci sont passes en paramtre  la fonction
	CDlgDecomp dlgDecomp((pdec==&oApp.m_dec), this);
	
	if (!pszRep || !*pszRep) {
		// Demander le dossier de destination et les options
		
		VERIF(dlgDecomp.DoModal()!=IDCANCEL);
		pszRep =  dlgDecomp.m_sDest;
		pdec   = &dlgDecomp.m_dec;
	}
	
	
	// Construire la liste des FICHIERDECOMP
	
	// - Composer la base du dossier de destination
	TCHAR pszDest[MAX_PATH];
	lstrcpy(pszDest, pszRep);
	TCHAR *const pcFin = PathAddBackslash(pszDest);
	
	// - Construire la liste
	CLstFichierDecomp lstFichierDecomp(true);
	POS pos = lstFichier.GetPosDeb();
	while (pos) {
		CFichier *const pFichier = lstFichier.GetSuiv(pos);
		
		// Grer le mode "dcompresser que les fichiers trouvs par la recherche"
		if (pdec->fQueTrouve && !pFichier->m_fTrouveAjout)
			continue;
		
		// Crer l'entre
		FICHIERDECOMP *const pFichierDecomp = new FICHIERDECOMP;
		pFichierDecomp->pFichier = pFichier;
		pFichierDecomp->pszDest  = pszDest;
		pFichierDecomp->pcFin    = pcFin;
		lstFichierDecomp.AddDeb(pFichierDecomp);
	}
	
	// Dcompresser cette liste
	return ArchiveDecomp(lstFichierDecomp, (pdec) ? pdec : &oApp.m_dec);
}

// Ajouter des fichiers  une liste de fichiers  dcompresser
void CLstFichierDecomp::Ajouter(CLstFichier& lstFichier, const DECOMP* pdec,
	LPTSTR pszDest, TCHAR* pcFin)
{
	POS pos = lstFichier.GetPosDeb();
	while (pos) {
		CFichier *const pFichier = lstFichier.GetSuiv(pos);
		
		// Grer le mode "dcompresser que les fichiers trouvs par la recherche"
		if (pdec && pdec->fQueTrouve && !pFichier->m_fTrouveRech)
			continue;
		
		// Crer l'entre
		FICHIERDECOMP *const pFichierDecomp = new FICHIERDECOMP;
		pFichierDecomp->pFichier = pFichier;
		pFichierDecomp->pszDest  = pszDest;
		pFichierDecomp->pcFin    = pcFin;
		AddDeb(pFichierDecomp);
	}
}



// Recompresser des fichiers
void CMuzipArc::ArchiveRecomp(CLstFichier& lstFichier)
{
	// Composer le filtre en insrant correctement les sparateurs de parties
	// Les fichiers  compresser seront ceux du cache
	CString sFiltre;
	POS pos = lstFichier.GetPosDeb();
	while (pos) {
		CString sFile;
		lstFichier.GetSuiv(pos)->GetCacheFile(sFile);
		
		if (!sFiltre.IsEmpty())
			sFiltre += cSepPartie;
		sFiltre += sFile;
	}
	
	// Recompresser
	ArchiveRecompInterne(sFiltre, NULL, &lstFichier);
}


// Recompresser un dossier
void CMuzipArc::ArchiveRecomp(CDossier* pDossier)
{
	// Composer le filtre
	CString sFiltre;
	pDossier->GetCacheDir(sFiltre);
	PathAppend(sFiltre.GetBuffer(MAX_PATH), _T("*.*"));
	sFiltre.ReleaseBuffer();
	
	// Recompresser
	ArchiveRecompInterne(sFiltre, pDossier, NULL);
}


// Recompresser, soit un dossier, soit une liste de fichiers
// (pas les deux  la fois)
void CMuzipArc::ArchiveRecompInterne(const CString& sFiltre,
	CDossier* pDossier, CLstFichier* plstFichier)
{
	ASSERT(ToBool(pDossier) ^ ToBool(plstFichier));
	
	CString sMdp1 = m_sMdp;
	CString sMdp2 = m_sMdp;
	
	// Demander les options de recompression
	CDlgRecomp dlgRecomp(this, sMdp1, sMdp2);
	if (dlgRecomp.DoModal()==IDCANCEL)
		return;
	
	// Mettre le dossier ou les fichiers slectionns en cache
	const CString sMdpOld = m_sMdp;
	m_sMdp = sMdp1;
	if ((pDossier)
			? pDossier->CacheAddRef()
			: CacheAddRef(*plstFichier)) {
		// Compresser avec les nouvelles options
		
		// Mot de passe
		m_sMdp = sMdp2;
		
		// Options
		CPlusItem pi;
		pi.SetDefautExt();
		pi.CO_nArbo   = arboSous;
		pi.CO_nNiveau = dlgRecomp.m_nNiveau;
		pi.DO_sFiltre = sFiltre;
		
		// Effectuer la compression
		CArchivePlus arPlus;
		arPlus.AddDeb(&pi);
		ArchivePlus(arPlus);
	}
	
	// Restaurer l'ancien mot de passe
	m_sMdp = sMdpOld;
	
	// Librer les fichiers du cache
	(pDossier)
		? pDossier->CacheRelease()
		: CacheRelease(*plstFichier);
}


// Tester
void CMuzipArc::ArchiveTester(CLstFichier& lstFichier)
{
	POS pos;
	
	// Passer en mode test
	SPACE_DECOMP::e_dm = dmTester;
	PrepareDecomp();
	
	CMuzipOp mop(this, STA_ARCHIVETESTERT);
	
	// Calculer la taille totale des fichiers en lecture
	DWORD dwTotal = 0;
	pos = lstFichier.GetPosDeb();
	while (pos)
		dwTotal += lstFichier.GetSuiv(pos)->m_dwTailleC;
	oApp.ShowStatusAvance(dwTotal);
	
	// Tester chaque fichier en grant la progression
	UINT iFichier = 0;
	pos = lstFichier.GetPosDeb();
	while (pos && oApp.UpdateProgressStatus()!=prsAnnuler) {
		CFichier *const pFichier = lstFichier.GetSuiv(pos);
		oApp.SetProgress(STA_ARCHIVETESTER, pFichier->m_pszNom,
			iFichier++, lstFichier.GetNb());
		
		pFichier->Decomp(NULL, NULL);
	}
	oApp.HideStatusAvance();
	
	// Revenir au mode normal
	SPACE_DECOMP::e_dm = dmNormal;
}


// Crer un auto-extractible
bool CMuzipArc::ArchiveSfx(LPCTSTR pszEXE, LPCTSTR pszDest,
	bool fEcraser, CFichier* pFichierExec)
{
	const HINSTANCE hInstRes = AfxGetResourceHandle();
	
	// L'auto-extractible est compos de trois parties, dans cet ordre:
	// 1) L'en-tte excutable, fixe, contenu sous forme compresse
	//    dans la DLL des ressources
	// 2) L'archive telle quelle, sans aucune modification
	// 3) Un "footer" qui contient des informations sur l'auto-extractible
	//    (position du dbut de l'archive dans le fichier, longueur de l'archive,
	//    et des informations sur la dcompression)
	
	
	// Charger la ressource contenant l'auto-extractible
	const HRSRC   hRSrc   = FindResource(hInstRes,
		MAKEINTRESOURCE(GetSfxID()), _T("SFX"));
	VERIF(hRSrc);
	const HGLOBAL hGlobal = LoadResource(hInstRes, hRSrc);
	VERIF(hGlobal);
	void *const pvHeader  = LockResource(hGlobal);
	VERIF(pvHeader);
	
	bool fOK = false;
	
	// crire la ressource dans un fichier temporaire
	CString sHeaderFile;
	{
		// Rcuprer le nom du fichier temporaire avec .gz comme extension
		TCHAR pszDossierTemp[MAX_PATH];
		VERIF(
			e_opt.GetDossier(repTemp, pszDossierTemp) &&
			GetFichierTemp(pszDossierTemp, sHeaderFile.GetBuffer(MAX_PATH)));
		sHeaderFile.ReleaseBuffer();
		sHeaderFile += _T(".gz");
		
		// crire l'header SFX dans le fichier
		CFileWrite fw;
		fOK =
			fw.Open(sHeaderFile) &&
			fw.Write(pvHeader, SizeofResource(hInstRes, hRSrc));
		fw.Close();
	}
	
	if (fOK) {
		
		//*** Dcompresser l'header SFX (qui est une archive GZip)
		
		// Ouvrir l'archive
		CMuzipArc *const parcHeader = OpenArchive(sHeaderFile, NULL, NULL, true);
		if (parcHeader) {
			
			// Effectuer la dcompression puis fermer l'archive
			DECOMP dec = e_decEcraseToujours;
			parcHeader->PrepareDecomp();
			fOK = parcHeader->m_lstFichier.GetDeb()->Decomp(pszEXE, &dec);
			parcHeader->Release(false);
			
			// Ajouter l'archive et le footer  l'header SFX
			CFileWrite fw;
			if (fOK && fw.Open(pszEXE, 0, OPEN_EXISTING)) {
				
				fw.Seek(0, FILE_END);
				m_fr.Seek(0, FILE_BEGIN);
				
				// Initialiser le footer
				SFXFOOTER sfx;
				sfx.id         = idSFXFooter;
				sfx.off        = fw.Tell();
				sfx.lenArchive = GetFileSize(m_fr, NULL);
				sfx.offExec    = (pFichierExec) ? pFichierExec->GetOffSfx() : (DWORD)-1;
				lstrcpy(sfx.pszDest, pszDest);
				sfx.dwFlags    = 0;
				if (fEcraser)      sfx.dwFlags |= sfxEcraser;
				if (pFichierExec)  sfx.dwFlags |= sfxExec;
				
				// crire l'archive, le footer, puis fermer le fichier
				fOK =
					fw.WriteFrom(m_fr, sfx.lenArchive) &&
					fw.Write(&sfx, sizeof(sfx));
				fw.Close();
			}
		}
	}
	
	// En cas d'erreur on supprime l'auto-extractible
	if (!fOK)
		DeleteFile(pszEXE);
	
	return fOK;
}


// Convertir l'archive dans un autre format
// fConvertirFichiers prcise s'il faut convertir l'archive toute entire
// en tant que fichier ou bien chaque fichier individuellement
// fConvertirFichiers ne peut tre  TRUE que si le format cible
// est multi-fichiers ou si l'archive source a un ou aucun fichier
void CMuzipArc::Convertir(LPCTSTR pszCible, CDocType& dt, bool fConvertirFichiers)
{
	// Ouvrir la nouvelle archive
	if (!FichierCreer(pszCible))
		return;
	CMuzipArc *const pArc = OpenArchive(pszCible, NULL, &dt);
	
	// Effectuer la conversion
	bool fOK = false;
	if (pArc) {
		fOK = Convertir(pArc, fConvertirFichiers);
		pArc->Release(false);
	}
	
	// En cas d'erreur supprimer le fichier cr
	if (!fOK)
		DeleteFile(pszCible);
}


// Convertir l'archive
bool CMuzipArc::Convertir(CMuzipArc* pArc, bool fConvertirFichiers)
{
	// Crer la requte de compression
	CArchivePlus arPlus;
	CDlgPlusItem dlgPI(pArc, NULL, false, pArc->m_sMdp);
	dlgPI.SetDefautExt();
	VERIF(dlgPI.DoModal()==IDOK);
	
	// Composer le filtre
	if (fConvertirFichiers) {
		// On doit compresser le contenu de l'archive et non l'archive elle-mme
		// Il faut donc dcompresser l'archive
		
		// Mettre en cache la totalit de l'archive
		CDossier::CacheAddRef();
		
		// On compresse le dossier de l'archive
		GetCacheDir(dlgPI.DO_sFiltre);
		PathAppend(dlgPI.DO_sFiltre.GetBuffer(MAX_PATH), _T("*.*"));
		dlgPI.DO_sFiltre.ReleaseBuffer();
		
	}else
		// On compresse l'archive elle-mme
		dlgPI.DO_sFiltre = m_sPath;
	
	
	// Effectuer la compression
	arPlus.AddDeb(&dlgPI);
	const bool fOK = pArc->ArchivePlus(arPlus) && pArc->Reconstruire();
	
	// Librer le cache si on a compress les fichiers individuellement
	if (fConvertirFichiers)
		CDossier::CacheRelease();
	
	return fOK;
}


// Ouvrir l'archive, crer un fichier temporaire
// et y copier les entres de la liste
// L'archive est ouverte en mode criture dans e_fw
bool CMuzipArc::OpenArchiveTemp()
{
	// Rcuprer le nom du fichier temporaire
	TCHAR pszTemp[MAX_PATH];
	TCHAR pszDossierTemp[MAX_PATH];
	VERIF(
		e_opt.GetDossier(repTemp, pszDossierTemp) &&
		GetFichierTemp(pszDossierTemp, pszTemp));
	
	// Aller au dbut de l'archive
	VERIF(m_fr.Seek(0, FILE_BEGIN));
	
	// Ouvrir le fichier temporaire
	VERIF(e_fw.Open(pszTemp));
	
	// Ouvrir l'ancienne archive pour crire les anciens fichiers
	CFichier::s_nbFichier = GetNbFichier();
	CFichier::s_nFichier  = 0;
	oApp.ShowStatusAvance(CFichier::s_nbFichier);
	oApp.SetStatusLabel(IDCLBL_2, STA2_FICHIER);
	const bool fOK = FichierCopier(m_fr);
	CFichier::s_nbFichier = 0;
	if (!fOK)
		e_fw.Close();
	oApp.HideStatusAvance();
	
	return fOK;
}

// Fermer une archive temporaire
bool CMuzipArc::CloseArchiveTemp()
{
	const CString sTemp = e_fw.GetPath();
	e_fw.Close();
	
	// Effacer l'ancienne archive
	ReadClose();
	if (!FichierSupprimer(m_sPath, false)) {
ErreurRemplacer:
		AfxMessageBox(ERR_REMPLACER);
		ReadOpen();
		return false;
	}
	
	// Le remplacer par le fichier temporaire
	while (!FichierDeplacer(sTemp, m_sPath)) {
		AfxFormatString2(e_sMsg, ERR_RENOMMER, sTemp, m_sPath);
		if (AfxMessageBox(e_sMsg, MB_RETRYCANCEL | MB_ICONEXCLAMATION)==IDCANCEL)
			goto ErreurRemplacer;
	}
	
	return ReadOpen();
}



// Initialiser la progression principale
// avec la taille totale des fichiers
void CMuzipArc::ShowStatusAvanceFichier()
{
	DWORD dwTailleTotal = 0;
	
	POS pos = s_lstFichierPlus.GetPosDeb();
	while (pos)
		dwTailleTotal += s_lstFichierPlus.GetSuiv(pos)->m_pFichier->m_dwTailleN;
	
	oApp.ShowStatusAvance(dwTailleTotal, true);
	oApp.SetStatusLabel(IDCLBL_2, STA2_FICHIER);
}



// Ouvrir une archive
CMuzipArc* OpenArchive(LPCTSTR pszFichier, CMuzipDoc* pDoc, CDocType* pdtForce, bool fSilent)
{
	// Rcuprer le chemin complet
	TCHAR pszPath[MAX_PATH];
	MyGetFullPathName(pszFichier, pszPath);
	
	CMuzipArc *pArc;
	
	// Regarder si l'archive est dj ouverte
	{
		POS pos = e_lstArc.GetPosDeb();
		while (pos) {
			pArc = e_lstArc.GetSuiv(pos);
			
			if (pArc->GetPath()==pszPath) {
				// L'archive est dj ouverte
				// Mais si elle contient une archive fille,
				// alors c'est celle-ci qu'il faut ouvrir en ralit
				
				// Rcuprer la dernire archive fille
				pArc = CMuzipArc::GetDerniereFille(pArc);
				
				// On a notre archive, il ne reste plus qu' l'instancier
				pArc->AddRef(pDoc);
				return pArc;
			}
		}
	}
	
	// Tester l'ouverture du fichier
	CFileRead fr;
	fr.SetVerbeux(!fSilent);
	if (!fr.Open(pszPath, FILE_SHARE_ALL))
		return NULL;
	fr.Close();
	
	
	// Ouvrir l'archive
	// Chercher d'abord le meilleur type de document
	pArc = NULL;
	if (!pdtForce) {
		POS pos = oApp.m_lstDocType.GetPosDeb();
		while (pos) {
			CDocType &dt = *oApp.m_lstDocType.GetSuiv(pos);
			
			CDocument *pDocMatch;
			if (dt.MatchDocType(pszPath, pDocMatch)==CDocTemplate::yesAttemptNative) {
				// Template trouv !
				pdtForce = &dt;
				break;
			}
		}
	}
	
	if (pdtForce) {
		// On sait avec quoi ouvrir l'archive
		
		// Dsactiver la fentre principale pour viter une fermeture
		// de Muzip pendant l'ouverture de l'archive
		if (oApp.m_pMainWnd)
			oApp.m_pMainWnd->EnableWindow(FALSE);
		
		// Ouvrir l'archive
		pArc = pdtForce->CreateArchive(
			(pDoc) ? new CInterfDoc(pDoc) : new CInterf);
		if (!pArc->OpenDocument(pszPath, fSilent)) {
			// Erreur  l'ouverture: librer l'archive
			pArc->Release(false);
			pArc = NULL;
		}
		
		// Ractiver la fentre principale
		if (oApp.m_pMainWnd)
			oApp.m_pMainWnd->EnableWindow();
		
		return CMuzipArc::GetDerniereFille(pArc);
	}
	
	
	// Erreur: aucun format ne correspond
	AfxFormatString1(e_sMsg, ERR_FORMATINCONNU, pszPath);
	AfxMessageBox(e_sMsg);
	
	return NULL;
}

// Rcuprer la dernire archive fille d'une archive donne
// Cette fonction est statique pour accepter un pArcParent nul
CMuzipArc* CMuzipArc::GetDerniereFille(CMuzipArc* pArcParent)
{
	// Rcuprer la fin de la file des archives filles
	while (pArcParent && pArcParent->m_parcFille)
		pArcParent = pArcParent->m_parcFille;
	
	return pArcParent;
}


// Crer un raccourci de dcompression pour une archive
// fVerbeux indique si un message d'erreur peut apparatre
//  la cration du raccourci
bool CreateArchiveShortcut(LPCTSTR pszArchive, LPCTSTR pszLink,
	LPCTSTR pszDest, LPCTSTR pszOptions, bool fDemanderOptions, bool fVerbeux)
{
	CString sLink = pszLink;
	sLink.GetBuffer(MAX_PATH);
	
	if (PathIsDirectory(sLink) && pszArchive && *pszArchive) {
		// La destination est un dossier:
		// y ajouter le nom de l'archive au dossier
		
		PathAppend((STRING)sLink, PathFindFileName(pszArchive));
		sLink.ReleaseBuffer();
	}
	
	// Ajouter l'extension "lnk" des raccourcis si ncessaire
	const LPCTSTR pszLinkExt = _T(".lnk");
	if (lstrcmpi(PathFindExtension(sLink), pszLinkExt))
		PathAddExtension((STRING)sLink, pszLinkExt);
	
	sLink.ReleaseBuffer();
	
	if (fVerbeux && !FichierEcraser(sLink))
		return true;
	
	
	// Rcuprer une interface IShellLink pour le futur raccourci
	IShellLink *psl;
	VERIF(SUCCEEDED(CoCreateInstance(
		CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER,
		IID_IShellLink, (void**)&psl)));
	
	bool fOK = false;
	
	// Rcuprer un IPersistFile pour stocker le raccourci
	IPersistFile* ppf;
	if (SUCCEEDED(psl->QueryInterface(IID_IPersistFile, (void**)&ppf))) {
		TCHAR pszTemp[MAX_PATH];
		
		// Excutable
		GetModuleFileName(e_em.hModule, pszTemp, nbArray(pszTemp));
		PathRemoveFileSpec(pszTemp);
		PathAppend(pszTemp, pszMuzipEXE);
		PathQuoteSpaces(pszTemp);
		fOK = SUCCEEDED(psl->SetPath(pszTemp));
		
		
		// Arguments
		// Ne pas oublier les guillemets aux noms de fichiers !
		// Syntaxe: /q /u [dossier destination] [/o=options] [/s] [archive]
		
		CString sArgs = _T("/q /u");
		
		// Dossier de destination
		if (pszDest && *pszDest) {
			lstrcpy(pszTemp, pszDest);
			PathQuoteSpaces(pszTemp);
			sArgs += _T(" /d ");
			sArgs += pszTemp;
		}
		
		// Options d'extraction
		if (pszOptions && *pszOptions) {
			sArgs += _T(" /o=");
			sArgs += pszOptions;
		}
		
		// Commutateur "forcer la demande des options"
		if (!fDemanderOptions)
			sArgs += _T(" /s");
		
		// Nom de l'archive
		if (pszArchive && *pszArchive) {
			lstrcpy(pszTemp, pszArchive);
			PathQuoteSpaces(pszTemp);
			sArgs += ' ';
			sArgs += pszTemp;
		}
		
		fOK &= SUCCEEDED(psl->SetArguments(sArgs));
		
		// Icne
		::GetModuleFileName(AfxGetResourceHandle(), pszTemp, nbArray(pszTemp));
		psl->SetIconLocation(pszTemp, 6);
		
		// Enregistrer le raccourci
#ifdef UNICODE
		fOK &= SUCCEEDED(ppf->Save(sLink, TRUE));
#else
		WCHAR wszLink[MAX_PATH];
		strtoW(wszLink, sLink, nbArray(wszLink));
		fOK &= SUCCEEDED(ppf->Save(wszLink, TRUE));
#endif
		ppf->Release();
	}
	
	psl->Release();
	return fOK;
}
