// MuzipArc.h


#ifndef _MUZIPARC_H_
#define _MUZIPARC_H_
#if _MSC_VER>1000
#pragma once
#endif

#include "Global.h"
#include "Fichier.h"
#include "Dossier.h"
#include "PlusItem.h"
#include "Decomp.h"
#include "Options.h"

#include <FileBuf.h>

class CDocType;


enum
{
	capNiveau           = 0x001,
	capCommentaire      = 0x002,
	capArbo             = 0x004,
	capAttribut         = 0x008,
	capCrypte           = 0x010,
	capRapideRenommer   = 0x020,
	capRapideModifProp  = 0x040,
	capSfx              = 0x080,
	capdtM              = 0x100,
	capMaxUnFichier     = 0x200,
	capLectureSeule     = 0x400,
	capDoitReconstruire = 0x800,
};

enum PRGSTATUS;

struct DECOMP;

class CMuzipDoc;
class CDocType;
class CInterf;
class CArchivePlus;
class CMuzipCmdLine;

class CFichierPlus;
typedef CListePtr1<CFichierPlus*> CLstFichierPlus;


class CMuzipArc : public CDossier
{
	friend class CFichier;
	friend CMuzipArc* OpenArchive(LPCTSTR pszFichier, CMuzipDoc* pDoc, CDocType* pdtForce, bool fSilent);
	
public:  // Fonctions
	
	void AddRef (CMuzipDoc* pDoc=NULL);
	void Release(bool fDoc);
	
	bool OpenDocument(LPCTSTR pszFichier, bool fSilent);
	bool IsArchiveGoodType(LPCTSTR pszFichier);
	
	bool FautReconstruire(RM_OPTION_AC rmo=rmoRien);
	
	bool ReadOpen();
	void ReadClose();
	bool WriteOpen();
	void WriteClose();
	
	bool ArchivePlus  (const CArchivePlus& arPlus);
	bool DoArchivePlus();
	bool ProgressArchivePlus(bool& rfOK, CFichier* pFichier, PRGSTATUS prs, POS pos, DWORD offOld);
	PRGSTATUS ErreurArchivePlus();
	void FinaliserAjout();
	void ArchiveMoins (CLstFichier& lstFichier);
	bool ArchiveDecomp(CLstFichierDecomp& lstFichierDecomp, const DECOMP* pdec);
	bool ArchiveDecomp(CLstFichier& lstFichier, LPCTSTR pszRep, const DECOMP* pdec);
	void ArchiveRecomp(CLstFichier& lstFichier);
	void ArchiveRecomp(CDossier* pDossier);
	void ArchiveTester(CLstFichier& lstFichier);
	bool ArchiveSfx   (LPCTSTR pszEXE, LPCTSTR pszDest, bool fEcraser, CFichier* pFichierExec);
	
	void Convertir(LPCTSTR pszCible, CDocType& dt, bool fConvertirFichiers);
	
	bool Reconstruire();
	
	
	bool IsSurDisqueAmovible() const;
	
	// Gestion du cache
	bool CacheAddRef (CLstFichier& lstFichier, bool fOK=true);
	void CacheRelease(CLstFichier& lstFichier);
	
	
public:  // Interface
	
	// Chemin d'accs
	const CString& GetPath() const { return m_sPath; }
	
	// Dossier du cache
	const CString& GetCachePath() const { return m_sCachePath; }
	
	
	// Capacits
	int GetCap() const;
	
	// Capacit "lecture seule"
	bool IsLectureSeule() const
		{ return ToBool(GetCap() & capLectureSeule); }
	void SetLectureSeule(bool fLectureSeule=true)
		{ m_fLectureSeule = fLectureSeule; }
	
	
	// Dfinir l'archive parent
	void SetArchiveParent(CMuzipArc* pArc) { m_parcParent = pArc; }
	
	// Prparer la dcompression
	void PrepareDecomp() { SPACE_DECOMP::e_pfr = &m_fr; }
	
	
public:  // Hritage
	
	virtual UINT GetSfxID() const { return 0; }
	
	
	virtual bool OpenDocument(bool fSilent) = 0;
	virtual void CloseDocument();
	
	virtual bool OpenArchiveTemp();
	virtual bool CloseArchiveTemp();
	
	virtual CFichier* NewFichier (LPCTSTR pszFichier, const WIN32_FIND_DATA& wfd) = 0;
	
	virtual void SetComment(LPCTSTR psz) { psz; }
	virtual LPCTSTR GetComment() const { return NULL; }
	
	
protected:  // Interne
	
	void ArchiveRecompInterne(const CString& sFiltre, CDossier* pDossier, CLstFichier* plstFichier);
	
	bool OpenArchiveOK(bool fSilent, LPCTSTR pszComment=NULL);
	
	virtual bool IsGoodType() = 0;
	
	virtual bool ArchivePlus();
	virtual bool ArchivePlusUn(CFichierPlus* pFichierPlus, CFileRead& fr) { pFichierPlus; fr;  return false; }
	
	virtual void IgnorerFichier(DWORD offOld, CFichier* pFichier);
	
	virtual bool Convertir(CMuzipArc* pArc, bool fConvertirFichiers);
	
	
	int m_nUse;   // Compteur d'utilisation
	CMuzipArc(CInterf* pi);
	virtual ~CMuzipArc();
	
	
	static void ShowStatusAvanceFichier();
	
	
public:  // Donnes
	
	CInterf    *m_pi;      // Interface
	CDocType   *m_pmdt;    // Type de document
	
	CString     m_sMdp;    // Mot de passe
	
	
	static CLstFichierPlus s_lstFichierPlus;   // Liste des fichiers  ajouter
	
	bool m_fRechFiltre;      // Filtrer recherche
	bool m_fRm;              // Reconstruction manuelle
	
	
protected:  // Donnes internes
	
	CFileRead   m_fr;           // Fichier en mode lecture
	
	CString     m_sPath;        // Chemin d'accs de l'archive
	CString     m_sCachePath;   // Dossier de cache (rpertoire temporaire d'extraction)
	
	bool m_fLectureSeule;       // TRUE si modification interdite
	                            // (s'ajoute  capLectureSeule)
	
	// Pour le cas de fichier d'une archive
	// ouvert comme archive  la place de l'archive elle-mme
	// Exemple: archive Tar compresse dans une archive GZip
	// Dans ce cas on a la structure suivante:
	//
	// ArchiveGZip
	//   parcParent = NULL;
	//   parcFille  = &ArchiveTar;
	//
	// ArchiveTar
	//   parcParent = &ArchiveGZip;
	//   parcFille  = NULL;
	//
	CMuzipArc *m_parcParent;    // Archive parent
	CMuzipArc *m_parcFille;     // Archive fille
	
	static CMuzipArc* GetDerniereFille(CMuzipArc* pArcParent);
	
	
	CLstArchivePlus m_lstRM;    // Actions d'ajout en attente de reconstruction manuelle
};

typedef CListe1<CMuzipArc*> CLstMuzipArc;


extern CLstMuzipArc e_lstArc;


class CDocType;
CMuzipArc* OpenArchive(LPCTSTR pszFichier, CMuzipDoc* pDoc=NULL, CDocType* pdtForce=NULL, bool fSilent=false);

bool CreateArchiveShortcut(LPCTSTR pszArchive, LPCTSTR pszLink,
	LPCTSTR pszDest, LPCTSTR pszOptions, bool fDemanderOptions, bool fVerbeux);


#endif
