// MuzipDoc.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "Muzip2.h"
#include "MuzipDoc.h"
#include "MuzipArc.h"
#include "InterfDoc.h"

#include "IOFichier.h"

#include "MainFrame.h"
#include "MuzipFrame.h"
#include "ArboView.h"
#include "MuzipView.h"
#include "ApercuView.h"
#include "BitmapView.h"

#include "PlusItem.h"
#include "Recherche.h"

#include "Decomp.h"

#include "dlgConfirm.h"
#include "dlgRech.h"
#include "dlgArchivePlus.h"
#include "dlgDecomp.h"
#include "dlgLister.h"
#include "dlgConvertir.h"
#include "dlgSfx.h"
#include "dlgMdp.h"
#include "dlgComment.h"
#include "dlgOptions.h"
#include "dlgResRech.h"

#include "Macro.h"
#include "Options.h"
#include "Reg.h"


IMPLEMENT_DYNCREATE(CMuzipDoc, CDocument)
BEGIN_MESSAGE_MAP(CMuzipDoc, CDocument)
	//{{AFX_MSG_MAP(CMuzipDoc)
	ON_COMMAND          (ID_FILE_DELETE,               OnFileDelete)
	ON_COMMAND          (ID_FILE_SEND_MAIL,            OnFileSendMail)
	ON_COMMAND          (ID_FILE_LISTER,               OnFileLister)
	ON_COMMAND          (ID_ARCHIVE_PLUS,              OnArchivePlus)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_PLUS,              OnUpdateFautModif)
	ON_COMMAND          (ID_ARCHIVE_CONVERTIR,         OnArchiveConvertir)
	ON_COMMAND          (ID_ARCHIVE_SFX,               OnArchiveSfx)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_SFX,               OnUpdateArchiveSfx)
	ON_COMMAND          (ID_ARCHIVE_NOUVEAUDOSSIER,    OnArchiveNouveauDossier)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_NOUVEAUDOSSIER,    OnUpdateArchiveNouveauDossier)
	ON_COMMAND          (ID_ARCHIVE_RM,                OnArchiveRm)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_RM,                OnUpdateArchiveRm)
	ON_COMMAND          (ID_ARCHIVE_MAJ,               OnArchiveMaj)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_MAJ,               OnUpdateArchiveMaj)
	ON_COMMAND          (ID_ARCHIVE_MDP,               OnArchiveMdp)
	ON_COMMAND          (ID_ARCHIVE_COMMENT,           OnArchiveComment)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_COMMENT,           OnUpdateArchiveComment)
	ON_COMMAND          (ID_ARCHIVE_RECHERCHER,        OnArchiveRechercher)
	ON_COMMAND          (ID_ARCHIVE_RECHERCHERSUIVANT, OnArchiveRechercherSuivant)
	ON_COMMAND          (ID_ARCHIVE_RECHERCHERFILTRER, OnArchiveRechercherFiltrer)
	ON_UPDATE_COMMAND_UI(ID_ARCHIVE_RECHERCHERFILTRER, OnUpdateArchiveRechercherFiltrer)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE,                 OnUpdateFileSave)
	ON_COMMAND          (ID_APERCU_DOS,                OnApercuDOS)
	ON_UPDATE_COMMAND_UI(ID_APERCU_DOS,                OnUpdateApercuDOS)
	ON_COMMAND          (ID_APERCU_RETOUR,             OnApercuRetour)
	ON_UPDATE_COMMAND_UI(ID_APERCU_RETOUR,             OnUpdateApercuRetour)
	ON_COMMAND          (ID_APERCU_POLICE,             OnApercuPolice)
	ON_COMMAND          (ID_APERCU_REDIM,              OnApercuRedim)
	ON_UPDATE_COMMAND_UI(ID_APERCU_REDIM,              OnUpdateApercuRedim)
	ON_COMMAND          (ID_APERCU_OPTIONS,            OnApercuOptions)
	ON_COMMAND          (ID_VIEW_RESRECH,              OnViewResRech)
	ON_UPDATE_COMMAND_UI(ID_VIEW_RESRECH,              OnUpdateViewResRech)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_COMMENT,         OnUpdateIndicatorComment)
	//}}AFX_MSG_MAP
	ON_UPDATE_COMMAND_UI(ID_FILE_SEND_MAIL,            CDocument::OnUpdateFileSendMail)
END_MESSAGE_MAP()


// Constructeur
CMuzipDoc::CMuzipDoc()
{
	m_pArc   = NULL;
	m_pCur   = NULL;
	m_pDragD = NULL;
}

// Destructeur
CMuzipDoc::~CMuzipDoc()
{
	if (m_pArc)
		m_pArc->Release(true);
}


// Retourner l'ID de ressource du document
UINT CMuzipDoc::GetIdType() const
{
	return ((CMuzipDocTemplate*)GetDocTemplate())->GetStringsID();
}



// Ouverture
BOOL CMuzipDoc::OnOpenDocument(LPCTSTR pszFichier)
{
	if (m_pArc)
		m_pArc->Release(true);
	
	m_pArc = OpenArchive(pszFichier, this);
	m_plst->m_plstMuzip->AttachArc(m_pArc);
	m_ptrv->             AttachArc(m_pArc);
	
	VERIF(m_pArc);
	
	// Excuter les macros automatiques
	if (e_opt.m_nMacroAutoOpen!=maoJamais) {
		CMacContexte ctx;
		CMacro mac;
		
		bool fPermissionOK = e_opt.m_nMacroAutoOpen!=maoDemander;
		
		// Lister les macros automatiques potentielles
		CLstFichier lstFichierMacro;
		m_pArc->ListMacro(lstFichierMacro);
		
		// Excuter les macros automatiques
		POS pos = lstFichierMacro.GetPosDeb();
		while (pos) {
			CFichier *const pFichier = lstFichierMacro.GetSuiv(pos);
			
			CString sFile;
			
			// Si erreur  la dcompression, il faut garder la macro dans la liste
			// au cas o l'erreur serait corrige par la suite
			if (pFichier->CacheAddRef(sFile)) {
				
				// Charger la macro
				// L'excuter si elle est automatique
				if (mac.FromINI(sFile)) {
					if (mac.m_fAutoOpen) {
						
						// Demander la permission
						if (!fPermissionOK) {
							BOOL bConfirm = TRUE;
							fPermissionOK = CDlgConfirm::GetReponseMan(bConfirm, DEM_AUTOOPEN);
							
							if (!bConfirm)
								e_opt.m_nMacroAutoOpen = (fPermissionOK)
									? maoToujours : maoJamais;
						}
						
						if (!fPermissionOK)
							break;
					}
					
					if (fPermissionOK)
						mac.Executer(ctx);
				}
				
				pFichier->CacheRelease();
				
				if (!fPermissionOK)
					break;
			}
		}
	}
	
	return TRUE;
}


// Supprimer le contenu
void CMuzipDoc::DeleteContents()
{
	m_pCur = NULL;
	if (m_pArc) {
		m_pArc->m_pi->OnDossierChange();
		m_pArc->Release(true);
	}
	
	m_pArc = ((CMuzipDocTemplate*)GetDocTemplate())->
		GetDocType()->CreateArchive(new CInterfDoc(this));
}


// Enregistrement
BOOL CMuzipDoc::OnSaveDocument(LPCTSTR)
{
	if (m_pArc)
		m_pArc->Reconstruire();
	
	return TRUE;
}


// Enregistrement
BOOL CMuzipDoc::DoSave(LPCTSTR, BOOL)
{
	return OnSaveDocument(NULL);
}


// Fermeture
void CMuzipDoc::OnCloseDocument()
{
	if (m_pArc)
		m_pArc->CloseDocument();
	
	CDocument::OnCloseDocument();
}


// Reconstruire si ncessaire, avec demande de confirmation
BOOL CMuzipDoc::SaveModified()
{
	if (IsModified()) {
		// Demander confirmation
		
		switch (AfxMessageBox(DEM_FERMER, MB_YESNOCANCEL)) {
			
			// Annuler
			case IDCANCEL:
				return FALSE;
			
			// Reconstruire: oui
			case IDYES:
				m_pArc->Reconstruire();
				break;
		}
	}
	
	return TRUE;
}


// Supprimer l'archive
void CMuzipDoc::OnFileDelete()
{
	// Supprimer l'archive et fermer le document
	m_pArc->ReadClose();
	if (CDlgConfirm::GetReponse(mcfFileDelete, DEM_FILEDELETE) &&
	    FichierSupprimer(GetPathName(), true))
		OnCloseDocument();
	else
		m_pArc->ReadOpen();
}


// Lister l'archive
void CMuzipDoc::OnFileLister()
{
	CDlgLister(m_pArc).DoModal();
}


// Envoyer un message
void CMuzipDoc::OnFileSendMail()
{
	const BOOL bModified = IsModified();
	if (SaveModified()) {
		SetModifiedFlag(FALSE);
		CDocument::OnFileSendMail();
		SetModifiedFlag(bModified);
	}
}


// Demander les fichiers  ajouter dans un dossier
// et passer le contrle au document pour la compression
void CMuzipDoc::OnArchivePlus()
{
	CArchivePlus arPlus(NULL, true);
	if (CDlgArchivePlus(m_pArc, NULL, arPlus, m_pArc->m_sMdp).DoModal()==IDOK)
		m_pArc->ArchivePlus(arPlus);
}



// Convertir en un autre format
void CMuzipDoc::OnArchiveConvertir()
{
	CDlgConvertir dlgConvertir(m_pArc);
	
	if (dlgConvertir.DoModal()==IDOK)
		m_pArc->Convertir(dlgConvertir.m_sCible,
			*dlgConvertir.m_pmdt, (dlgConvertir.m_nType==0));
}


// Crer un auto-extractible
void CMuzipDoc::OnArchiveSfx()
{
	CDlgSfx dlgSfx(m_plst->m_plstMuzip->GetFichierSel());
	dlgSfx.m_sEXE = m_pArc->GetPath();
	PathRenameExtension(dlgSfx.m_sEXE.GetBuffer(MAX_PATH), _T(".exe"));
	dlgSfx.m_sEXE.ReleaseBuffer();
	
	// Demander avant d'craser
	// Crer l'archive SFX
	// Afficher un message en cas d'erreur
	if (dlgSfx.DoModal()==IDOK && FichierEcraser(dlgSfx.m_sEXE) &&
	   !m_pArc->ArchiveSfx(dlgSfx.m_sEXE, dlgSfx.m_sDest,
				ToBool(dlgSfx.m_fEcraser), dlgSfx.m_pFichierExec))
		AfxMessageBox(ERR_SFX);
}
void CMuzipDoc::OnUpdateArchiveSfx(CCmdUI* pCmdUI)
{
	pCmdUI->Enable((m_pArc->GetCap() & capSfx) && !m_pArc->IsLectureSeule());
}


// Nouveau dossier
void CMuzipDoc::OnArchiveNouveauDossier()
{
	m_ptrv->ArchiveNouveauDossier(m_pCur);
}
void CMuzipDoc::OnUpdateArchiveNouveauDossier(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(
		(m_pArc->GetCap() & capCommentaire) &&
		!m_pArc->IsLectureSeule());
}


// Rechercher
void CMuzipDoc::OnArchiveRechercher()
{
	RECHOPTIONS &rch = oApp.m_rch;
	if (CDlgRech(true, rch).DoModal()==IDCANCEL)
		return;
	
	// Effectuer la recherche dans toutes les archives
	POS pos = e_lstArc.GetPosDeb();
	while (pos) {
		CMuzipArc *const pArc = e_lstArc.GetSuiv(pos);
		CMuzipDoc *const pDoc = pArc->m_pi->GetDoc();
		if (!pDoc)
			continue;
		
		pArc->m_pi->Rechercher();
		if (pDoc->m_plst && pArc->m_fRechFiltre)
			pDoc->m_plst->OnUpdate(NULL, NULL,NULL);
	}
	
	// Rechercher suivant
	if (!m_pArc->m_fRechFiltre)
		OnArchiveRechercherSuivant();
}

// Rechercher suivant
void CMuzipDoc::OnArchiveRechercherSuivant()
{
	if (m_plst)
		m_plst->OnArchiveRechercherSuivant();
}

// Filtrer recherche
void CMuzipDoc::OnArchiveRechercherFiltrer()
{
	m_pArc->m_fRechFiltre = !m_pArc->m_fRechFiltre;
	m_plst->OnUpdate(NULL,NULL,NULL);
}
void CMuzipDoc::OnUpdateArchiveRechercherFiltrer(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_pArc->m_fRechFiltre);
}


// Mettre  jour
void CMuzipDoc::OnArchiveMaj()
{
	m_pArc->Reconstruire();
}
void CMuzipDoc::OnUpdateArchiveMaj(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(IsModified() && !m_pArc->IsLectureSeule());
}

// Reconstruction manuelle
void CMuzipDoc::OnArchiveRm()
{
	if (m_pArc->m_fRm && IsModified()) {
		if (!CDlgConfirm::GetReponse(mcfMaj, DEM_MAJ))
			return;
		m_pArc->Reconstruire();
	}
	m_pArc->m_fRm = !m_pArc->m_fRm;
}
void CMuzipDoc::OnUpdateArchiveRm(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_pArc->m_fRm);
	pCmdUI->Enable(!m_pArc->IsLectureSeule());
}

// Mot de passe
void CMuzipDoc::OnArchiveMdp()
{
	CDlgMdp(m_pArc->m_sMdp).DoModal();
}

// Commentaires
void CMuzipDoc::OnArchiveComment()
{
	CDlgComment dlgComment(m_pArc);
	if (dlgComment.DoModal()==IDOK && !m_pArc->IsLectureSeule())
		m_pArc->SetComment(dlgComment.m_s);
}
void CMuzipDoc::OnUpdateArchiveComment(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(ToBool(m_pArc->GetCap() & capCommentaire));
}
void CMuzipDoc::OnUpdateIndicatorComment(CCmdUI*)
{
	const LPCTSTR pszComment = m_pArc->GetComment();
	
	if (oApp.m_pMainWnd->IsKindOf(RUNTIME_CLASS(CMainFrame)))
		((CMainFrame*)oApp.m_pMainWnd)->SetComment(pszComment && *pszComment);
}


// Enregistrer
void CMuzipDoc::OnUpdateFileSave(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(FALSE);
}


//*** Pour CApercuView

// Retour  la ligne automatique
void CMuzipDoc::OnApercuRetour()
{
	m_papr->m_fRetour = !m_papr->m_fRetour;
	m_papr->m_rtf.SetTargetDevice(NULL, !m_papr->m_fRetour);
}
void CMuzipDoc::OnUpdateApercuRetour(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_papr->m_fRetour);
}

// Texte DOS
void CMuzipDoc::OnApercuDOS()
{
	m_papr->m_fDOS = !m_papr->m_fDOS;
	m_papr->m_pApercu->MAJ();
}
void CMuzipDoc::OnUpdateApercuDOS(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_papr->m_fDOS);
}

// Police
void CMuzipDoc::OnApercuPolice()
{
	if (m_papr->m_pApercu->GetToolBarID()==IDR_APERCUPOLICE) {
		m_papr->m_pApercu->OnSetPoliceFont();
		return;
	}
	
	LOGFONT lf;
	oApp.m_fntVoir.GetLogFont(&lf);
	
	// Demander la police
	CFontDialog dlg(&lf, CF_BOTH);
	if (dlg.DoModal()==IDOK) {
		dlg.GetCurrentFont(&lf);
		oApp.SetFontVoir(&lf);
	}
}

// Redimensionnement automatique
void CMuzipDoc::OnApercuRedim()
{
	m_papr->m_fRedim = !m_papr->m_fRedim;
	m_papr->m_pBmpView->MAJRedim();
}
void CMuzipDoc::OnUpdateApercuRedim(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_papr->m_fRedim);
}


// Options
void CMuzipDoc::OnApercuOptions()
{
	CDlgOptions(pgOptionsApercu).DoModal();
}


// Rsultats de la recherche
void CMuzipDoc::OnViewResRech()
{
	if (e_pdlgResRech) {
		// Fermer la fentre
		e_pdlgResRech->DestroyWindow();
		e_pdlgResRech = NULL;
	}else{
		// Crer la fentre et mettre  jour la liste
		e_pdlgResRech = new CDlgResRech;
		e_pdlgResRech->Create(e_pdlgResRech->IDD, oApp.m_pMainWnd);
		
		m_pArc->m_pi->MAJResRech();
	}
}
void CMuzipDoc::OnUpdateViewResRech(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(ToBool(e_pdlgResRech));
}


// Ouvrir un dossier
void CMuzipDoc::OuvrirDossier(const CDossier* pDossier, bool fDebut)
{
	CTreeCtrl &trv = *m_ptrv->m_ptrv;
	
	if (!pDossier->m_fOuvert) {
		// Ouvrir le dossier parent
		OuvrirDossier(pDossier->m_pParent, false);
		
		// Dvelopper le dossier parent
		trv.Expand(pDossier->m_pParent->m_hti, TVE_EXPAND);
	}
	
	if (fDebut) {
		// Slectionner le dossier
		trv.EnsureVisible(pDossier->m_hti);
		trv.SelectItem(pDossier->m_hti);
	}
}

// Slectionner un fichier
void CMuzipDoc::SelFichier(const CFichier* pFichier)
{
	// Ouvrir le dossier contenant le fichier
	SetActiveDocument(m_pArc->m_pi->GetDoc());
	m_plst->GetParentFrame()->SetActiveView(m_plst);
	OuvrirDossier(pFichier->m_pDossier);
	
	CListCtrl &lst = *m_plst->m_plst;
	
	// Tout dslectionner
	LVITEM lvi;
	lvi.iItem     = lst.GetItemCount();
	lvi.mask      = LVIF_STATE;
	lvi.stateMask = LVIS_SELECTED | LVIS_FOCUSED;
	lvi.state     = 0;
	while (--lvi.iItem>=0)  lst.SetItem(&lvi);
	
	// Slectionner uniquement l'item voulu
	LVFINDINFO lvfi;
	lvfi.flags  = LVFI_PARAM;
	lvfi.lParam = (LPARAM)pFichier;
	lvi.iItem = lst.FindItem(&lvfi);
	lvi.stateMask =
	lvi.state     = LVIS_SELECTED | LVIS_FOCUSED;
	lst.EnsureVisible(lvi.iItem, FALSE);
	lst.SetItem(&lvi);
	
	// Donner le focus  la vue
	m_plst->SetFocus();
}


// MAJ: L'archive doit tre modifiable
void CMuzipDoc::OnUpdateFautModif(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(!m_pArc->IsLectureSeule());
}
