// MuzipView.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "MuzipDoc.h"
#include "MuzipView.h"
#include "ArboView.h"
#include "ApercuView.h"
#include "MuzipArc.h"

#include "Options.h"
#include "Decomp.h"

#include "dlgConfirm.h"
#include "lstMuzip.h"

#include "Resource.h"


IMPLEMENT_DYNCREATE(CMuzipView, CListView)
BEGIN_MESSAGE_MAP(CMuzipView, CListView)
	//{{AFX_MSG_MAP(CMuzipView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_COMMAND          (ID_EDIT_SELECT_ALL,           OnEditSelectAll)
	ON_COMMAND          (ID_EDIT_SELECT_INVERT,        OnEditSelectInvert)
	ON_COMMAND          (ID_ARCHIVE_RECHERCHERSUIVANT, OnArchiveRechercherSuivant)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_SEL,             OnUpdateIndicatorEtat)
	ON_NOTIFY_REFLECT   (LVN_ITEMCHANGED,              OnItemChanged)
	ON_NOTIFY_REFLECT   (LVN_BEGINLABELEDIT,           OnBeginLabelEdit)
	ON_NOTIFY_REFLECT   (LVN_ENDLABELEDIT,             OnEndLabelEdit)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_TOTAL,           OnUpdateIndicatorEtat)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


// Constructeur
CMuzipView::CMuzipView() : m_plstMuzip(NULL)
{
}

// Cration
int CMuzipView::OnCreate(CREATESTRUCT* pcs)
{
	m_plst = &GetListCtrl();
	m_plstMuzip = new CLstMuzip;
	m_plstMuzip->m_hWnd         = *this;
	m_plstMuzip->m_fDansArchive = true;
	
	// Slection toujours visible
	pcs->style |= LVS_REPORT | LVS_SHOWSELALWAYS | LVS_EDITLABELS |
		LVS_SHAREIMAGELISTS;
	if (CListView::OnCreate(pcs)==-1) 
		return -1;
	m_plstMuzip->Init(0);
	CLstMuzip::InitListCtrl(*m_plst);
	
	CMuzipDoc *const pDoc = GetDocument();
	
	// Enregistrer la cible de glisser-dplacer
	m_mdt.m_pDoc = pDoc;
	m_mdt.Register(m_plst);
	
	// Passer la vue au document
	pDoc->m_plst = this;
	
	return 0;
}


// Mettre  jour
void CMuzipView::OnUpdate(CView*, LPARAM, CObject*)
{
	// Vider la liste
	m_plst->DeleteAllItems();
	
	// Arrter l si aucun dossier n'est slectionn
	CMuzipDoc *const pDoc = GetDocument();
	CMuzipArc *const pArc = pDoc->m_pArc;
	if (!pDoc->m_pCur)
		return;
	
	// Prparer l'ajout des items
	const bool fRechFiltre = pArc->m_fRechFiltre;
	
	// Ajouter les items
	CLstFichier &lstFichier = pDoc->m_pCur->m_lstFichier;
	POS pos = lstFichier.GetPosDeb();
	int nItem = 0;
	while (pos) {
		CFichier *const pFichier = lstFichier.GetSuiv(pos);
		
		// Appliquer le filtre si ncessaire
		if (!fRechFiltre || pFichier->m_fTrouveRech)
			pFichier->lstLister(*m_plst, nItem++);
	}
	
	// Retrier
	m_plstMuzip->Trier();
	
	// Mettre  jour l'aperu
	OnApercu();
}


// Destruction
void CMuzipView::OnDestroy()
{
	m_plstMuzip->m_hWnd = NULL;
	
	CListView::OnDestroy();
	
	GetDocument()->m_plst = NULL;
	delete m_plstMuzip;
}


// Slectionner tout
void CMuzipView::OnEditSelectAll()
{
	LVITEM lvi;
	lvi.iItem     = m_plst->GetItemCount();
	lvi.mask      = LVIF_STATE;
	lvi.stateMask =
	lvi.state     = LVIS_SELECTED | LVIS_FOCUSED;
	while (--lvi.iItem>=0)  m_plst->SetItem(&lvi);
}

// Inverser la slection
void CMuzipView::OnEditSelectInvert()
{
	LVITEM lvi;
	lvi.iItem     = m_plst->GetItemCount();
	lvi.mask      = LVIF_STATE;
	lvi.stateMask = LVIS_SELECTED;
	
	while (--lvi.iItem>=0) {
		m_plst->GetItem(&lvi);
		lvi.stateMask = (lvi.stateMask==LVIS_SELECTED)
			? 0 : LVIS_SELECTED;
		m_plst->SetItem(&lvi);
	}
}


// Barre d'tat
void CMuzipView::OnUpdateIndicatorEtat(CCmdUI* pCmdUI)
{
	pCmdUI->Enable();
	pCmdUI->SetText(m_plstMuzip->OnUpdateIndicatorEtat(pCmdUI->m_nID));
}


// Slection modifie
void CMuzipView::OnItemChanged(NMHDR*, LRESULT*)
{
	const CFichier *const pFichier = m_plstMuzip->GetFichierSel();
	
	if ((e_opt.m_fApercuMAJAuto || !m_plst->GetSelectedCount()) &&
	   (!pFichier || (pFichier->m_dwTailleN>>10)<e_opt.m_nApercuTaille))
		OnApercu();
	else
		GetDocument()->m_papr->PeutApercu();
	
	m_plstMuzip->m_fPeutDrag = ToBool(GetAsyncKeyState(
		GetSystemMetrics(SM_SWAPBUTTON) ? VK_RBUTTON : VK_LBUTTON) & 0x8000);
}

// Mettre  jour l'apercu
void CMuzipView::OnApercu()
{
	GetDocument()->m_papr->SetApercu(m_plstMuzip->GetFichierSel());
}


// Renommage - dbut
void CMuzipView::OnBeginLabelEdit(NMHDR*, LRESULT* pResult)
{
	*pResult = m_plstMuzip->m_fPasEdit = false;
}

// Renommage - fin
void CMuzipView::OnEndLabelEdit(NMHDR* pNMHDR, LRESULT* pResult)
{
	m_plstMuzip->m_fPasEdit = true;
	
	*pResult = FALSE;
	LVITEM &item = ((NMLVDISPINFO*)pNMHDR)->item;
	
	// Si on a annul le renommage, ne rien faire
	const LPTSTR pszNom = item.pszText;
	if (pszNom) {
		CFichier *const pFichier = (CFichier*)item.lParam;
		
		if (pFichier->Renommer(pszNom)) {
			// Renommage russi
			
			*pResult = TRUE;
			
			// Raffecter l'icne car il peut y avoir eu changement d'extension
			SHFILEINFO shfi;
			SHGetFileInfo(pFichier->m_pszNom, 0, &shfi, sizeof(shfi),
				SHGFI_USEFILEATTRIBUTES | SHGFI_SYSICONINDEX | SHGFI_SMALLICON);
			item.mask   = LVIF_IMAGE;
			item.iImage = shfi.iIcon;
			m_plst->SetItem(&item);
		}
	}
}


// Poursuivre la recherche
void CMuzipView::OnArchiveRechercherSuivant()
{
	GetDocument()->m_pCur->RechercherSuivant(m_plst->GetSelectionMark());
}


// Glisser-dplacer - Dbut de survol
DROPEFFECT CMuzipView::OnDragEnter(COleDataObject* pDataObject, DWORD dwKeyState, CPoint pt)
{
	return m_mdt.OnDragEnter(pDataObject, dwKeyState, pt);
}

// Glisser-dplacer - Survol
DROPEFFECT CMuzipView::OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint pt)
{
	GetDocument()->m_pDragD = GetDocument()->m_pCur;
	return m_mdt.OnDragOver(pDataObject, dwKeyState, pt);
}

// Glisser-dplac - Sortie
void CMuzipView::OnDragLeave()
{
	m_mdt.OnDragLeave();
}


// Glisser-dplacer - Lcher
BOOL CMuzipView::OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint pt)
{
	const BOOL bRet = m_mdt.OnDrop(pDataObject, dropEffect, pt);
	
	CFrameWnd *const pParentFrame = GetParentFrame();
	pParentFrame->ActivateFrame();
	pParentFrame->SetActiveView(this);
	
	return bRet;
}

// Glisser-dplacer - Lcher tendu
DROPEFFECT CMuzipView::OnDropEx(COleDataObject* pDataObject, DROPEFFECT dropDefault, DROPEFFECT dropList, CPoint pt)
{
	const BOOL bRet = m_mdt.OnDropEx(pDataObject, dropDefault, dropList, pt);
	
	CFrameWnd *const pParentFrame = GetParentFrame();
	pParentFrame->ActivateFrame();
	pParentFrame->SetActiveView(this);
	
	return bRet;
}



// Notification de la liste
BOOL CMuzipView::OnChildNotify(UINT uMsg, WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
	return (*m_plstMuzip &&
	         m_plstMuzip->OnChildNotify(uMsg, wParam, lParam, pResult))
		? TRUE
		: CListView::OnChildNotify(uMsg, wParam, lParam, pResult);
}


// Message
LRESULT CMuzipView::WindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if (m_plstMuzip) {
		m_plstMuzip->OnWindowProc(uMsg, wParam, lParam);
		
		LRESULT lResult;
		if (m_plstMuzip->m_sm.WindowProc(uMsg, wParam, lParam, &lResult))
			return lResult;
	}
	
	return CListView::WindowProc(uMsg, wParam, lParam);
}


// Commande
BOOL CMuzipView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
	switch (nID) {
		
		case ID_ARCHIVE_MOINS:
		case ID_ARCHIVE_DECOMP:
		case ID_ARCHIVE_RENOMMER:
		case ID_ARCHIVE_PROPRIETES:
		case ID_ARCHIVE_RECOMP:
		case ID_ARCHIVE_CACHER:
		case ID_ARCHIVE_TESTER:
			return m_plstMuzip->OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
		
		default:
			return CListView::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
	}
}
