// Recherche.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "Recherche.h"

#include "Dossier.h"
#include "Fichier.h"
#include "PlusItem.h"
#include "FichierPlus.h"
#include "MuzipArc.h"

#include "Exceptions.h"
#include "IOFichier.h"

#include <GUnicode.h>


// Constructeur
CRecherche::CRecherche(CMuzipArc* pArc)
	: m_pArc(pArc), m_pfpDeb(NULL)
{
}

// Changer de requte d'ajout
void CRecherche::SetArchivePlus(const CArchivePlus& arPlus)
{
	m_parPlus = &arPlus;
	m_pParent = m_pArc->GetDossier(arPlus.m_sDossier, gdCreerTemp);
}


// Commencer une recherche
// Retourner TRUE pour annuler la recherche
bool CRecherche::Rechercher(CPlusItem* ppi)
{
	m_ppi = ppi;
	
	// Composer les masques d'attributs
	m_ppi->GetMskAttr(m_mskAttrO, m_mskAttrN);
	m_mskAttrOPasComp = (m_ppi->CO_fArchiveQue)
		? FILE_ATTRIBUTE_ARCHIVE : 0;
	
	bool fOK = true;
	
	// Effectuer chaque recherche, chacune tant spare
	// de la suivante par un point-virgule ';'
	CTokenize tok(ppi->DO_sFiltre, cSepPartie);
	while (tok.Next()) {
		CString &sPartie = tok.GetToken();
		PathUnquoteSpaces(sPartie.GetBuffer(0));
		sPartie.ReleaseBuffer();
		
		// Regarder s'il s'agit d'un fichier,
		// auquel cas pas besoin d'extraire de spcification et autres
		const DWORD dwAttr = GetFileAttributes(sPartie);
		const bool fVraieRecherche =
			dwAttr==(DWORD)-1 ||
			(dwAttr & FILE_ATTRIBUTE_DIRECTORY);   // TRUE si dossier ou wildcards
		
		TCHAR pszCommun[MAX_PATH];
		
		const TCHAR *const pcSpec = PathFindFileName(sPartie);
		bool fCompresseDossier = false;
		
		if (fVraieRecherche) {
			// Vritable recherche, pas spcification directe de fichier
			
			if (dwAttr!=(DWORD)-1 && (dwAttr & FILE_ATTRIBUTE_DIRECTORY)) {
				// La recherche porte sur un dossier sans spcification
				
				fCompresseDossier = true;
				
				// Prendre la partie tout entire comme tronc commun
				// et "*.*" comme spcification
				lstrcpy(pszCommun, sPartie);
				m_sSpec = _T("*.*");
				
			}else{
				// Recherche classique: isoler la spcification du dossier
				
				lstrcpyn(pszCommun, sPartie, (pcSpec-(LPCTSTR)sPartie)+1);
				m_sSpec = pcSpec;
			}
			
		}else{
			// Spcification de fichier
			// Les traiter spcifiquement gagne du temps
			// car les spcifications sont transformes en "*.*"
			// ( cause de la gestion des sous-dossiers: "*.exe" ne trouve
			// pas tous les sous-dossiers par exemple)
			
			lstrcpy(pszCommun, sPartie);
			m_sSpec.Empty();
		}
		
		//*** Former le chemin d'accs du dossier de destination
		
		// Composer le nom du dossier parent
		// On permet:
		// - les double sparateurs qui seront supprims plus tard
		// -  la fois '\\' et '/' comme sparateurs, on choisira aprs
		
		CDossier *pDossier = m_pParent;
		
		switch (ppi->CO_nArbo) {
			
			// - Pas d'arborescence: on se limite au rpertoire parent
			// ou
			// - Sous-dossiers seulement
			// Si on a une spcification de type "nom de dossier seul"
			// ("C:\Chemin\Dossier" sans "*.*"  la fin par exemple),
			// prendre le nom du dossier compress comme sous-rpertoire
			case arboNon:
			case arboSous:
				if (fCompresseDossier)
					pDossier = pDossier->GetDossier(pcSpec, gdCreerTemp);
				break;
			
			// Chemin d'accs complet (mais sans la lettre de lecteur)
			case arboTout:
				{
					TCHAR pszDossier[MAX_PATH];
					lstrcpy(pszDossier, PathSkipRoot(sPartie));
					if (!fCompresseDossier)
						PathRemoveFileSpec(pszDossier);
					pDossier = pDossier->GetDossier(pszDossier, gdCreerTemp);
				}
				break;
		}
		
		
		// Commencer la recherche
		CString sDossier;
		if (!Rechercher(pszCommun, fVraieRecherche, pDossier, sDossier)) {
			fOK = false;
			break;
		}
	}
	
	return fOK;
}


// Suite d'une recherche (fonction rcursive)
// Retourner FALSE pour annuler la recherche
//
// pDossier et sDossier servent  ne pas crer de dossiers inutiles
// Exemple: on fait une recherche sur tout le disque pour ne compresser
// que quelques fichiers. Avec ce systme, les dossiers ne sont crs
// QUE si on en a rellement besoin, en vue d'un ajout de fichier par exemple.
// Cela vite de crer plein de dossiers qui ne serviraient  rien.
//
// sDossier spcifie les sous-dossiers qu'il faudrait crer pour avoir
// le dossier correspondant rellement au dossier courant dans la recherche.
// Ds qu'on cre un dossier, sDossier est vid compltement puisque pDossier
// contient alors le dossier courant exact et non un de ses parents.
//
//
// pszParent       - chemin d'accs complet physique (pas dans l'archive)
// fVraieRecherche - TRUE si on a une spcification, FALSE si c'est un fichier seul
// pDossier        - dossier dj cr le plus profond dans l'arborescence
// sDossier        - sous-dossier  crer  partir de pDossier pour rcuprer
//                   le dossier correspondant au chemin d'accs courant
bool CRecherche::Rechercher(LPTSTR pszParent, bool fVraieRecherche, CDossier* pDossier, CString& sDossier)
{
	TCHAR *pcParentFin;
	TCHAR pszRecherche[MAX_PATH];
	
	
	// Rcuprer la chane de recherche (passe  CMyFindFile)
	// ainsi que la position dans pszParent o pourront tre copis
	// les noms des fichiers du dossier courant
	// Il suffira de copier un nom de fichier dans pcParentFin
	// pour obtenir son chemin d'accs completdans pszParent
	
	if (fVraieRecherche) {
		// Spcification
		
		// Rajouter un antislash  pszParent si ncessaire
		// et rcuprer la position de fin de la chane
		pcParentFin = PathAddBackslash(pszParent);
		
		// Lancer la recherche
		lstrcpy(pszRecherche,                         pszParent);
		lstrcpy(pszRecherche+(pcParentFin-pszParent), _T("*.*"));
		
	}else{
		// Fichier seul
		
		pcParentFin = PathFindFileName(pszParent);
		lstrcpy(pszRecherche, pszParent);
	}
	
	
	// Lancer la recherche
	CMyFindFile ff(pszRecherche);
	if (ff.IsError()) {
		CMuzipException(ERR_OPEN, pszRecherche).ReportError();
		return true;
	}
	
	
	bool fOK = true;
	while (ff.Next()) {
		if (oApp.UpdateProgressStatus()==prsAnnuler) {
			fOK = false;
			break;
		}
		
		if (ff.m_wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
			// Rpertoire
			
			if (!fVraieRecherche || !m_ppi->OP_fSous)
				continue;
			if ((ff.m_wfd.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN) &&
			    !m_ppi->OP_fCache)
				continue;
		
			// Ignorer les dossiers "." et ".."
			if (ff.IsDots())
				continue;
			
			
			//*** Poursuivre la recherche dans le sous-dossier
			
			// Conserver le chemin complet
			lstrcpy(pcParentFin, ff.m_wfd.cFileName);
			
			// Passer au sous-rpertoire
			CString sDossierSous = sDossier;
			if (m_ppi->CO_nArbo!=arboNon) {
				if (!sDossierSous.IsEmpty())
					sDossierSous += '/';
				sDossierSous += ff.m_wfd.cFileName;
			}
			
			// Poursuivre la recherche dans le sous-dossier
			VERIF(Rechercher(pszParent, fVraieRecherche, pDossier, sDossierSous));
			
			
		}else{
			// Fichier
			
			if (!PathMatchSpec(ff.m_wfd.cFileName, m_sSpec))
				continue;
			
			// Attributs
			if (!(ff.m_wfd.dwFileAttributes & m_mskAttrO))  continue;
			if (  ff.m_wfd.dwFileAttributes & m_mskAttrN )  continue;
			
			// Taille
			if (m_ppi->TA_nActive) {
				const DWORD lenAjuste = ff.m_wfd.nFileSizeLow>>(m_ppi->TA_nUnite*10);
				if       (m_ppi->TA_nOp==0)   {  // ==
					if (m_ppi->TA_uTaille!=lenAjuste)  continue;
				}else if (m_ppi->TA_nOp==1)   {  // >=
					if (m_ppi->TA_uTaille>=lenAjuste)  continue;
				}else /* (m_ppi->TA_nOp==2)*/ {  // <=
					if (m_ppi->TA_uTaille<=lenAjuste)  continue;
				}
			}
			
			// Date
			if (m_ppi->DA_rdC.fActive &&
			   !m_ppi->DA_rdC.Correspond(ff.m_wfd.ftCreationTime))   continue;
			if (m_ppi->DA_rdM.fActive &&
			   !m_ppi->DA_rdM.Correspond(ff.m_wfd.ftLastWriteTime))  continue;
			
			// Le fichier correspond au filtre
			// Composer le chemin d'accs
			lstrcpy(pcParentFin, ff.m_wfd.cFileName);
			
			
			// Regarder si le fichier est dj dans la liste
			// des fichiers  ajouter
			// Pour a on fait un bsearch() sur l'arbre
			CFichierPlus  *pfpParent =  NULL;
			CFichierPlus **ppfpCur   = &m_pfpDeb;
			while (*ppfpCur) {
				CFichierPlus *const pfpCur = *ppfpCur;
				
				const int iDiff = lstrcmpi(pszParent, pfpCur->GetNomES());
				if (!iDiff)
					break;
				pfpParent = pfpCur;
				ppfpCur   = (iDiff<0) ? &pfpCur->m_pfpGauche : &pfpCur->m_pfpDroite;
			}
			
			
			// Rcuprer le sous-dossier si ncessaire
			if (!sDossier.IsEmpty()) {
				pDossier = pDossier->GetDossier(sDossier, gdCreerTemp);
				sDossier.Empty();
			}
			
			
			CFichierPlus *pfpCur = *ppfpCur;
			
			// Regarder si le fichier est prsent dans l'archive
			CFichier *pFichierOld;
			POS posFichierOld;
			if (pfpCur) {
				// Le fichier tait dans la liste des fichiers  ajouter
				// On a donc dj trouv le fichier correspondant de l'archive
				
				pFichierOld   = pfpCur->m_pFichierOld;
				posFichierOld = pfpCur->m_posFichierOld;
				
			}else{
				// Il faut chercher dans le dossier du fichier
				// voir si notre fichier y est ou non
				
				CLstFichier &lstFichier = pDossier->m_lstFichier;
				POS pos = lstFichier.GetPosDeb();
				do{
					posFichierOld = pos;
					if (!pos) {
						// On n'a pas trouv
						pFichierOld = NULL;
						break;
					}
					
					// Continuer tant que le fichier n'est pas trouv
					pFichierOld = lstFichier.GetSuiv(pos);
				}while (lstrcmpi(ff.m_wfd.cFileName, pFichierOld->m_pszNom));
			}
			
			
			
			//*** Ici posFichierOld et pFichierOld contiennent
			//*** l'index du fichier et le fichier lui-mme
			//*** dans l'archive active
			
			int nAction = m_ppi->CO_nAction;
			
			if (m_mskAttrOPasComp!=(ff.m_wfd.dwFileAttributes & m_mskAttrOPasComp)) {
				// Le fichier ne satisfait pas au critres d'attributs
				// qui s'ils ne sont pas respects on doit passer
				// en mode "ne pas compresser"
				// Si on est en mode "exclure de la recherche"
				// alors on fait comme si le fichier n'existait pas sur le disque,
				// sinon on passe en mode "ne pas compresser"
				if (nAction == coaMoins)
					continue;
				nAction = coaPasComp;
			}
			
			
			// Comportement par dfaut, modifi dans le switch qui suit:
			// - le fichier doit tre compress
			// - le fichier est considr comme trouv
			bool fFautComp = true;
			if (pFichierOld)
				pFichierOld->m_fTrouveAjout = true;
			
			// Critre sur la nouveaut du fichier:
			// ne compresser que si le fichier est plus rcent
			// que son homologue dans l'archive, ou s'il est
			// carrment nouveau (absent de l'archive)
			if (m_ppi->CO_fQueNouveau && pFichierOld &&
			   (m_pArc->GetCap() & capdtM) &&
			   pFichierOld->m_dtM >= ff.m_wfd.ftLastWriteTime)
				continue;
			
			switch (nAction) {
				
				// Ajouter et mettre  jour
				case coaPlusMAJ:
					break;
				
				
				// Mettre  jour
				// Le fichier doit avoir un homologue dans l'archive
				case coaMAJ:
					fFautComp = ToBool(pFichierOld);
					break;
				
				
				// Ajouter
				// Le fichier ne doit pas avoir d'homologue dans l'archive
				case coaPlus:
					fFautComp = !pFichierOld;
					break;
				
				
				// Ne pas compresser
				case coaPasComp:
					fFautComp = false;
					break;
				
				
				// Exclure de la recherche
				case coaMoins:
					
					// Considrer le fichier comme non-trouv
					// (c'est le but de l'option "Exclure de la recherche")
					if (pFichierOld)
						pFichierOld->m_fTrouveAjout = false;
					fFautComp = false;
					break;
			}
			
			
			if (pFichierOld)
				pFichierOld->m_fFautComp = fFautComp;
			
			
			if (!fFautComp) {
				// Le fichier ne doit pas tre compress,
				// donc il faut le supprimer de l'arbre
				
				if (pfpCur)
					pfpCur->Supprimer();
				
			}else{
				// Le fichier doit tre compress:
				// il faut l'ajouter  l'arbre
				
				
				if (!pfpCur) {
					// Ajouter une nouvelle entre  l'arbre
					// car aucune n'existait pour le fichier
					
					*ppfpCur = pfpCur = new CFichierPlus(pFichierOld,posFichierOld);
				}
				pfpCur->m_ppi = m_ppi;
				
				
				// Crer une structure pour le futur fichier
				// si elle n'existe pas dj
				if (!pfpCur->m_pFichier) {
					pfpCur->m_pFichier = m_pArc->NewFichier(pszParent, ff.m_wfd);
					pfpCur->m_pFichier->m_pDossier = pDossier;
					delete [] pfpCur->m_pszNomES;
					pfpCur->m_pszNomES = NULL;
				}
				pfpCur->m_pFichier->m_fFautComp = true;
			}
		}
	}
	
	return fOK;
}


// Finaliser la recherche en rcuprant la liste des fichiers
void CRecherche::Finaliser(CLstFichierPlus& lstFichierPlus)
{
	// Fixer la limite du nombre de fichiers  ajouter
	int iLimite = m_parPlus->m_iLimite;
	if (!iLimite)
		iLimite = UINT_MAX;
	
	m_pfpDeb->ArbreLister(lstFichierPlus, iLimite);
}
