// Tar/TarArc.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "../Muzip.h"
#include "../Muzip2.h"
#include "TarArc.h"
#include "TarFichier.h"
#include "Tar.h"

#include "../Interf.h"
#include "../FichierPlus.h"

#include "../Options.h"
#include "../Exceptions.h"
#include "../IOFichier.h"

#include <GUnicode.h>


// Crer une archive
CMuzipArc* CTarArc::CreateArchive(CInterf* pi)
{
	return new CTarArc(pi);
}


// Validation du type
bool CTarArc::IsGoodType()
{
	TARRECORD tr;
	return m_fr.Read(&tr, sizeof(tr)) && Validate(tr);
}


// Valider un en-tte
bool CTarArc::Validate(TARRECORD& tr, int nReparer)
{
	// Rcuprer le checksum
	int i;
	for (i = 0; i < sizeof(tr.th.pszCheckSum); i++) {
		if (tr.th.pszCheckSum[i] != ' ') {
			DWORD dwCheckSum = 0;
			for (;;) {
				const char c = tr.th.pszCheckSum[i];
				if (!c || c==' ')
					break;
				if (i==8 || c<'0' || c>'7') {
					if (nReparer < repAltere)
						return false;
					break;
				}
				
				// Enregistrer le nouveau chiffre
				dwCheckSum = (dwCheckSum*8) + c-'0';
				i++;
			}
			
			// Valider le checksum
			if (nReparer <= repPresumer) {
				memset(tr.th.pszCheckSum, cCheckSumVide, sizeof(tr.th.pszCheckSum));
				
				DWORD dwSigned, dwUnsigned;
				for (dwSigned = dwUnsigned = 0, i = 0; i < sizeof(tr); i++) {
					dwSigned   += (unsigned char&)tr.pb[i];
					dwUnsigned += (signed char&)  tr.pb[i];
				}
				
				VERIF(dwCheckSum ==dwSigned || dwCheckSum == dwUnsigned);
			}
			break;
		}
	}
	
	return true;
}



// Ouverture
bool CTarArc::OpenDocument(bool fSilent)
{
	DWORD off = 0;
	TARRECORD tr;
	TCHAR pszPath[bufTarPath];
	for (;;) {
		
		// Lire l'en-tte du fichier
		m_fr.SetVerbeux(false);
		const bool fOK = m_fr.Read(&tr, sizeof(tr));
		m_fr.SetVerbeux(!fSilent);
		if (!fOK || !tr.th.pszPath[0])
			break;
		
		if (!Validate(tr, e_opt.m_nReparer))
			goto ErreurFormat;
		
		// Identifier le champ magique GNU
		const bool fGNU = !lstrcmpA(tr.th.pszMagic, pszMagicGNU);
		
		// Valider le type du fichier
		if (e_opt.m_nReparer==repNon) {
			if (tr.th.cType!=tartNormalUnix && (tr.th.cType<'0' || tr.th.cType>'7')) {
				if (!fGNU ||
				   (tr.th.cType!=tartRepListe &&
				    tr.th.cType!=tartMultiVol &&
				    tr.th.cType!=tartHeaderVol))
					goto ErreurFormat;
			}
		}
		
		off += sizeof(tr);
		
		// Rcuprer la taille
		DWORD dwTaille;
		if (!GetOctal12(tr.th.taille, dwTaille))
			break;
		
		switch (tr.th.cType) {
			
			// Fichier normal
			case tartNormalUnix:
			case tartNormal:
			case tartContigu:
				break;
			
			// Rpertoire
			case tartRep:
				break;
			
			// Fichier sur rparti sur plusieurs volumes
			case tartMultiVol:
				break;
			
			// Fichier  sauter
			default:
				goto Suite;
		}
		
		{
			// Convertir le chemin d'accs et rcuprer le nom du fichier
			tr.th.pszPath[bufTarPath] = 0;
			strfromA(pszPath, tr.th.pszPath, nbArray(pszPath));
			
			if (tr.th.cType==tartRep) {
				// Dossier
				
				CDossier *const pDossier = GetDossier(pszPath, gdCreer);
				time_t dtM;
				if (GetOctal12(tr.th.dtM, (DWORD&)dtM))
					pDossier->m_dtM = dtM;
				goto Suite;
			}
			
			
			// Fichier
			
			const LPTSTR pcFichier = PathFindFileName(pszPath);
			if (!pcFichier) {
				if (e_opt.m_nReparer<=repPresumer)
					goto ErreurFormat;
				goto Suite;
			}
			
			CTarFichier *const ptf = new CTarFichier;
			ptf->m_pszNom    = newDup(pcFichier);
			time_t dtM;
			if (GetOctal12(tr.th.dtM, (DWORD&)dtM))
				ptf->m_dtM = dtM;
			ptf->m_dwTailleC =
			ptf->m_dwTailleN = dwTaille;
			ptf->m_dwFlags   = 0;
			ptf->m_dwAttr    = 0;
			ptf->m_off       = off;
			ptf->MAJ();
			
			// Ajouter le fichier  son dossier
			ptf->m_pArc      = this;
			(ptf->m_pDossier = GetDossier(pszPath, gdCreer, true))->m_lstFichier.AddFin(ptf);
		}
		
Suite:
		
		// Sauter le fichier
		if (dwTaille) {
			// Arrondir dwTaille  sizeof(tr) prs
			if (dwTaille & sizeof(tr)-1)
				dwTaille = (dwTaille | (sizeof(tr)-1)) + 1;
			// Effectuer le dplacement
			if (!m_fr.Seek((long)dwTaille, FILE_CURRENT))
				goto Erreur;
			off += dwTaille;
		}
	}
	
	return OpenArchiveOK(fSilent);
	
ErreurFormat:
	if (!fSilent)
		CMuzipException(ERR_FORMAT, m_fr.GetPath()).ReportError();
Erreur:
	m_lstFichier.DeleteAll();
	m_lstDossier.DeleteAll();
	return false;
}


// Initialiser la cration d'une archive temporaire
bool CTarArc::OpenArchiveTemp()
{
	CTarFichier::s_off = 0;
	return CMuzipArc::OpenArchiveTemp();
}


// Ajouter un fichier  la liste
CFichier* CTarArc::NewFichier(LPCTSTR pszFichier, const WIN32_FIND_DATA& wfd)
{
	CTarFichier *const pf = new CTarFichier;
	pf->NewFichier(pszFichier, wfd);
	
	pf->m_pDossier = this;
	pf->m_pArc     = this;
	
	return pf;
}


// Ajouter des fichiers  l'archive
bool CTarArc::ArchivePlus()
{
	CTarFichier::s_fComp = true;
	bool fOK = true;
	
	ShowStatusAvanceFichier();
	UINT iFichier = 0;
	POS pos = s_lstFichierPlus.GetPosDeb();
	while (pos) {
		CFichierPlus *const pFichierPlus = s_lstFichierPlus.GetSuiv(pos);
		CTarFichier  *const ptf = (CTarFichier*)pFichierPlus->m_pFichier;
		oApp.SetProgress(STA_ARCHIVEPLUS, ptf->m_pszNomES,
			iFichier++, s_lstFichierPlus.GetNb());
		
		PRGSTATUS prs = prsOK;
		
		// Si le fichier est l'archive elle-mme,
		// afficher un message d'erreur et passer au fichier suivant
		const DWORD offOld = e_fw.Tell();
		if (!ptf->CompresseArchive()) {
			if (ptf->m_fFautComp) {
				
				// Ouvrir le fichier en vue de lecture
				CFileRead fr;
				if (!fr.Open(ptf->m_pszNomES, FILE_SHARE_ALL))
					prs = prsErreur;
				else{
					
					// Remplir la classe CTarFichier:
					CMyFindFile ff(ptf->m_pszNomES);
					if (ff.IsError()) {
						CMuzipException(ERR_READ, ptf->m_pszNomES).ReportError();
						prs = prsErreur;
					}else{
						
						// Rcuprer la taille et la date de modification
						ptf->m_dwTailleC =
						ptf->m_dwTailleN = ff.m_wfd.nFileSizeLow;
						ptf->m_dtM       = ff.m_wfd.ftCreationTime;
						
						// crire le fichier aprs son en-tte
						ptf->m_fFautComp = false;
						if (!ptf->FichierCopier(fr))
							prs = ErreurArchivePlus();
						ptf->m_fFautComp = true;
					}
					fr.Close();
				}
			}
		}
		
		if (!ProgressArchivePlus(fOK, ptf, prs, pos, offOld))
			break;
	}
	CTarFichier::s_fComp = false;
	
	return fOK;
}



// Traduire une chane de 11 caractres d'ASCII octal termine par un espace
// en un DWORD, et retourner TRUE si succs
bool CTarArc::GetOctal12(LPCSTR psz, DWORD& dw)
{
	bool fDeb = true;
	UINT64 ddw = 0;
	for (int i=0;; i++) {
		const char c = psz[i];
		if (c==' ' || !c) {
			if (c && fDeb)
				continue;
			dw = (DWORD)ddw;
			return true;
		}else
			fDeb = false;
		
		VERIF(i<12 && c>='0' && c<='7');
		
		// Enregistrer le nouveau chiffre
		ddw = (ddw*8) + c-'0';
	}
}
