// UnArj.cpp
//
// Copyright (c) 1991-97 by ARJ Software, Inc.  All rights reserved.
//
// R. Jung
// M. Adler


#include "StdAfx.h"
#pragma hdrstop
#include "UnArj.h"


DWORD   crc;
FILE   *arcfile;
FILE   *outfile;
WORD bitbuf;
long   compsize;
long   origsize;
BYTE  subbitbuf;
BYTE  header[HEADERSIZE_MAX];
char   arc_name[FNAME_MAX];
int    command;
int    bitcount;
int    file_type;
bool   fNoOutput;
int    error_count;

// Messages

static LPCSTR M_USAGE[] =
{
"Usage:  UNARJ archive[.arj]    (list archive)\n",
"        UNARJ e archive        (extract archive)\n",
"        UNARJ l archive        (list archive)\n",
"        UNARJ t archive        (test archive)\n",
"        UNARJ x archive        (extract with pathnames)\n",
"\n",
"This is an ARJ demonstration program and ** IS NOT OPTIMIZED ** for speed.\n",
"You may freely use, copy and distribute this program, provided that no fee\n",
"is charged for such use, copying or distribution, and it is distributed\n",
"ONLY in its original unmodified state.  UNARJ is provided as is without\n",
"warranty of any kind, express or implied, including but not limited to\n",
"the implied warranties of merchantability and fitness for a particular\n",
"purpose.  Refer to UNARJ.DOC for more warranty information.\n",
"\n",
"ARJ Software, Inc.      Internet address:  robjung@world.std.com\n",
"P.O. Box 249                    Web site:  www.arjsoft.com\n",
"Norwood MA 02062\n",
"USA\n",
NULL
};

static const char M_VERSION [] = "UNARJ (Demo version) 2.43 Copyright (c) 1991-97 ARJ Software, Inc.\n\n";

static const char M_ARCDATE [] = "Archive created: %s";
static const char M_ARCDATEM[] = ", modified: %s";
static const char M_BADCOMND[] = "Bad UNARJ command: %s";
static const char M_BADCOMNT[] = "Invalid comment header";
static const char M_BADHEADR[] = "Bad header";
static const char M_CANTOPEN[] = "Can't open %s";
static const char M_CANTREAD[] = "Can't read file or unexpected end of file";
static const char M_CANTWRIT[] = "Can't write file. Disk full?";
static const char M_CRCERROR[] = "CRC error!\n";
static const char M_CRCOK   [] = "CRC OK\n";
static const char M_DIFFHOST[] = "  Binary file!";
static const char M_ENCRYPT [] = "File is password encrypted, ";
static const char M_ERRORCNT[] = "%sFound %5d error(s)!";
static const char M_EXTRACT [] = "Extracting %-25s";
static const char M_FEXISTS [] = "%-25s exists, ";
static const char M_HEADRCRC[] = "Header CRC error!";
static const char M_NBRFILES[] = "%5d file(s)\n";
static const char M_NOTARJ  [] = "%s is not an ARJ archive";
static const char M_PROCARC [] = "Processing archive: %s\n";
static const char M_SKIPPED [] = "Skipped %s\n";
static const char M_SUFFIX  [] = ".ARJ";
static const char M_TESTING [] = "Testing    %-25s";
static const char M_UNKNMETH[] = "Unsupported method: %d, ";
static const char M_UNKNTYPE[] = "Unsupported file type: %d, ";
static const char M_UNKNVERS[] = "Unsupported version: %d, ";

#define getByte()      ((BYTE)(*get_ptr++))

#define BUFFERSIZE      4096

// Local functions

#ifndef __TURBOC__
static void  make_crctable();
#endif
static void  crc_buf(BYTE* pbBuf, size_t len);
static void  strparity(BYTE* pbBuf);
static BYTE  fgetByte(FILE* pf);
static WORD  fgetWord(FILE* pf);
static DWORD fgetDWord(FILE* pf);
static void  fread_crc(BYTE* pbBuf, size_t len, FILE* pf);
static void  get_date_str(LPSTR str, DWORD tstamp);
static WORD  getWord();
static DWORD getDWord();
static long  find_header(FILE *pf);
static bool  read_header(bool fPrem, FILE *pf, LPSTR name);
static void  skip();
static void  unstore();
static bool  check_flags();
static bool  extract();
static bool  test();
static UINT  ratio(long a, long b);
static void  list_arc(int count);
static void  execute_cmd();
static void  help();


// Local variables

static char   filename[FNAME_MAX];
static char   comment[COMMENT_MAX];
static char   *hdr_filename;
static char   *hdr_comment;

static WORD headersize;
static BYTE  first_hdr_size;
static BYTE  arj_nbr;
static BYTE  arj_x_nbr;
static BYTE  host_os;
static BYTE  arj_flags;
static short  method;
static UINT   file_mode;
static DWORD  time_stamp;
static short  entry_pos;
static WORD host_data;
static BYTE  *get_ptr;
static DWORD  file_crc;
static DWORD  header_crc;

static long   first_hdr_pos;
static long   torigsize;
static long   tcompsize;

static int    clock_inx;

static char   *writemode[2]  = { "wb",  "w" };

static DWORD	crctable[UCHAR_MAX + 1];



#ifndef __TURBOC__
static void make_crctable()
{
	UINT i, j;
	DWORD r;

	for (i=0; i<=UCHAR_MAX; i++) {
		r = i;
		for (j=CHAR_BIT; j>0; j--) {
			if (r & 1)
				r = (r >> 1) ^ 0xEDB88320L;
			else
				r >>= 1;
		}
		crctable[i] = r;
	}
}
#endif

static void crc_buf(BYTE* pbBuf, size_t len)
{
	while (len--) {
		crc = crctable[(BYTE)((BYTE)(crc) ^ (BYTE)(*pbBuf++))] ^ (crc>>CHAR_BIT);
	}
}

void disp_clock()
{
	static char clock_str[4] = { '|', '/', '-', '\\' };
	printf("(%c)\b\b\b", clock_str[clock_inx]);
	clock_inx = (clock_inx + 1) & 0x03;
}

void error(LPCSTR pszFormat, LPCSTR pszArg)
{
	putchar('\n');
	printf(pszFormat, pszArg, error_count);
	putchar('\n');
	exit(EXIT_FAILURE);
}

static void strparity(BYTE* pbBuf)
{
	while (*pbBuf)  *pbBuf++ &= 0x7F;
}

static BYTE fgetByte(FILE* pf)
{
	const int c = getc(pf);
	if (c==EOF)  error(M_CANTREAD, "");
	return (BYTE)c;
}

static WORD fgetWord(FILE* pf)
{
	return fgetByte(pf) | ((WORD)fgetByte(pf)<<8);
}

static DWORD fgetDWord(FILE* pf)
{
	return		fgetByte(pf)      |
		((DWORD)fgetByte(pf)<< 8) |
		((DWORD)fgetByte(pf)<<16) |
		((DWORD)fgetByte(pf)<<24);
}

static void fread_crc(BYTE* pbBuf, size_t len, FILE* pf)
{
	len = fread(pbBuf, 1,len, pf);
	origsize += len;
	crc_buf(pbBuf, len);
}

void fwrite_txt_crc(BYTE* pbBuf, size_t len)
{
	crc_buf(pbBuf, len);
	if (fNoOutput)  return;

	if (file_type==TEXT_TYPE && host_os!=OS) {
		while (len--) {
			if (putc(*pbBuf++ & 0x7F, outfile)==EOF)  error(M_CANTWRIT, "");
		}
	}else{
		if (fwrite(pbBuf, 1,len, outfile)!=len)  error(M_CANTWRIT, "");
	}
}

void init_getbits()
{
	bitbuf    = 0;
	subbitbuf = 0;
	bitcount  = 0;
	fillbuf(2 * CHAR_BIT);
}

// Shift bitbuf n bits left, read n bits
void fillbuf(int n)
{
	bitbuf = (bitbuf << n) & 0xFFFF;  // lose the first n bits
	while (n > bitcount) {
		bitbuf |= subbitbuf << (n -= bitcount);
		if (compsize) {
			compsize--;
			subbitbuf = (BYTE)getc(arcfile);
		}else
			subbitbuf = 0;
		bitcount = CHAR_BIT;
	}
	bitbuf |= subbitbuf >> (bitcount -= n);
}

WORD getbits(int n)
{
	WORD x = bitbuf >> (2 * CHAR_BIT - n);
	fillbuf(n);
	return x;
}

static void get_date_str(LPSTR str, DWORD tstamp)
{
	sprintf(str, "%04u-%02u-%02u %02u:%02u:%02u",
		   ts_year(tstamp), ts_month(tstamp), ts_day(tstamp),
		   ts_hour(tstamp), ts_min(tstamp), ts_sec(tstamp));
}

static WORD getWord()
{
	return getByte() | ((WORD)getByte()<<8);
}

static DWORD getDWord()
{
	return   getByte()      |
		((WORD)getByte()<< 8) |
		((WORD)getByte()<<16) |
		((WORD)getByte()<<24);
}

static long find_header(FILE* pf)
{
	long arcpos, lastpos;
	int c;

	arcpos = ftell(pf);
	fseek(pf, 0L, SEEK_END);
	lastpos = ftell(pf) - 2;
	if (lastpos > MAXSFX)  lastpos = MAXSFX;
	for (; arcpos<lastpos; arcpos++) {
		fseek(pf, arcpos, SEEK_SET);
		c = fgetByte(pf);
		while (arcpos<lastpos) {
			if (c!=HEADER_ID_LO)  // low order first
				c = fgetByte(pf);
			else if ((c = fgetByte(pf))==HEADER_ID_HI)
				break;
			arcpos++;
		}
		if (arcpos>=lastpos)  break;

		headersize = fgetWord(pf);
		if (headersize<=HEADERSIZE_MAX) {
			crc = CRC_MASK;
			fread_crc(header, headersize, pf);
			if ((crc ^ CRC_MASK)==fgetDWord(pf)) {
				fseek(pf, arcpos, SEEK_SET);
				return arcpos;
			}
		}
	}
		return -1;          // could not find a valid header
}

static bool read_header(bool fPrem, FILE* pf, LPSTR name)
{
	if (fgetWord(pf)!=HEADER_ID) {
		if (fPrem)  error(M_NOTARJ, name);
		else        error(M_BADHEADR, "");
	}

	WORD lenHeader = fgetWord(pf);
	if (!lenHeader)  return false;        // end of archive
	if (lenHeader>HEADERSIZE_MAX)  error(M_BADHEADR, "");

	crc = CRC_MASK;
	fread_crc(header, lenHeader, pf);
	header_crc = fgetDWord(pf);
	if ((crc ^ CRC_MASK)!=header_crc)  error(M_HEADRCRC, "");

	get_ptr = header;
	first_hdr_size = getByte();
	arj_nbr        = getByte();
	arj_x_nbr      = getByte();
	host_os        = getByte();
	arj_flags      = getByte();
	method         = getByte();
	file_type      = getByte();
	getByte();
	time_stamp     = getDWord();
	compsize       = getDWord();
	origsize       = getDWord();
	file_crc       = getDWord();
	entry_pos      = getWord();
	file_mode      = getWord();
	host_data      = getWord();

	hdr_filename = (LPSTR)&header[first_hdr_size];
	strncpy(filename, hdr_filename, sizeof(filename));
	if (host_os!=OS)  strparity((BYTE*)filename);
	if (arj_flags & PATHSYM_FLAG) {
		TCHAR *pc = filename;
		while (*pc) {
			if (*pc++=='/')  *pc = '\\';
		}
	}

	hdr_comment = (LPSTR)&header[first_hdr_size + strlen(hdr_filename) + 1];
	strncpy(comment, hdr_comment, sizeof(comment));
	if (host_os!=OS)  strparity((BYTE*)comment);

	// if extheadersize==0 then no CRC
	// otherwise read extheader data and read 4 bytes for CRC

	while (ToBool(lenHeader = fgetWord(pf)))
		fseek(pf, (long)(lenHeader+4), SEEK_CUR);

	return true;
}

static void skip()
{
	fseek(arcfile, compsize, SEEK_CUR);
}

static void unstore()
{
	char buffer[BUFFERSIZE];
	long pos = ftell(arcfile);
	disp_clock();
	int n = (int)(BUFFERSIZE - (pos % BUFFERSIZE));
	n = (compsize>(long)n) ? n : (int)compsize;
	while (compsize > 0) {
		if (fread(buffer, 1, n, arcfile)!=n)  error(M_CANTREAD, "");
		disp_clock();
		compsize -= n;
		fwrite_txt_crc((BYTE*)buffer, n);
		n = compsize > BUFFERSIZE ? BUFFERSIZE : (int)compsize;
	}
}

static bool check_flags()
{
	if (arj_x_nbr > ARJ_X_VERSION) {
		printf(M_UNKNVERS, arj_x_nbr);
Erreur:
		printf(M_SKIPPED, filename);
		skip();
		return false;
	}
	if (arj_flags & GARBLE_FLAG) {
		printf(M_ENCRYPT);
		goto Erreur;
	}
	if (method<0 || method>MAXMETHOD || (method==4 && arj_nbr==1)) {
		printf(M_UNKNMETH, method);
		goto Erreur;
	}
	if (file_type!=BINARY_TYPE && file_type!=TEXT_TYPE) {
		printf(M_UNKNTYPE, file_type);
		goto Erreur;
	}
	return true;
}

static bool extract()
{
	if (!check_flags()) {
		error_count++;
		return false;
	}

	fNoOutput = false;

	char pszNom[FNAME_MAX];
	strcpy(pszNom, (command=='E') ? &filename[entry_pos] : filename);

	if (host_os!=OS)  strupr(pszNom);

	if (!access(pszNom, 0)) {
		printf(M_FEXISTS, pszNom);
		printf(M_SKIPPED, pszNom);
Erreur:
		skip();
		error_count++;
		return false;
	}
	outfile = fopen(pszNom, writemode[file_type & 1]);
	if (!outfile) {
		printf(M_CANTOPEN, pszNom);
		putchar('\n');
		goto Erreur;
	}
	printf(M_EXTRACT, pszNom);
	if (host_os!=OS && file_type==BINARY_TYPE)  printf(M_DIFFHOST);
	printf("  ");

	crc = CRC_MASK;

	if      (method==0)               unstore();
	else if (method>=1 && method<=3)  decode();
	else if (method==4)               decode_f();
	fclose(outfile);

	{
		const int hf = open(pszNom, O_WRONLY | O_BINARY);
		if (hf!=-1) {
			setftime(hf, (ftime*)&time_stamp);
			close(hf);
			if (host_os==OS)  _chmod(pszNom, 1, file_mode & 0x27);
		}
	}

	if ((crc ^ CRC_MASK)==file_crc)
		printf(M_CRCOK);
	else{
		printf(M_CRCERROR);
		error_count++;
	}
	return true;
}

static bool test()
{
	if (!check_flags())  return false;

	fNoOutput = true;
	printf(M_TESTING, filename);
	printf("  ");

	crc = CRC_MASK;

	if      (method== 0)               unstore();
	else if (method>= 1 && method<=3)  decode();
	else if (method==4)                decode_f();

	if ((crc ^ CRC_MASK)==file_crc)
		printf(M_CRCOK);
	else{
		printf(M_CRCERROR);
		error_count++;
	}
	return true;
}

static UINT ratio(long a, long b)
{
	for (int i=0; i<3; i++) {
		if (a<=LONG_MAX/10)  a *= 10;
		else                 b /= 10;
	}
	if ((long)(a+(b>>1))<a) {
		a >>= 1;
		b >>= 1;
	}
	if (!b)  return 0;
	return (UINT) ((a+(b>>1)) / b);
}


static void list_arc(int count)
{
	bool garble_mode, path_mode, volume_mode, extfil_mode, ftype, bckf_mode;
	char date_str[20], fmode_str[10];
	static const char mode[5] = { 'B', 'T', '?', 'D', 'V' };
	static const char pthf[2] = { ' ', '+' };
	static const char pwdf[2] = { ' ', 'G' };  // plain, encrypted
	static const char volf[2] = { ' ', 'V' };
	static const char extf[2] = { ' ', 'X' };
	static const char bckf[2] = { ' ', '*' };

	if (!count) {
		printf("Filename       Original Compressed Ratio DateTime modified CRC-32   AttrBTPMGVX\n");
		printf("------------ ---------- ---------- ----- ----------------- -------- -----------\n");
	}

	garble_mode = ToBool(arj_flags & GARBLE_FLAG);
	volume_mode = ToBool(arj_flags & VOLUME_FLAG);
	extfil_mode = ToBool(arj_flags & EXTFILE_FLAG);
	bckf_mode   = ToBool(arj_flags & BACKUP_FLAG);
	path_mode   = (entry_pos > 0);
	UINT r = ratio(compsize, origsize);
	torigsize += origsize;
	tcompsize += compsize;
	ftype = file_type;
	if (ftype != BINARY_TYPE && ftype != TEXT_TYPE && ftype != DIR_TYPE &&
			ftype != LABEL_TYPE)
		ftype = 3;
	get_date_str(date_str, time_stamp);
	strcpy(fmode_str, "    ");
	if (host_os==OS) {
		strcpy(fmode_str, "---W");
		if (file_mode & FA_ARCH)     fmode_str[0] = 'A';
		if (file_mode & FA_SYSTEM)   fmode_str[1] = 'S';
		if (file_mode & FA_HIDDEN)   fmode_str[2] = 'H';
		if (file_mode & FA_RDONLY)   fmode_str[3] = 'R';
	}
	if (strlen(&filename[entry_pos])>12)
		printf("%-12s\n             ", &filename[entry_pos]);
	else
		printf("%-12s ", &filename[entry_pos]);
	printf("%10ld %10ld %u.%03u %s %08lX %4s%c%c%c%u%c%c%c\n",
		origsize, compsize, r / 1000, r % 1000, &date_str[2], file_crc,
		fmode_str, bckf[bckf_mode], mode[ftype], pthf[path_mode], method,
		pwdf[garble_mode], volf[volume_mode], extf[extfil_mode]);
}

static void execute_cmd()
{
	int file_count;
	char date_str[22];
	UINT r;

	first_hdr_pos = 0;
	time_stamp = 0;
	first_hdr_size = FIRST_HDR_SIZE;

	arcfile = fopen(arc_name, "rb");
	if (!arcfile)  error(M_CANTOPEN, arc_name);

	printf(M_PROCARC, arc_name);

	first_hdr_pos = find_header(arcfile);
	if (first_hdr_pos < 0)
		error(M_NOTARJ, arc_name);
		fseek(arcfile, first_hdr_pos, SEEK_SET);
	if (!read_header(true, arcfile, arc_name))
		error(M_BADCOMNT, "");
	get_date_str(date_str, time_stamp);
	printf(M_ARCDATE, date_str);
	if (arj_nbr >= ARJ_M_VERSION)
	{
		get_date_str(date_str, (DWORD) compsize);
		printf(M_ARCDATEM, date_str);
	}
	printf("\n");

	file_count = 0;
	while (read_header(false, arcfile, arc_name)) {
		switch (command) 	{
			case 'E':
			case 'X':
				if (extract())  file_count++;
				break;
			case 'L':
				list_arc(file_count++);
				skip();
				break;
			case 'T':
				if (test())  file_count++;
				break;
		}
	}

	if (command=='L') {
		printf("------------ ---------- ---------- ----- -----------------\n");
		r = ratio(tcompsize, torigsize);
		printf(" %5d files %10ld %10ld %u.%03u %s\n",
			file_count, torigsize, tcompsize, r / 1000, r % 1000, &date_str[2]);
	}else
		printf(M_NBRFILES, file_count);

	fclose(arcfile);
}

static void help()
{
	for (int i=0; M_USAGE[i]; printf(M_USAGE[i++]));
}

int main(int argc, LPSTR argv[])
{
	int i, j, lastc;
	LPSTR arc_p;

		printf(M_VERSION);

	if (argc==1) {
		help();
		return EXIT_SUCCESS;
	}else if (argc==2) {
		command = 'L';
		arc_p = argv[1];
	}else if (argc==3)	{
		if (strlen(argv[1])>1)
			error(M_BADCOMND, argv[1]);
		command = toupper(*argv[1]);
		if (strchr("ELTX", command)==NULL)
			error(M_BADCOMND, argv[1]);
		arc_p = argv[2];
	}else{
		help();
		return EXIT_FAILURE;
	}

	strncpy(arc_name, arc_p, FNAME_MAX);
	strupr(arc_name);
	i = strlen(arc_name);
	arc_p = arc_name;
	for (TCHAR *pc=_T("\\:/"); *pc; pc++) {
		TCHAR *const pcTrouve = _tcschr(arc_p, *pc);
		if (pcTrouve && (!arc_p || pcTrouve>arc_p))  arc_p = pcTrouve;
	}
	j = (arc_p) ? arc_p+1-arc_name : 0;
	lastc = arc_name[i - 1];
	if (lastc=='.')
		arc_name[i - 1] = 0;
	else if (!strchr(&arc_name[j], '.'))
		strcat(arc_name, M_SUFFIX);

#ifndef __TURBOC__
	make_crctable();
#endif

	error_count = 0;
	clock_inx   = 0;
	arcfile     = NULL;
	outfile     = NULL;

	execute_cmd();

	if (error_count>0)  error(M_ERRORCNT, "");

	return EXIT_SUCCESS;
}
