// UnArj.h


#ifndef _UNARJ_H_
#define _UNARJ_H_


/* ********************************************************* */
/* Environment definitions (implementation dependent)        */
/* ********************************************************* */

#define OS                  0
#define MAXSFX              25000L
#define FNAME_MAX           512

#define ERROR_OK        0       /* success */
#define ERROR_WARN      1       /* minor problem (file not found) */
#define ERROR_FAIL      2       /* fatal error */
#define ERROR_CRC       3       /* CRC error */
#define ERROR_SECURE    4       /* ARJ security invalid or not found */
#define ERROR_WRITE     5       /* disk full */
#define ERROR_OPEN      6       /* can't open file */
#define ERROR_USER      7       /* user specified bad parameters */
#define ERROR_MEMORY    8       /* not enough memory */



/* ********************************************************* */
/*
/* Structure of archive main header (low order byte first):
/*
/*  2  header id (comment and local file) = 0x60, 0xEA
/*  2  basic header size (from 'first_hdr_size' thru 'comment' below)
/*	     = first_hdr_size + strlen(filename) + 1 + strlen(comment) + 1
/*	     = 0 if end of archive
/*
/*  1  first_hdr_size (size up to 'extra data')
/*  1  archiver version number
/*  1  minimum archiver version to extract
/*  1  host OS	 (0 = MSDOS, 1 = PRIMOS, 2 = UNIX, 3 = AMIGA, 4 = MACDOS)
/*               (5 = OS/2, 6 = APPLE GS, 7 = ATARI ST, 8 = NEXT)
/*               (9 = VAX VMS)
/*  1  arj flags (0x01 = GARBLED_FLAG, 0x02 = OLD_SECURED_FLAG)
/*               (0x04 = VOLUME_FLAG,  0x08 = EXTFILE_FLAG)
/*               (0x10 = PATHSYM_FLAG, 0x20 = BACKUP_FLAG)
/*               (0x40 = SECURED_FLAG)
/*  1  arj security version (2 = current)
/*  1  file type            (2 = comment header)
/*  1  ?                   ]
/*  4  date time stamp created
/*  4  date time stamp modified
/*  4  archive size up to the end of archive marker
/*  4  file position of security envelope data
/*  2  entryname position in filename
/*  2  length in bytes of trailing security data
/*  2  host data
/*  ?  extra data
/*
/*  ?  archive filename (null-terminated)
/*  ?  archive comment  (null-terminated)
/*
/*  4  basic header CRC
/*
/*  2  1st extended header size (0 if none)
/*  ?  1st extended header
/*  4  1st extended header's CRC
/*  ...
/*
/*
/* Structure of archive file header (low order byte first):
/*
/*  2  header id (comment and local file) = 0x60, 0xEA
/*  2  basic header size (from 'first_hdr_size' thru 'comment' below)
/*	     = first_hdr_size + strlen(filename) + 1 + strlen(comment) + 1
/*	     = 0 if end of archive
/*
/*  1  first_hdr_size (size up to 'extra data')
/*  1  archiver version number
/*  1  minimum archiver version to extract
/*  1  host OS	 (0 = MSDOS, 1 = PRIMOS, 2 = UNIX, 3 = AMIGA, 4 = MACDOS)
/*               (5 = OS/2, 6 = APPLE GS, 7 = ATARI ST, 8 = NEXT)
/*               (9 = VAX VMS)
/*  1  arj flags (0x01 = GARBLED_FLAG, 0x02 = NOT USED)
/*               (0x04 = VOLUME_FLAG,  0x08 = EXTFILE_FLAG)
/*               (0x10 = PATHSYM_FLAG, 0x20 = BACKUP_FLAG)
/*               (0x40 = NOT USED)
/*  1  method    (0 = stored, 1 = compressed most ... 4 compressed fastest)
/*  1  file type (0 = binary, 1 = text, 2 = comment header, 3 = directory)
/*               (4 = label)
/*  1  garble password modifier
/*  4  date time stamp modified
/*  4  compressed size
/*  4  original size
/*  4  original file's CRC
/*  2  entryname position in filename
/*  2  file access mode
/*  2  host data
/*  ?  extra data
/*     4 bytes for extended file position
/*
/*  ?  filename (null-terminated)
/*  ?  comment	(null-terminated)
/*
/*  4  basic header CRC
/*
/*  2  1st extended header size (0 if none)
/*  ?  1st extended header
/*  4  1st extended header's CRC
/*  ...
/*  ?  compressed file
/*
/* ********************************************************* */
/* ********************************************************* */
/*                                                           */
/*     Time stamp format:                                    */
/*                                                           */
/*      31 30 29 28 27 26 25 24 23 22 21 20 19 18 17 16      */
/*     |<---- year-1980 --->|<- month ->|<--- day ---->|     */
/*                                                           */
/*      15 14 13 12 11 10  9  8  7  6  5  4  3  2  1  0      */
/*     |<--- hour --->|<---- minute --->|<- second/2 ->|     */
/*                                                           */
/* ********************************************************* */

#define CODE_BIT          16

#define MAXMETHOD          4

#define ARJ_VERSION        3
#define ARJ_M_VERSION      6    /* ARJ version that supports modified date. */
#define ARJ_X_VERSION      3    /* decoder version */
#define ARJ_X1_VERSION     1
#define DEFAULT_METHOD     1
#define DEFAULT_TYPE       0    /* if type_sw is selected */
#define HEADER_ID     0xEA60
#define HEADER_ID_HI    0xEA
#define HEADER_ID_LO    0x60
#define FIRST_HDR_SIZE    30
#define FIRST_HDR_SIZE_V  34
#define COMMENT_MAX     2048
#define HEADERSIZE_MAX   (FIRST_HDR_SIZE + 10 + FNAME_MAX + COMMENT_MAX)
#define BINARY_TYPE        0    /* This must line up with binary/text strings */
#define TEXT_TYPE          1
#define COMMENT_TYPE       2
#define DIR_TYPE           3
#define LABEL_TYPE         4

#define GARBLE_FLAG     0x01
#define VOLUME_FLAG     0x04
#define EXTFILE_FLAG    0x08
#define PATHSYM_FLAG    0x10
#define BACKUP_FLAG     0x20

#define CRC_MASK        0xFFFFFFFFL

#define FA_RDONLY       0x01            /* Read only attribute */
#define FA_HIDDEN       0x02            /* Hidden file */
#define FA_SYSTEM       0x04            /* System file */
#define FA_LABEL        0x08            /* Volume label */
#define FA_DIREC        0x10            /* Directory */
#define FA_ARCH         0x20            /* Archive */

#define HOST_OS_NAMES1 "MS-DOS","PRIMOS","UNIX","AMIGA","MAC-OS","OS/2"
#define HOST_OS_NAMES2 "APPLE GS","ATARI ST","NEXT","VAX VMS"
#define HOST_OS_NAMES  { HOST_OS_NAMES1, HOST_OS_NAMES2, NULL }

/* Timestamp macros */

#define get_tx(m,d,h,n) (((DWORD)m<<21)+((DWORD)d<<16)+((DWORD)h<<11)+(n<<5))
#define get_tstamp(y,m,d,h,n,s) ((((DWORD)(y-1980))<<25)+get_tx(m,d,h,n)+(s/2))

#define ts_year(ts)  ((UINT)((ts >> 25) & 0x7f) + 1980)
#define ts_month(ts) ((UINT)(ts >> 21) & 0x0f)      /* 1..12 means Jan..Dec */
#define ts_day(ts)   ((UINT)(ts >> 16) & 0x1f)      /* 1..31 means 1st..31st */
#define ts_hour(ts)  ((UINT)(ts >> 11) & 0x1f)
#define ts_min(ts)   ((UINT)(ts >> 5) & 0x3f)
#define ts_sec(ts)   ((UINT)((ts & 0x1f) * 2))

/* unarj.c */

extern long origsize;
extern long compsize;

extern DWORD crc;

extern FILE *arcfile;
extern FILE *outfile;

extern WORD bitbuf;

extern BYTE subbitbuf;
extern BYTE header[HEADERSIZE_MAX];

extern char arc_name[FNAME_MAX];

extern int bitcount;
extern int file_type;
extern int error_count;

/* Global functions */

/* unarj.c */

void   disp_clock();
void   error(LPCSTR pszFormat, LPCSTR pszArg);
void   fillbuf(int n);
WORD   getbits(int n);
void   fwrite_txt_crc(BYTE* pbBuf, size_t n);
void   init_getbits();

/* decode.c */

void   decode();
void   decode_f();

#endif
