// Uue/UueFichier.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "AppResource.h"
#include "UueFichier.h"
#include "Uue.h"

#include "../Decomp.h"
using namespace SPACE_DECOMP;

#include "../Exceptions.h"

#include <FileBuf.h>
#include <GUnicode.h>


const BYTE bErr = 0xFF;

static const BYTE aDecoder[0x100] =
{
	bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr,
	bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr,
	bErr, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F,
	0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F,
	0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F,
	0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F,
	0x00, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr,
	bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr,
	bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr,
	bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr,
	bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr,
	bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr,
	bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr,
	bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr,
	bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr,
	bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr, bErr,
};


// Dcompresser le fichier (spcifique  l'extraction de dossiers)
#pragma warning (disable: 4701)
bool CUueFichier::DecompInterne(LPCTSTR pszDest)
{
	bool fOK = false;
	
	if (e_dm==dmNormal) {
		// Mode rel (sortie sur fichier)
		
		// Crer le fichier (sans attributs car on ne les connait pas)
		SetFileAttributes(pszDest, 0);
		VERIF(e_fw.Open(pszDest));
		
		// Dfinir la date de modification du fichier
		const FILETIME ftM = m_dtM;
		SetFileTime(e_fw, NULL, NULL, &ftM);
	}
	
	VERIF(e_pfr->Seek(m_off, FILE_BEGIN));
	
	
	BYTE b;
	
	// Sauter la premire ligne
	do{
		if (!e_pfr->Read(b))
			goto Fin;
		OnRead(1);
	}while (b!='\r' && b!='\n');
	
	// Dcompresser ici
	char psz[256];
	int lenLigne;
	BYTE abOut[maxLigneD], *pb;
	for (;;) {
		
		// Lire une ligne
		bool fDebLigne = true;
		char *pc = psz;
		for (;;) {
			if (!e_pfr->Read(b))
				goto Fin;
			OnRead(1);
			if (b=='\r' || b=='\n') {
				if (!fDebLigne)
					break;
				fDebLigne = false;
				continue;
			}
			
			*pc++ = (char)b;
			
			// Erreur si la ligne est trop longueur
			if (pc-psz==nbArray(psz)) {
ErreurFormat:
				CMuzipException(ERR_FORMAT, e_pfr->GetPath()).ReportError();
				goto Fin;
			}
		}
		*pc = 0;
		
		// Regarder si le token de fin est trouv
		if (!lstrcmpiA(psz, pszUueEnd))
			break;
		
		// Dcoder la longueur de la ligne
		lenLigne = aDecoder[psz[0]];
		if (!lenLigne)
			break;
		
		// Tester une fin de ligne inattendue
		if (lstrlenA(psz)<lenLigne)
			goto ErreurFormat;
		
		pc = psz+1;
		if (e_dm==dmTester) {
			// Plus rapide comme a !
			
			for (int i=0; i<lenLigne; i+=3,pb+=3, pc+=4) {
				if (aDecoder[pc[0]]==bErr ||
				    aDecoder[pc[1]]==bErr ||
				    aDecoder[pc[2]]==bErr ||
				    aDecoder[pc[3]]==bErr)
					goto ErreurFormat;
			}
			
		}else{
			// Dcoder
			
			pb = abOut;
			for (int i=0; i<lenLigne; i+=3, pc+=4,pb+=3) {
				const BYTE b0 = aDecoder[pc[0]];
				const BYTE b1 = aDecoder[pc[1]];
				const BYTE b2 = aDecoder[pc[2]];
				const BYTE b3 = aDecoder[pc[3]];
				if (b0==bErr || b1==bErr || b2==bErr || b3==bErr)
					goto ErreurFormat;
				
				// Dcoder les 4 caractres en 3 octets
				pb[0] = (BYTE)((b0<<2) | ((b1>>4) & 0x03));
				pb[1] = (BYTE)((b1<<4) | ((b2>>2) & 0x0F));
				pb[2] = (BYTE)((b2<<6) |   b3);
			}
			
			// crire le contenu du tampon
			if (!Flush(abOut, lenLigne))
				goto Fin;
		}
	}
	
	
	fOK = true;
Fin:
	e_fw.Close();
	
	return fOK;
}
#pragma warning (default: 4701)
