// Z/Compress.cpp
//
// Routines gnrales de compression


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#define FEN_COMP

#include "../CodeRapide.h"
#include "../Muzip.h"
#include "ZArc.h"
#include "Fen.h"
#include "Huffman.h"

#include "../PlusItem.h"


// Configuration courante
static int    s_nNiveau;
static CONFIG s_config;




// Les correspondances de longueur 3 sont ignores
// si leur distance excde TOO_FAR
const UINT maxDistMinMatch = 4096;

// lenHash et lenFen doivent tre des puissances de deux
const UINT bitsHash = 15;
const UINT lenHash  = 1<<bitsHash;
const UINT mskHash  = lenHash-1;
const UINT mskFen   = lenFen-1;
const UINT shfHash  = (bitsHash+minMatch-1)/minMatch;


// Lien  la plus ancienne chane avec le mme index de hachage
// Pour limiter la taille de ce tableau  64ko, ce lien n'est maintenu
// que pour les derniers 32ko de chanes
// Un index dans ce tableau est donc un index e_pbFen modulo 32ko
static UINT s_aAvant[lenFen];

// Ttes des chanes de hachage
static UINT s_aTete[lenHash];

// FALSE si le fichier d'entre est dj en mmoire
static bool s_fSlide;

// Index de hachage pour la chane  insrer
static UINT s_nIns;


static UINT s_offMatch;    // Dbut de la chane de correspondance


// Longueur de la meilleure correspondance au pas prcdent
// Les correspondances n'tant pas suprieures  cela sont ignores
// Sert  l'valuation paresseuse des correspondances
static UINT s_lenAvant;


static bool s_fEOF;           // Fin du fichier d'entre ?
static UINT s_nResteDevant;   // Nombre d'octets valides ahead dans e_pbFen



// Flusher le bloc courant, avec le flag fin-de-fichier donn
// e_offStr doit contenir la fin de la correspondance courante
#define BLOC_FLUSH(fEOF) \
	trBlocFlush((e_debBloc>=0) \
		? &e_pbFen[(UINT)e_debBloc] \
		: NULL, \
	(long)e_offStr-e_debBloc, (fEOF))


// Mettre  jour une valeur de hachage avec l'octet d'entre donn
// Tous les appels  cette macro sont faits avec des caractres
// d'entre conscutifs, de manire  ce que chaque cl de hachage
// courante puisse tre calcule  partir de la cl prcdente
// au lieu d'effectuer un recalcul complet  chaque fois
#define HASH_MAJ(h, c)   (h = (((h)<<shfHash) ^ (c)) & mskHash)


// Insrer la chane psz dans le dictionnaire et placer dans match_head
// la tte prcdente de la chane de hachage (la plus rcente chane
// ayant la mme cl de hachage)
// Retourner la longueur prcdente de la chane de hachage
// Tous les appels  cette macro sont faits avec des caractres
// d'entre conscutifs et les minMatch premiers octets de s sont
// valides (sauf les minMatch-1 derniers octets du fichier d'entre
#define CHAINE_INS(s, debMatch) \
	(HASH_MAJ(s_nIns, e_pbFen[(s) + (minMatch-1)]), \
	s_aAvant[(s) & mskFen] = debMatch = s_aTete[s_nIns], \
	s_aTete[s_nIns] = (s))


// Initialiser les routines "plus longue correspondance"
// pour un nouveau fichier
// e_lenFen>0 si le fichier d'entre est dj lu, ou gal  0 sinon
// Dans le premier cas, e_lenFen est suffisant pour contenir
// la totalit du fichier plus minResteDevant octets (pour viter
// des accs  de la mmoire au-del de la fin de e_pbFen[] lors de
// la recherche de correspondances dans la fin)
void CZArc::lmInit(int nNiveau)
{
	// Ne pas diviser la fentre si la totalit de l'entre
	// est dj en mmoire (e_lenFen>0)
	if (e_lenFen)
		s_fSlide = false;
	else{
		s_fSlide = true;
		e_lenFen = 2*lenFen;
	}
	
	// Initialiser la table de hachage
	// s_aAvant[] sera initialis au passage
	s_aTete[lenHash-1] = 0;
	ZeroMemory(s_aTete, (lenHash-1)*sizeof(*s_aTete));
	
	// Dfinir les paramtre de configuration par dfaut
	s_nNiveau = nNiveau;
	s_config  = e_aConfig[s_nNiveau];
	
	e_offStr  = 0;
	e_debBloc = 0;
	
	register UINT len = lenFen;
	len <<= 1;
	s_nResteDevant = LireFichier(e_pbFen, len);
	oApp.AddStatusAvance(0, s_nResteDevant);
	oApp.AddStatusAvance(1, s_nResteDevant);
	
	if (!s_nResteDevant || s_nResteDevant==(UINT)EOF) {
		s_fEOF = true;
		s_nResteDevant = 0;
		return;
	}
	s_fEOF = false;
	
	// S'assurer que l'on a toujours assez de s_nResteDevant
	if (s_nResteDevant<minResteDevant)
		FillFen();
	
	// Si s_nResteDevant<minMatch, s_nIns est erron, mais cela n'est pas important
	// car seuls des octets littraux seront mis
	s_nIns = 0;
	for (len=0; len<minMatch-1; len++)
		HASH_MAJ(s_nIns, e_pbFen[len]);
}


// Place dans s_offMatch le dpart de la plus longue correspondance
// de la chane donne, et retourne la longueur de cette chane
// Les correspondnces infrieures ou gales  s_lenAvant sont rejetes,
// auquel cas le rsultat est gal  s_lenAvant et s_offMatch est erron
// nCurMatch est la tte de la chane de hachage pour la chane courant
// (e_offStr), sa distance est <=maxDist et s_lenAvant>=1
int MatchPlusLongue(UINT nCurMatch)
{
	UINT lenChaine = s_config.maxChaine;        // Longueur maximale des chanes de hachage
	register BYTE *pbScan = e_pbFen+e_offStr;   // Chane courante
	register BYTE *pbMatch;                     // Chane de correspondance
	register int   len;                         // Longueur de la correspondance courante
	int lenBest = s_lenAvant;                   // Meilleure longueur de correspondance
	
	// Attter quand nCurMatch<=nLimite
	// Pour simplifier le code, on empche les correspondance avec la chane
	// de e_pbFen index 0
	const UINT nLimite = (e_offStr>(UINT)maxDist) ? e_offStr-(UINT)maxDist : 0;
	
#ifdef UNALIGNED_OK
	// Comparer deux octets  la fois
	// Note: cela n'est pas toujours bnfique; essayer avec et sans UNALIGNED_OK
	register BYTE *pbFin  = e_pbFen + e_offStr + maxMatch - 1;
	register WORD debScan = *(WORD*)pbScan;
	register WORD finScan = *(WORD*)(pbScan+lenBest-1);
#else
	register BYTE *pbFin  = e_pbFen + e_offStr + maxMatch;
	register BYTE finScan1 = pbScan[lenBest-1];
	register BYTE finScan  = pbScan[lenBest];
#endif
	
	// Ne pas gcher trop de temps si on a dj une bonne correspondance
	if (s_lenAvant>=s_config.lenOK)
		lenChaine >>= 2;
	ASSERT(e_offStr<=e_lenFen-minResteDevant); // Plus assez de s_nResteDevant
	
	do{
		ASSERT(nCurMatch<e_offStr); // Plus de futur
		pbMatch = e_pbFen + nCurMatch;
		
		// Passer la correspondance suivante si sa longueur ne peut tre augmente
		// ou si elle est infrieure  2
#ifdef     UNALIGNED_OK
		if (*(WORD*)(pbMatch+lenBest-1)!=finScan ||
		    *(WORD*) pbMatch           !=debScan)  continue;
		
		// Il n'est pas ncessaire de comparer pbScan[2]  pbMatch[2]
		// car ils sont toujours gaux si les autres octets correspondent,
		// tant donn que les cls de hachage sont gales et que bitsHash>=8
		// Comparer 2 octets  la fois  e_offStr+3, +5, ... e_offStr+257
		// On vrifie l'insuffisance de s_nResteDevant que toutes les 4 comparaisons;
		// la 128e vrification sera faite  e_offStr+257
		// Si maxMatch-2 n'est pas un multiple de 8, il est ncessaire de mettre
		// plus d'octets de garde  la fin de e_pbFen, ou de vrifier plus
		// souvent l'insuffisance de s_nResteDevant
		pbScan++;
		pbMatch++;
		
		// do{}while gnre souvent un meilleur code sur les compilateurs
		do{
		}while (*(WORD*)(pbScan+=2)==*(WORD*)(pbMatch+=2) &&
		       *(WORD*)(pbScan+=2)==*(WORD*)(pbMatch+=2) &&
		       *(WORD*)(pbScan+=2)==*(WORD*)(pbMatch+=2) &&
		       *(WORD*)(pbScan+=2)==*(WORD*)(pbMatch+=2) &&
		        pbScan<pbFin);
		
		// Ici, pbScan<=e_pbFen+e_offStr+257
		ASSERT(pbScan<=e_pbFen+(UINT)(e_lenFen-1)); // pbScan trange
		if (*pbScan==*pbMatch)
			pbScan++;
		
		len = (maxMatch-1) - (int)(pbFin-pbScan);
		pbScan = pbFin - (maxMatch-1);
		
#else  // !UNALIGNED_OK
		
		if (pbMatch[lenBest]   !=finScan  ||
		    pbMatch[lenBest-1] !=finScan1 ||
		    *pbMatch           !=*pbScan ||
		    *++pbMatch         != pbScan[1])
			continue;
		
		// La vrification  lenBest-1 peut tre enleve car elle sera
		// effectue de nouveau plus tard (cette heuristique n'est pas
		// toujours un gain)
		// Il n'est pas ncessaire de comparer pbScan[2]  pbMatch[2]
		// car ils sont toujours gaux si les autres octets correspondent,
		// tant donn que les cls de hachage sont gales et que bitsHash>=8
		pbScan += 2, pbMatch++;
		
		// On vrifie l'insuffisance de s_nResteDevant que toutes les
		// 8 comparaisons; la 256e vrification sera faite  e_offStr+258
		do{
		}while (*++pbScan==*++pbMatch && *++pbScan==*++pbMatch &&
		        *++pbScan==*++pbMatch && *++pbScan==*++pbMatch &&
		        *++pbScan==*++pbMatch && *++pbScan==*++pbMatch &&
		        *++pbScan==*++pbMatch && *++pbScan==*++pbMatch &&
		        pbScan<pbFin);
		
		ASSERT(pbScan<=e_pbFen+(UINT)(e_lenFen-1));
		
		len = maxMatch - (int)(pbFin - pbScan);
		pbScan = pbFin - maxMatch;
#endif // ?UNALIGNED_OK
		
		if (len>lenBest) {
			s_offMatch = nCurMatch;
			lenBest = len;
			if (len>=s_config.lenFin) break;
#ifdef UNALIGNED_OK
			finScan = *(WORD*)(scan+lenBest-1);
#else
			finScan1  = pbScan[lenBest-1];
			finScan   = pbScan[lenBest];
#endif
		}
	}while ((nCurMatch = s_aAvant[nCurMatch & mskFen])>nLimite && --lenChaine);
	
	return lenBest;
}

#ifdef     _DEBUG
// Vrifier que la correspondance  s_offMatch
// soit  la place d'une correspondance
void CZArc::MatchVerif(UINT nDeb, UINT nMatch, int len)
{
	if (memcmp(e_pbFen+nMatch, e_pbFen+nDeb, len)) {
		TRACE3("!!! Correspondance invalide - nDeb %d, nMatch %d, len %d\n",
			nDeb, nMatch, len);
		AfxDebugBreak();
	}
}
#else  // !_DEBUG
#define MatchVerif(nDeb, nMatch, len)
#endif // ?_DEBUG


// Remplir la fentre quand s_nResteDevant devient trop petit
// Mettre  jour e_offStr et s_nResteDevant, puis dfinir s_fEOF
// si la fin du fichier d'entre est atteinte
// e_pbFen non vide
// s_nResteDevant<minResteDevant && e_offStr+s_nResteDevant>0
// A la sortie, on a:
// e_offStr<=e_lenFen-minResteDevant
// Au moins un octet a t lu, ou s_fEOF a t dfini
void CZArc::FillFen()
{
	register UINT n, m;
	UINT nPlus;    // Espace disponible  la fin de e_pbFen
	
	do{
		nPlus = (UINT)(e_lenFen - (DWORD)s_nResteDevant - (DWORD)e_offStr);
		
		// Si la fentre est presque pleine et qu'il ne reste plus d'espace devant,
		// dplacer la moiti suprieure dans l'infrieure pour faire de la place
		// dans la moiti suprieure
		if (nPlus==(UINT)EOF)
			// Trs dplaisant, mais possible sur les machines 16 bits
			// si e_offStr==0 et s_nResteDevant==1 (entre octet par octet)
			nPlus--;
			
		else if (e_offStr>=lenFen+maxDist && s_fSlide) {
			
			// e_pbFen n'est pas vide (par convention d'appel)
			// nPlus==0 avec nPlus==64ko sur machines 16 bits
			memcpy(e_pbFen, e_pbFen+lenFen, (size_t)lenFen);
			s_offMatch -= lenFen;
			e_offStr   -= lenFen;  // On a ici e_offStr>=maxDist:
			e_debBloc  -= (long)lenFen;
			
			for (n=0; n<lenHash; n++) {
				m = s_aTete[n];
				s_aTete[n] = (UINT)(m >= lenFen ? m-lenFen : 0);
			}
			for (n=0; n<lenFen; n++) {
				// Si n n'est pas une chane de hachage,
				// s_aAvant[n] est erron mais sa valeur ne sera jamais utilise
				m = s_aAvant[n];
				s_aAvant[n] = (UINT)(m >= lenFen ? m-lenFen : 0);
			}
			nPlus += lenFen;
		}
		if (s_fEOF) return;
		
		n = LireFichier(e_pbFen+e_offStr+s_nResteDevant, nPlus);
		if (!n)
			s_fEOF = true;
		else{
			oApp.AddStatusAvance(0, n);
			oApp.AddStatusAvance(1, n);
			s_nResteDevant += n;
		}
		
	}while (s_nResteDevant<minResteDevant && !s_fEOF);
}


// Processes a new input file and return its compressed length. This
// function does not perform lazy evaluationof matches and inserts
// new strings in the dictionary only for unmatched strings or for short
// matches. It is used only for the fast compression options.
DWORD CZArc::DeflateFast()
{
	UINT debHash;        // Head of the hash chain
	bool fDoitFlush;     // Set if current block must be flushed
	UINT lenMatch = 0;   // Length of best match
	s_lenAvant = minMatch-1;
	while (s_nResteDevant) {
		// Insert the string e_pbFen[e_offStr .. e_offStr+2] in the
		// dictionary, and set debHash to the s_aTete of the hash chain:
		CHAINE_INS(e_offStr, debHash);
		
		// Find the longest match, discarding those <= s_lenAvant.
		// At this point we have always lenMatch < minMatch
		if (debHash && e_offStr-debHash <= maxDist) {
			// To simplify the code, we prevent matches with the string
			// of e_pbFen index 0 (in particular we have to avoid a match
			// of the string with itself at the start of the input file).
			lenMatch = MatchPlusLongue(debHash);
			
			// MatchPlusLongue() dfinit s_offMatch
			if (lenMatch>s_nResteDevant)
				lenMatch = s_nResteDevant;
		}
		if (lenMatch >= minMatch) {
			MatchVerif(e_offStr, s_offMatch, lenMatch);
			
			fDoitFlush = ctCompter(e_offStr-s_offMatch, lenMatch - minMatch);
			s_nResteDevant -= lenMatch;
			// Insert new strings in the hash table only if the match length
			// is not too large. This saves time but degrades compression.
			if (lenMatch <= s_config.maxParesseux) {
				lenMatch--; // string at e_offStr already in hash table
				do{
					e_offStr++;
					CHAINE_INS(e_offStr, debHash);
					// e_offStr never exceeds lenFen-maxMatch, so there are
					// always minMatch bytes ahead. If s_nResteDevant < minMatch
					// these bytes are garbage, but it does not matter since
					// the next s_nResteDevant bytes will be emitted as literals.
				}while (--lenMatch);
				e_offStr++;
			}else{
				e_offStr += lenMatch;
				lenMatch = 0;
				s_nIns = e_pbFen[e_offStr];
				HASH_MAJ(s_nIns, e_pbFen[e_offStr+1]);
			}
		}else{
			// Pas de correspondance, un octet littral en sortie
			fDoitFlush = ctCompter(0, e_pbFen[e_offStr]);
			s_nResteDevant--;
			e_offStr++;
		}
		if (fDoitFlush) {
			BLOC_FLUSH(false);
			e_debBloc = e_offStr;
		}
		// Make sure that we always have enough s_nResteDevant, except
		// at the end of the input file. We need maxMatch bytes
		// for the next match, plus minMatch bytes to insert the
		// string following the next match.
		if (s_nResteDevant<minResteDevant)
			FillFen();
	}
	return BLOC_FLUSH(true);
}

// Same as above, but achieves better compression. We use a lazy
// evaluation for matches: a match is finally adopted only if there is
// no better match at the next e_pbFen position.
DWORD CZArc::Deflate()
{
	UINT debHash;       // Tte de la chane de hachage
	UINT nMatchAvant;   // Correspondance prcdente
	bool fDoitFlush;    // Bloc courant doit tre flush ?
	bool fMatchAvant = false;   // Correspondance prcdente existe ?
	register UINT lenMatch = minMatch-1;   // Longueur meilleure correspondance
	
	if (s_nNiveau<2)
		return DeflateFast();
	
	// Traiter le bloc d'entre
	while (s_nResteDevant) {
		// Insrer la chane e_pbFen[e_offStr..e_offStr+2] dans le
		// dictionnaire, et mettre la tte de la chane de hachage
		// dans debHash
		CHAINE_INS(e_offStr, debHash);
		
		// Chercher la plus longue correspondance
		// en liminant celles <= s_lenAvant
		s_lenAvant  = lenMatch;
		nMatchAvant = s_offMatch;
		lenMatch    = minMatch-1;
		if (debHash && s_lenAvant<s_config.maxParesseux && e_offStr-debHash<=maxDist) {
			// Pour simplifier le code, on limine les correspondances
			// avec la chane de e_pbFen d'index 0 (en particulier on
			// doit viter une correspondance de la chane avec elle-mme
			// au dbut du fichier d'entre)
			lenMatch = MatchPlusLongue(debHash);
			
			// MatchPlusLongue() dfinit s_offMatch
			if (lenMatch>s_nResteDevant)
				lenMatch = s_nResteDevant;
			
			// Ignorer une correspondance de longueur 3 si trop distante
			if (lenMatch==minMatch && e_offStr-s_offMatch>maxDistMinMatch)
				// If nMatchAvant is also minMatch, s_offMatch is garbage
				// but we will ignore the current match anyway.
				lenMatch = minMatch-1;
		}
		// If there was a match at the previous step and the current
		// match is not better, output the previous match:
		if (s_lenAvant>=minMatch && lenMatch<=s_lenAvant) {
			
			MatchVerif(e_offStr-1, nMatchAvant, s_lenAvant);
			fDoitFlush = ctCompter(e_offStr-1-nMatchAvant, s_lenAvant-minMatch);
			
			// Insert in hash table all strings up to the end of the match.
			// e_offStr-1 and e_offStr are already inserted.
			s_nResteDevant -= s_lenAvant-1;
			s_lenAvant   -= 2;
			do{
				e_offStr++;
				CHAINE_INS(e_offStr, debHash);
				// e_offStr never exceeds lenFen-maxMatch, so there are
				// always minMatch bytes ahead. If s_nResteDevant < minMatch
				// these bytes are garbage, but it does not matter since the
				// next s_nResteDevant bytes will always be emitted as literals.
			}while (--s_lenAvant);
			fMatchAvant = false;
			lenMatch = minMatch-1;
			e_offStr++;
			
			if (fDoitFlush) {
				BLOC_FLUSH(false);
				e_debBloc = e_offStr;
			}
			
		}else if (fMatchAvant) {
			// If there was no match at the previous position, output a
			// single literal. If there was a match but the current match
			// is longer, truncate the previous match to a single literal.
			if (ctCompter(0, e_pbFen[e_offStr-1]))
				BLOC_FLUSH(false), e_debBloc = e_offStr;
			
			e_offStr++;
			s_nResteDevant--;
		}else{
			// There is no previous match to compare with, wait for
			// the next step to decide.
			fMatchAvant = true;
			e_offStr++;
			s_nResteDevant--;
		}
		ASSERT(e_offStr<=e_lenIn && s_nResteDevant<=e_lenIn); // Un peu trop loin
		
		// Make sure that we always have enough s_nResteDevant, except
		// at the end of the input file. We need maxMatch bytes
		// for the next match, plus minMatch bytes to insert the
		// string following the next match.
		if (s_nResteDevant<minResteDevant)
			FillFen();
	}
	if (fMatchAvant)
		ctCompter(0, e_pbFen[e_offStr-1]);
	
	return BLOC_FLUSH(true);
}
