// Z/Crypt.cpp
//
// Cryptage de l'archive compresse (format Zip seulement)


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#ifndef MUZIPSFX
#include "../CodeRapide.h"
#include "../Muzip.h"
#include "ZArc.h"
#endif
#include "Crypt.h"
#include "UnZ.h"

#include "../CRC32.h"

#include "../Zip/Zip.h"
#include "../Zip/ZipFichier.h"

#ifndef MUZIPSFX
#include "../Exceptions.h"
#endif


bool e_fCrypte;


// Pour le cryptage, on doit initialiser l'algorithme avec des octets
// relativement imprvisibles, voir la fonction cyptEnTete().
// La fonction rand() de la bibliothque standard est utilise pour fournir
// ces octets "au hasard"; cette fonction est initialiss par un appel de
// srand(). Cette fonction prend un "UINT" (d'au moins 16 bits) comme
// gnrateur de nombres pseudo-alatoires.
// Ce gnrateur (Seed) est construit avec "Seed = Seed1 XOR Seed2"
// o Seed1 est fourni par l'heure du jour (Seed1 = "(UINT)time()"),
// et Seed2 l'identificateur du processus actif, c'est--dire un nombre
// totalement imprvisible.
//
// Note
// 1) Cette implmentation requiert les fonctions suivantes de la bibliothque
//    standard du C: time(), rand(), srand().
// 2) C'est une trs mauvaise ide d'appeler une seconde fois time() pour
//    initialiser le nombre Seed2! Dans ce cas, Seed1 et Seed2 seraient
//    presque toujours gaux et le XOR donnerait comme rsultat 0.


static DWORD s_aClef[3];   // Cls dfinisant la squence pseudo-alatoire


#define CRC32(nCRC, b) (e_aCRC[((int)(nCRC) ^ (b)) & 0xFF] ^ ((nCRC) >> 8))


// Retourner l'octet suivant de la squence pseudo-alatoire
BYTE cryptDecrypter()
{
	const UINT nTemp = ((UINT)(s_aClef[2] & 0xFFFF)) | 2;
	return (BYTE)((nTemp * (nTemp^1))>>8);
}


// Mettre  jour les cls de cryptage avec l'octet suivant du texte
BYTE keyMAJ(BYTE b)
{
	s_aClef[0] = CRC32(s_aClef[0], b);
	s_aClef[1] = (s_aClef[1] + ((BYTE)s_aClef[0])) * 134775813L + 1;
	
	register int shfClef = (int)(s_aClef[1]>>24);
	s_aClef[2] = CRC32(s_aClef[2], shfClef);
	
	return b;
}


// Initialiser les cls de cryptage et l'en-tte alatoire
// avec le mot de passe donn
void keyInit(LPCTSTR pszMDP)
{
	if (pszMDP) {
		e_fCrypte = true;
		s_aClef[0] = 305419896L;
		s_aClef[1] = 591751049L;
		s_aClef[2] = 878082192L;
		while (*pszMDP)  keyMAJ((BYTE)*pszMDP++);
	}else
		e_fCrypte = false;
}


#ifndef MUZIPSFX

// crire l'en-tte de cryptage du fichier pf en utilisant
// le mot de passe et le CRC donns
void cryptEnTete(LPCTSTR pszMDP, DWORD nCRC)
{
	int n, nTemp;
	BYTE b;
	BYTE abEnTete[lenCryptEnTete-2];  // En-tte altoire
	
	// Gnrer d'abord lenCryptEnTete-2 octets alatoires
	// On crypte la sortie de rand() pour rduire la prdictabilit,
	// car la fonction rand() est souvent mal implmente
	keyInit(pszMDP);
	for (n=0; n<lenCryptEnTete-2; n++) {
		b = (BYTE)(rand()>>7);
		abEnTete[n] = zipCrypter(b, nTemp);
	}
	
	// Crypter l'en-tte alatoire
	// Les deux derniers octets sont le mot de poids fort du CRC
	keyInit(pszMDP);
	for (n=0; n<lenCryptEnTete-2; n++)
		e_fw.Write(zipCrypter(abEnTete[n], nTemp));
	
	// Crypter les deux derniers octets du CRC
	b = (BYTE)(nCRC>>16);
	e_fw.Write(zipCrypter(b, nTemp));
	b = (BYTE)(nCRC>>24);
	e_fw.Write(zipCrypter(b, nTemp));
}


// Si demand, crypter les donnes dans pbBuf,
// et dans tous les cas crire les donnes dans le fichier e_fw
void CZArc::Write(BYTE* pbBuf, UINT len)
{
	if (e_fCrypte) {
		// Crypter les donnes dans le tampon
		int nTemp;
		BYTE *pb = pbBuf;
		for (UINT lenBuf=len; lenBuf; pb++, lenBuf--)
			*pb = zipCrypter(*pb, nTemp);
	}
	
	// crire le tampon
	if (!e_fw.Write(pbBuf, len) || oApp.UpdateProgressStatus()!=prsOK)
		THROW(new CMuzipException((UINT)0));
}

#endif



// Tester un mot de passe sur un en-tte
#ifndef MUZIPSFX
bool UNZ::TesterMDP(const BYTE* pbEnTete, CZipFichier* pzf)
#else
bool UNZ::TesterMDP(const BYTE* pbEnTete)
#endif
{
	int n;
	
	BYTE abEnTete[lenCryptEnTete];
	
	// Dfinir les cls et sauver l'en-tte crypt
#ifndef MUZIPSFX
	keyInit(pzf->m_pArc->m_sMdp);
#else
	keyInit(e_pszMdp);
#endif
	memcpy(abEnTete, pbEnTete, lenCryptEnTete);
	
	// Vrifier le mot de passe
	for (n=0; n<lenCryptEnTete; n++)
		zipDecrypter(abEnTete[n]);
	
#ifndef MUZIPSFX
	if (abEnTete[lenCryptEnTete-1]!=((pzf->m_nFlags & locfEtendu)
			? ((WORD)(pzf->m_cen.dt   >>  8)) & 0xFF
			:  (WORD)(pzf->m_cen.nCRC >> 24)))
#else
	if (abEnTete[lenCryptEnTete-1]!=((phCenSig->nFlags & locfEtendu)
			? ((WORD)(phCenSig->dt   >>  8)) & 0xFF
			:  (WORD)(phCenSig->nCRC >> 24)))
#endif
		return false;
	
	// Mot de passe OK: dcrypter le contenu du tampon courant avant de quitter
	BYTE *pIn = e_pbIn;
	for (n=((long)e_lenIn>e_lenComp) ? (int)e_lenComp : e_lenIn; n; n--)
		zipDecrypter(*pIn++);
	
	return true;
}
