// Zip/ZipArc.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "../Muzip.h"
#include "../Muzip2.h"
#include "ZipArc.h"
#include "ZipFichier.h"
#include "Zip.h"

#include "../Z/Crypt.h"
#include "../Z/Huffman.h"

#include "../Interf.h"
#include "../FichierPlus.h"
#include "../Options.h"
#include "../Exceptions.h"

#include "../IOFichier.h"

#include <GUnicode.h>
#include <FileBuf.h>
#include <sys/stat.h>

#include "../MuzipSfx/MuzipSfx.h"


// Constructeur
CZipArc::CZipArc(CInterf* pi) : CZArc(pi)
{
	m_pszCom = NULL;
	m_lenCom = 0;
}


// Crer une archive
CMuzipArc* CZipArc::CreateArchive(CInterf* pi)
{
	return new CZipArc(pi);
}


// ID de l'EXE SFX
UINT CZipArc::GetSfxID() const
{
	return IDSFX_ZIP;
}


// Lit l'en-tte de l'auto-extractible de l'archive
bool CZipArc::ReadSfxFooter(SFXFOOTER& sfx)
{
	return
		m_fr.Seek(-(long)sizeof(sfx), FILE_END) &&
		m_fr.Read(&sfx, sizeof(sfx)) &&
		sfx.id == idSFXFooter;
}


// Validation du type
bool CZipArc::IsGoodType()
{
	DWORD sig;
	VERIF(m_fr.Read(sig));
	
	return (sig == sigLoc || sig == sigCen || sig == sigFin);
}



// Ouverture
bool CZipArc::OpenDocument(bool fSilent)
{
	CLstZipFichier lstTrouve;
	
	// Ouvrir l'archive et appeler OuvrirNormal() ou OuvrirReparer()
	CMuzipException *const pmex =
		(e_opt.m_nReparer<=repPresumer)
			? OuvrirNormal (lstTrouve)
			: OuvrirReparer(lstTrouve);
	if (pmex) {
		lstTrouve.DeleteAll();
		if (!fSilent)
			pmex->ReportError();
		pmex->Delete();
		return false;
	}
	
	return OpenArchiveOK(fSilent, m_pszCom);
}

// Ouvrir l'archive en mode normal (vrification active)
CMuzipException* CZipArc::OuvrirNormal(CLstZipFichier& lstTrouve)
{
#define phFin         ((HEADERFIN*)      abBuf)
#define phLocSig      ((HEADERLOCSIG*)   abBuf)
#define phLoc         ((HEADERLOC*)      abBuf)
#define phCenSig      ((HEADERCENSIG*)   abBuf)
	
	BYTE abBuf[sizeof(HEADERCENSIG)];  // Tampon des en-ttes du central
	DWORD nFlags;
	
	m_offDebut = 0;
	DWORD offFin = 0;
	
	// Grer l'en-tte de l'auto-extractible
	if (m_pmdt->GetIdType()==IDR_SFXTYPE) {
		SFXFOOTER sfx;
		if (ReadSfxFooter(sfx))
			m_offDebut = sfx.off;
	}
	
	
	BYTE *const abBufTemp = new BYTE[4096+4];
	BYTE *pb = &abBufTemp[4096];
	pb[1] = 0;
	pb[2] = 0;
	pb[3] = 0;
	
	
	// Rechercher la signature finale  partir de la fin,
	// en lisant des tranches de 4ko  la fois
	// On doit en effet ignorer ce qu'il y a aprs la fin des structures Zip,
	// et ne pas prsumer que les structures Zip se terminent avec le fichier
	//  la fin du processus, offFin contiendra la position de l'en-tte final
	size_t len;   // Utilis plus tard, mais les goto demandent une dclaration avant
	m_fr.SetVerbeux(false);
	if (m_fr.Seek(-4096, FILE_END)) {
		offFin = m_fr.Tell();
		for (;;) {
			abBufTemp[4096] = pb[1];
			abBufTemp[4097] = pb[2];
			abBufTemp[4098] = pb[3];
			
			m_fr.Read(abBufTemp, 4096);
			m_fr.Seek(-2*4096, FILE_CURRENT);
			pb = &abBufTemp[4095];
			
			// Rechercher la signature finale
			do{
				if (*(DWORD*)pb==sigFin)
					goto FinTrouveOK;
			}while (--pb>=abBufTemp);
			if (offFin<4096)
				break;
			offFin -= 4096;
		}
		if (!offFin)
			goto ErreurFormat;
		
		m_fr.Seek(0, FILE_BEGIN);
		len = m_fr.ReadLen(abBufTemp, offFin);
		
	}else
		// Lire le maximum d'octets (4ko - 1 octet)
		len = m_fr.ReadLen(abBufTemp, 4096-1);
	
	m_fr.SetVerbeux();
	
	// Rechercher la signature finale dans les tous-premiers octets,
	// car soit le fichier faisait moins de 4ko,
	// soit la signature se trouve dans les 4095 premiers octets (= 4ko - 1octet)
	
	abBufTemp[len  ] = pb[1];
	abBufTemp[len+1] = pb[2];
	abBufTemp[len+2] = pb[3];
	pb = &abBufTemp[len-1];
	do{
		if (*(DWORD*)pb==sigFin)
			goto FinTrouveOK;
	}while (--pb>=abBufTemp);
	
	// Erreur: Signature finale non-trouve
	delete [] abBufTemp;
	
ErreurFormat:
	return new CMuzipException(ERR_FORMAT, m_fr.GetPath());
	
FinTrouveOK:  // Fin du central trouve
	delete [] abBufTemp;
	
	// Signature finale trouve
	// Calculer la position de la fin du central, et y aller
	offFin += (DWORD)(pb - abBufTemp);
	m_fr.Seek(offFin+lenSig, FILE_BEGIN);
	
	// Lire l'en-tte final
	if (!m_fr.Read(abBuf, sizeof(HEADERFIN))) {
ErreurFileRead:
		return new CMuzipException;
	}
#ifdef _DEBUG
	if (phFin->nDisque || phFin->nDisqueD || phFin->nbEntry!=phFin->nbEntryT)
		TRACE0("Attention: multi-volumes ignor\n");
#endif
	
	// Lire les commentaires
	m_lenCom = phFin->lenCom;
	if (m_lenCom) {
		m_pszCom = FromZip(m_lenCom, verCur, false);
		if (!m_pszCom)
			goto ErreurFileRead;
	}
	
	// On prsume que le central prcde immdiatement l'en-tte final
	m_offCen = offFin - phFin->nTailleT;
	const long offDecal = m_offDebut +
		((e_opt.m_nReparer==repPresumer) ? 0 : m_offCen - phFin->off);
	if (!m_fr.Seek(m_offCen, FILE_BEGIN))
		goto ErreurFileRead;
	
	CZipFichier *pzf;
	WORD lenNom;
	
	// Lire le central
	// La partie des en-ttes centraux qui doit tre commune
	// avec les en-ttes locaux est lue brute, pour une comparaison future
	// (cela demande que l'offset de m_pbExtra dans CZipFichier soit suprieur
	// ou gal  LOCHEAD)
	DWORD offCentral = m_offCen;   // Offset courant
	for (;;) {
		
		// Lire la signature: ce doit tre la signature des en-ttes centraux
		m_fr.SetVerbeux(false);
		DWORD sig;
		const bool fOK = m_fr.Read(sig);
		m_fr.SetVerbeux();
		if (!fOK) {
			if (m_fr.IsEOF())
				break;
			return new CMuzipException(m_fr);
		}
		
		// Valider la signature
		if (sig!=sigCen)
			break;
		
		// Lire le central
		pzf = new CZipFichier;
		if (!m_fr.Read(&pzf->m_cen, sizeof(pzf->m_cen))) {
			delete pzf;
			goto ErreurFormat;
		}
		
		lstTrouve.AddFin(pzf);
		pzf->m_cen.off    += offDecal;
		pzf->m_offCentral  = offCentral;
		WORD lenNom = pzf->m_cen.lenNom;
		if (!lenNom)
			goto ErreurFormat;
		
		const WORD verC    = pzf->m_cen.nVerC;
		const bool fPasOEM = !HIWORD(pzf->m_cen.nAttrE);
		pzf->m_fDOS        = !(verC & 0xF00);
		
		// Lire le nom du fichier
		pzf->m_pszNom = FromZip(lenNom, verC, fPasOEM);
		if (!pzf->m_pszNom)
			goto ErreurFileRead;
		
		// Lire le champ extra
		if (pzf->m_cen.lenExtraC) {
			pzf->m_pbExtraC = new BYTE[pzf->m_cen.lenExtraC];
			if (!m_fr.Read(pzf->m_pbExtraC, pzf->m_cen.lenExtraC))
				goto ErreurFileRead;
		}
		
		// Lire les commentaires
		const WORD lenCom = pzf->m_cen.lenCom;
		if (lenCom) {
			pzf->m_pszCom = FromZip(pzf->m_cen.lenCom, verC, fPasOEM);
			if (!pzf->m_pszCom)
				goto ErreurFileRead;
		}
		
		offCentral += sizeof(HEADERCENSIG) + lenNom + pzf->m_cen.lenExtraC + lenCom;
	}
	
	// m_offDebut pointe maintenant vers l'en-tte local
	// du premier fichier de l'archive en position
	
	// Lire les en-ttes locaux, et les comparer avec les en-ttes centraux
	POS pos = lstTrouve.GetPosDeb();
	while (pos) {
		pzf = lstTrouve.GetSuiv(pos);
		
		// Lire la signature
		if (!m_fr.Seek(pzf->m_cen.off, FILE_BEGIN) ||
		    !m_fr.Read((DWORD&)*abBuf))
			goto ErreurFileRead;
		
		if (phLocSig->sig!=sigLoc)
			// Erreur: en-tte local non-trouv
			goto ErreurFormat;
		
		// Lire l'en-tte proprement dit
		if (!m_fr.Read(abBuf, sizeof(HEADERLOC)))
			goto ErreurFileRead;
		pzf->m_nFlags   = phLoc->nFlags;
		lenNom          = phLoc->lenNom;
		pzf->m_lenExtra = phLoc->lenExtra;
		
		// Comparer la longueur des noms
		if (lenNom!=pzf->m_cen.lenNom)
			// Noms de longueur diffrente dans local et central
			goto ErreurFormat;
		
		// Lire le nom (local) et le comparer avec l'ancien (central)
		const LPTSTR pszNom = FromZip(lenNom, pzf->m_cen.nVerC, false);
		if (!pszNom)
			goto ErreurFormat;
		const bool fNomDiff = ToBool(lstrcmpi(pszNom, pzf->m_pszNom));
		delete [] pszNom;
		if (fNomDiff)
			goto ErreurFormat;
		
		// Comparer les champs extra
		if (pzf->m_lenExtra) {
			pzf->m_pbExtra = new BYTE[pzf->m_lenExtra];
			if (!m_fr.Read(pzf->m_pbExtra, pzf->m_lenExtra)) {
				delete [] pzf->m_pbExtra;
				goto ErreurFileRead;
			}
			if (pzf->m_lenExtra==pzf->m_cen.lenExtraC &&
			    !memcmp(pzf->m_pbExtra, pzf->m_pbExtraC, pzf->m_lenExtra)) {
				// En-ttes local et central identiques:
				// tout garder dans l'en-tte central
				delete [] pzf->m_pbExtra;
				pzf->m_pbExtra = pzf->m_pbExtraC;
			}
		}
		
		// Vrifier l'en-tte local tendu s'il y en a un
		if (pzf->m_nFlags & locfEtendu) {
			HEADERLOCEXTSIG hLocExtSig;
			
			DWORD nTailleC = phLoc->nTailleC;
			if (!nTailleC)
				nTailleC = pzf->m_cen.nTailleC;
			
			if (!m_fr.Seek(pzf->m_cen.off + sizeof(HEADERLOCSIG) +
			     pzf->m_cen.lenNom + pzf->m_lenExtra + nTailleC, FILE_BEGIN) ||
			    !m_fr.Read(&hLocExtSig, sizeof(hLocExtSig)))
				goto ErreurFileRead;
			
			if (hLocExtSig.sig==sigLocExt)
				// En-tte local tendu trouv:
				// craser les valeurs de l'en-tte local
				memcpy(&phLoc->nCRC, &hLocExtSig.nCRC, sizeof(HEADERLOCEXT));
		}
		
		// Comparer l'en-tte local avec l'en-tte central
		// (sauf les bits rservs des flags gnraux
		// et les entres dj compares)
		
		// Sauver les flags du central et masquer les bits rservs
		pb = (BYTE*)&pzf->m_cen.nVerE;
#define phCen   ((HEADERCEN*)(pb-cenVerE))
		nFlags = phCen->nFlags;
		pb[cenFlag-cenVerE+1] &= 0x1F;
		abBuf[locFlag     +1] &= 0x1F;
		
		// Terminer l'initialisation de l'entre
		pzf->m_cen.nTailleN  = phCen->nTailleN;
		pzf->m_cen.nTailleC  = phCen->nTailleC;
		pzf->m_cen.nCRC      = phCen->nCRC;
		pzf->SetUnixDateM(phCen->dt);
		pzf->m_cen.nMethode  = phCen->nMethode;
		pzf->m_cen.nFlags    = (WORD)nFlags;
		pzf->m_cen.nVerE     = phCen->nVerE;
#undef phCen
	}
	
	
#if 0  // TODO Grer la rparation
	if (m_offDebut) {
		// m_nReparer==0  m_offDebut octets trouvs avant le dbut
		// m_nReparer==1 - Ajustement des offsets car dcalage de m_offDebut octets au dbut
	}
#endif
	
	// Ajouter les fichiers dans leurs dossiers
	TCHAR pszNom[MAX_PATH*2];
	const TCHAR *pcNom;
	pos = lstTrouve.GetPosDeb();
	while (pos) {
		pzf = lstTrouve.GetSuiv(pos);
		
		// Copier le nom et convertir les '\' en '/'
		lstrcpyn(pszNom, pzf->m_pszNom, nbArray(pszNom));
		for (pcNom=pszNom; *pcNom; pcNom++)
			if (*pcNom=='\\')
				*(LPTSTR)pcNom = '/';
		
		// Traiter sparment les rpertoires
		lenNom = pzf->m_cen.lenNom;
		if (pszNom[lenNom-1]=='/') {
			CDossier *const pDossier = GetDossier(pszNom, gdCreer);
			pDossier->m_dtM = pzf->m_dtM;
			
			delete pzf;
			continue;
		}
		
		// Rcuprer le dernier sparateur '/'
		// pour ne garder que le nom du fichier dans pzf->m_pszNom
		pcNom = pszNom+lenNom;
		do{
			pcNom--;
		}while (pcNom>=pszNom && *pcNom!='/');
		delete [] pzf->m_pszNom;
		pzf->m_pszNom = newDup(pcNom+1);
		
		// Rcuprer le dossier correspondant au chemin d'accs
		// pour y ajouter le fichier
		pzf->m_dwTailleC = pzf->m_cen.nTailleC;
		pzf->m_dwTailleN = pzf->m_cen.nTailleN;
		pzf->m_dwAttr    = pzf->m_cen.nAttrE;
		pzf->MAJ();
		pzf->m_pArc      = this;
		(pzf->m_pDossier = GetDossier(pszNom, gdCreer, true))->m_lstFichier.AddFin(pzf);
	}
	lstTrouve.RemoveAll();
	
	return NULL;
	
#undef phFin
#undef phLocSig
#undef phLoc
#undef phLocExtSig
#undef phCenSig
}

// Ouvrir l'archive en mode rparation (vrification minimale)
CMuzipException* CZipArc::OuvrirReparer(CLstZipFichier& lstTrouve)
{
#define phLocSig      ((HEADERLOCSIG*)   abBuf)
#define phLoc         ((HEADERLOC*)      abBuf)
#define phLocExtSig   ((HEADERLOCEXTSIG*)abBuf)
#define phCenSig      ((HEADERCENSIG*)   abBuf)
	
	
	BYTE abBuf[sizeof(HEADERCENSIG)];
	DWORD len;
	
	BYTE b;
	
	DWORD off = 0;
	m_offDebut = 0;
	
	// Trouver le dbut des structures Zip
	// 'P' est le 1er caractre commun  toutes les signatures
	m_fr.SetVerbeux(false);
	for (;;) {
		while (m_fr.Read(b) && b!='P')  off++;
		abBuf[0] = 'P';
		if (!m_fr.Read(abBuf+1, lenSig-1))
			break;
		
#define sig  len
		sig = *(DWORD*)abBuf;
		if (sig==sigLoc || sig==sigFin)
			break;
#undef sig
		
		if (!m_fr.Seek(-3, FILE_CURRENT)) {
ErreurFileRead:
			return new CMuzipException(m_fr);
		}
		off++;
	}
	
	m_offDebut = off;
	
	// Lire les en-ttes locaux
	TCHAR pszNom[MAX_PATH*2];
	const TCHAR *pcNom;
	
	while (*(DWORD*)abBuf==sigLoc) {
		if (!m_fr.Read(abBuf, sizeof(HEADERLOC)))
			break;
		
		bool fLireSig = true;
		CZipFichier *const pzf = new CZipFichier;
		lstTrouve.AddFin(pzf);
		
		*(HEADERLOC*)pzf->m_cen = *phLoc;
		pzf->m_cen.nVerC   = verCur;
		pzf->m_cen.nFlags  = 0;
		pzf->SetUnixDateM(pzf->m_cen.dt);
		pzf->m_fDOS        = false;
		pzf->m_lenExtra    = pzf->m_cen.lenExtraC;
		
		pzf->m_cen.lenCom  = 0;
		pzf->m_cen.nDisque = 0;
		pzf->m_offCentral  = 0;
		
		// Ne pas prsumer de la taille compresse
		len = (e_opt.m_nReparer<repAltere) ? pzf->m_cen.nTailleC : 0;
		
		// Tout mettre  NULL
		pzf->m_pszNom   =
		pzf->m_pszCom   = NULL;
		pzf->m_pbExtra  =
		pzf->m_pbExtraC = NULL;
		
		// Lire le nom du fichier
		WORD lenNom = pzf->m_cen.lenNom;
		if (!lenNom) {
			// Nom vide
ErreurFormatFR:
			return new CMuzipException(ERR_REPARER);
		}
		pzf->m_pszNom = FromZip(lenNom, pzf->m_cen.nVerC, false);
		if (!pzf->m_pszNom)
			goto ErreurFileRead;
		
		// Lire le champ extra
		if (pzf->m_lenExtra) {
			pzf->m_pbExtra = new BYTE[pzf->m_lenExtra];
			if (!m_fr.Read(pzf->m_pbExtra, pzf->m_lenExtra))
				goto ErreurFileRead;
		}
		
		// Ignorer les commentaires
		
		// S'il y a un en-tte local tendu,
		// len est soit zro soit la taille compresse correcte
		pzf->m_pbExtraC = pzf->m_pbExtra;
		
		// Sauver l'offset, mettre  jour pour l'en-tte suivant
		pzf->m_cen.off = off;
		off += sizeof(HEADERLOCSIG) + lenNom + pzf->m_lenExtra + len;
		
		if (len && !m_fr.Seek(len, FILE_CURRENT))
			goto ErreurFileRead;
		
		// Sauter l'en-tte local tendu s'il y en a un
		if (pzf->m_nFlags & locfEtendu) {
			// Sauter les donnes compresses si la taille compresse est inconnue
			// Pour plus de scurit, on devrait utiliser le central
			if (!len || m_fr.Tell()!=off) {
				for (;;) {
					
					while (m_fr.Read(b) && b!='P')
						;
					
					abBuf[0] = b;
					if (!m_fr.Read(abBuf+1, 15) ||
					    *(DWORD*)abBuf==sigLocExt)
						break;
					
					if (!m_fr.Seek(-15L, FILE_CURRENT))
						goto ErreurFileRead;
				}
				len = phLocExtSig->nTailleC;
				off += len;
				if (m_fr.Tell()!=off+lenEtendu)
					goto ErreurFormatFR;
				
			}else{
				// Taille compresse non-nulle, assumer qu'elle est valide
				
				if (!m_fr.Seek(off, FILE_BEGIN) ||
				    !m_fr.Read(abBuf, lenEtendu))
					goto ErreurFileRead;
				if (phLocExtSig->sig==sigLocExt)
					goto ApresLocExt;
			}
			
			// Rcuprer les valeurs inconnues de l'en-tte local tendu
			pzf->m_cen.nCRC     = phLocExtSig->nCRC;
			pzf->m_cen.nTailleC = len;
			pzf->m_cen.nTailleN = phLocExtSig->nTailleN;
			
			off += lenEtendu;
			
		}else if (e_opt.m_nReparer==repAltere) {
			// Ne pas prsumer de la taille compresse
			
			for (;;) {
				
				while (m_fr.Read(b) && b!='P')
					;
				abBuf[0] = b;
				
				if (!m_fr.Read(abBuf+1, 3) ||
				    *(DWORD*)abBuf==sigLoc)
					break;
				
				if (!m_fr.Seek(-3, FILE_CURRENT))
					goto ErreurFileRead;
				off++;
			}
			
			len = off - (pzf->m_cen.off + sizeof(HEADERLOCSIG) + lenNom + pzf->m_lenExtra);
			if (len!=pzf->m_cen.nTailleC)
				// Taille compresse corrige
				pzf->m_cen.nTailleC = len;
			
			// Prochain sigLoc juste l, ne pas le relire
			fLireSig = false;
		}
	ApresLocExt:
		
		// Copier le nom et convertir les '\' en '/'
		lstrcpyn(pszNom, pzf->m_pszNom, nbArray(pszNom));
		for (pcNom=pszNom; *pcNom; pcNom++)
			if (*pcNom=='\\')
				*(LPTSTR)pcNom = '/';
		
		// Supprimer les rpertoires
		if (pszNom[lenNom-1]=='/') {
			delete pzf;
			goto Suite;
		}
		
		// Rcuprer le dernier sparateur '/'
		// pour ne garder que le nom du fichier dans pzf->m_pszNom
		pcNom = pszNom+lenNom;
		while (pcNom-- > pszNom && *pcNom!='/')
			;
		delete [] pzf->m_pszNom;
		pzf->m_pszNom = newDup(pcNom+1);
		
		// Rcuprer le dossier correspondant au chemin d'accs
		// pour y ajouter le fichier
		pzf->m_dwTailleC = len;
		pzf->m_dwTailleN = pzf->m_cen.nTailleN;
		pzf->m_dwAttr    = 0;
		pzf->MAJ();
		pzf->m_pArc      = this;
		(pzf->m_pDossier = GetDossier(pszNom, gdCreer, true))->m_lstFichier.AddFin(pzf);
		
		// Le fichier est dans m_lstFichier: il sera supprim en cas d'exception
		// On DOIT le retirer de lstTrouve pour ne pas le supprimer deux fois
		lstTrouve.RemoveAll();
		
Suite:
		
		// Lire la signature suivante
		if (fLireSig && !m_fr.Read((DWORD&)*abBuf))
			break;
	}
	
	// Sauver la position du central
	m_offCen = off;
	
#if 0
	if (phCen->sig!=sigCen)   // Troncature
	if (m_offDebut)           // Ajustement de m_offDebut octets au dbut du fichier
#endif
	
	return NULL;
	
#undef phLocExtSig
}

// Fermer le document
void CZipArc::CloseDocument()
{
	CZArc::CloseDocument();
	
	delete [] m_pszCom;
	m_pszCom = NULL;
}


// Ajouter un fichier  la liste
CFichier* CZipArc::NewFichier(LPCTSTR pszFichier, const WIN32_FIND_DATA& wfd)
{
	// Ajouter le fichier  la liste du Zip
	CZipFichier *const pzf = new CZipFichier;
	
	pzf->NewFichier(pszFichier, wfd);
	pzf->m_cen.nTailleC = pzf->m_dwTailleC;
	pzf->m_cen.nTailleN = pzf->m_dwTailleN;
	pzf->m_pArc     = this;
	pzf->m_pDossier = this;
	pzf->m_fDOS     = false;
	pzf->m_cen.dt   = pzf->m_dtM;
	
	return pzf;
}


// Ajouter des fichiers  l'archive
bool CZipArc::ArchivePlus()
{
	bool fOK = true;
	
	ShowStatusAvanceFichier();
	
	UINT iFichier = 0;
	const UINT nbFichier = s_lstFichierPlus.GetNb();
	POS pos = s_lstFichierPlus.GetPosDeb();
	while (pos) {
		CFichierPlus *const pFichierPlus = s_lstFichierPlus.GetSuiv(pos);
		CZipFichier  *const pzf = (CZipFichier*)pFichierPlus->m_pFichier;
		oApp.SetProgress(STA_ARCHIVEPLUS, pzf->m_pszNomES,
			iFichier++, nbFichier);
		
		pzf->m_lenExtra      =
		pzf->m_cen.lenExtraC =
		pzf->m_cen.lenCom    = 0;
		
		PRGSTATUS prs = prsOK;
		
		// Si le fichier est l'archive elle-mme,
		// afficher un message d'erreur et passer au fichier suivant
		const DWORD offOld = e_lenEcrit;
		if (!pzf->CompresseArchive()) {
			pzf->m_pbExtra  =
			pzf->m_pbExtraC = NULL;
			
			// Compresser le fichier
			if (pzf->m_fFautComp && !Compresser(pFichierPlus->m_ppi, pzf))
				prs = ErreurArchivePlus();
		}
		
		if (!ProgressArchivePlus(fOK, pzf, prs, pos, offOld))
			break;
	}
	SetEndOfFile(e_fw);
	
	return fOK;
}


// Ignorer un fichier
void CZipArc::IgnorerFichier(DWORD offOld, CFichier* pFichier)
{
	e_lenEcrit = offOld;
	CMuzipArc::IgnorerFichier(offOld, pFichier);
}


// Convertir l'archive  un autre format
// On gre la conversion rapide d'auto-extractible en Zip
// avec conversion individuelle des fichiers
bool CZipArc::Convertir(CMuzipArc* pArc, bool fConvertirFichiers)
{
	if (!(    m_pmdt->GetIdType()==IDR_SFXTYPE &&
	    pArc->m_pmdt->GetIdType()==IDR_ZIPTYPE &&
	    fConvertirFichiers))
		// Autre type de conversion: appliquer la mthode gnrale
		return CMuzipArc::Convertir(pArc, fConvertirFichiers);
	
	// Lire l'en-tte de l'auto-extractible
	SFXFOOTER sfx;
	VERIF(ReadSfxFooter(sfx));
	
	// Ouvrir l'archive en mode criture,
	// et y recopier l'intgralit de l'archive Zip
	VERIF(pArc->WriteOpen());
	m_fr.Seek(m_offDebut, FILE_BEGIN);
	const bool fOK = e_fw.WriteFrom(m_fr, sfx.lenArchive);
	pArc->WriteClose();
	
	return fOK;
}



// Dfinir les commentaires
void CZipArc::SetComment(LPCTSTR psz)
{
	// Recopier les commentaires
	m_lenCom = (WORD)lstrlen(psz);
	delete [] m_pszCom;
	if (m_lenCom) {
		m_pszCom = new TCHAR[m_lenCom+1];
		lstrcpy(m_pszCom, psz);
	}else
		m_pszCom = NULL;
	
	// Mettre  jour
	FautReconstruire();
}

// Retourner les commentaires
LPCTSTR CZipArc::GetComment() const
{
	return m_pszCom;
}


// Initialiser la cration d'une archive temporaire
bool CZipArc::OpenArchiveTemp()
{
	e_lenEcrit = 0;
	return CZArc::OpenArchiveTemp();
}


// crire le rpertoire central du Zip
// et remplacer l'ancien Zip par le fichier temporaire
bool CZipArc::CloseArchiveTemp()
{
	// crire le central
	CFichier::s_nbFichier = GetNbFichier();
	CFichier::s_nFichier  = 0;
	oApp.SetMessageText(STA_PUTCENTRAL);
	oApp.ShowStatusAvance(CFichier::s_nbFichier);
	oApp.SetStatusLabel(IDCLBL_2, STA2_FICHIER);
	bool fOK = PutCentral(this, putLocalDossier);
	const DWORD offDeb = e_lenEcrit;
	if (fOK)
		fOK =
			PutCentral(this, putCentralDossier) &&
			PutCentral(this, putCentralFichier);
	oApp.HideStatusAvance();
	if (!fOK) {
Erreur:
		CFichier::s_nbFichier = 0;
		e_fw.Close();
		return false;
	}
	
	// crire le rpertoire final
	HEADERFINSIG headerFin;
	headerFin.sig      = sigFin;
	headerFin.nDisque  =
	headerFin.nDisqueD = 0;
	headerFin.nbEntry  =
	headerFin.nbEntryT = (WORD)(CFichier::s_nbFichier + GetNbDossier());
	headerFin.nTailleT = e_lenEcrit-offDeb;
	headerFin.off      = offDeb;
	headerFin.lenCom   = m_lenCom;
	CFichier::s_nbFichier = 0;
	if (!e_fw.Write(&headerFin, sizeof(headerFin)) ||
	    !e_fw.Write(m_pszCom,   m_lenCom))
		goto Erreur;
	
	return CZArc::CloseArchiveTemp();
}



// Compresser l'entre pzf dans le fichier pf
// Une "entre zip" comporte un en-tte local, un en-tte de cryptage
// (si ncessaire), les donnes compresses, et ventuellement
// un en-tte local tendu
// Met dans e_lenEcrit le nombre d'octets crits
bool CZipArc::Compresser(CPlusItem* ppi, CZipFichier* pzf)
{
	CMyFindFile ff(pzf->m_pszNomES);
	if (ff.IsError()) {
		CMuzipException(ERR_OPEN, pzf->m_pszNomES).ReportError();
		return false;
	}
	
	// Rcuprer les attributs Dos
	DWORD dwAttr = (ff.m_wfd.dwFileAttributes & FILE_ATTRIBUTE_READONLY)
		? _S_IFREG | _S_IREAD
		: _S_IFREG | _S_IREAD | _S_IWRITE;
	{
		LPCTSTR pszExt = PathFindExtension(pzf->m_pszNomES);
		if (pszExt) {
			pszExt++;
			if (!lstrcmpi(pszExt, _T("exe")) ||
			    !lstrcmpi(pszExt, _T("cmd")) ||
			    !lstrcmpi(pszExt, _T("bat")) ||
			    !lstrcmpi(pszExt, _T("com")))
				dwAttr |= _S_IEXEC;
		}
	}
	dwAttr |= (dwAttr & 0700)>>3;
	dwAttr |= (dwAttr & 0700)>>6;
	dwAttr  = (dwAttr<<16) | ff.m_wfd.dwFileAttributes;
	
	// Convertir la date au format Dos
	DWORD dtDosM;
	FILETIME ft;
	FileTimeToLocalFileTime(&ff.m_wfd.ftLastWriteTime, &ft);
	FileTimeToDosDateTime(&ft, (WORD*)&dtDosM+1, (WORD*)&dtDosM);
	SYSTEMTIME st;
	FileTimeToSystemTime(&ft, &st);
	
	pzf->m_cen.nAttrI = attInconnu;   // Changera plus tard
	pzf->m_cen.nAttrE = 0;            // Peut tre modifi
	
	// Librer les anciens champs extra probablement obsoltes
	if (pzf->m_lenExtra)
		delete [] pzf->m_pbExtra;
	if (pzf->m_cen.lenExtraC && pzf->m_pbExtra!=pzf->m_pbExtraC)
		delete [] pzf->m_pbExtraC;
	pzf->m_lenExtra      =
	pzf->m_cen.lenExtraC = 0;
	
	// Choisir la mthode en se basant sur le suffixe du fichier
	// et la mthode choisie par l'utilisateur
	ZMETHODE nMethode;
	if (ppi->CO_nNiveau==0)
		nMethode = zmStore;
	else{
		// Regarder si le fichier a une extension  ne pas compresser
		
		TCHAR pszExt[_MAX_EXT];
		lstrcpy(pszExt, PathFindExtension(pzf->m_pszNomES));
		if (*pszExt) {
			CharLower(pszExt);
			CLstCString2 &lst = e_opt.m_lstPasCompExt;
			POS pos = lst.GetPosDeb();
			while (pos) {
				if (lst.GetSuiv(pos)==pszExt) {
					nMethode = zmStore;
					goto FinMethode;
				}
			}
		}
		nMethode = zmDeflate;
	}
FinMethode:
	
	CFileRead fr;
	VERIF(fr.Open(pzf->m_pszNomES, FILE_SHARE_ALL));
	pzf->m_cen.dt = dtDosM;
	pzf->m_dtM    = ff.m_wfd.ftLastWriteTime;
	
	if (!ff.m_wfd.nFileSizeLow)
		nMethode = zmStore;
	
	// Remplir les informations de l'en-tte et crire l'en-tte local
	// Cet en-tte sera rcrit plus tard car la longueur compresse
	// et le CRC sont inconnus  ce stade
	
	pzf->m_cen.nVerC = verCur;
	
	// PKZIP 1.0 (mthode Store) ou 2.0 (Deflate) requis pour extraction
	pzf->m_cen.nVerE = (nMethode==zmStore) ? (WORD)10 : (WORD)20;
	pzf->m_cen.nCRC  = 0;   // Mis  jour plus tard
	
	e_fCrypte = !m_sMdp.IsEmpty();
	
	// Assumer d'abord que l'on a besoin d'un en-tte local tendu
	pzf->m_cen.nFlags = locfEtendu;  // Mis  jour plus tard
	if (e_fCrypte) {
		pzf->m_cen.nFlags |= locfCrypte;
		// Comme on ne connat pas le CRC ici,
		// on prend la date de modification  la place
		pzf->m_cen.nCRC = pzf->m_cen.dt<<16;
	}
	pzf->m_nFlags       = pzf->m_cen.nFlags;
	pzf->m_cen.nMethode = (WORD)nMethode;             // Peut tre chang plus tard
	pzf->m_dwTailleC    =
	pzf->m_cen.nTailleC = (nMethode==zmStore)   // Chang plus tard
		? ff.m_wfd.nFileSizeLow : 0;
	pzf->m_dwTailleN    =
	pzf->m_cen.nTailleN = ff.m_wfd.nFileSizeLow;     // Chang plus tard
	pzf->m_cen.nDisque  = 0;
	const bool fType = (pzf->m_cen.nAttrI==attInconnu);
	if (fType)
		pzf->m_cen.nAttrI = attBinaire;
	
	pzf->m_cen.nAttrE  = (pzf->m_fDOS) ? dwAttr & 0xFF : dwAttr | (pzf->m_cen.nAttrE & 0xFF00);
	pzf->m_cen.off     = e_lenEcrit;
	VERIF(pzf->PutLocal());
	e_lenEcrit += sizeof(HEADERLOCSIG)+pzf->m_cen.lenNom+pzf->m_lenExtra;
	
	if (fType && nMethode==zmDeflate)
		pzf->m_cen.nAttrI = attInconnu; // Sera modifi dans Deflate() plus tard
	
	pzf->m_dwTailleC = 0;
	if (e_fCrypte) {
		cryptEnTete(m_sMdp, pzf->m_cen.nCRC);
		pzf->m_dwTailleC  = lenCryptEnTete;  // Mettre  jour plus tard
		e_lenEcrit       += lenCryptEnTete;
	}
	
	pzf->m_cen.nFlags |= e_aConfig[ppi->CO_nNiveau].vit;
	
	DWORD lenC = CZArc::Compresser(pzf, fr,
		ppi->CO_nNiveau,&nMethode, &pzf->m_cen.nAttrI);
	
	fr.Close();
	
	// Vrifier la taille d'entre (annulation ou erreur d'criture)
	if ((!lenC && ff.m_wfd.nFileSizeLow>0) || e_lenIn!=ff.m_wfd.nFileSizeLow) {
		e_lenEcrit = e_fw.Tell();
		return false;
	}
	
	e_lenEcrit += lenC;
	
	DWORD off2 = e_lenEcrit; // Garder pour Seek() futur
	ASSERT(e_lenEcrit == e_fw.Tell());
	
	// Essayer de rcrir l'en-tte local avec les informations correctes
	pzf->m_cen.nCRC      = s_nCRCIn;
	pzf->m_dwTailleC    += lenC;
	pzf->m_cen.nTailleC  = pzf->m_dwTailleC;
	pzf->m_dwTailleN     =
	pzf->m_cen.nTailleN  = e_lenIn;
	
	if (!e_fw.Seek(pzf->m_cen.off, FILE_BEGIN)) {
		pzf->PutEtendu();
		e_lenEcrit += lenEtendu;
		pzf->m_cen.nFlags = pzf->m_nFlags;  // Si m_cen.nFlags modifi par inflate
		
	}else{
		ASSERT(e_fw.Tell() == pzf->m_cen.off);
		
		// Vrifier la taille compresse
		pzf->m_cen.nMethode = (WORD)nMethode;
		pzf->m_cen.nVerE = (nMethode==zmStore) ? (WORD)10 : (WORD)20;   // PKUNZIP 2.0 requis pour Store
		// RAZ le flag d'en-tte local tendu
		if (!(pzf->m_cen.nFlags & locfCrypte))
			pzf->m_cen.nFlags &= ~locfEtendu;
		pzf->m_nFlags = pzf->m_cen.nFlags;
		// Rcrire l'en-tte local
		VERIF(pzf->PutLocal());
		e_fw.Seek(off2, FILE_BEGIN);
		if (pzf->m_cen.nFlags & locfEtendu) {
			// Fichier crypt, en-tte tendu toujours ncessaire
			pzf->PutEtendu();
			e_lenEcrit += lenEtendu;
		}
	}
	
	// Mettre  jour CFichier
	pzf->MAJ();
	
	return true;
}


// crire le central de tous les fichiers du dossier
bool CZipArc::PutCentral(CDossier* pDossier, PUTCENTRAL put)
{
	if (put!=putCentralFichier) {
		// Dossier
		
		if (pDossier!=pDossier->m_pArc) {
			// Ignorer la racine
			
			CString sDossier = pDossier->m_pszPath;
			sDossier += '/';
			
			HEADERCEN cen;
			ZeroMemory(&cen, sizeof(cen));
			cen.nVerC = verCur;
			cen.nVerE = (WORD)10;
			cen.dt    = CFichier::GetUnixDate(pDossier->m_dtM);
			const LPSTR aszNom = cen.GetZipNom(sDossier);
			
			bool fOK = true;
			if (put==putLocalDossier) {
				// Local
				
				pDossier->m_offLoc = e_lenEcrit;
				fOK = e_fw.Write(sigLoc) && e_fw.Write((HEADERLOC*)cen, sizeof(HEADERLOC));
				e_lenEcrit += sizeof(HEADERLOC);
				
			}else /* (put==putCentralDossier) */ {
				// Central
				
				cen.off = pDossier->m_offLoc;
				fOK = e_fw.Write(sigCen) && e_fw.Write(&cen, sizeof(cen));
				e_lenEcrit += sizeof(cen);
			}
			
			e_lenEcrit += sizeof(DWORD);   // Signature
			
			if (fOK) {
				fOK = e_fw.Write(aszNom, cen.lenNom);
				e_lenEcrit += cen.lenNom;
			}
			delete [] aszNom;
			
			VERIF(fOK);
		}
		
	}else{
		// Fichier
		
		POS pos = pDossier->m_lstFichier.GetPosDeb();
		while (pos) {
			CZipFichier *const pzf =
				(CZipFichier*)pDossier->m_lstFichier.GetSuiv(pos);
			if (pzf->m_fIgnorer)
				continue;
			
			oApp.SetProgress(STA_TERMINER, pzf->m_pszNom,
				CFichier::s_nFichier++, CFichier::s_nbFichier);
			
			VERIF(oApp.UpdateProgressStatus()!=prsAnnuler);
			
			pzf->m_offCentral = e_lenEcrit;
			VERIF(pzf->PutCentral());
			e_lenEcrit += sizeof(HEADERCENSIG)+pzf->m_cen.lenNom+pzf->m_cen.lenExtraC+pzf->m_cen.lenCom;
			
			oApp.SetStatusAvance(CFichier::s_nFichier);
		}
	}
	
	
	// Poursuivre dans les sous-dossiers
	POS pos = pDossier->m_lstDossier.GetPosDeb();
	while (pos)
		VERIF(PutCentral(pDossier->m_lstDossier.GetSuiv(pos), put));
	
	return true;
}


// crire l'en-tte local de pzf dans un fichier
bool CZipFichier::PutLocal()
{
	VERIF(e_fw.Write(sigLoc));
	
	bool fOK = false;
	const LPSTR aszNom = GetZipNom();
	
	// crire les donnes  longueur fixe
	const WORD nFlagsOld = m_cen.nFlags;
	m_cen.nFlags = m_nFlags;
	if (!e_fw.Write((HEADERLOC*)m_cen, loclenExtra-locVerE))
		goto Fin;
	m_cen.nFlags = nFlagsOld;
	
	// crire donnes  longueur variable
	fOK =
		e_fw.Write(m_lenExtra) &&
		e_fw.Write(  aszNom,  m_cen.lenNom) &&
		e_fw.Write(m_pbExtra, m_lenExtra);
	
Fin:
	delete [] aszNom;
	
	return fOK;
}

// crire l'en-tte local tendu de pzf dans le fichier pf
void CZipFichier::PutEtendu() const
{
	const HEADERLOCEXTSIG hLocExt =
	{
		sigLocExt,
		m_cen.nCRC,
		m_dwTailleC,
		m_dwTailleN,
	};
	e_fw.Write(&hLocExt, sizeof(hLocExt));
}

// crire l'en-tte central de pzf dans le fichier pf
bool CZipFichier::PutCentral()
{
	VERIF(e_fw.Write(sigCen));
	
	bool fOK = false;
	const LPSTR aszNom = GetZipNom();
	
	// crire les donnes  longueur fixe
	const WORD nFlagsOld = m_cen.nFlags;
	m_cen.nFlags = m_nFlags;
	if (!e_fw.Write(&m_cen, sizeof(HEADERCEN)))
		goto Fin;
	m_cen.nFlags = nFlagsOld;
	
	// crire donnes  longueur variable
	fOK =
		e_fw.Write(  aszNom,   m_cen.lenNom) &&
		e_fw.Write(m_pbExtraC, m_cen.lenExtraC) &&
		e_fw.Write(m_pszCom,   m_cen.lenCom);
	
	
Fin:
	
	delete [] aszNom;
	
	return fOK;
}


// Convertit une chane  partir du fichier ouvert (m_fr)
// Utilise la plate-forme et la version spcifies
// pour dterminer si la chane est stocke en OEM
LPTSTR CZipArc::FromZip(WORD& len, WORD ver, bool fPasOEM)
{
	const LPTSTR psz = new TCHAR[len+1];
	
#ifdef UNICODE
	char *const pszTemp = new char[len+1];
	if (!m_fr.Read(pszTemp, len)) {
#else
	if (!m_fr.Read(psz, len)) {
#endif
		delete [] psz;
		len = 0;
		return NULL;
	}
	
	const BYTE bOS  = VER_GETOS (ver);
	const BYTE bNum = VER_GETNUM(ver);
	
#ifdef UNICODE
	pszTemp[len] = 0;
#else
	psz[len] = 0;
#endif
	
	if ((bOS==ziposFAT &&
	    !(fPasOEM && (bNum==25 || bNum==26 || bNum==40))) ||
	    (bOS==ziposNTFS && bNum==50) ||
	     bOS==ziposHPFS)
#ifdef UNICODE
		// Convertir  partir d'OEM
		OemToChar(pszTemp, psz);
	else
		// Convertir en TCHAR
		strfromA(psz, pszTemp, len+1);
	
	delete [] pszTemp;
	
#else
		// Convertir  partir d'OEM
		OemToChar(psz, psz);
#endif
	
	return psz;
}

// Convertit le nom du fichier au format Zip
// Utilise la plate-forme et la version spcifies
// pour dterminer si la chane doit tre stocke en OEM
// Attention ! sNom sera modifi par la fonction !
// La fonction modifie len en y mettant la longueur de la chane retourne
LPSTR HEADERCEN::GetZipNom(CString& sNom)
{
	// Convertir le chemin d'accs: transformer les \ en /
	sNom.Replace('\\', '/');
	
	// Convertir la chane en ANSI
	this->lenNom = (WORD)sNom.GetLength();
	const LPSTR aszNom = new CHAR[this->lenNom+1];
	strtoA(aszNom, sNom, this->lenNom+1);
	
	
	const BYTE bOS  = VER_GETOS (this->nVerC);
	const BYTE bNum = VER_GETNUM(this->nVerC);
	
	if ((bOS==ziposFAT &&
	    !(!HIWORD(nAttrE) && (bNum==25 || bNum==26 || bNum==40))) ||
	    (bOS==ziposNTFS && bNum==50) ||
	     bOS==ziposHPFS)
		// Convertir en OEM
		CharToOemA(aszNom, aszNom);
	
	return aszNom;
}


// Mme chose que la fonction prcdente mais en non-statique (pour les fichiers)
LPSTR CZipFichier::GetZipNom()
{
	// Convertir le chemin d'accs: transformer les \ en /
	CString sNom = m_pDossier->m_pszPath;
	if (!sNom.IsEmpty())
		sNom += '/';
	sNom += m_pszNom;
	
	return m_cen.GetZipNom(sNom);
}
