// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// dlgAjouter.cpp
//
// * Objectif:
// - Muzip doit permettre l'ajout de dossiers en marquant leur nom
// - On doit utiliser CFileDialog pour la bote de dialogue d'ajout
//   car on ne veut pas rinventer la roue
//
// * Expos du problme:
//   Dans CFileDialog, quand on marque un nom de dossier et qu'on clique
//   sur OK, la bote de dialogue ne se ferme pas en validant le nom du
//   dossier comme on le voudrait, mais le dossier en question est ouvert
//
// * Solution:
// - OFN_NOVALIDATE permet de court-circuiter la gestion du bouton OK
//   par Windows qui se contente d'appeler OnFileNameOK() pour nous
//   demander si la spcification est valide
// - Quand l'utilisateur clique sur OK:
//   - si la zone o on tape le nom du fichier est vide,
//     alors on ajoute le dossier tout entier (pas de spcification)
//   - si elle n'est pas vide alors on ajoute la spcification entre
//     applique au dossier courant
// - Consquence: il faut vider la zone du nom de fichier  chaque fois
//   que l'utilisateur change de dossier


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "AppResource.h"
#include "dlgAjouter.h"

#include "Options.h"
#include "PlusItem.h"

#include "IOFichier.h"

#include <GUnicode.h>
#include <Dlgs.h>


BEGIN_MESSAGE_MAP(CDlgAjouter, CFileDialog)
	//{{AFX_MSG_MAP(CDlgAjouter)
	ON_COMMAND(IDCCMD_AJOUTERDOSSIER, OnAjouterDossier)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


// Constructeur
CDlgAjouter::CDlgAjouter(CPlusItem* ppi, LPCTSTR pszArchive)
	: CFileDialogEx(TRUE, NULL,NULL, fdFlagsCommon | OFN_NOVALIDATE | OFN_ALLOWMULTISELECT),
	  m_ppi(ppi)
{
	m_wndParent.m_pFileDialog = this;
	
	const LPCTSTR pszFiltre = m_ppi->DO_sFiltre;
	
	// Charger le titre
	m_sTitle.LoadString(IDS_AJOUTER);
	m_ofn.lpstrTitle = m_sTitle;
	
	// Charger le template
	SetTemplate(0, IDD_AJOUTER);
	
	
	// Dfinir la spcification initiale
	// Ce n'est pas si simple car le format de CFileDialog est compliqu:
	// - si un seul fichier, c'est facile (pas de guillemets)
	// - si plusieurs fichiers, il faut ajouter les guillemets et les espaces
	//
	if (!*pszFiltre) {
		// Aucune spcification: *.* par dfaut
		
		// Dossier par dfaut
		m_sInit = _T("*.*");
		m_ofn.lpstrInitialDir = new TCHAR[MAX_PATH];
		if (!e_opt.GetDossier(repAjout, (LPTSTR)m_ofn.lpstrInitialDir, pszArchive))
			*(LPTSTR)m_ofn.lpstrInitialDir = 0;
		
	}else{
		// Former la spcification
		
		CString sDebut;
		CLstCString2 lstPartie;
		m_ppi->ListerPartie(sDebut, lstPartie);
		
		// Enregistrer le rpertoire commun
		m_ofn.lpstrInitialDir = newDup(sDebut);
		
		if (lstPartie.GetNb()==1)
			// Une seule partie: tout simple, pas de guillemets
			
			m_sInit = PathFindFileName(lstPartie.GetDeb());
			
		else if (lstPartie.GetNb()>0) {
			// Plusieurs parties, entoures de guillemets
			// et spares par des espaces
			
			const int lenDebut = sDebut.GetLength();
			POS pos = lstPartie.GetPosDeb();
			for (;;) {
				
				// Copier la partie, en enlevant la partie commune
				// et si ncessaire en ajoutant des guillemets
				m_sInit += MyPathQuoteSpacesCopy(
					(LPCTSTR)lstPartie.GetSuiv(pos) + lenDebut);
				
				if (!pos)
					break;
				
				// Rajouter un sparateur
				m_sInit += ' ';
			}
		}
	}
}

// Destructeur
CDlgAjouter::~CDlgAjouter()
{
	// Nettoyer le rpertoire initial
	delete [] (LPTSTR)m_ofn.lpstrInitialDir;
	m_ofn.lpstrInitialDir = NULL;
}



// Initialisation
void CDlgAjouter::OnInitDone()
{
	CWnd *const pwndParent = GetParent();
	
	// Redimensionner le template et le bouton "Ajouter dossier"
	//  la taille du bouton "Annuler"
	CWnd *const pwndAnnuler = pwndParent->GetDlgItem(IDCANCEL);
	if (pwndAnnuler) {
		
		// Rcuprer la taille du template et du bouton "Annuler"
		CRect rcTemplate;
		GetClientRect(rcTemplate);
		CRect rcAnnuler;
		pwndAnnuler->GetClientRect(rcAnnuler);
		
		// Redimensionner le template puis le bouton
		SetWindowPos(NULL, 0,0, rcAnnuler.right,rcTemplate.bottom, SWP_NOZORDER | SWP_NOMOVE);
		GetDlgItem(IDCCMD_AJOUTERDOSSIER)->
			SetWindowPos(NULL, 0,0, rcAnnuler.right,rcAnnuler.bottom, SWP_NOZORDER | SWP_NOMOVE);
	}
	
	
	// Sous-classer la fentre parent (voir au dbut de ce fichier)
	// On doit le faire APRS avoir redimensionn le bouton "Annuler"
	// car CWndFileDialogParent sauvegarde la largeur du template
	m_wndParent.SubclassWindow(*pwndParent);
	
	
	// Spcification initiale
	SetControlText(edt1,  (LPCSTR)(LPCTSTR)m_sInit);
	SetControlText(cmb13, (LPCSTR)(LPCTSTR)m_sInit);
	
	// Redfinir le label du bouton OK en Ajouter
	// et masquer la liste droulante du type de fichier ainsi que son label
	// Il y a un bug dans la dfinition de SetControlText:
	// le paramtre chane est de type LPCTSTR et non LPCSTR
	SetControlText(IDOK, (LPCSTR)(LPCTSTR)CStringRes(IDS_CMDAJOUTER));
	HideControl(stc2);
	HideControl(cmb1);
}


// Ajouter le dossier courant (on ignore la spcification)
void CDlgAjouter::OnAjouterDossier()
{
	SetFiltre(GetFolderPath());
	
	// Forcer la fermeture de la bote de dialogue
	((CDialog&)m_wndParent).EndDialog(IDOK);
}


// Valider la spcification
// Attention ! 
// return FALSE --> OK
// return TRUE  --> erreur
BOOL CDlgAjouter::OnFileNameOK()
{
	const int bufSpec = 64 * MAX_PATH;
	
	// Rcuprer la spcification
	CString sSpec;
	if (m_wndParent.SendMessage(CDM_GETSPEC,
			(WPARAM)bufSpec, (LPARAM)sSpec.GetBuffer(bufSpec)) < 0)
		// Erreur
		return TRUE;
	
	
	//*** Vrifier la validit de la recherche, partie par partie
	
	// Transformer les ';' en espaces, avec gestion des guillemets
	// S'il y a au moins un ';', alors on ajoute des guillemets
	// autour de chaque partie
	// Sinon l'espace devient le sparateur et on n'ajoute aucun guillemet
	//
	// On a donc  la fin:
	// sSpec   = spcification telle que donne par l'utilisateur
	// sSpecOK = spcification corrige (';' devenus espaces)
	//
	CString sSpecOK;
	if (sSpec.Find(';')>=0) {
		CTokenize tok(sSpec, ';');
		while (tok.Next()) {
			if (!sSpecOK.IsEmpty())
				sSpecOK += ' ';
			
			// On ajoute la partie spare par un espace
			// et entoure de guillemets
			sSpecOK += MyPathQuoteSpacesCopy(tok.GetToken());
		}
	}else
		sSpecOK = sSpec;
	
	
	// sFiltre contiendra la spcification finale:
	// chaque partie sera un chemin d'accs absolu de chaque partie
	CString sFiltre;
	
	CTokenize tok(sSpec);
	while (tok.Next()) {
		
		// Supprimer les espaces autour de la partie
		tok.GetToken().TrimLeft();
		tok.GetToken().TrimRight();
		
		// Rcuprer le chemin d'accs absolu de la partie
		TCHAR pszPartie[MAX_PATH];
		if (!MyGetFullPathName(tok.GetToken(), pszPartie)) {
			// Spcification incorrecte
Erreur:
			AfxMessageBox(ERR_AJOUTERSPEC);
			return TRUE;
		}
		
		// Vrifier la validit de la spcification
		if (CMyFindFile(pszPartie).IsError()) {
			const DWORD dwErreur = GetLastError();
			if (dwErreur!=ERROR_FILE_NOT_FOUND && dwErreur!=ERROR_PATH_NOT_FOUND)
				goto Erreur;
		}
		
		// Rajouter le sparateur si ncessaire
		if (!sFiltre.IsEmpty())
			sFiltre += cSepPartie;
		
		// Rajouter la partie, "tendue"  la spcification complte
		PathUnquoteSpaces(pszPartie);
		sFiltre += pszPartie;
	}
	sSpec.ReleaseBuffer();
	
	
	// Mmoriser le filtre
	SetFiltre(sFiltre);
	
	
	// OK, tout va bien !
	return FALSE;
}


// Mmoriser un filtre
void CDlgAjouter::SetFiltre(LPCTSTR pszFiltre)
{
	// Rcuprer le dossier courant
	const CString sDossier = GetFolderPath();
	
	// Si aucune spcification n'est fournie,
	// utiliser le dossier lui-mme comme spcification
	m_ppi->DO_sFiltre = (*pszFiltre) ? pszFiltre : sDossier;
	
	// Mmoriser le dossier comme tant le dernier choisi
	// si on est en mode "dernier dossier"
	e_opt.SetDossierDernier(repAjout, sDossier);
}

