// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// dlgLister.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "AppResource.h"
#include "dlgLister.h"

#include <FileBuf.h>
#include "FileDialogEx.h"

#include "Options.h"
#include "MuzipArc.h"
#include "Fichier.h"
#include "Dossier.h"


enum{ alignLeft, alignCenter, alignRight };

static const LPCTSTR apszAlign[] =
{
	_T(""),
	_T(" ALIGN=center"),
	_T(" ALIGN=right"),
};

static const int aAlign[] =
{
	alignLeft,   alignLeft,  alignLeft,
	alignRight,  alignRight,
	alignCenter, alignCenter,
	alignLeft,
};



BEGIN_MESSAGE_MAP(CDlgLister, CDialogAide)
	//{{AFX_MSG_MAP(CDlgLister)
	ON_CBN_SELCHANGE(IDCCBO_TYPE, OnTypeSelChange)
	ON_BN_CLICKED   (IDCCHK_NOM,  OnChkNom)
	ON_BN_CLICKED   (IDCCHK_PATH, OnChkPath)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


// Constructeur
CDlgLister::CDlgLister(CMuzipArc* pArc, CWnd* pParent)
	: CDialogAide(IDD, pParent), m_pArc(pArc)
{
	//{{AFX_DATA_INIT(CDlgLister)
	m_nType    = 0;
	m_fColonne = TRUE;
	m_fNom     = TRUE;
	m_fPath    = TRUE;
	m_fType    = TRUE;
	m_fTailleN = TRUE;
	m_fTailleC = TRUE;
	m_fPC      = TRUE;
	m_fDtModif = TRUE;
	m_fAttr    = TRUE;
	//}}AFX_DATA_INIT
}


// Contrles
void CDlgLister::DoDataExchange(CDataExchange* pDX)
{
	CDialogAide::DoDataExchange(pDX);
	
	//{{AFX_DATA_MAP(CDlgLister)
	DDX_CBIndex(pDX, IDCCBO_TYPE,    m_nType);
	DDX_Check  (pDX, IDCCHK_COLONNE, m_fColonne);
	DDX_Check  (pDX, IDCCHK_NOM,     m_fNom);
	DDX_Check  (pDX, IDCCHK_PATH,    m_fPath);
	DDX_Check  (pDX, IDCCHK_TYPE,    m_fType);
	DDX_Check  (pDX, IDCCHK_TAILLEN, m_fTailleN);
	DDX_Check  (pDX, IDCCHK_TAILLEC, m_fTailleC);
	DDX_Check  (pDX, IDCCHK_PC,      m_fPC);
	DDX_Check  (pDX, IDCCHK_DTMODIF, m_fDtModif);
	DDX_Check  (pDX, IDCCHK_ATTR,    m_fAttr);
	//}}AFX_DATA_MAP
	
	if (!pDX->m_bSaveAndValidate)
		OnTypeSelChange();
}


// Nouveau format
void CDlgLister::OnTypeSelChange()
{
	const bool fPeutInfo = !IsQueUneSeuleColonne(((CComboBox*)GetDlgItem(IDCCBO_TYPE))->GetCurSel());
	
	for (int nID=IDCCHK_COLONNE; nID<=IDCCHK_ATTR; nID++)
		::EnableWindow(::GetDlgItem(*this, nID), fPeutInfo);
	
	MajChk(IDCCHK_NOM, IDCCHK_PATH);
}


// Nom/chemin d'accs
void CDlgLister::OnChkNom()
{
	MajChk(IDCCHK_NOM, IDCCHK_PATH);
}
void CDlgLister::OnChkPath()
{
	MajChk(IDCCHK_PATH, IDCCHK_NOM);
}
void CDlgLister::MajChk(int idChkClic, int idChkAutre)
{
	const bool fClic  = ToBool(IsDlgButtonChecked(idChkClic));
	const bool fAutre = ToBool(IsDlgButtonChecked(idChkAutre));
	
	if (IsQueUneSeuleColonne(((CComboBox*)GetDlgItem(IDCCBO_TYPE))->GetCurSel()))
		// Arborescence et liste: une seule case permise
		CheckDlgButton(idChkAutre, !fClic);
		
	else if (!fClic && !fAutre)
		// Autre: au moins une des deux cases
		CheckDlgButton(idChkAutre, TRUE);
}


// OK
void CDlgLister::OnOK()
{
#define VERIFOUT(f)  if (!(f))  return
	
	// Dcocher les cases dsactives
	if (IsQueUneSeuleColonne(((CComboBox*)GetDlgItem(IDCCBO_TYPE))->GetCurSel())) {
		for (int nID=IDCCHK_COLONNE; nID<=IDCCHK_ATTR; nID++)
			CheckDlgButton(nID, FALSE);
	}
	VERIFOUT(UpdateData());
	
	// Initialiser la bote de dialogue d'exportation
	// avec la bonne extension par dfaut et le bon filtre
	LPCTSTR pszDefExt;
	int     idsFiltre;
	if      (m_nType<=ltHTMLListe) {
		pszDefExt = _T("htm");
		idsFiltre = IDS_FILTREHTML;
	}else if (m_nType<=ltTexteTab) {
		pszDefExt = _T("txt");
		idsFiltre = IDS_FILTRETXT;
	}else /* (m_nType==ltCSV)*/ {
		pszDefExt = _T("csv");
		idsFiltre = IDS_FILTRECSV;
	}
	CFileDialogEx dlgExporter(FALSE, pszDefExt, NULL, fdFlagsSave,
		CStringRes(idsFiltre), this);
	VERIFOUT(dlgExporter.DoModal()!=IDCANCEL);
	
	int nCol;
	
	
	// Initialiser les tableaux des colonnes
	const CString sCols = MAKEINTRESOURCE(IDS_COL_LISTER);
	m_nbColF = m_nbColD = 0;
	for (nCol=0; nCol<maxCol; nCol++) {
		if ((&m_fNom)[nCol]) {
			if (nCol<2)
				m_nbColD++;
			m_aCol[m_nbColF++] = nCol;
			AfxExtractSubString(m_asCol[nCol], sCols, nCol, '\n');
		}
	}
	
	
	//*** Initialiser l'exportation
	VERIFOUT(m_fw.Open(dlgExporter.m_ofn.lpstrFile));
	
	// Rcuprer le sparateur
	if      (m_nType==ltTexteTab)  m_cSep = '\t';   // Tabul
	else if (m_nType==ltCSV)       m_cSep = ';';    // CSV
	
	// En-tte
	if (m_nType<=ltHTMLListe) {
		// HTML
		VERIFOUT(m_fw.Write(
			_T("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\"")
			_T("\"http://www.w3.org/TR/html4/strict.dtd\">\r\n")));
	}
	
	if (m_nType==ltHTMLTableau) {
		// crire la liste des colonnes avec leur alignement: <COLGROUP>
		
		VERIFOUT(m_fw.Write(_T("<TABLE>\r\n<COLGROUP>\r\n")));
		
		for (nCol=0; nCol<m_nbColF; nCol++) {
			// En-tte
			CString s;
			s.Format(_T("\t<COL%s>\r\n"), apszAlign[aAlign[m_aCol[nCol]]]);
			VERIFOUT(m_fw.Write(s));
		}
		
		VERIFOUT(m_fw.Write(_T("<THEAD>\r\n")));
	}
	
	// Liste des champs
	if (m_fColonne) {
		
		if (m_nType==ltTexteFixe) {
			// Prendre en compte les colonnes de l'en-tte
			
			for (nCol=0; nCol<m_nbColF; nCol++) {
				const int iCol = m_aCol[nCol];
				m_alenCol[iCol] = m_asCol[iCol].GetLength();
			}
			
			// Initialiser avec les largeurs des colonnes des autres lignes
			m_nType = ltTexteFixeInit;
			Lister(m_pArc);
			m_nType = ltTexteFixe;
			
		}
		
		VERIFOUT(AddDeb(0));
		
		// crire l'en-tte des colonnes
		for (nCol=0; nCol<m_nbColF; nCol++) {
			const int iCol = m_aCol[nCol];
			VERIFOUT(AddSep(iCol,nCol,false, m_asCol[iCol], true));
		}
		
		if (m_nType==ltHTMLTableau)
			VERIFOUT(m_fw.Write(_T("<TBODY>\r\n")));
		
	}else if (m_nType==ltTexteFixe) {
		
		// Initialiser avec les largeurs des colonnes des autres lignes
		m_nType = ltTexteFixeInit;
		Lister(m_pArc);
		m_nType = ltTexteFixe;
	}
	
	// Effectuer l'exportation
	VERIFOUT(Lister(m_pArc));
	
	// Pied de page
	if (m_nType==ltHTMLTableau)
		VERIFOUT(m_fw.Write(_T("</TABLE>")));
	
	// OK
	m_fw.Close();
	EndDialog(IDOK);
	
#undef VERIFOUT
}


// Ajouter un dbut de ligne
bool CDlgLister::AddDeb(int nProf)
{
	if (m_nType==ltTexteFixeInit)
		return true;
	
	if (m_nType<=ltHTMLTableau)
		VERIF(m_fw.Write(_T("<TR>\r\n")));
	
	// Arborescence: insrer le dcalage  gauche
	if (m_nType==ltHTMLArbo || m_nType==ltTexteArbo) {
		const LPCTSTR pszSep = (m_nType==ltHTMLArbo)
			? _T("&nbsp   &nbsp") : _T("  ");
		for (int nbReste=nProf; nbReste; nbReste--)
			VERIF(m_fw.Write(pszSep));
	}
	return true;
}


// Lister un dossier
bool CDlgLister::Lister(const CDossier* pDossier, int nProf)
{
	POS pos;
	
	if (pDossier!=pDossier->m_pArc) {
		// Afficher le nom du dossier
		
		AddDeb(nProf++);
		
		int nCol = 0;
		
		// Nom
		if (m_fNom)
			VERIF(AddSep(0,nCol++,true, pDossier->m_pszNom));
		
		// Chemin d'accs
		if (m_fPath)
			VERIF(AddSep(1,nCol++,true, pDossier->m_pszPath));
	}
	
	// Lister les fichiers
	pos = pDossier->m_lstFichier.GetPosDeb();
	while (pos) {
		AddDeb(nProf);
		VERIF(Lister(pDossier->m_lstFichier.GetSuiv(pos)));
	}
	
	// Poursuivre dans les sous-dossiers
	pos = pDossier->m_lstDossier.GetPosDeb();
	while (pos)
		VERIF(Lister(pDossier->m_lstDossier.GetSuiv(pos), nProf));
	
	return true;
}


// Lister un fichier
bool CDlgLister::Lister(const CFichier* pFichier)
{
	int nCol = 0;
	TCHAR psz[MAX_PATH];
	
	// Nom
	if (m_fNom)
		VERIF(AddSep(0,nCol++,false, pFichier->m_pszNom));
	
	// Chemin d'accs
	if (m_fPath) {
		PathCombine(psz, pFichier->m_pDossier->m_pszPath, pFichier->m_pszNom);
		VERIF(AddSep(1,nCol++,false, psz));
	}
	
	// Type
	if (m_fType) {
		SHFILEINFO shfi;
		SHGetFileInfo(pFichier->m_pszNom, 0, &shfi, sizeof(shfi),
			SHGFI_USEFILEATTRIBUTES | SHGFI_TYPENAME);
		VERIF(AddSep(2,nCol++,false, shfi.szTypeName));
	}
	
	// Taille
	if (m_fTailleN) {
		StrFormatByteSize(pFichier->m_dwTailleN, psz, nbArray(psz));
		VERIF(AddSep(3,nCol++,false, psz));
	}
	
	// Compress
	if (m_fTailleC) {
		StrFormatByteSize(pFichier->m_dwTailleC, psz, nbArray(psz));
		VERIF(AddSep(4,nCol++,false, psz));
	}
	
	// %
	if (m_fPC) {
		wsprintf(psz, _T("%hd %%"), pFichier->m_nPC);
		VERIF(AddSep(5,nCol++,false, psz));
	}
	
	// Modifi
	if (m_fDtModif) {
		SYSTEMTIME st = pFichier->m_dtM;
		*psz = 0;
		const int len = GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &st, NULL, psz, nbArray(psz));
		if (e_opt.m_fAffichageModifHeure) {
			// Rajouter l'heure  la date
			if (len)
				psz[len-1] = ' ';
			GetTimeFormat(LOCALE_USER_DEFAULT, 0, &st, NULL, &psz[len], nbArray(psz)-len);
		}
		VERIF(AddSep(6,nCol++,false, psz));
	}
	
	// Attributs
	if (m_fAttr)
		VERIF(AddSep(7,nCol++,false, pFichier->m_pszAttr));
	
	return true;
}


// Ajouter un sparateur aprs une colonne
bool CDlgLister::AddSep(int iCol, int nCol, bool fDossier, LPCTSTR psz, bool fHeader)
{
	const int nbCol = (fDossier) ? m_nbColD : m_nbColF;
	
	CString s;
	if (fDossier && m_nType<=ltHTMLListe)
		s.Format(_T("<B>%s</B>"), psz);
	else
		s = psz;
	
	
	if (m_nType==ltTexteFixeInit) {
		// Initialisation taille des colonnes en texte fixe
		const int len = s.GetLength();
		if (m_alenCol[iCol]<len)
			m_alenCol[iCol] = len;
		return true;
	}
	
	TCHAR pszOut[MAX_PATH+2];
	
	if (nCol>=nbCol-1) {
		// Dernire colonne
		
		if (m_nType==ltHTMLTableau)
			wsprintf(pszOut, _T("\t<T%c>%s\r\n"), (fHeader) ? 'H' : 'D', s);
		else{
			s.Format(_T("%s%s\r\n"), psz,
				(m_nType<=ltHTMLListe && m_nType!=ltHTMLTableau)
				? _T("<BR>") : _T(""));
			return m_fw.Write(s);
		}
		
	}else{
		// Autres colonnes
		
		if (m_nType==ltHTMLTableau)
			// HTML - tableau
			wsprintf(pszOut, _T("\t<T%c>%s  &nbsp;\r\n"), (fHeader) ? 'H' : 'D', s);
			
		else{
			
			// Texte
			if (m_nType==ltTexteFixe) {
				// Longueur fixe
				
				// Calculer le nombre d'espaces  ajouter  gauche et  droite
				const int nbManque = m_alenCol[iCol]-s.GetLength();
				int nbG;
				if      (aAlign[iCol]==alignLeft)     nbG = 0;
				else if (aAlign[iCol]==alignCenter)   nbG = nbManque/2;
				else /* (aAlign[iCol]==alignRight)*/  nbG = nbManque;
				int nbD = nbManque-nbG + 2;
				
				// Ajouter les espaces
				TCHAR *pc = pszOut;
				while (nbG>0) {
					nbG--;
					*pc++ = ' ';
				}
				while (*psz)
					*pc++ = *psz++;
				while (nbD>0) {
					nbD--;
					*pc++ = ' ';
				}
				*pc = 0;
				
			}else // (m_nType>=ltTexteTab)
				// Sparateur
				wsprintf(pszOut, _T("%s%c"), psz, m_cSep);
		}
	}
	
	m_fw.Write(pszOut);
	m_fw.Flush();
	return true;
}
