// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// dlgMacroGerer.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "Muzip2.h"
#include "dlgMacroGerer.h"

#include "dlgOptions.h"
#include "dlgMacroAction.h"

#include "Macro.h"

#include "Interf.h"
#include "MuzipArc.h"
#include "MuzipDoc.h"
#include "MuzipView.h"

#include "MacroDoc.h"
#include "MacroView.h"



enum MACROITEMTYPE
{
	mitArchive,
	mitMacro,
	mitAction,
};


// Plusieurs cas de figure:
//
// mit==mitArchive
//   L'item est une archive
//   pArc est valide
//
// mit==mitMacro
//   L'item est une macro, valide ou non
//   pmac valide pFichier==NULL
//     => la macro est valide et dans pmac
//   pmac==NULL et pFichier valide
//     => la macro est peut-tre invalide, elle est contenue dans une archive,
//        dans le fichier pFichier (fMacroDoc==false)
//
// mit==mitAction
//   L'item est une action appartenant  une macro valide
//   pact est valide
//   fMacroDoc peut tre gal  TRUE ou  FALSE

class CMacroItem
{
public:  // Fonctions
	
	CMacroItem(CDlgMacroGerer& dlg) : m_dlg(dlg)
	{
	}
	
	
	bool CacheMacro(CString& sFile);
	void OnMacroModified();
	
	CMacroDoc* OpenMacroDoc();
	bool       LoadMacro();
	
	
public:  // Donnes
	
	CDlgMacroGerer& m_dlg;
	
	
	HTREEITEM m_hti;
	
	MACROITEMTYPE m_mit;
	union
	{
		CMuzipArc  *m_pArc;   // Si mit==mitArchive, toujours valide
		CMacro     *m_pmac;   // Si mit==mitMacro (NULL si macro non charge)
		CMacAction *m_pact;   // Si mit==mitAction, toujours valide
	};
	
	
	bool m_fMacroDoc;   // TRUE si l'objet vient d'un document macro, FALSE sinon
	union
	{
		CMacroDoc *m_pDoc;
		CFichier  *m_pFichier;
	};
};


BEGIN_MESSAGE_MAP(CDlgMacroGerer, CDialogAide)
	//{{AFX_MSG_MAP(CDlgMacroGerer)
	ON_COMMAND(IDCCMD_MAC_EXECUTER,  OnExecuter)
	ON_COMMAND(IDCCMD_MAC_EDITER,    OnEditer)
	ON_COMMAND(IDCCMD_MAC_EDITERTXT, OnEditerTxt)
	ON_COMMAND(IDCCMD_MAC_LOCALISER, OnLocaliser)
	ON_COMMAND(IDCCMD_MAC_NOUVEAU,   OnNouveau)
	ON_COMMAND(IDCCMD_MAC_OPTIONS,   OnOptions)
	ON_NOTIFY(TVN_SELCHANGED,    IDCTRV_MACRO, OnTrvSelChanged)
	ON_NOTIFY(TVN_ITEMEXPANDING, IDCTRV_MACRO, OnTrvItemExpanding)
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


// Contrles
void CDlgMacroGerer::DoDataExchange(CDataExchange* pDX)
{
	CDialogAide::DoDataExchange(pDX);
	
	//{{AFX_DATA_MAP(CDlgMacroGerer)
	DDX_Control(pDX, IDCTRV_MACRO, m_trv);
	//}}AFX_DATA_MAP
}


// Initialisation
BOOL CDlgMacroGerer::OnInitDialog()
{
	CDialogAide::OnInitDialog();
	
	//*** Remplir l'arborescence des macros
	
	CMacroItem itemBase(*this);
	itemBase.m_pmac = NULL;
	
	// Macros des archives
	itemBase.m_fMacroDoc = false;
	POS pos = e_lstArc.GetPosDeb();
	while (pos) {
		CMuzipArc *const pArc = e_lstArc.GetSuiv(pos);
		CMuzipDoc *const pDoc = pArc->m_pi->GetDoc();
		if (!pDoc)
			// Il faut une archive visible
			continue;
		
		// Item de l'archive
		CMacroItem *const pItem = new CMacroItem(itemBase);
		pItem->m_mit  = mitArchive;
		pItem->m_pArc = pArc;
		TVINSERTSTRUCT tvis;
		tvis.item.mask    = TVIF_TEXT | TVIF_PARAM;
		tvis.hParent      = TVI_ROOT;
		tvis.hInsertAfter = TVI_SORT;
		tvis.item.pszText = (STRING)pDoc->GetTitle();
		tvis.item.lParam  = (LPARAM)pItem;
		pItem->m_hti = m_trv.InsertItem(&tvis);
		
		// Lister les macros de l'archive
		ListMacro(pItem->m_hti, pArc, itemBase);
	}
	
	
	// Macros ouvertes comme document
	POSITION pos2 = oApp.m_pmdtMacro->GetFirstDocPosition();
	itemBase.m_fMacroDoc = true;
	while (pos2) {
		itemBase.m_pDoc = DYNAMIC_DOWNCAST(CMacroDoc,
			oApp.m_pmdtMacro->GetNextDoc(pos2));
		
		if (itemBase.m_pDoc) {
			// Ajouter et lister la macro
			
			// Nom de la macro
			CMacroItem *const pItem = new CMacroItem(itemBase);
			pItem->m_pmac = &itemBase.m_pDoc->m_mac;
			ListMacro(AddMacro(TVI_ROOT, pItem->m_pmac->m_sName, pItem));
		}
	}
	
	
	return TRUE;
}


// Excuter
void CDlgMacroGerer::OnExecuter()
{
	CMacroItem *const pItem = GetSelItem();
	ASSERT(pItem);
	
	// Si c'est une macro, elle doit tre charge
	if (pItem->m_mit==mitMacro && !pItem->LoadMacro())
		return;
	
	// Fermer la bote de dialogue
	EndDialog(IDCANCEL);
	
	// Excuter la macro
	CMacContexte ctx;
	switch (pItem->m_mit) {
		
		// Macro
		case mitMacro:
			pItem->m_pmac->Executer(ctx);
			break;
		
		// Action
		case mitAction:
			pItem->m_pact->Executer(ctx);
			break;
	}
}

// diter
void CDlgMacroGerer::OnEditer()
{
	CMacroItem *const pItem = GetSelItem();
	ASSERT(pItem);
	
	
	switch (pItem->m_mit) {
		
		// Macro
		case mitMacro:
			if (pItem->LoadMacro() &&
			    pItem->m_pmac->Editer())
				pItem->OnMacroModified();
			break;
		
		// Action
		case mitAction:
			if (pItem->m_pact->Editer(true))
				pItem->OnMacroModified();
			break;
	}
}

// diter en texte
void CDlgMacroGerer::OnEditerTxt()
{
	// Fermer la bote de dialogue
	EndDialog(IDCANCEL);
	
	// diter la macro
	CMacroDoc *const pDoc = GetSelItem()->OpenMacroDoc();
	if (pDoc) {
		SetActiveDocument(pDoc);
		POSITION pos = pDoc->GetFirstViewPosition();
		if (pos)
			((CMacroView*)pDoc->GetNextView(pos))->EditMacroText();
	}
}

// Localiser
void CDlgMacroGerer::OnLocaliser()
{
	CMacroItem *const pItem = GetSelItem();
	ASSERT(pItem);
	
	EndDialog(IDCANCEL);
	
	
	switch (pItem->m_mit) {
		
		// Archive
		case mitArchive:
			SetActiveDocument(pItem->m_pArc->m_pi->GetDoc());
			break;
		
		// Macro ou action
		case mitMacro:
		case mitAction:
			if (pItem->m_fMacroDoc)
				SetActiveDocument(pItem->m_pDoc);
			else if (pItem->m_pFichier) {
				CFichier *const pFichier = pItem->m_pFichier;
				pFichier->m_pArc->m_pi->SelFichier(pFichier);
			}else
				SetActiveDocument(pItem->m_pArc->m_pi->GetDoc());
			break;
	}
}


// Nouvelle macro
void CDlgMacroGerer::OnNouveau()
{
	EndDialog(IDCANCEL);
	
	oApp.m_pmdtMacro->OpenDocumentFile(NULL);
}

// Options
void CDlgMacroGerer::OnOptions()
{
	CDlgOptions(pgOptionsMacro).DoModal();
}


// Ajouter les macros d'un dossier  l'arborescence
// Poursuivre la recherche dans les sous-dossiers
void CDlgMacroGerer::ListMacro(HTREEITEM htiParent, CDossier* pDossier, CMacroItem& itemBase)
{
	// Chercher les macros dans les fichiers
	CLstFichier lstFichierMacro;
	pDossier->ListMacro(lstFichierMacro);
	
	// Ajouter  l'arborescence les macros trouves
	POS pos = lstFichierMacro.GetPosDeb();
	while (pos) {
		CFichier *const pFichier = lstFichierMacro.GetSuiv(pos);
		
		// Lister le nom de la macro uniquement
		// Si l'utilisateur dcide de voir ses actions,
		// alors on dcompressera la macro et on la regardera de plus prs
		// Pour l'instant on ne sait pas s'il s'agit d'une macro valide
		CMacroItem *const pItem = new CMacroItem(itemBase);
		pItem->m_mit      = mitMacro;
		pItem->m_pFichier = pFichier;
		AddMacro(htiParent, pFichier->m_pszNom, pItem);
	}
}


// Ajouter une macro  l'arborescence
// La structure CMacroItem doit tre entirement remplie,
//  l'exception de CMacroItem::mit
HTREEITEM CDlgMacroGerer::AddMacro(HTREEITEM htiParent, LPCTSTR pszName, CMacroItem* pItem)
{
	TVINSERTSTRUCT tvis;
	tvis.item.mask = TVIF_TEXT | TVIF_PARAM | TVIF_CHILDREN;
	
	// C'est une macro
	pItem->m_mit = mitMacro;
	
	// Ajouter l'item
	tvis.hParent        = htiParent;
	tvis.hInsertAfter   = TVI_SORT;
	tvis.item.pszText   = (LPTSTR)pszName;
	tvis.item.lParam    = (LPARAM)pItem;
	tvis.item.cChildren = TRUE;   // Pour afficher le '+'
	pItem->m_hti = m_trv.InsertItem(&tvis);
	
	return pItem->m_hti;
}


// Lister le contenu d'une macro (si cela n'a pas dj t fait)
void CDlgMacroGerer::ListMacro(HTREEITEM htiMacro)
{
	// Si on a dj ajout les actions, ne pas le refaire
	if (m_trv.GetChildItem(htiMacro))
		return;
	
	
	CMacroItem *const pItem = (CMacroItem*)m_trv.GetItemData(htiMacro);
	ASSERT(pItem);
	
	// Il faut une macro
	if (pItem->m_mit!=mitMacro)
		return;
	
	
	// Charger la macro si ncessaire
	if (!pItem->LoadMacro())
		return;
	
	
	//*** Actions de la macro
	
	CMacroItem itemBase = *pItem;
	itemBase.m_mit = mitAction;
	
	TVINSERTSTRUCT tvis;
	tvis.item.mask    = TVIF_TEXT | TVIF_PARAM;
	tvis.hParent      = htiMacro;
	tvis.hInsertAfter = TVI_SORT;
	
	CMacro *const pmac = pItem->m_pmac;
	POS pos = pmac->GetPosDeb();
	while (pos) {
		CMacAction *const pact = pmac->GetSuiv(pos);
		if (pact->m_fHidden)
			continue;
		
		CMacroItem *const pItem = new CMacroItem(itemBase);
		pItem->m_pact = pact;
		CString sItem;
		pact->GetItem(sItem);
		
		// Ajouter l'item
		tvis.item.pszText = (STRING)sItem;
		tvis.item.lParam  = (LPARAM)pItem;
		pItem->m_hti = m_trv.InsertItem(&tvis);
	}
	
	// a y est, on a list la macro
	TVITEM tvi;
	tvi.mask = TVIF_CHILDREN;
	tvi.hItem = htiMacro;
	VERIFY(m_trv.GetItem(&tvi));
	tvi.cChildren = pmac->GetNb() > 0;
	m_trv.SetItem(&tvi);
}


// Arborescence - Changement de slection
void CDlgMacroGerer::OnTrvSelChanged(NMHDR* pNMHDR, LRESULT*)
{
	const CMacroItem *const pItem = GetSelItem();
	
	const bool fLocaliser = ToBool(((NMTREEVIEW*)pNMHDR)->itemNew.hItem);
	const bool fManip     = pItem->m_mit!=mitArchive;
	
	// Activer/dsactiver les boutons selon la slection:
	// - Localiser: tout objet
	// - Excuter, diter: macro ou action
	const UINT aid[] =
	{
		IDCCMD_MAC_EXECUTER,
		IDCCMD_MAC_EDITER,
		IDCCMD_MAC_EDITERTXT,
		IDCCMD_MAC_LOCALISER,
	};
	for (int i=0; i<nbArray(aid); i++)
		GetDlgItem(aid[i])->EnableWindow(
			(aid[i]==IDCCMD_MAC_LOCALISER) ? fLocaliser : fManip);
}

// Arborescence - Ouverture d'item
// Il faut grer le listage des actions des macros qui n'ont jamais t ouvertes
void CDlgMacroGerer::OnTrvItemExpanding(NMHDR* pNMHDR, LRESULT* pResult)
{
	NMTREEVIEW *const pNMTRV = (NMTREEVIEW*)pNMHDR;
	
	if (pNMTRV->action & TVE_EXPAND)
		// C'est bien une expansion
		
		ListMacro(pNMTRV->itemNew.hItem);
	
	
	// Permettre l'action
	*pResult = FALSE;
}


// Rcuprer l'item slectionn
CMacroItem* CDlgMacroGerer::GetSelItem()
{
	const HTREEITEM hti = m_trv.GetSelectedItem();
	return (hti) ? (CMacroItem*)m_trv.GetItemData(hti) : NULL;
}

// Rcuprer la macro slectionne
CMacro* CDlgMacroGerer::GetSelMacro()
{
	const CMacroItem *const pItem = GetSelItem();
	
	if (pItem) {
		switch (pItem->m_mit) {
			
			// Macro
			case mitMacro:
				return pItem->m_pmac;
			
			// Action
			case mitAction:
				return pItem->m_pact->GetMacro();
		}
	}
	
	return NULL;
}


// Destruction
void CDlgMacroGerer::OnDestroy()
{
	POS pos;
	
	// Recompresser les fichiers correspondant aux macros modifies
	// On cre une seule requte de compression par fichier,
	// mais on groupe les requtes par archive pour grer
	// la reconstruction manuelle
	
	pos = m_lstFichierModif.GetPosDeb();
	while (pos) {
		POS pos2 = pos;
		CMuzipArc *const pArc = m_lstFichierModif.GetSuiv(pos)->m_pArc;
		
		// Regarder si on tait en reconstruction manuelle
		const bool fOldRm = pArc->m_fRm;
		
		// On veut tous les fichiers appartenant  l'archive
		while (pos2) {
			CFichier *const pFichier = m_lstFichierModif.GetSuiv(pos2);
			if (pFichier->m_pArc!=pArc)
				// Pas la bonne archive
				continue;
			
			// Enregistrer la macro
			
			
			// Crer une requte d'ajout, en mode reconstruction manuelle
			CArchivePlus arPlus(pFichier->m_pDossier);
			arPlus.m_fAjoutRm = true;
			CPlusItem pi;
			pFichier->ToPlusItem(pi);
			pFichier->GetCacheFile(pi.DO_sFiltre);
			
			// Drfrencer le cache EN GARDANT le fichier du cache
			// mme si c'tait la dernire instance du cache
			pFichier->CacheReleaseNoDelete();
			
			arPlus.AddDeb(&pi);
			pArc->ArchivePlus(arPlus);
		}
		
		// Reconstruire l'archive si ncessaire
		if (!fOldRm)
			pArc->FautReconstruire(rmoAjouter);
	}
	
	
	
	// Nettoyer
	CleanChild(TVI_ROOT);
	
	CDialogAide::OnDestroy();
}

// Nettoyer un item et ses fils
void CDlgMacroGerer::Clean(HTREEITEM hti)
{
	if (!hti)
		return;
	
	// Nettoyer l'item
	if (hti!=TVI_ROOT)
		delete (CMacroItem*)m_trv.GetItemData(hti);
	
	// Nettoyer les fils
	CleanChild(hti);
	
	// Dtruire l'item
	m_trv.DeleteItem(hti);
}

// Nettoyer les fils d'un item
void CDlgMacroGerer::CleanChild(HTREEITEM hti)
{
	if (!hti)
		return;
	
	hti = m_trv.GetChildItem(hti);
	while (hti) {
		const HTREEITEM htiNext = m_trv.GetNextSiblingItem(hti);
		
		// Nettoyer l'item et ses fils
		Clean(hti);
		
		// On passe au frre
		hti = htiNext;
	}
}




//*** CMacroItem

// Mettre la macro dans le cache
// La fonction est suppose tre appele lorsque cela est ncessaire uniquement
bool CMacroItem::CacheMacro(CString& sFile)
{
	VERIF(m_pFichier->CacheAddRef(sFile));
	
	// Ajouter le fichier  la liste des fichiers modifis
	if (!m_dlg.m_lstFichierModif.Find(m_pFichier))
		m_dlg.m_lstFichierModif.AddFin(m_pFichier);
	
	return true;
}

// La macro a t modifie
// Si elle est dans le cache, alors il faut modifier le fichier du cache
// Attention: le pointeur 'this' n'est plus valide aprs l'appel de cette fonction
void CMacroItem::OnMacroModified()
{
	if (!m_fMacroDoc) {
		
		CMacro *pmacTemp = NULL;
		
		switch (m_mit) {
			
			// Macro
			case mitMacro:
				pmacTemp = m_pmac;
				break;
			
			// Action
			case mitAction:
				pmacTemp = m_pact->GetMacro();
				break;
		}
		
		ASSERT(pmacTemp);
		
		// crire la macro dans le fichier du cache
		CString s;
		pmacTemp->ToINI(s);
		CString sFile;
		m_pFichier->GetCacheFile(sFile);
		CFileWrite fw;
		if (fw.Open(sFile)) {
			fw.Write(s);
			fw.Close();
		}
	}
	
	
	// Recrer l'arborescence de l'item
	// - Pour les macros, on reconstruit la liste des actions
	//   mais on n'a pas besoin de recrer la macro elle-mme
	//   car dans l'arborescence on ne voit que le nom de la macro
	//   qui ne peut pas changer
	// - Pour les actions, on a besoin de recrer l'item correspondant  l'action
	//   car dans l'arborescence on voit aussi le nom complet qui a pu changer
	
	// On a besoin de copier *this car
	// CleanChild() supprime l'action courante si m_mit==mitAction
	CMacroItem itemCopy = *this;
	const HTREEITEM htiMacro = (m_mit==mitMacro)
		? m_hti : m_dlg.m_trv.GetParentItem(m_hti);
	m_dlg.CleanChild(htiMacro);
	
	switch (itemCopy.m_mit) {
		
		// Macro
		case mitMacro:
			itemCopy.m_dlg.ListMacro(htiMacro);
			break;
		
		// Action
		case mitAction:
			itemCopy.m_dlg.ListMacro(htiMacro);
			break;
	}
}


// Ouvrir le document contenant la macro
CMacroDoc* CMacroItem::OpenMacroDoc()
{
	if (m_fMacroDoc)
		// Macro dj ouverte
		return m_pDoc;
	
	
	// Macro dans l'archive
	CString sFile;
	if (CacheMacro(sFile)) {
		
		CDocument *const pDoc = pApp->OpenDocumentFile(sFile);
		CMacroDoc *const pDocMacro = DYNAMIC_DOWNCAST(CMacroDoc, pDoc);
		
		if (pDocMacro)
			return pDocMacro;
		
		pDoc->OnCloseDocument();
	}
	
	return NULL;
}


// Charger la macro
bool CMacroItem::LoadMacro()
{
	if (m_pmac)
		// Macro dj charge
		return true;
	
	
	// Dcompresser la macro
	ASSERT(m_pFichier);
	CString sFile;
	
	// Si erreur  la dcompression, il faut garder la macro dans la liste
	// au cas o l'erreur serait corrige par la suite
	VERIF(CacheMacro(sFile));
	
	// Charger la macro
	m_pmac = new CMacro;
	if (!m_pmac->FromINI(sFile)) {
		// La macro est invalide: il faut l'enlever de la liste
		
		// Supprimer l'item de l'arborescence puis nettoyer la mmoire
		m_dlg.m_trv.DeleteItem(m_hti);
		delete m_pmac;
		delete this;
		return false;
	}
	
	// La macro est valide
	m_dlg.m_lstMacroArc.AddFin(m_pmac);
	
	return true;
}
