// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// dlgOptions.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "dlgOptions.h"


BEGIN_MESSAGE_MAP(CDlgOptions, CDialog)
	//{{AFX_MSG_MAP(CDlgOptions)
	ON_NOTIFY(TVN_SELCHANGING, IDCTRV, OnTrvSelChanging)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


// Constructeur
CDlgOptions::CDlgOptions(int pgDebut)
	: CDialogAide(IDD), m_pgDebut(pgDebut)
{
	//{{AFX_DATA_INIT(CDlgOptions)
	//}}AFX_DATA_INIT
	
	m_fRegister = true;
	m_pPageCur  = NULL;
	
	// Ajouter les pages
	#define PageClasse(page)  m_aPage[pgOptions##page] = new CPgOptions##page;
	ManipulerPage(PageClasse)
	#undef PageClasse
}

// Destructeur
CDlgOptions::~CDlgOptions()
{
	for (int iPage=0; iPage<nbPageOptions; iPage++)
		delete m_aPage[iPage];
}


// Contrles
void CDlgOptions::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	
	//{{AFX_DATA_MAP(CDlgOptions)
	DDX_Control(pDX, IDCTRV,     m_trv);
	DDX_Control(pDX, IDCLBL_DLG, m_lblDlg);
	//}}AFX_DATA_MAP
}


// Initialisation
BOOL CDlgOptions::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	const CString sPages(MAKEINTRESOURCE(IDS_OPTIONS_PAGES));
	
	// Ajouter les pages
	// Rcuprer le handle de l'item correspondant  la page initiale
	HTREEITEM htiParent = NULL;
	HTREEITEM htiDebut  = NULL;
	CTokenize tok(IDS_OPTIONS_PAGES, '\n');
	while (tok.Next()) {
		
		// Prparer l'ajout d'un item dans l'arborescence
		TVINSERTSTRUCT tvis;
		tvis.hParent      = TVI_ROOT;
		tvis.hInsertAfter = TVI_LAST;
		if (tok.GetToken()[0]=='\t') {
			tvis.hParent    = htiParent;
			tok.GetToken().Delete(0);
		}
		
		tvis.item.mask    = TVIF_TEXT | TVIF_PARAM;
		tvis.item.pszText = (STRING)tok.GetToken();
		tvis.item.lParam  = tok.GetIndex();
		
		// Ajouter l'item
		const HTREEITEM hti = m_trv.InsertItem(&tvis);
		if (tvis.hParent==TVI_ROOT) 
			htiParent = hti;
		else
			m_trv.Expand(htiParent, TVE_EXPAND);
		
		if (tok.GetIndex()==m_pgDebut)
			// On a trouv la page de dpart
			htiDebut = hti;
	}
	
	m_trv.SelectItem(htiDebut);
	SetCurPage(m_pgDebut);
	
	return TRUE;
}


// Arborescence: nouvelle slection
void CDlgOptions::OnTrvSelChanging(NMHDR* pNMHDR, LRESULT* pResult)
{
	*pResult = !SetCurPage(((NMTREEVIEW*)pNMHDR)->itemNew.lParam);
}


// Afficher une nouvelle page d'options
bool CDlgOptions::SetCurPage(int iPage)
{
	// Si pas de changement de page, ne rien faire
	VERIF((iPage<0) ? ToBool(m_pPageCur) : m_pPageCur!=m_aPage[iPage]);
	
	// Supprimer l'ancienne page
	if (m_pPageCur) {
		VERIF(m_pPageCur->UpdateData());
		m_pPageCur->DestroyWindow();
	}
	
	// Rcuprer l'emplacement et la position des pages d'options
	CRect rcDlg;
	m_lblDlg.GetWindowRect(rcDlg);
	ScreenToClient(rcDlg);
	
	if (iPage<0)
		// Pas de nouvelle page
		m_pPageCur = NULL;
		
	else{
		
		// Afficher la nouvelle page
		m_pPageCur = m_aPage[iPage];
		m_pPageCur->Create(m_pPageCur->m_psp.pszTemplate, this);
		m_pPageCur->SetWindowPos(NULL,
			rcDlg.left,rcDlg.top,
			rcDlg.Width(),rcDlg.Height(),
			SWP_NOZORDER | SWP_SHOWWINDOW);
		
		// Lui donner un ID pour l'aide
		m_pPageCur->SetDlgCtrlID((UINT)m_pPageCur->m_psp.pszTemplate);
		
		// Redonner le focus  l'arborescence
		m_trv.SetFocus();
	}
	
	return true;
}


// Affichage en modal
int CDlgOptions::DoModal()
{
	POS pos;
	int i;
	
	CPgOptionsGeneral  *const ppgGeneral  = (CPgOptionsGeneral*) m_aPage[pgOptionsGeneral];
	CPgOptionsDossiers *const ppgDossiers = (CPgOptionsDossiers*)m_aPage[pgOptionsDossiers];
	CPgOptionsRm       *const ppgRm       = (CPgOptionsRm*)      m_aPage[pgOptionsRm];
	
	
	// Initialiser les botes de dialogue
	#define CopierMembres(page, type, prefixe, nom, defaut) \
		((CPgOptions##page*)m_aPage[pgOptions##page])->m_##prefixe##nom = \
			e_opt.m_##prefixe##page##nom;
	ManipulerMembres(CopierMembres)
	#undef CopierMembres
	
	// Copier les options des dossiers
	for (i=0; i<repTotal; i++)
		ppgDossiers->m_aDossier[i] = e_opt.m_aDossier[i];
	
	// Copier les options de la reconstruction manuelle
	memcpy(ppgRm->m_aRmOption, e_opt.m_aRmOption,
		sizeof(e_opt.m_aRmOption));
	
	// Copier les options des extensions par dfaut
	// Ne prendre que les formats ayant plusieurs extensions
	pos = oApp.m_lstDocType.GetPosDeb();
	while (pos) {
		CDocType &dt = *oApp.m_lstDocType.GetSuiv(pos);
		
		if (dt.IsMultiExt()) {
			EXTDEFAUT ed;
			ed.pdt        = &dt;
			ed.iExtDefaut = dt.m_iExtDefaut;
			ppgGeneral->m_lstExtDefaut.AddFin(ed);
		}
	}
	
	
	// Afficher la bote de dialogue
	const int nRet = CDialog::DoModal();
	if (nRet==IDOK) {
		// OK
		
		#define ChargerMembres(page, type, prefixe, nom, defaut) \
			e_opt.m_##prefixe##page##nom = \
				((CPgOptions##page*)m_aPage[pgOptions##page])->m_##prefixe##nom;
		ManipulerMembres(ChargerMembres)
		#undef ChargerMembres
		
		// Copier les options des dossiers
		for (i=0; i<repTotal; i++)
			e_opt.m_aDossier[i] = ppgDossiers->m_aDossier[i];
		
		// Copier les options de la reconstruction manuelle
		memcpy(e_opt.m_aRmOption, ppgRm->m_aRmOption,
			sizeof(e_opt.m_aRmOption));
		
		// Copier les options des extensions par dfaut
		pos = ppgGeneral->m_lstExtDefaut.GetPosDeb();
		while (pos) {
			EXTDEFAUT &ed = ppgGeneral->m_lstExtDefaut.GetSuiv(pos);
			ed.pdt->m_iExtDefaut = ed.iExtDefaut;
		}
		
		// Recrer la liste des fichiers rcents
		// avec la nouvelle taille
		oApp.RebuildRecentFileList();
		
		// Mettre  jour les options
		e_opt.MAJAffichageStyleEx();
		e_opt.MAJPasCompExt();
		e_opt.Appliquer();
		
		// Enregistrer Muzip et la DLL si demand
		// ("Dsenregistrer" dans l'onglet Intgration n'a pas t cliqu)
		if (m_fRegister) {
			oApp.Register();
			DllRegisterServer();
		}
	}
	
	return nRet;
}


// OK
void CDlgOptions::OnOK()
{
	// Valider la page courante pour ne pas perdre
	// les dernires modifications
	if (!m_pPageCur || m_pPageCur->UpdateData())
		CDialog::OnOK();
}
