// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// DocType.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "Muzip2.h"

#include "MuzipFrame.h"
#include "ArboView.h"

#include "Reg.h"

#include "Zip/ZipArc.h"
#include "GZip/GZipArc.h"
#include "Tar/TarArc.h"
#include "Uue/UueArc.h"

#include "MuzipDoc.h"
#include "Interf.h"


const LPCTSTR eszGeneralExt = _T("ExtDefaut");


//*** CDocType


// Ajouter les types de document
void CDocType::AddDocTypes()
{
	struct DOCTYPE
	{
		UINT              idType;
		PFN_CREATEARCHIVE pfnCreateArchive;
		int               cap;
	};
	
	static const DOCTYPE aDocType[] =
	{
		{
			IDR_ZIPTYPE, CZipArc::CreateArchive,
			capArbo | capdtM | capNiveau | capCommentaire |
			capAttribut | capCrypte | capRapideModifProp | capSfx |
			capDoitReconstruire
		},
		
		{
			IDR_GZIPTYPE, CGZipArc::CreateArchive,
			capMaxUnFichier | capdtM | capNiveau | capCommentaire |
			capRapideModifProp | capDoitReconstruire
		},
		
		{
			IDR_TARTYPE, CTarArc::CreateArchive,
			capArbo | capdtM | capRapideRenommer | capRapideModifProp |
			capDoitReconstruire
		},
		
		{
			IDR_UUETYPE, CUueArc::CreateArchive,
			capMaxUnFichier
		},
		
		{
			IDR_SFXTYPE, CZipArc::CreateArchive,
			capArbo | capdtM | capNiveau | capCommentaire |
			capAttribut | capCrypte | capRapideModifProp | capLectureSeule
		},
	};
	
	oApp.m_lstDocType.DeleteAll();
	for (int i=0; i<nbArray(aDocType); i++) {
		const DOCTYPE &rdt = aDocType[i];
		
		CDocType *const pdt = new CDocType;
		
		pdt->m_idType           = rdt.idType;
		pdt->m_prcDoc           = RUNTIME_CLASS(CMuzipDoc);
		pdt->m_pfnCreateArchive = rdt.pfnCreateArchive;
		pdt->m_cap              = rdt.cap;
		pdt->AddAssocs();
		pdt->LoadSaveOptions(true);
		oApp.m_lstDocType.AddFin(pdt);
	}
}


// Crer le modle de document associ au type de document
void CDocType::AddDocTemplate()
{
	// Crer le CMuzipDocTemplate
	m_pmdt =
		new CMuzipDocTemplate(this,
			m_idType, IDR_MUZIPTYPE,
			m_prcDoc,
			RUNTIME_CLASS(CMuzipFrame),
			RUNTIME_CLASS(CArboView));
	
	// L'enregistrer
	oApp.AddDocTemplate(m_pmdt);
}

// Ajouter les associations rattaches au type de document
void CDocType::AddAssocs()
{
	// S'occuper des extensions
	CString sFilterExt;
	GetDocString(sFilterExt, CDocTemplate::filterExt);
	
	// Rcuprer la liste des extensions
	CTokenize tok(sFilterExt, ';');
	while (tok.Next()) {
		
		// Crer l'association
		CAssoc *const pAssoc = new CAssoc;
		oApp.m_lstAssoc.AddFin(pAssoc);
		
		// Copier l'extension
		tok.GetToken().MakeLower();
		lstrcpy(pAssoc->m_pszExt, tok.GetToken());
		*pAssoc->m_pszOldProg = 0;
	}
}

// Charger ou enregistrer les options (extension par dfaut)
void CDocType::LoadSaveOptions(bool fLoad)
{
	if (!IsMultiExt())
		return;
	
	CString sFilterExt;
	GetDocString(sFilterExt, CDocTemplate::filterExt);
	
	// Composer le nom de la cl
	CString esz;
	AfxExtractSubString(esz, sFilterExt, 0, ';');
	esz = eszGeneralExt+esz;
	
	// Charger ou enregistrer la valeur, selon ce qui est demand
	if (fLoad)
		m_iExtDefaut = oApp.GetProfileInt(kszGeneral, esz, 0);
	else
		oApp.WriteProfileInt(kszGeneral, esz, m_iExtDefaut);
}


// Rcuprer une chane caractristique du document
bool CDocType::GetDocString(CString& s, int i) const
{
	return AfxExtractSubString(s, CStringRes(m_idType), i) && !s.IsEmpty();
}

// Retourne TRUE si le type de document a plusieurs extensions
bool CDocType::IsMultiExt() const
{
	CString sFilterExt;
	GetDocString(sFilterExt, CDocTemplate::filterExt);
	return sFilterExt.Find(';')>=0;
}

// Rcuprer l'extension par dfaut, avec un point
void CDocType::GetExtDefaut(CString& sExt) const
{
	// Rcuprer la liste des extensions
	CString sFilterExt;
	GetDocString(sFilterExt, CDocTemplate::filterExt);
	
	// Prendre l'extension par dfaut
	// En cas d'chec (ne devrait jamais arriver)
	// prendre la premire extension
	if (!AfxExtractSubString(sExt, sFilterExt, m_iExtDefaut, ';'))
		AfxExtractSubString(sExt, sFilterExt, 0, ';');
}


// Indiquer si le modle de document correspond  un fichier
// en grant les extensions multiples
CDocTemplate::Confidence CDocType::MatchDocType(
	LPCTSTR pszPathName, CDocument*& rpDocMatch) const
{
	ASSERT(ToBool(pszPathName));
	rpDocMatch = NULL;
	
	// Vrifier que le document n'est pas ouvert
	if (m_pmdt) {
		POSITION pos = m_pmdt->GetFirstDocPosition();
		while (pos) {
			CDocument *const pDoc = m_pmdt->GetNextDoc(pos);
			if (pDoc->GetPathName()==pszPathName) {
				// Dj ouvert
				rpDocMatch = pDoc;
				return CDocTemplate::Confidence::yesAlreadyOpen;
			}
		}
	}
	
	// Rcuprer la chane du filtre
	CString sFilterExt;
	if (!GetDocString(sFilterExt, CDocTemplate::filterExt))
		return CDocTemplate::Confidence::yesAttemptForeign;
	
	// Regarder si le fichier a une des extensions proposes
	const LPCTSTR pszExt = PathFindExtension(pszPathName);
	if (pszExt) {
		CTokenize tok(sFilterExt, ';');
		while (tok.Next()) {
			ASSERT(!tok.GetToken().IsEmpty() && tok.GetToken()[0]=='.');
			
			// Regarder si l'extension correspond
			if (!tok.GetToken().CompareNoCase(pszExt))
				return CDocTemplate::Confidence::yesAttemptNative;
		}
	}
	
	// Extension inconnue: essayer de deviner le type du fichier
	CMuzipArc *const pArc = CreateArchive(new CInterf);
	const bool bGoodType = pArc->IsArchiveGoodType(pszPathName);
	pArc->Release(false);
	
	// L'extension ne correspond  aucun template
	return (bGoodType)
		? CDocTemplate::Confidence::yesAttemptNative
		: CDocTemplate::Confidence::yesAttemptForeign;
}

// Crer une nouvelle archive
CMuzipArc* CDocType::CreateArchive(CInterf* pi) const
{
	CMuzipArc *const pArc = m_pfnCreateArchive(pi);
	
	if (pArc)
		pArc->m_pmdt = (CDocType*)this;
	
	return pArc;
}



//*** CAssoc


// Copier une association
CAssoc& CAssoc::operator = (CAssoc& rAssoc)
{
	ASSERT(!lstrcmp(m_pszExt, rAssoc.m_pszExt));
	
	lstrcpy(m_pszOldProg, rAssoc.m_pszOldProg);
	
	return *this;
}


// Rcuprer la cl de l'association et l'association courante
// (pszValeur doit avoir une taille de MAX_PATH caractres)
// Retourner TRUE si l'association courante est Muzip
bool CAssoc::GetAssocCur(HKEY& hKey, LPTSTR pszValeur)
{
	*pszValeur = 0;
	
	// Ouvrir la cl correspondant  l'association
	if (RegCreateKeyEx(HKEY_CLASSES_ROOT, m_pszExt, 0, NULL, 0,
			KEY_QUERY_VALUE | KEY_NOTIFY | KEY_SET_VALUE,
			NULL, &hKey, NULL)!=ERROR_SUCCESS)
		// Erreur
		hKey = NULL;
	else{
		// Lire la valeur par dfaut de la cl (l'association courante)
		DWORD dwType;
		DWORD dwLen = MAX_PATH;
		if (RegQueryValueEx(hKey, NULL, NULL, &dwType,
				(BYTE*)pszValeur, &dwLen)==ERROR_SUCCESS && dwType!=REG_SZ)
			*pszValeur = 0;
	}
	
	return !lstrcmpi(pszValeur, pszClasseMuzipArchive);
}


// Affecter ou dsaffecter l'extension  Muzip
void CAssoc::Affecter(bool fToMuzip, HKEY hKey, LPCTSTR pszValeur)
{
	if (fToMuzip) {
		// Sauvegarder l'ancienne valeur
		// et mettre pszClasseMuzipArchive  la place
		
		lstrcpy(m_pszOldProg, pszValeur);
		RegSetValueEx(hKey, NULL, NULL, REG_SZ,
			(const BYTE*)pszClasseMuzipArchive, lstrlen(pszClasseMuzipArchive)+1);
		
	}else{
		// Restaurer l'ancienne valeur dans le registre
		
		RegSetValueEx(hKey, NULL, NULL, REG_SZ,
			(const BYTE*)m_pszOldProg, lstrlen(m_pszOldProg)+1);
		*m_pszOldProg = 0;
	}
}



// Notifier Windows de changements concernant les extensions
void CAssoc::NotifyChange()
{
	SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, 0,0);
}
