// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// Exceptions.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "AppResource.h"
#include "Exceptions.h"

#include <FileBuf.h>


// Constructeurs
CMuzipException::CMuzipException(UINT nID) : m_p(NULL), m_s(MAKEINTRESOURCE(nID))
{
}
CMuzipException::CMuzipException(LPCTSTR pszFichier) : m_p(NULL)
{
	Init(ERR_OPEN, pszFichier);
}
CMuzipException::CMuzipException(UINT nID, LPCTSTR pszFichier) : m_p(NULL)
{
	Init(nID, pszFichier);
}
CMuzipException::CMuzipException(CFileRead& fr) : m_pfr(&fr)
{
	Init(ERR_READ, fr.GetPath());
}
CMuzipException::CMuzipException(CFileWrite& fw) : m_pfw(&fw)
{
	Init(ERR_WRITE, fw.GetPath());
}

// Initialiser l'exception
void CMuzipException::Init(UINT nID, LPCTSTR pszFichier)
{
	m_nID = nID;
	
	// Rcuprer le message d'erreur de Windows
	LPTSTR pszMessage;
	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
		NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPTSTR)&pszMessage, 0, NULL);
	
	// L'inclure dans le message d'erreur donn
	AfxFormatString2(m_s, nID, pszFichier, pszMessage);
	
	LocalFree(pszMessage);
}

// Retourner le message d'erreur
BOOL CMuzipException::GetErrorMessage(LPTSTR pszError, UINT nMaxError, UINT* pnHelpContext)
{
	lstrcpyn(pszError, m_s, nMaxError);
	if (pnHelpContext)  *pnHelpContext = 0;
	return TRUE;
}

// Afficher le message d'erreur
int CMuzipException::ReportError(UINT nType, UINT nMessageID)
{
	if (m_s.IsEmpty()) {
		switch (nType) {
			
			case MB_ABORTRETRYIGNORE:
				return IDIGNORE;
			
			case MB_RETRYCANCEL:
				return IDCANCEL;
			
			case MB_YESNO:
			case MB_YESNOCANCEL:
				return IDYES;
			
			default:
				return IDOK;
		}
	}
	
	// S'il s'agit d'un fichier, on peut ressayer
	if (m_p)
		nType = MB_RETRYCANCEL | MB_ICONEXCLAMATION;
	
	const int nRet = CException::ReportError(nType, nMessageID);
	
	if (nRet==IDCANCEL && m_p) {
		// On a rpondu Annuler: fermer le fichier ouvert
		if      (m_nID==ERR_READ)  { if (m_pfr->GetPeutFermer())  m_pfr->Close(); }
		else if (m_nID==ERR_WRITE) { if (m_pfw->GetPeutFermer())  m_pfw->Close(); }
		m_p = NULL;
	}
	
	return nRet;
}
