// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// IOFichier.cpp
//
// Des fonctions utilitaires pour grer les fichiers et les dossiers
// affichant les messages d'erreurs appropris quand il y a lieu


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "IOFichier.h"

#include "Options.h"
#include "dlgConfirm.h"
#include "MuzipDll/Resource.h"

#include <FileBuf.h>


static bool DossierSupprimerInterne(bool fQueVide, LPTSTR pszPath);


// Rcupre la partie suivante d'un chemin d'accs
bool GetPathPartieSuiv(LPCTSTR& pszPath, CString& sPart)
{
	VERIF(pszPath && *pszPath);
	while (*pszPath=='\\')  pszPath++;
	
	// Rcuprer la fin de la partie
	TCHAR *const pcPartieFin = PathFindNextComponent(pszPath);
	if (pcPartieFin && *pcPartieFin) {
		// Partie intermdiaire
		
		const int buf = pcPartieFin - pszPath;
		lstrcpyn(sPart.GetBuffer(buf), pszPath, buf);
		sPart.ReleaseBuffer();
		pszPath = pcPartieFin;
		
	}else{
		// Dernire partie
		
		sPart   = pszPath;
		pszPath = NULL;
	}
	
	return true;
}



// Crer un fichier en grant les messages d'erreur
// Le fichier n'est pas vid s'il existe
bool FichierCreer(LPCTSTR pszFichier)
{
	if (PathFileExists(pszFichier))
		return true;
	
	CFileWrite fw;
	VERIF(fw.Open(pszFichier));
	fw.Close();
	return true;
}

// Crer un fichier et demander confirmation en cas d'crasement
bool FichierEcraser(LPCTSTR pszFichier)
{
	if (PathFileExists(pszFichier)) {
		AfxFormatString1(e_sMsg, DEM_ECRASER, pszFichier);
		VERIF(CDlgConfirm::GetReponse(mcfEcraser, 0, cfmNonDefaut));
		SetFileAttributes(pszFichier, 0);
	}
	
	CFileWrite fw;
	VERIF(fw.Open(pszFichier));
	fw.Close();
	return true;
}



// Supprimer un fichier en grant les messages d'erreur
bool FichierSupprimer(LPCTSTR pszFichier, bool fPeutCorbeille)
{
	if (!PathFileExists(pszFichier))
		return true;
	
	// Rajouter un second nul final
	TCHAR pszFichier2[MAX_PATH+1];
	lstrcpy(pszFichier2, pszFichier);
	pszFichier2[lstrlen(pszFichier2)+1] = 0;
	
	// Supprimer le fichier
	SHFILEOPSTRUCT shfo;
	shfo.hwnd   = *oApp.m_pMainWnd;
	shfo.wFunc  = FO_DELETE;
	shfo.pFrom  = pszFichier2;
	shfo.pTo    = NULL;
	shfo.fFlags = (fPeutCorbeille && e_opt.m_fGeneralCorbeille)
		? (FILEOP_FLAGS)(FOF_SILENT | FOF_NOCONFIRMATION | FOF_ALLOWUNDO)
		: (FILEOP_FLAGS)(FOF_SILENT | FOF_NOCONFIRMATION);
	return !SHFileOperation(&shfo) && !shfo.fAnyOperationsAborted;
}

// Dplacer un fichier
// Supporte les changement de volumes
bool FichierDeplacer(LPCTSTR pszDe, LPCTSTR pszVers)
{
	// Rajouter un second nul final
	TCHAR pszDe2[MAX_PATH+1];
	lstrcpy(pszDe2, pszDe);
	pszDe2[lstrlen(pszDe2)+1] = 0;
	
	// Dplacer le fichier le fichier
	SHFILEOPSTRUCT shfo;
	shfo.hwnd   = *oApp.m_pMainWnd;
	shfo.wFunc  = FO_MOVE;
	shfo.pFrom  = pszDe2;
	shfo.pTo    = pszVers;
	shfo.fFlags = FOF_SILENT | FOF_NOERRORUI | FOF_NOCONFIRMATION;
	return !SHFileOperation(&shfo) && !shfo.fAnyOperationsAborted;
}


// Crer un dossier en grant les messages d'erreur
bool DossierCreer(LPCTSTR pszDossier, bool bVerbose)
{
	if (!pszDossier || !*pszDossier)
		return true;
	
	// Formatter le message d'erreur ventuel
	if (bVerbose)
		AfxFormatString1(e_sMsg, ERR_DOSSIERCREER, pszDossier);
	
	// Rcuprer le chemin d'accs complet
	TCHAR pszTemp[MAX_PATH];
	while (!MyGetFullPathName(pszDossier, pszTemp))
		VERIF(bVerbose && AfxMessageBox(e_sMsg, MB_RETRYCANCEL | MB_ICONEXCLAMATION) != IDCANCEL);
	
	
	// On va composer le chemin d'accs partie par partie
	TCHAR pszCur[MAX_PATH];
	lstrcpy(pszCur, pszTemp);
	PathStripToRoot(pszCur);
	
	// Pour chaque partie...
	CString sPartie;
	TCHAR *pc = pszTemp;
	while (GetPathPartieSuiv(pc, sPartie)) {
		PathAppend(pszCur, sPartie);
		
		// Si le rpertoire existe dj, poursuivre avec la partie suivante
		if (PathFileExists(pszCur))
			continue;
		
		// Crer la partie
		while (!CreateDirectory(pszCur, NULL) &&
		       GetLastError() != ERROR_ALREADY_EXISTS)
			VERIF(bVerbose && AfxMessageBox(e_sMsg, MB_RETRYCANCEL | MB_ICONEXCLAMATION)!=IDCANCEL);
	}
	
	return true;
}


// Dtruire un dossier et son contenu
bool DossierSupprimer(LPCTSTR pszDossier, bool fQueVide)
{
	if (!PathFileExists(pszDossier))
		return true;
	
	TCHAR pszPath[MAX_PATH];
	lstrcpy(pszPath, pszDossier);
	return DossierSupprimerInterne(fQueVide, pszPath);
}

// Fonction interne rcursive utilise par DossierSupprimer()
bool DossierSupprimerInterne(bool fQueVide, LPTSTR pszPath)
{
	bool fOK = true;
	
	PathAddBackslash(pszPath);
	TCHAR *const pcPathFin = pszPath+lstrlen(pszPath);
	
	// Effectuer la recherche
	lstrcpy(pcPathFin, _T("*.*"));
	for (CMyFindFile ff(pszPath); ff.Next();) {
		if (ff.IsDots())
			continue;
		
		lstrcpy(pcPathFin, ff.m_wfd.cFileName);
		if (ff.m_wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
			// Sous-dossier
			fOK &= DossierSupprimerInterne(fQueVide, pszPath);
			lstrcpy(pcPathFin, ff.m_wfd.cFileName);
		}else{
			// Fichier
			if (fQueVide || !DeleteFile(pszPath))
				fOK = false;
		}
	}
	
	// Supprimer le dossier lui-mme et en notifier le shell
	*pcPathFin = 0;
	PathRemoveBackslash(pszPath);
	if (fOK && RemoveDirectory(pszPath))
		SHChangeNotify(SHCNE_RMDIR, SHCNF_PATH, pszPath, 0);
	else
		fOK = false;
	
	return fOK;
}

// Indiquer si le nom pass (sans chemin d'accs) est valide
// Enlever les espaces des deux cts du nom
bool NomCorriger(LPTSTR pszNom)
{
	// Supprimer les espaces avant/aprs
	TCHAR *pcIn  = pszNom;
	TCHAR *pcOut = pszNom;
	while (_istspace(*pcIn))
		*pcIn++;
	if (*pcIn) {
		lstrcpy(pcOut, pcIn);
		pcOut = pcOut+lstrlen(pcOut);
		while (_istspace(pcOut[-1]))
			pcOut--;
	}
	*pcOut = 0;
	
	if (!*pszNom) {
		// Nom vide
		AfxMessageBox(ERR_NOMVIDE);
		return false;
	}
	
	// Vrifier l'absence de caractres interdits
	for (const TCHAR *pc=pszNom; *pc; pc++) {
		const TCHAR c = *pc;
		if (c=='\\' || c=='/' || c==':' || c=='*' || c=='?'||
		    c=='\"' || c=='<' || c=='>' || c=='|') {
			AfxMessageBox(ERR_NOMINVALIDE);
			return false;
		}
	}
	
	return true;
}


// Crer un fichier temporaire et rcuprer son nom
bool GetFichierTemp(LPCTSTR pszDossier, LPTSTR pszFichier)
{
	while (!GetTempFileName(pszDossier, _T("MUZ"), 0, pszFichier)) {
		AfxFormatString1(e_sMsg, ERR_TEMP, pszDossier);
		if (AfxMessageBox(e_sMsg, MB_RETRYCANCEL | MB_ICONEXCLAMATION)==IDCANCEL) {
			*pszFichier = 0;
			return false;
		}
	}
	
	return true;
}

// Crer un dossier temporaire
bool GetDossierTemp(LPCTSTR pszParent, LPTSTR pszDossier)
{
	// Rcuprer un dossier temporaire
	*pszDossier = 0;
	return
		GetFichierTemp(pszParent, pszDossier) &&
		FichierSupprimer(pszDossier, false) &&
		DossierCreer(pszDossier);
}


// Rcuprer la version d'un fichier
bool GetInstanceVersion(HINSTANCE hInst, VERSION& ver)
{
	// Charger la ressource
	HRSRC hRes = ::FindResource(hInst,
		MAKEINTRESOURCE(VS_VERSION), RT_VERSION);
	if (!hRes)
		hRes = ::FindResource(hInst, MAKEINTRESOURCE(VS_VERSION_DLL), RT_VERSION);
	const HGLOBAL hGlobal = ::LoadResource(hInst, hRes);
	void *const pvInfo = ::LockResource(hGlobal);
	
	if (pvInfo) {
		VS_FIXEDFILEINFO *pfdi;
		UINT len;
		
		if (VerQueryValue(pvInfo, _T("\\"), (void**)&pfdi, &len)) {
			// Version rcupre
			
			ver.verH = pfdi->dwProductVersionMS;
			ver.verL = pfdi->dwProductVersionLS;
			return true;
		}
	}
	
	ver.verH = ver.verL = 0x0000;
	return false;
}


// Rcuprer le nom complet d'un fichier/dossier
// Renvoyer TRUE si le nom du fichier est non-vide
bool MyGetFullPathName(LPCTSTR pszIn, LPTSTR pszOut)
{
	*pszOut = 0;
	VERIF(pszIn && *pszIn);
	
	const int ret =
		GetFullPathName(pszIn, MAX_PATH, pszOut, NULL);
	VERIF(ret>0);
	
	if (ret>MAX_PATH)
		lstrcpy(pszOut, pszIn);
	
	return true;
}


// Ajouter si ncessaire des guillemets autour d'une spcification
// Les espaces  gauche et  droite sont supprims
// La chane originale n'est pas modifie
CString MyPathQuoteSpacesCopy(LPCTSTR pszPath)
{
	CString s = pszPath;
	s.TrimLeft();
	s.TrimRight();
	PathQuoteSpaces(s.GetBuffer(s.GetLength() + 3));
	s.ReleaseBuffer();
	
	return s;
}




//*** CMyFindFile

// Commencer la recherche
void CMyFindFile::Start(LPCTSTR pszRech)
{
	Close();
	
	m_hff    = ::FindFirstFile(pszRech, &m_wfd);
	m_fStart = true;
}

// Poursuivre la recherche
bool CMyFindFile::Next()
{
	VERIF(m_hff!=INVALID_HANDLE_VALUE);
	
	if (m_fStart) {
		m_fStart = false;
		return true;
	}
	
	return ToBool(::FindNextFile(m_hff, &m_wfd));
}

// Terminer la recherche
void CMyFindFile::Close()
{
	if (m_hff!=INVALID_HANDLE_VALUE) {
		::FindClose(m_hff);
		m_hff = INVALID_HANDLE_VALUE;
	}
}


// Indiquer si le fichier trouv est un rpertoire '.' ou '..'
bool CMyFindFile::IsDots() const
{
	return
	   m_wfd.cFileName[0]=='.' &&
	 (!m_wfd.cFileName[1] ||
	 ( m_wfd.cFileName[1]=='.' && !m_wfd.cFileName[2]));
}
