// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// LBPlusItem.cpp
//
// Bote de liste avec dessin personnalis
// o chaque ligne reprsente un objet CPlusItem
// Affiche une icne reprsentant l'action  gauche
// de la spcification du ou des fichier(s)  compresser


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "AppResource.h"
#include "LBPlusItem.h"

#include "PlusItem.h"
#include "dlgArchivePlus.h"


const int cIcone       = 16;
const int cxMarge      =  5;
const int cxMargeEntre =  3;
const int cyMarge      =  3;


BEGIN_MESSAGE_MAP(CLBPlusItem, CListBox)
	//{{AFX_MSG_MAP(CLBPlusItem)
	ON_WM_DROPFILES()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


// Constructeur
CLBPlusItem::CLBPlusItem()
{
	m_cyItem = -1;
	
	m_iml.Create(IDB_COA, cIcone, nbCOA, RGB(192,192,192));
}

// Wrapper d'un cast de GetItemDataPtr()
// avec gestion des dpassements de nItem
CPlusItem* CLBPlusItem::GetItemPI(int nItem)
{
	if (nItem<0 || nItem>=GetCount())
		return NULL;
	
	return (CPlusItem*)GetItemDataPtr(nItem);
}


// Mettre  jour la largeur de la barre de dfilement horizontale
void CLBPlusItem::UpdateHorizontalExtent()
{
	// Calculer la largeur maximale des textes (sans l'icne et les marges)
	
	int cxTexteMax = 0;
	
	// Rcuprer un DC avec la bonne police
	CDC *const pdc = GetDC();
	pdc->SelectStockObject(DEFAULT_GUI_FONT);
	
	const int nbItem = GetCount();
	for (int nItem=0; nItem<nbItem; nItem++) {
		
		// Calculer la largeur du texte de l'item
		CString sItem;
		GetText(nItem, sItem);
		const int cxTexte = pdc->GetTextExtent(sItem).cx;
		
		// On veut le maximum
		if (cxTexteMax < cxTexte)
			cxTexteMax = cxTexte;
	}
	
	ReleaseDC(pdc);
	
	// Calculer la largeur maximale (icne et marge comprises)
	const int cxMax = cxMarge + cIcone + cxMargeEntre + cxTexteMax + cxMarge;
	
	// La largeur minimale est celle de la liste
	// (pour que le rectangle de la slection fasse toute la largeur)
	CRect rc;
	GetClientRect(rc);
	SetHorizontalExtent(__max(cxMax, rc.right));
}

// Insrer un item
// ATTENTION: cette fonction n'appelle pas UpdateHorizontalExtent()
int CLBPlusItem::InsertString(int nItem, CPlusItem* ppi)
{
	CString sItem;
	
	// Rcuprer le dossier et les parties
	CLstCString2 lstPartie;
	ppi->ListerPartie(sItem, lstPartie);
	const int lenDebut = sItem.GetLength();
	
	if (lstPartie.GetNb()==1)
		// Une seule partie
		
		sItem = lstPartie.GetDeb();
		
	else{
		// Plusieurs parties
		
		// Grer le cas des racines:
		// X: doit tre affich sous la forme X:\ 
		if (lenDebut>0 && sItem[lenDebut-1]==':')
			sItem += '\\';
		sItem += _T(" - ");
		
		// Ajouter les parties
		POS pos = lstPartie.GetPosDeb();
		while (pos) {
			
			// Ajouter la partie
			LPCTSTR pszPartie = (LPCTSTR)lstPartie.GetSuiv(pos) + lenDebut;
			if (*pszPartie=='\\')
				pszPartie++;
			sItem += pszPartie;
			
			if (!pos)
				break;
			sItem += _T(" ; ");
		}
	}
	
	// Insrer l'item
	nItem = CListBox::InsertString(nItem, sItem);
	SetItemDataPtr(nItem, ppi);
	
	return nItem;
}


// Dimensions
void CLBPlusItem::MeasureItem(MEASUREITEMSTRUCT* pmis)
{
	CalcY();
	
	pmis->itemHeight = m_cyItem;
}

// Dessin
void CLBPlusItem::DrawItem(DRAWITEMSTRUCT* pdis)
{
	CalcY();
	
	CDC dc;
	dc.Attach(pdis->hDC);
	const UINT state = pdis->itemState;
	
	// Choisir la bonne couleur de fond et de texte
	int iclFond, iclTexte;
	if (state & ODS_SELECTED) {
		iclFond  = COLOR_HIGHLIGHT;
		iclTexte = COLOR_HIGHLIGHTTEXT;
	}else{
		iclFond  = COLOR_WINDOW;
		iclTexte = COLOR_WINDOWTEXT;
	}
	dc.SetBkColor  (GetSysColor(iclFond));
	dc.SetTextColor(GetSysColor(iclTexte));
	
	// crire le texte
	const int lenItem = GetTextLen(pdis->itemID);
	CString sItem;
	GetText(pdis->itemID, sItem.GetBuffer(lenItem+1));
	sItem.ReleaseBuffer();
	dc.ExtTextOut(
		pdis->rcItem.left + cxMarge + cIcone + cxMargeEntre,
		pdis->rcItem.top  + m_yoffTexte,
		ETO_OPAQUE | ETO_CLIPPED, &pdis->rcItem,
		sItem, NULL);
	
	if (lenItem>=0) {
		// Dessiner l'icne
		ImageList_DrawEx(m_iml,
			2*((CPlusItem*)pdis->itemData)->CO_nAction + ToBool(state & ODS_SELECTED),
			dc, pdis->rcItem.left+cxMarge,pdis->rcItem.top+m_yoffIcone,
			cIcone,cIcone, CLR_NONE,CLR_NONE, ILD_TRANSPARENT);
	}
	
	// Dessiner le rectangle de focus si ncessaire
	// Quand il n'y a pas d'item dans la liste, le rectangle de focus
	// devrait tre plat ; on corrige ce comportement pour que le focus
	// soit visible en donnait la hauteur des autres items
	// au rectangle du focus
	if (state & ODS_FOCUS) {
		if (pdis->rcItem.bottom==pdis->rcItem.top)
			pdis->rcItem.bottom += m_cyItem;
		dc.DrawFocusRect(&pdis->rcItem);
	}
	
	dc.Detach();
}


// Effectuer les calculs de hauteur
void CLBPlusItem::CalcY()
{
	if (m_cyItem>=0)
		return;
	
	// Calculer la hauteur d'un item
	CDC *const pdc = GetDC();
	pdc->SelectStockObject(DEFAULT_GUI_FONT);
	const int cyTexte = pdc->GetTextExtent(_T("A"), 1).cy;
	ReleaseDC(pdc);
	
	// Ne pas oublier les marges et la hauteur de l'icne
	m_cyItem = cyMarge + __max(cyTexte, cIcone) + cyMarge;
	
	// Calculer l'offset vertical du texte et de l'icne
	m_yoffTexte = (m_cyItem - cyTexte) / 2;
	m_yoffIcone = (m_cyItem - cIcone)  / 2;
}



// Glisser-dplacer de fichiers
void CLBPlusItem::OnDropFiles(HDROP hDropInfo)
{
	((CDlgArchivePlus*)GetParent())->OnLstDropFiles(hDropInfo);
	DragFinish(hDropInfo);
}
