// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// LstBouton.cpp
//
// Gre une liste d'icnes pour les boutons Ajouter, Supprimer,
// Dplacer vers le haut/bas, etc.
// Gre galement le dplacement d'items vers le haut/bas dans une liste


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "AppResource.h"
#include "LstBouton.h"

#include "LBPlusItem.h"


// Constructeur
CLstBouton::CLstBouton(CDialog* pdlg, CListBox& lst, UINT idDebut)
	: m_pdlg(pdlg), m_lst(lst), m_idDebut(idDebut)
{
	// Charger les icnes des boutons
	m_imlBouton.Create(IDB_ARCHIVEPLUS, cBouton, nbBouton, RGB(192,192,192));
}


// Affecter les icnes aux boutons
void CLstBouton::SetIcons()
{
	for (int btn=0; btn<nbBouton; btn++) {
		CButton *const pbtn = ((CButton*)m_pdlg->GetDlgItem(btn+m_idDebut));
		if (pbtn)
			pbtn->SetIcon(m_imlBouton.ExtractIcon(btn));
	}
}


// Traiter un message de bouton IDCCMD_HAUT ou IDCCMD_BAS
void CLstBouton::OnHautBas(UINT nID) const
{
	nID -= m_idDebut;
	if (nID!=btnHaut && nID!=btnBas)
		return;
		
	const int nbItem = m_lst.GetCount();
	
	m_lst.SetRedraw(FALSE);
	
	const bool fCheckBoxList =
		ToBool(m_lst.IsKindOf(RUNTIME_CLASS(CCheckListBox)));
	
	
	// On doit dcaler la slection vers le haut ou vers le bas
	// Pour cela, on recherche chaque squence continue d'items
	// slectionns, et on la dplace virtuellement en supprimant
	// un lment non-slectionn  un bout de la squence
	// et en remettant cet lment  l'autre bout.
	//
	// Mettons qu'on veuille veut dcaler la liste suivante vers la droite:
	// 1 [2] [3] 4   ([X] signifie que X est slectionn)
	//
	// On supprime l'lment 4 et on le met entre 1 et [2].
	//
	// Avantages de ce systme:
	// - pas besoin de s'occuper de la slection (les items slectionns
	//   le restent car ne sont pas supprims)
	// - rapidit: meilleur que dplacer un par un les items slectionns
	//
	// Le caret est mis en tte sur le premier lment slectionn
	// dans le sens du dplacement (le premier si dplacement vers le haut,
	// le dernier si dplacement vers le bas)
	
	
	int nItem;     // Item courant
	int derItem;   // Dernier item  parcourir
	int offSens;   // Sens de parcours (+/- 1)
	int nCaret;    // Nouveau caret
	
	nCaret = -1;
	if (nID==btnHaut) {
		nItem   = nbItem;
		derItem = -1;
		offSens = -1;
	}else /* (nID==btnBas)*/ {
		nItem   = -1;
		derItem = nbItem;
		offSens = +1;
	}
	
	const bool fListePlusItem = (m_idDebut==IDCCMD_AJOUTER && !fCheckBoxList);
	
	for (;;) {
		
		// Trouver le premier item slectionn
		for (;;) {
			nItem += offSens;
			if (nItem==derItem)
				goto Fin;
			if (m_lst.GetSel(nItem))
				break;
		}
		
		int nItemSel = nItem;
		
		// Aller jusqu'au dernier item slectionn de la squence
		for (;;) {
			nCaret = nItem;
			if (!m_lst.GetSel(nItem))
				break;
			
			nItem += offSens;
			if (nItem==derItem)
				goto Fin;
		}
		
		CString sItem;
		int     iChecked = FALSE;
		
		// On va dplacer l'item nItem
		// Sauver d'abord son tat
		if (!fListePlusItem) {
			m_lst.GetText(nItem, sItem);
			if (fCheckBoxList)
				iChecked = ((CCheckListBox&)m_lst).GetCheck(nItem);
		}
		const DWORD dwItemData = m_lst.GetItemData(nItem);
		
		// Supprimer l'item
		m_lst.DeleteString(nItem);
		
		// Remettre l'item supprim  la position nItemSel
		if (nItemSel==nbItem)
			nItemSel = -1;
		int nItemNew;
		if (fListePlusItem)
			// Liste de CPlusItem
			
			nItemNew = ((CLBPlusItem&)m_lst).InsertString(
				nItemSel, (CPlusItem*)dwItemData);
		else{
			// Liste classique (sans InsertString personnalis)
			
			nItemNew = m_lst.InsertString(nItemSel, sItem);
			m_lst.SetItemData(nItemNew, dwItemData);
			
			if (fCheckBoxList)
				// Liste avec cases  cocher
				((CCheckListBox&)m_lst).SetCheck(nItemNew, iChecked);
		}
	}
Fin:
	
	// Donner le caret au bon item
	if (fCheckBoxList)  m_lst.SetCurSel    (nCaret);
	else                m_lst.SetCaretIndex(nCaret);
	
	m_lst.SetFocus();
	m_lst.SetRedraw();
}


// Slection modifie dans la liste
void CLstBouton::OnLstSelChange(bool fMultiSel) const
{
	const bool fSel = (fMultiSel)
		? m_lst.GetSelCount()>0
		: m_lst.GetCurSel()!=LB_ERR;
	
	// Activer ou dsactiver les boutons qui demandent une slection
	// selon s'il y a une slection ou non
	for (int btn=btnEditer; btn<=btnBas; btn++) {
		CWnd *const pwnd = m_pdlg->GetDlgItem(btn+m_idDebut);
		if (pwnd)
			pwnd->EnableWindow(fSel);
	}
}
