// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// Macro.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "App.h"
#include "AppResource.h"
#include "Macro.h"

#include "MuzipDoc.h"
#include "ScriptDoc.h"
#include "MuzipArc.h"
#include "ApercuView.h"

#include "Divers.h"
#include "IOFichier.h"

#include "dlgConfirm.h"
#include "dlgDecomp.h"
#include "dlgMacroEditer.h"
#include "dlgMacroAction.h"

#include "MacroDoc.h"


#define kszMuzip    _T("Muzip")


static const LPCTSTR s_apszBoolean[] = { _T("No"), _T("Yes") };

const LPCTSTR e_apszCmd[nbMacCmd] =
{
	_T("include"),
	_T("unzip"),
	_T("view"),
	_T("exec"),
	_T("open"),
	_T("close"),
};


static BOOL GetBoolFromString(LPCTSTR psz, BOOL fDefaut);



// Convertir une chane en boolen
// Grer "Yes" et "No" ainsi que les valeurs gales (TRUE)
// ou diffrentes (FALSE) de zro
BOOL GetBoolFromString(LPCTSTR psz, BOOL fDefaut)
{
	if (!*psz)
		return fDefaut;
	
	return (lstrcmpi(psz, s_apszBoolean[TRUE]))
		? ToBool(_tcstol(psz, NULL, 10))
		: TRUE;
}



//*** CIniKey


// Constructeur de lecture
// Rcuprer le contenu d'une section du fichier INI
CIniKey::CIniKey(LPCTSTR pszINI, LPCTSTR pszSection) : m_psOut(NULL)
{
	CString s;
	
	// Rcuprer la section toute entire
	for (DWORD buf=0;;) {
		buf += 1024;
		
		const DWORD len = ::GetPrivateProfileSection(pszSection,
			s.GetBuffer(buf),buf, pszINI);
		s.ReleaseBuffer();
		
		if (len != buf-2)
			break;
	}
	
	// numrer les valeurs
	INI_KEY ik;
	LPCTSTR pc = s;
	while (*pc) {
		
		// Rcuprer le nom de la cl et la valeur associe
		CTokenize tok(pc, '=');
		if (tok.Next()) {
			ik.sKey = tok.GetToken();
			if (tok.Next()) {
				ik.sValeur = tok.GetToken();
				
				// Ajouter le couple (cl,valeur)  la liste
				AddDeb(ik);
			}
		}
		
		// Passer au couple suivant
		pc += lstrlen(pc) + 1;
	}
}


// Grer les entres-sorties pour une chane
void CIniKey::Serialize(LPCTSTR pszKey, CString& rs, LPCTSTR pszDefaut)
{
	if (IsStoring()) {
		// criture
		
		CString sPlus;
		sPlus.Format(_T("%s=%s\n"), pszKey, (LPCTSTR)rs);
		*m_psOut += sPlus;
		
	}else{
		// Lecture
		
		POS pos = GetPosDeb();
		while (pos) {
			const INI_KEY &ik = GetSuiv(pos);
			
			if (!lstrcmpi(pszKey, ik.sKey)) {
				// Trouv !
				rs = ik.sValeur;
				return;
			}
		}
		
		// Non-trouv, utiliser la valeur par dfaut
		rs = pszDefaut;
	}
}

// Grer les entres-sorties pour un entier
void CIniKey::Serialize(LPCTSTR pszKey, UINT& ri, UINT iDefaut)
{
	if (IsStoring()) {
		// criture
		
		CString sPlus;
		sPlus.Format(_T("%s=%u\n"), pszKey, ri);
		*m_psOut += sPlus;
		
	}else{
		// Lecture
		
		CString sValeur;
		Serialize(pszKey, sValeur, NULL);
		if (!sValeur.IsEmpty()) {
			LPTSTR pcFin;
			ri = _tcstoul(sValeur, &pcFin, 10);
			if (pcFin)
				// Valeur correcte trouve
				return;
		}
		
		// Valeur incorrecte: utiliser la valeur par dfaut
		ri = iDefaut;
	}
}

// Grer les entres-sorties pour un boolen
void CIniKey::Serialize(LPCTSTR pszKey, BOOL& rf, BOOL fDefaut)
{
	if (IsStoring()) {
		// criture
		
		CString sPlus;
		sPlus.Format(_T("%s=%s\n"), pszKey, s_apszBoolean[ToBool(rf)]);
		*m_psOut += sPlus;
		
	}else{
		// Lecture
		
		CString sValeur;
		Serialize(pszKey, sValeur);
		rf = GetBoolFromString(sValeur, fDefaut);
	}
}



//*** CMacro


// Constructeur
CMacro::CMacro() : CLstMacAction(true), m_fModified(false), m_pDoc(NULL)
{
	// Mettre les valeur par dfaut
	CIniKey ini;
	Serialize(ini);
}


// Affectation
CMacro& CMacro::operator = (const CMacro& mac)
{
	DeleteAll();
	
	m_fAuto     = mac.m_fAuto;
	m_fUnzip    = mac.m_fUnzip;
	m_fAutoOpen = mac.m_fAutoOpen;
	
	POS pos = mac.GetPosDeb();
	while (pos) {
		CMacAction *const pactNew = new CMacAction(this);
		*pactNew = *mac.GetSuiv(pos);
		AddFin(pactNew);
	}
	
	return *this;
}


// E/S
void CMacro::Serialize(CIniKey& ini)
{
	ini.Serialize(_T("Version"),  m_iVersion,  0);
	ini.Serialize(_T("Auto"),     m_fAuto,     FALSE);
	ini.Serialize(_T("Unzip"),    m_fUnzip,    FALSE);
	ini.Serialize(_T("AutoOpen"), m_fAutoOpen, FALSE);
}


// crire la macro au format INI dans une chane
void CMacro::ToINI(CString& s)
{
	// Section gnrale
	s = _T("[") kszMuzip _T("]\n");
	CIniKey ini(&s);
	Serialize(ini);
	
	// Actions
	POS pos = GetPosDeb();
	while (pos)
		GetSuiv(pos)->ToINI(s);
	
	// On demande des \r\n pour les sauts de ligne
	s.Replace(_T("\n"), _T("\r\n"));
}



// Lire la macro dans un fichier INI
bool CMacro::FromINI(LPCTSTR pszINI)
{
	DeleteAll();
	
	// Lire le fichier INI
	CIniKey ini(pszINI, kszMuzip);
	GetFileTitle(pszINI, m_sName.GetBuffer(MAX_PATH), MAX_PATH);
	Serialize(ini);
	
	
	// Vrifier la version
	VERIF(m_iVersion==100);
	
	
	// Rcuprer la liste des sections
	CString s;
	for (DWORD buf=0;;) {
		buf += 256;
		
		const DWORD lenLu = ::GetPrivateProfileSectionNames(
			s.GetBuffer(buf),buf, pszINI);
		
		if (lenLu!=buf-2) {
			s.ReleaseBuffer(lenLu);
			break;
		}
		
		s.ReleaseBuffer(0);
	}
	
	// Lire les actions
	// Ignorer la section gnrale, nomme [Muzip]
	for (LPCTSTR pc=s; *pc; pc+=lstrlen(pc)+1) {
		if (!lstrcmpi(pc, kszMuzip))
			continue;
		
		CMacAction *const pact = new CMacAction(this);
		pact->m_sId = pc;
		pact->FromINI(pszINI);
		AddFin(pact);
	}
	
	return true;
}


// diter la macro, retourner TRUE si elle a t modifie
bool CMacro::Editer()
{
	CDlgMacroEditer dlg;
	(CMacro&)dlg = *this;
	
	VERIF(dlg.DoModal()==IDOK);
	
	*this = dlg;
	m_fModified = true;
	
	// Mettre  jour les vues du document
	if (m_pDoc)
		m_pDoc->ToView();
	
	return true;
}


// Excuter les actions point d'entre
bool CMacro::Executer(CMacContexte& ctx)
{
	if (m_fAuto)
		oApp.m_nConfirm--;
	
	bool fOK = true;
	
	POS pos = GetPosDeb();
	while (pos && fOK) {
		CMacAction *const pact = GetSuiv(pos);
		
		if (pact->m_fEntryPoint)
			fOK = pact->Executer(ctx);
	}
	
	if (m_fAuto)
		oApp.m_nConfirm++;
	
	return fOK;
}




//*** CMacAction


// Constructeur
CMacAction::CMacAction(CMacro* pmac) : CLstMacCmd(true), m_pmac(pmac)
{
	// Mettre les valeur par dfaut
	CIniKey ini;
	Serialize(ini);
}


// Affectation
CMacAction& CMacAction::operator = (const CMacAction& act)
{
	DeleteAll();
	
	m_sId         = act.m_sId;
	m_sName       = act.m_sName;
	m_sComment    = act.m_sComment;
	m_fHidden     = act.m_fHidden;
	m_fEntryPoint = act.m_fEntryPoint;
	
	POS pos = act.GetPosDeb();
	while (pos) {
		CMacCmd *const pcmdNew = new CMacCmd(this);
		*pcmdNew = *act.GetSuiv(pos);
		AddFin(pcmdNew);
	}
	
	return *this;
}


// Rcuprer l'intitul lisible de l'action
void CMacAction::GetItem(CString& s) const
{
	if (m_sName.IsEmpty())
		s = m_sId;
	else
		s.Format(_T("%s (%s)"), (LPCTSTR)m_sId, (LPCTSTR)m_sName);
}


// crire l'action au format INI dans une chane
void CMacAction::ToINI(CString& s)
{
	// Crer la nouvelle section
	CString sPlus;
	sPlus.Format(_T("\n[%s]\n"), (LPCTSTR)m_sId);
	s += sPlus;
	
	// Ajouter les donnes
	CIniKey ini(&s);
	Serialize(ini);
	
	// crire les commandes
	POS pos = GetPosDeb();
	while (pos)
		GetSuiv(pos)->ToINI(s);
}


// E/S
void CMacAction::Serialize(CIniKey& ini)
{
	ini.Serialize(_T("Name"),       m_sName);
	ini.Serialize(_T("Comment"),    m_sComment);
	ini.Serialize(_T("Hidden"),     m_fHidden,     FALSE);
	ini.Serialize(_T("EntryPoint"), m_fEntryPoint, FALSE);
}


// Lire l'action dans un fichier INI
void CMacAction::FromINI(LPCTSTR pszINI)
{
	DeleteAll();
	
	// Lire le fichier INI
	CIniKey ini(pszINI, m_sId);
	Serialize(ini);
	
	// Lire les commandes
	// Pour cela il faut d'abord rcuprer tous les numros de commandes
	// et les trier avant de les lire dans l'ordre
	int *const aiCmd = new int[ini.GetNb()];
	int nbCmd = 0;
	
	POS pos = ini.GetPosDeb();
	while (pos) {
		
		// Rcuprer le numro  droite
		CString &sKey = ini.GetSuiv(pos).sKey;
		LPCTSTR pszNumero = (LPCTSTR)sKey + sKey.GetLength();
		while (pszNumero>(LPCTSTR)sKey && isdigit(pszNumero[-1]))
			pszNumero--;
		
		// Il faut que la commande ait le bon nom de base
		// c'est--dire le nom sans numro
		if (lstrcmpi(sKey.Left(pszNumero-(LPCTSTR)sKey), _T("Cmd")))
			continue;
		
		aiCmd[nbCmd++] = _tcstol(pszNumero, NULL, 10);
	}
	
	// Trier
	qsort(aiCmd, nbCmd, sizeof(aiCmd[0]), ComparerINT);
	
	// Lire les commandes
	int iPrec = 0;
	for (int i=0; i<nbCmd; i++) {
		
		// Valider le numro
		// Les zro sont ignors car iPrec==0 initialement
		if (aiCmd[i]==iPrec)
			continue;
		iPrec = aiCmd[i];
		
		// Composer le nom de la cl
		CString sKey;
		sKey.Format(_T("Cmd%d"), iPrec);
		
		// Lire et ajouter la commande
		CMacCmd *const pcmd = new CMacCmd(this);
		pcmd->m_id = (UINT)iPrec;
		
		CString sCmd;
		ini.Serialize(sKey, sCmd);
		pcmd->FromINI(sCmd);
		if (pcmd->m_mcm==nbMacCmd)
			delete pcmd;
		else
			AddFin(pcmd);
	}
	
	delete [] aiCmd;
}

// Comparer deux INT
int CMacAction::ComparerINT(const void* p1, const void* p2)
{
	return *(int*)p1 - *(int*)p2;
}


// diter l'action
// fSetModified indique si CMacro::m_fModified peut tre mis  TRUE
// Cela est le cas si l'action est dite directement  partir de CDlgMacroGerer
bool CMacAction::Editer(bool fSetModified)
{
	CDlgMacroAction dlg(m_pmac);
	(CMacAction&)dlg = *this;
	
	VERIF(dlg.DoModal()==IDOK);
	
	*this = dlg;
	m_pmac->m_fModified |= fSetModified;
	
	// Mettre  jour les vues du document
	if (m_pmac->m_pDoc)
		m_pmac->m_pDoc->ToView();
	
	return true;
}


// Excuter l'action
bool CMacAction::Executer(CMacContexte& ctx)
{
	// Empcher la rcursivit
	for (int i=0; i<ctx.m_pilAction.GetNb(); i++) {
		if (ctx.m_pilAction[i]==this)
			return ctx.Erreur(ERR_MACRO_RECURSIF);
	}
	
	ctx.m_pilAction.Push(this);
	
	// Excuter les commandes
	bool fOK = true;
	POS pos = GetPosDeb();
	while (fOK && pos)
		fOK = GetSuiv(pos)->Executer(ctx);
	
	ctx.m_pilAction.Pop();
	
	return fOK;
}



//*** CMacCmd


// Affectation
CMacCmd& CMacCmd::operator = (const CMacCmd& cmd)
{
	m_mcm = cmd.m_mcm;
	m_id  = cmd.m_id;
	
	for (int i=0; i<nbArray(m_aArgs); i++)
		m_aArgs[i] = cmd.m_aArgs[i];
	
	return *this;
}


// Rcuprer l'intitul lisible de la commande
// L'intitul est AJOUT  la chane SANS la supprimer
void CMacCmd::GetItem(CString& s) const
{
	// Ajouter le nom de la commande
	s += e_apszCmd[m_mcm];
	
	// Compter les arguments spcifis
	int derArg;
	for (derArg=nbArray(m_aArgs); derArg>0;)
		if (!m_aArgs[--derArg].IsEmpty())
			break;
	
	// Ajouter les arguments avec des guillemets si ncessaire
	// On ajoute systmatiquement des guillemets autour des arguments vides
	// pour ne pas qu'ils soient ignors
	for (int i=0; i<=derArg; i++) {
		s += ' ';
		const bool fGuillemet =
			m_aArgs[i].IsEmpty() || m_aArgs[i].Find(' ')>=0;
		
		if (fGuillemet)  s += '"';
		s += m_aArgs[i];
		if (fGuillemet)  s += '"';
	}
}


// crire la commande au format INI dans une chane
void CMacCmd::ToINI(CString& s) const
{
	CString sPlus;
	sPlus.Format(_T("Cmd%u="), m_id);
	s += sPlus;
	
	GetItem(s);
	
	s += '\n';
}

// Lire la commande dans un fichier INI
void CMacCmd::FromINI(LPCTSTR pszValeur)
{
	LPCTSTR pc;
	
	// Rcuprer le nom de la commande
	CString sCmd;
	pc = _tcschr(pszValeur, ' ');
	if (!pc)
		sCmd = pszValeur;
	else{
		const int buf = pc-pszValeur+1;
		lstrcpyn(sCmd.GetBuffer(buf), pszValeur, buf);
		sCmd.ReleaseBuffer();
	}
	
	// Convertir le nom en index
	for (((int&)m_mcm)=0;; ((int&)m_mcm)++) {
		if (m_mcm==nbMacCmd)
			return;
		if (!lstrcmpi(sCmd, e_apszCmd[m_mcm]))
			break;
	}
	
	// Pas d'arguments: on a fini
	if (!pc)
		return;
	
	CTokenize tok(pc);
	while (tok.Next()) {
		
		if (tok.GetIndex()==maxArgs) {
			// Trop d'arguments: ignorer la commande
			m_mcm = nbMacCmd;
			return;
		}
		
		m_aArgs[tok.GetIndex()] = tok.GetToken();
	}
}


// Excuter la commande
bool CMacCmd::Executer(CMacContexte& ctx)
{
	ctx.m_pcmd = this;
	
	CDocument *pDoc = NULL;
	CMuzipArc *pArc = NULL;
	
	// Rcuprer le document courant et l'archive courante
	CMDIChildWnd *const pwndChild =
		((CMDIFrameWnd*)oApp.m_pMainWnd)->MDIGetActive();
	if (pwndChild) {
		pDoc = pwndChild->GetActiveDocument();
		if (pDoc->IsKindOf(RUNTIME_CLASS(CMuzipDoc)))
			pArc = ((CMuzipDoc*)pDoc)->m_pArc;
	}
	
	bool fOK = true;
	
	switch (m_mcm) {
		
		// include action [macro]
		// Excuter une autre action
		case mcmInclude:
			{
				CMacro *pmac = m_pact->GetMacro();
				bool fDeleteMac = false;
				
				if (!m_aArgs[1].IsEmpty()) {
					// Une macro est spcifie
					
					pmac = new CMacro;
					CString sMacro;
					ctx.GetFinalPath(pArc, m_aArgs[1], sMacro, this);
					if (pmac->FromINI(sMacro))
						// OK
						fDeleteMac = true;
					else{
						// Erreur
						
						fOK = ctx.Erreur(ERR_MACRO_EXECUTE);
						
						delete pmac;
						pmac = NULL;
					}
				}
				
				fOK = false;
				bool fStop = false;
				POS pos = pmac->GetPosDeb();
				while (pos) {
					CMacAction *const pact = pmac->GetSuiv(pos);
					if (!lstrcmpi(pact->m_sId, m_aArgs[0])) {
						// Excuter l'action
						fOK   = true;
						fStop = pact->Executer(ctx);
						break;
					}
				}
				
				if (!fOK)
					fOK = ctx.Erreur(ERR_MACRO_EXECUTE);
				
				if (fDeleteMac)
					delete pmac;
			}
			break;
		
		
		// unzip fichiers/dossier destination [options]
		// Dcompresser des fichiers
		case mcmUnzip:
			{
				if (!pArc) {
					fOK = ctx.Erreur(ERR_MACRO_UNZIP);
					break;
				}
				
				if (m_aArgs[0].IsEmpty())
					break;
				
				// Options de dcompression
				CDlgDecomp dlgDecomp(false, pArc);
				if (dlgDecomp.Init(m_aArgs[1], m_aArgs[2]) ||
				    dlgDecomp.DoModal()==IDOK) {
					
					CDossier *const pDossier = pArc->GetDossier(m_aArgs[0], gdGet);
					if (pDossier)
						// C'est un dossier
						
						pDossier->DossierDecomp(dlgDecomp.m_sDest, &dlgDecomp.m_dec);
						
					else{
						// C'est un fichier
						
						CFichier *const pFichier = ctx.GetArcFichier(pArc, m_aArgs[0]);
						if (pFichier) {
							CLstFichier lstFichier;
							lstFichier.AddFin(pFichier);
							pArc->ArchiveDecomp(lstFichier,
								dlgDecomp.m_sDest, &dlgDecomp.m_dec);
						}
					}
				}
			}
			break;
		
		
		// view fichier [aperu]
		// Slectionner un fichier
		case mcmView:
			{
				CFichier *const pFichier = ctx.GetArcFichier(pArc, m_aArgs[0]);
				if (pFichier) {
					ASSERT(pDoc);
					((CMuzipDoc*)pDoc)->SelFichier(pFichier);
					
					if (GetBoolFromString(m_aArgs[1], FALSE))
						// Afficher l'aperu
						((CMuzipDoc*)pDoc)->m_papr->SetApercu(pFichier);
				}
			}
			break;
		
		
		// exec programme/fichier
		// Excuter un programme, ouvrir un fichier
		case mcmExec:
			{
				CString sExec;
				if (ctx.GetFinalPath(pArc, m_aArgs[0], sExec, this))
					ShellExecute(NULL, NULL, sExec, m_aArgs[1], NULL, SW_NORMAL);
			}
			break;
		
		
		// open fichier
		// Ouvrir une archive, un script ou une macro
		case mcmOpen:
			{
				// Ouvrir le document
				CDocument *const pDoc = pApp->OpenDocumentFile(m_aArgs[0]);
				if (!pDoc)
					break;
				
				if (pDoc->IsKindOf(RUNTIME_CLASS(CScriptDoc))) {
					// Excuter puis fermer le script
					
					((CScriptDoc*)pDoc)->OnScriptExecuter();
					pDoc->OnCloseDocument();
				}
			}
			break;
		
		
		// close
		// Fermer le document courant
		case mcmClose:
			if (pDoc)
				pDoc->OnCloseDocument();
			break;
	}
	
	ctx.m_pcmd = NULL;
	return fOK;
}


// Rcuprer un fichier dans une archive d'aprs son nom
CFichier* CMacContexte::GetArcFichier(CMuzipArc* pArc, LPCTSTR pszPath)
{
	if (!pArc)
		return NULL;
	
	// Ignorer les slashs initiaux
	while (*pszPath=='/')
		pszPath++;
	
	// Enlever les double-slashs
	// Convertir les slashs en antislashs
	CString sPath = pszPath;
	while (sPath.Replace(_T("//"), _T("/")));
	sPath.Replace('/', '\\');
	
	// Rcuprer le dossier
	CDossier *const pDossier = pArc->GetDossier(sPath, gdGet, true);
	if (!pDossier)
		return NULL;
	
	// Rcuprer le fichier
	const LPCTSTR pszFichier = PathFindFileName(sPath);
	POS pos = pDossier->m_lstFichier.GetPosDeb();
	while (pos) {
		CFichier *const pFichier = pDossier->m_lstFichier.GetSuiv(pos);
		
		if (!lstrcmpi(pFichier->m_pszNom, pszFichier))
			// Trouv !
			return pFichier;
	}
	
	return NULL;
}



// Afficher un message d'erreur concernant la macro
// Si nIDS==0, utilise e_sMsg comme message d'erreur au lieu
// de la chane d'ID nIDS
bool CMacContexte::Erreur(UINT nIDS)
{
	//*** Composer le message d'erreur
	
	CString sFormat(MAKEINTRESOURCE(ERR_MACRO_DEBUT));
	CString sMacro, sAction, sLigne;
	CString sErreur((nIDS) ? MAKEINTRESOURCE(nIDS) : e_sMsg);
	
	if (m_pilAction.GetNb()) {
		sFormat += CStringRes(ERR_MACRO_ACTION);
		
		// Rcuprer le nom de la macro et celui de l'action
		sMacro = m_pilAction[0]->GetMacro()->m_sName;
		m_pilAction[0]->GetItem(sAction);
	}
	
	// Si l'erreur concerne une commande prcise,
	// ajouter son intitul dans le message
	if (m_pcmd) {
		sFormat += CStringRes(ERR_MACRO_LIGNE);
		m_pcmd->GetItem(sLigne);
	}
	
	// Ajouter la description de l'erreur
	sFormat += CStringRes(ERR_MACRO_FIN);
	
	// Formatter la commande finale
	const LPCTSTR apszArgs[] = { sMacro, sAction, sLigne, sErreur };
	AfxFormatStrings(e_sMsg, sFormat, apszArgs, nbArray(apszArgs));
	
	return CDlgConfirm::GetReponse(mcfMacro | cfmForcerConfirm);
}


// Convertir un chemin d'accs ventuellement interne  une archive
// en chemin physique
// TRUE n'est retourn que si le fichier existe
bool CMacContexte::GetFinalPath(CMuzipArc* pArc,
	LPCTSTR pszIn, CString& sOut, CMacCmd* pcmd)
{
	if (*pszIn!='/') {
		// Chemin physique
		
		// Demander confirmation
		CString sItem;
		pcmd->GetItem(sItem);
		AfxFormatString1(e_sMsg, DEM_MACROEXTERNE, sItem);
		VERIF(CDlgConfirm::GetReponse(mcfMacroExterne, 0, cfmNonDefaut | cfmForcerConfirm));
		
		sOut = pszIn;
		
		if (PathFileExists(pszIn))
			// Le fichier existe
			return true;
		
		// Chercher un excutable
		TCHAR pszBuf[MAX_PATH];
		return FindExecutable(pszIn, NULL, pszBuf)>=(HINSTANCE)32;
	}
	
	
	// Chemin interne  l'archive
	CFichier *const pFichier = GetArcFichier(pArc, pszIn);
	VERIF(pFichier);
	return pFichier->CacheAddRef(sOut);
}
