// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// Macro.h


#ifndef _MACRO_H_
#define _MACRO_H_
#if _MSC_VER>1000
#pragma once
#endif

#include <Pile.h>


enum MACCMD
{
	mcmInclude,
	mcmUnzip,
	mcmView,
	mcmExec,
	mcmOpen,
	mcmClose,
	
	nbMacCmd
};


extern const LPCTSTR e_apszCmd[nbMacCmd];


class CMacro;
class CMacAction;
class CMacContexte;


struct INI_KEY
{
	CString sKey;
	CString sValeur;
};


//*** Classe de srialisation dans un fichier INI


class CIniKey : public CListe1<INI_KEY>
{
public:  // Fonctions
	
	CIniKey(CString* psOut=NULL) : m_psOut(psOut) {}
	CIniKey(LPCTSTR pszINI, LPCTSTR pszSection);
	
	void Serialize(LPCTSTR pszKey, CString& rs, LPCTSTR pszDefaut=NULL);
	void Serialize(LPCTSTR pszKey, UINT&    ri, UINT    iDefaut);
	void Serialize(LPCTSTR pszKey, BOOL&    rf, BOOL    fDefaut);
	
	
	bool IsStoring() const { return ToBool(m_psOut); }
	
	
protected:  // Interne
	
	CString *m_psOut;   // Chane d'criture (NULL si mode lecture)
};



//*** Commande dans une action de macro
//*** Une commande est la brique de base des macros

class CMuzipDoc;
class CMuzipArc;
class CFichier;

class CMacCmd
{
public:  // Donnes
	
	enum{ maxArgs = 3 };
	
	MACCMD  m_mcm;              // Type de la commande
	UINT    m_id;               // ID de la commande (numro d'ordre)
	CString m_aArgs[maxArgs];   // Arguments
	
	
public:  // Fonctions
	
	CMacCmd(CMacAction* pact) : m_pact(pact) {}
	
	CMacCmd& operator = (const CMacCmd& cmd);
	
	
	void GetItem(CString& s) const;
	
	
	void ToINI(CString& s) const;
	void FromINI(LPCTSTR pszValeur);
	
	
	bool Executer(CMacContexte& ctx);
	
	
protected:  // Interne
	
	CMacAction *m_pact;   // Action contenant la commande
};
typedef CListePtr2<CMacCmd*> CLstMacCmd;



//*** Action de macro, c'est--dire ensemble de commandes

class CMacAction : public CLstMacCmd
{
public:  // Donnes
	
	CString m_sId;          // Identificateur
	CString m_sName;        // Nom complet
	CString m_sComment;     // Commentaires
	
	BOOL    m_fHidden;      // Action invisible dans la liste
	                        // (excutable uniquement par 'include')
	BOOL    m_fEntryPoint;  // Point d'entre de la macro
	
	
public:  // Fonctions
	
	CMacAction(CMacro* pmac);
	
	CMacAction& operator = (const CMacAction& act);
	
	
	void GetItem(CString& s) const;
	CMacro* GetMacro() { return m_pmac; }
	
	void Serialize(CIniKey& ini);
	void ToINI(CString& s);
	void FromINI(LPCTSTR pszINI);
	
	bool Editer(bool fSetModified);
	
	bool Executer(CMacContexte& ctx);
	
	
protected:  // Interne
	
	CMacro *m_pmac;         // Macro contenant l'action
	
	
	static int ComparerINT(const void* p1, const void* p2);
};
typedef CListePtr2<CMacAction*> CLstMacAction;



//*** Macro, c'est--dire ensemble d'actions

class CMacroDoc;

class CMacro : public CLstMacAction
{
public:  // Donnes
	
	CMacroDoc *m_pDoc;    // Document ditant la macro
	
	CString m_sName;      // Nom de la macro (interne  Muzip)
	
	UINT    m_iVersion;   // Version de la macro
	BOOL    m_fAuto;      // viter les confirmations
	BOOL    m_fUnzip;     // Dcompression automatique
	BOOL    m_fAutoOpen;  // Excuter lma macro ds l'ouverture de l'archive
	                      // ou le lancement de l'auto-extractible
	
	bool    m_fModified;  // TRUE si la macro a t modifie
	                      // (quivalent de SetModifiedFlag/IsModified)
	
	
public:  // Fonctions
	
	CMacro();
	
	CMacro& operator = (const CMacro& mac);
	
	
	void Serialize(CIniKey& ini);
	void ToINI(CString& s);
	bool FromINI(LPCTSTR pszINI);
	

	bool Editer();
	
	bool Executer(CMacContexte& ctx);
};



//*** Contexte d'excution de macros

class CMacContexte
{
public:  // Fonctions
	
	CMacContexte() : m_pcmd(NULL) {}
	
	
	static CFichier* GetArcFichier(CMuzipArc* pArc, LPCTSTR pszPath);
	
	bool Erreur(UINT nIDS=0);
	
	static bool GetFinalPath(CMuzipArc* pArc, LPCTSTR pszIn, CString& sOut, CMacCmd* pcmd);
	
	
public:  // Donnes
	
	CPileCh<CMacAction*>  m_pilAction;   // Pile des appels d'actions
	CMacCmd              *m_pcmd;        // Commande en cours, NULL si aucune
};


#endif
