// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// MainFrame.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "Muzip2.h"
#include "MuzipDll/MuzipDll.h"
#include "MainFrame.h"

#include "MuzipDoc.h"
#include "MuzipArc.h"

#include "Options.h"
#include "Reg.h"
#include "IOFichier.h"

#include <SysObject.h>


const LPCTSTR eszVersionH = _T("VersionH");
const LPCTSTR eszVersionL = _T("VersionL");


// Message pour que MuzipDll puisse demander l'excution de commandes
// par les instances de Muzip.exe ouvertes

const UINT MSG_MUZIPCMD = RegisterWindowMessage(pszMsgMuzipCmd);

#pragma comment(linker, "/SECTION:.shared,RWS")
#pragma data_seg(".shared")
static TCHAR s_pszMuzipCmdArg[MAX_PATH] = _T("");
static bool  s_bMuzipFound = false;
#pragma data_seg()


static bool SendMuzipCmdInternal(HWND hWnd, MUZIPCMD muc);


IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)
BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SETTINGCHANGE()
	ON_WM_SYSCOMMAND()
	ON_WM_INITMENUPOPUP()
	ON_NOTIFY           (NM_CLICK, AFX_IDW_STATUS_BAR, OnIndicatorClick)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_SEL,             OnUpdateIndicatorEtat)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_COMMENT,         OnUpdateIndicatorComment)
	ON_COMMAND          (ID_VIEW_TOOLBAR,              OnViewToolbar)
	ON_COMMAND          (ID_VIEW_CUSTOMIZE,            OnViewCustomize)
	ON_COMMAND          (ID_VIEW_APERCUAUTO,           OnViewApercuAuto)
	ON_UPDATE_COMMAND_UI(ID_VIEW_TOOLBAR,              OnUpdateViewToolbar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_APERCUAUTO,           OnUpdateViewApercuAuto)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_TOTAL,           OnUpdateIndicatorEtat)
	//}}AFX_MSG_MAP
	ON_COMMAND          (ID_HELP_FINDER,               CMDIFrameWnd::OnHelpFinder)
	ON_COMMAND          (ID_HELP,                      CMDIFrameWnd::OnHelp)
	ON_COMMAND          (ID_CONTEXT_HELP,              CMDIFrameWnd::OnContextHelp)
	ON_COMMAND          (ID_DEFAULT_HELP,              CMDIFrameWnd::OnHelpFinder)
	ON_REGISTERED_MESSAGE(MSG_MUZIPCMD,                OnMuzipCmd)
END_MESSAGE_MAP()


// Liste des barres-d'outils (une par menu)
static struct
{
	UINT    nID;      // ID de la barre-d'outils
	LPCTSTR pszReg;   // Cl d'enregistrement dans le registre
	
} const s_aToolBar[] =
{
	{ IDR_MAINFRAME,  _T("BarreOutilsPrincipale") },
	{ IDR_MUZIPTYPE,  _T("BarreOutilsArchive") },
	{ IDR_SCRIPTTYPE, _T("BarreOutilsScript") },
	{ IDR_MACROTYPE,  _T("BarreOutilsMacro") },
};


// Indicateurs sur la barre d'tat
static const UINT s_StatusBar[] =
{
	ID_SEPARATOR,
	ID_INDICATOR_SEL,
	ID_INDICATOR_TOTAL,
	ID_INDICATOR_COMMENT,
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
};


static BOOL CALLBACK prcEnumGetMuzipWndMain(HWND hWnd, LPARAM lParam);


// Constructeur
CMainFrame::CMainFrame() : m_nToolBar(0)
{
}


// Cration
int CMainFrame::OnCreate(CREATESTRUCT* pcs)
{
	// Activer le toujours visible si ncessaire
	if (e_opt.m_fGeneralTjVisible)
		pcs->dwExStyle |= WS_EX_TOPMOST;
	
	if (CMDIFrameWnd::OnCreate(pcs)==-1)
		return -1;
	
	// Sous-classer la fentre cliente
	m_wndClient.SubclassWindow(m_hWndMDIClient);
	
	
	// Rcuprer l'ancienne version et stocker la version courante
	bool fNouvelleVersion;
	{
		VERSION verOld, verCur;
		
		// Rcuprer l'ancienne version dans le registre
		verOld.verH = oApp.GetProfileInt(kszGeneral, eszVersionH, 0x0000);
		verOld.verL = oApp.GetProfileInt(kszGeneral, eszVersionL, 0x0000);
		
		// Rcuprer la version courante dans les informations de version
		GetInstanceVersion(oApp.m_hInstance, verCur);
		
		// Stocker la version courante
		oApp.WriteProfileInt(kszGeneral, eszVersionH, verCur.verH);
		oApp.WriteProfileInt(kszGeneral, eszVersionL, verCur.verL);
		
		// Regarder si on est  une version diffrente
		fNouvelleVersion = (verCur.verH!=verOld.verH || verCur.verL!=verOld.verL);
	}
	
	
	// Barres d'outils
	for (int i=0; i<nbArray(s_aToolBar); i++) {
		
		// Crer la barre d'outils
		m_wndToolBar[i].SetSizes(CSize(23,23), CSize(16,16));
		if (!m_wndToolBar[i].Create(this, WS_CHILD | CBRS_SIZE_DYNAMIC |
		     CBRS_TOP | CBRS_TOOLTIPS | CBRS_FLYBY, s_aToolBar[i].nID) ||
		    !m_wndToolBar[i].LoadToolBar(s_aToolBar[i].nID))
			return -1;
		m_wndToolBar[i].ShowWindow(SW_HIDE);
		
		// La rendre ditable et la charger
		m_wndToolBar[i].SetEditable();
		if (!fNouvelleVersion)
			m_wndToolBar[i].FromReg(s_aToolBar[i].pszReg, false);
	}
	
	m_wndToolBar[m_nToolBar = 0].ShowWindow(SW_SHOW);
	
	// Barre d'tat
	m_fComment = false;
	if (!m_wndStatusBar.CreateEx(this, SBT_TOOLTIPS) ||
	    !m_wndStatusBar.SetIndicators(s_StatusBar, nbArray(s_StatusBar)))
		return -1;
	
	// Initialiser l'icne des commentaires:
	// dfinir sa largeur, enlever le texte, et lui affecter une bulle d'aide
	// On n'enlve pas rellement le texte mais on le remplace par des espaces,
	// comme a l'info bulle pourra tre affiche mme s'il n'y a pas
	// l'icne (selon le MSDN, l'info bulle ne peut tre affiche que s'il y
	// a une icne ou du texte qui dborde du panel: ici on met plein d'espaces
	// pour tre sr que a sort des 16+1 pixels disponibles)
	const int nPane = m_wndStatusBar.CommandToIndex(ID_INDICATOR_COMMENT);
	m_wndStatusBar.SetPaneInfo(nPane, ID_INDICATOR_COMMENT,
		m_wndStatusBar.GetPaneStyle(nPane), 16+1);
	m_wndStatusBar.SetPaneText(nPane, _T("       "), FALSE);
	m_wndStatusBar.GetStatusBarCtrl().SetTipText(nPane,
		CStringRes(ID_INDICATOR_COMMENT));
	
	
	// Menu systme: rajouter un item "Toujours visible"
	ASSERT((IDS_SCTOUJOURSVISIBLE & 0xFFF0)==IDS_SCTOUJOURSVISIBLE);
	ASSERT( IDS_SCTOUJOURSVISIBLE < 0xF000);
	const CString s = MAKEINTRESOURCE(IDS_SCTOUJOURSVISIBLE);
	CMenu *const pMenu = GetSystemMenu(FALSE);
	pMenu->AppendMenu(MF_SEPARATOR);
	pMenu->AppendMenu(MF_STRING, IDS_SCTOUJOURSVISIBLE, (LPCTSTR)s+(s.Find('\n')+1));
	
	return 0;
}


// Destruction
void CMainFrame::OnDestroy()
{
	// Enregistrer l'tat des barres-d'outils
	for (int i=0; i<nbArray(s_aToolBar); i++)
		m_wndToolBar[i].ToReg(s_aToolBar[i].pszReg, false);
	
	CMDIFrameWnd::OnDestroy();
}


// Changement de paramtre systme
void CMainFrame::OnSettingChange(UINT uFlags, LPCTSTR lpszSection)
{
	CMDIFrameWnd::OnSettingChange(uFlags, lpszSection);
	
	// Mettre  jour les couleurs systme
	// ainsi que les options (ncessaire si on se base
	// sur les paramtres de Windows)
	CSysObject::MAJ();
	e_opt.MAJAffichageStyleEx();
	e_opt.Appliquer();
}


// Commande systme
void CMainFrame::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0)==IDS_SCTOUJOURSVISIBLE) {
		// Grer la commande "Toujours visible"
		
		const bool fTjVisible = !ToBool(GetExStyle() & WS_EX_TOPMOST);
		e_opt.m_fGeneralTjVisible = fTjVisible;
		SetWindowPos((fTjVisible)
			? &wndTopMost : &wndNoTopMost, 0,0, 0,0, SWP_NOMOVE | SWP_NOSIZE);
		
	}else
		CMDIFrameWnd::OnSysCommand(nID, lParam);
}

// Menu droul
void CMainFrame::OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu)
{
	CMDIFrameWnd::OnInitMenuPopup(pPopupMenu, nIndex, bSysMenu);
	
	// Cocher/dcocher la commande "Toujours visible"
	if (bSysMenu)
		pPopupMenu->CheckMenuItem(IDS_SCTOUJOURSVISIBLE,
			(GetExStyle() & WS_EX_TOPMOST)
				? MF_BYCOMMAND | MF_CHECKED
				: MF_BYCOMMAND);
}


// Barre d'tat - aucun texte
void CMainFrame::OnUpdateIndicatorEtat(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(FALSE);   // pCmdUI->SetText(NULL) en plus rapide
}

// Barre d'tat - Icne commentaire
void CMainFrame::OnIndicatorClick(NMHDR*, LRESULT*)
{
	// Le panneau cliqu doit tre celui de l'icne
	CPoint pt;
	GetCursorPos(&pt);
	m_wndStatusBar.ScreenToClient(&pt);
	CStatusBarCtrl &sb = m_wndStatusBar.GetStatusBarCtrl();
	const int nPane = m_wndStatusBar.CommandToIndex(ID_INDICATOR_COMMENT);
	CRect rc;
	sb.GetRect(nPane, rc);
	if (!rc.PtInRect(pt))
		return;
	
	// Le document actif doit tre une archive
	CFrameWnd *const pFrame = GetActiveFrame();
	if (!pFrame)
		return;
	CView *const pView = pFrame->GetActiveView();
	if (pView) {
		CMuzipDoc *const pDoc = DYNAMIC_DOWNCAST(CMuzipDoc, pView->GetDocument());
		if (pDoc && (pDoc->m_pArc->GetCap() & capCommentaire))
			pView->PostMessage(WM_COMMAND, ID_ARCHIVE_COMMENT);
	}
}
void CMainFrame::OnUpdateIndicatorComment(CCmdUI* pCmdUI)
{
	if (m_fComment) {
		m_fComment = false;
		m_wndStatusBar.GetStatusBarCtrl().SetIcon(
			m_wndStatusBar.CommandToIndex(pCmdUI->m_nID), NULL);
	}
}


// Dfinir l'affichage de l'icne Commentaire
void CMainFrame::SetComment(bool fComment)
{
	if (m_fComment==fComment)
		return;
	
	static HICON s_hIcon = NULL;
	if (!s_hIcon)
		s_hIcon = (HICON)::LoadImage(AfxGetResourceHandle(),
			MAKEINTRESOURCE(IDI_COMMENT), IMAGE_ICON, 16,16, 0);
	
	m_fComment = fComment;
	m_wndStatusBar.GetStatusBarCtrl().SetIcon(
		m_wndStatusBar.CommandToIndex(ID_INDICATOR_COMMENT),
		(m_fComment) ? s_hIcon : NULL);
}


// Charger la bonne barre d'outils selon le type du document actif
void CMainFrame::SwitchToolBar()
{
	// Rechercher l'ID du document actif
	// (IDR_MAINFRAME s'il n'y a aucun document)
	UINT nID = IDR_MAINFRAME;
	CMDIChildWnd *const pFrame = MDIGetActive();
	if (pFrame) {
		CDocument *const pDoc = pFrame->GetActiveDocument();
		nID = ((CMuzipDocTemplate*)pDoc->GetDocTemplate())->GetResourceID();
	}
	
	// En dduire l'index de la barre d'outils  afficher
	int nToolBar;
	for (nToolBar=0; s_aToolBar[nToolBar].nID!=nID; nToolBar++);
	if (m_nToolBar==nToolBar)
		// Aucun changement
		return;
	
	// Masquer l'ancienne barre d'outils et afficher la nouvelle
	m_wndToolBar[m_nToolBar].ShowWindow(SW_HIDE);
	m_nToolBar = nToolBar;
	m_wndToolBar[m_nToolBar].ShowWindow(SW_SHOW);
	RecalcLayout();
}

// Afficher ou masquer la barre d'outils
void CMainFrame::OnViewToolbar()
{
	m_wndToolBar[m_nToolBar].ShowWindow(
		(m_wndToolBar[m_nToolBar].IsWindowVisible()) ? SW_HIDE : SW_SHOW);
	RecalcLayout();
}
void CMainFrame::OnUpdateViewToolbar(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_wndToolBar[m_nToolBar].IsWindowVisible());
}

// Personnaliser la barre d'outils
void CMainFrame::OnViewCustomize()
{
	m_wndToolBar[m_nToolBar].Customize();
}

// Aperu automatique
void CMainFrame::OnViewApercuAuto()
{
	e_opt.m_fApercuMAJAuto = !e_opt.m_fApercuMAJAuto;
}
void CMainFrame::OnUpdateViewApercuAuto(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(e_opt.m_fApercuMAJAuto);
}


// Rcuprer la barre d'outils courante
CToolBar& CMainFrame::GetToolBar()
{
	return m_wndToolBar[m_nToolBar];
}


struct MUZIPCMDEXEC
{
	MUZIPCMD muc;
	HWND     hwndMain;
};


// Envoyer un message  une fentre principale de Muzip.exe visible
// Retourner la fentre trouve si tout s'est droul comme prvu
// (commande excute avec succs), NULL sinon
HWND SendMuzipCmd(MUZIPCMD muc, LPCTSTR pszArg, HWND hwndMain)
{
	// Copier la chane dans la mmoire partage
	lstrcpyn(s_pszMuzipCmdArg, pszArg, nbArray(s_pszMuzipCmdArg));
	
	// Fentre impose (pas de vrification)
	if (hwndMain)
		return (SendMuzipCmdInternal(hwndMain, muc)) ? hwndMain : NULL;
	
	// Essayer avec toutes les fentres principales de Muzip
	MUZIPCMDEXEC mce;
	mce.muc      = muc;
	mce.hwndMain = NULL;
	EnumWindows(prcEnumGetMuzipWndMain, (LPARAM)&mce);
	return mce.hwndMain;
}


// Envoyer une commande  une fentre potentiellement celle de Muzip
// Grer un timeout pour viter de figer
bool SendMuzipCmdInternal(HWND hWnd, MUZIPCMD muc)
{
	DWORD_PTR lResult;
	
	if (muc == mucIsMuzip) {
		// Message avec timeout
		
		VERIF(::SendMessageTimeout(hWnd, MSG_MUZIPCMD, mucIsMuzip, 0, SMTO_ABORTIFHUNG, 100, &lResult)
			&& s_bMuzipFound);
		s_bMuzipFound = false;
		
	}else
		// Message sans timeout
		
		lResult = ::SendMessage(hWnd, MSG_MUZIPCMD, muc, 0);
	
	
	return lResult == mucMagicResult;
}


// Chercher une fentre principale de Muzip
BOOL CALLBACK prcEnumGetMuzipWndMain(HWND hWnd, LPARAM lParam)
{
	const MUZIPCMD muc = ((MUZIPCMDEXEC*)lParam)->muc;
	
	if ((muc == mucIsMuzip || SendMuzipCmdInternal(hWnd, mucIsMuzip)) &&
	    SendMuzipCmdInternal(hWnd, muc)) {
		// Une fentre principale de Muzip a t trouve
		// Conserver le handle de la fentre
		// et arrter l'numration
		((MUZIPCMDEXEC*)lParam)->hwndMain = hWnd;
		return FALSE;
	}
	
	return TRUE;
}

// Message de communication entre les instances de Muzip
// Retourner TRUE si la commande a t traite, avec succs ou non
LRESULT CMainFrame::OnMuzipCmd(WPARAM wParam, LPARAM)
{
	switch (wParam) {
		
		// Retourner TRUE pour indiquer que la fentre appartient  Muzip
		case mucIsMuzip:
			s_bMuzipFound = true;
			break;
		
		// Excuter une ligne de commande
		case mucCmdLine:
			{
				CWnd *const pwndMain = oApp.m_pMainWnd;
				
				// La fentre principale doit tre active et visible
				// Si ce n'est pas le cas, alors une bote de dialogue est affiche
				// ou une opration longue est en cours (compression, etc.)
				// Si Muzip est occup, on DOIT ignorer la requte
				VERIF(pwndMain &&
					pwndMain->IsWindowEnabled() &&
					pwndMain->IsWindowVisible());
				
				// Parser la ligne de commande
				CMuzipCmdLine cmdInfo;
				CTokenize tok(s_pszMuzipCmdArg, cSepPartie);
				while (tok.Next()) {
					CString &sArg = tok.GetToken();
					if (sArg.IsEmpty())
						continue;
					
					// Si on a un flag, alors il faut supprimer le caractre
					// situ devant le flag (tiret ou slash)
					const bool fFlag = sArg[0]=='-' || sArg[0]=='/';
					if (fFlag)
						sArg.Delete(0);
					
					cmdInfo.ParseParam(sArg, fFlag, tok.IsLast());
				}
				
				// Enlever le flag "quitter" qui n'a pas lieu d'tre
				// car Muzip tait dj lanc
				cmdInfo.m_fQuitter = false;
				
				/// Excuter la ligne de commande
				oApp.ProcessShellCommand(cmdInfo);
				pwndMain->SetForegroundWindow();
			}
			break;
		
		
		// Autres messages
		default:
			return 0;
	}
	
	return mucMagicResult;
}
