// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// Muzip2.cpp
//
// Classes auxiliaires dclares dans Muzip.h


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "Muzip2.h"
#include "MuzipDll/MuzipApp.h"

#include "MuzipArc.h"
#include "PlusItem.h"
#include "Reg.h"
#include "Options.h"

#include "dlgConfirm.h"
#include "dlgProgress.h"
#include "dlgResRech.h"

#include "IOFichier.h"

#include <afxPriv.h>
#include <Pidl.h>


const LPCTSTR kszAppPaths  = _T("Software\\Microsoft\\Windows\\CurrentVersion\\App Paths\\Muzip.exe");
const LPCTSTR kszScriptExt = _T(".mus");
const LPCTSTR kszScript    = _T("Muzip.Script");
const LPCTSTR kszMacroExt  = _T(".mum");
const LPCTSTR kszMacro     = _T("Muzip.Macro");
const LPCTSTR kszArchive   = _T("Muzip.Archive");


// Enregistrer Muzip dans le Registre
// Ne pas enregistrer la DLL
void CMuzipApp::Register()
{
	// Chemin d'accs de l'excutable (Muzip.exe)
	CString sPath;
	GetModuleFileName(oApp.m_hInstance, sPath.GetBuffer(MAX_PATH), MAX_PATH);
	PathQuoteSpaces((STRING)sPath);
	sPath.ReleaseBuffer();
	
	// Enregistrer Muzip.exe dans les "App Paths"
	HKEY hKey;
	if (RegCreateKeyEx(HKEY_LOCAL_MACHINE, kszAppPaths,
		  0, NULL, 0, KEY_WRITE, NULL, &hKey, NULL)==ERROR_SUCCESS) {
		RegSetValueEx(hKey, NULL, 0, REG_SZ,
			(const BYTE*)(LPCTSTR)sPath, (sPath.GetLength()+1)*sizeof(sPath[0]));
		RegCloseKey(hKey);
	}
	
	
	//*** Associations
	
	const LPCTSTR kszIcone   = _T("DefaultIcon");
	
	const LPCTSTR kszShell   = _T("shell");
	const LPCTSTR kszOpen    = _T("open");
	const LPCTSTR kszEdit    = _T("edit");
	const LPCTSTR kszCmd     = _T("command");
	const LPCTSTR kszDDE     = _T("ddeexec");
	const LPCTSTR kszShellEx = _T("shellex");
	const LPCTSTR pszCLSID   = _T("{06712C80-5BDC-11D4-B8EE-B5DE61CA987B}");
	
	// Rechercher le chemin de la DLL contenant les ressources
	// (pour les icnes des archives et des scripts)
	TCHAR pszResDll[MAX_PATH];
	::GetModuleFileName(AfxGetResourceHandle(), pszResDll, nbArray(pszResDll));
	
	
	// Chanes dynamiques dont on a besoin:
	// - icnes 2,3,4
	// - intituls
	// - commandes d'ouverture, dition, etc.
	
	CString asIcone[3];
	for (int i=0; i<3; i++)
		asIcone[i].Format(_T("%s,%d"), pszResDll, i+2);
	
	const CString sScript (MAKEINTRESOURCE(REG_SCRIPT));
	const CString sMacro  (MAKEINTRESOURCE(REG_MACRO));
	const CString sExec   (MAKEINTRESOURCE(REG_EXEC));
	const CString sEdit   (MAKEINTRESOURCE(REG_EDIT));
	const CString sArchive(MAKEINTRESOURCE(REG_ARCHIVE));
	
	// Pour ouvrir une archive:
	// Muzip.exe "%1"
	CString sArchiveOpenCmd = sPath;
	sArchiveOpenCmd += _T(" \"%1\"");
	
	// Pour lancer un script ou une macro: Muzip.exe "%1" /q
	// Pour diter un script ou une macro: Muzip.exe "%1" /e
	CString sExecCmd = sArchiveOpenCmd;
	sExecCmd += _T(" /q");
	CString sEditCmd = sArchiveOpenCmd;
	sEditCmd += _T(" /e");
	
	struct REG
	{
		LPCTSTR pszKey1;
		LPCTSTR pszKey2;
		LPCTSTR pszKey3;
		LPCTSTR pszKey4;
		LPCTSTR pszValue;
	};
	
	
	// Liste des valeurs  enregistrer
	const REG aReg[] =
	{
		// Script
		{ kszScriptExt, NULL,       NULL,     NULL,   kszScript },
		{ kszScript,    NULL,       NULL,     NULL,   sScript },
		{ kszScript,    kszIcone,   NULL,     NULL,   asIcone[3-2] },
		{ kszScript,    kszShell,   kszOpen,  NULL,   sExec },
		{ kszScript,    kszShell,   kszOpen,  kszCmd, sExecCmd },
		{ kszScript,    kszShell,   kszOpen,  kszDDE, NULL },
		{ kszScript,    kszShell,   kszEdit,  NULL,   sEdit },
		{ kszScript,    kszShell,   kszEdit,  kszCmd, sEditCmd },
		{ kszScript,    kszShell,   kszEdit,  kszDDE, NULL },
		
		// Macro
		{ kszMacroExt,  NULL,       NULL,     NULL,   kszMacro },
		{ kszMacro,     NULL,       NULL,     NULL,   sMacro },
		{ kszMacro,     kszIcone,   NULL,     NULL,   asIcone[4-2] },
		{ kszMacro,     kszShell,   kszOpen,  NULL,   sExec },
		{ kszMacro,     kszShell,   kszOpen,  kszCmd, sExecCmd },
		{ kszMacro,     kszShell,   kszOpen,  kszDDE, NULL },
		{ kszMacro,     kszShell,   kszEdit,  NULL,   sEdit },
		{ kszMacro,     kszShell,   kszEdit,  kszCmd, sEditCmd },
		{ kszMacro,     kszShell,   kszEdit,  kszDDE, NULL },
		
		// Archive
		{ kszArchive,   NULL,       NULL,     NULL,   sArchive },
		{ kszArchive,   kszIcone,   NULL,     NULL,   asIcone[2-2] },
		{ kszArchive,   kszShell,   kszOpen,  kszCmd, sArchiveOpenCmd },
		{ kszArchive,   kszShellEx, _T("{00021500-0000-0000-c000-000000000046}"), NULL, pszCLSID },
		{ kszArchive,   kszShell,   kszOpen,  kszDDE, NULL },
	};
	
	
	// Enregistrer les valeurs de la liste
	for (int i=0; i<nbArray(aReg); i++) {
		const REG &reg = aReg[i];
		
		// Formatter la cl
		CString sKey = reg.pszKey1;
		if (reg.pszKey2) {
			sKey += '\\';
			sKey += reg.pszKey2;
			if (reg.pszKey3) {
				sKey += '\\';
				sKey += reg.pszKey3;
				if (reg.pszKey4) {
					sKey += '\\';
					sKey += reg.pszKey4;
				}
			}
		}
		
		if (reg.pszValue)
			// Enregistrer la valeur
			RegSetValue(HKEY_CLASSES_ROOT, sKey, REG_SZ,
				reg.pszValue, lstrlen(reg.pszValue)*sizeof(reg.pszValue[0]));
		else
			// Supprimer la cl
			AfxDelRegTreeHelper(HKEY_CLASSES_ROOT, sKey);
	}
}


// Dsenregistrer Muzip et la DLL du Registre
void CMuzipApp::Unregister()
{
	// Dsenregistrer toutes les associations
	POS pos = m_lstAssoc.GetPosDeb();
	while (pos) {
		CAssoc *const pAssoc = m_lstAssoc.GetSuiv(pos);
		
		HKEY hKey;
		TCHAR pszValeur[MAX_PATH];
		if (pAssoc->GetAssocCur(hKey, pszValeur))
			pAssoc->Affecter(false, hKey, pszValeur);
		if (hKey)
			RegCloseKey(hKey);
	}
	
	// Dsenregistrer la DLL
	DllUnregisterServer();
	
	// Supprimer toutes les cls
	AfxDelRegTreeHelper(HKEY_LOCAL_MACHINE, kszAppPaths);
	AfxDelRegTreeHelper(HKEY_CLASSES_ROOT,  kszScriptExt);
	AfxDelRegTreeHelper(HKEY_CLASSES_ROOT,  kszScript);
	AfxDelRegTreeHelper(HKEY_CLASSES_ROOT,  kszArchive);
	
	// Notifier le changement d'association
	CAssoc::NotifyChange();
}



//*** CMuzipOp

// Constructeur
CMuzipOp::CMuzipOp(const CMuzipArc* pArc, UINT nIDSTitre)
{
	if (!oApp.m_ndlgProgress++) {
		// Afficher le curseur sablier
		// et dsactiver les fentres
		// (principale et rsultats de la recherche)
		oApp.BeginWaitCursor();
		oApp.m_pMainWnd->EnableWindow(FALSE);
		if (e_pdlgResRech) {
			e_pdlgResRech->EnableWindow(FALSE);
			e_pdlgResRech->m_lst.EnableWindow(FALSE);
		}
		
		// Activer la fentre de progression
		oApp.m_pdlgProgress->EnableWindow(TRUE);
	}
	
	oApp.InitStatusAvance(pArc, nIDSTitre);
}

// Destructeur
CMuzipOp::~CMuzipOp()
{
	if (oApp.m_ndlgProgress==1) {
		oApp.HideStatusAvance();
		
		// Enlever le curseur sablier
		// et ractiver les fentres
		// (principale et rsultats de la recherche)
		oApp.EndWaitCursor();
		oApp.m_pMainWnd->EnableWindow(TRUE);
		if (e_pdlgResRech) {
			e_pdlgResRech->EnableWindow(TRUE);
			e_pdlgResRech->m_lst.EnableWindow(TRUE);
		}
		
	}else
		oApp.m_ndlgProgress--;
}


//*** CMuzipCmdLine
//
// Gre la compression/dcompression en ligne de commande

// Constructeur
CMuzipCmdLine::CMuzipCmdLine()
	: m_arPlus(NULL, true), m_fQuitter(false)
{
	m_nShellCommand    = FileNothing;
	m_fDemanderOptions = true;
	m_dwAttend         = 0;
}


// Parser une ligne de commande
void CMuzipCmdLine::ParseCommandLine(LPCTSTR pszCmdLine, bool fPeutInclude)
{
	CTokenize tok(pszCmdLine);
	while (tok.Next()) {
		CString &sArg = tok.GetToken();
		if (sArg.IsEmpty())
			continue;
		
		const TCHAR cDebut = sArg[0];
		if (cDebut=='@' && fPeutInclude) {
			// Lien vers un fichier
			
			// Ouvrir le fichier
			CFileRead fr;
			if (fr.Open((LPCTSTR)sArg+1, FILE_SHARE_ALL)) {
				
				// Lire son contenu
				CString s;
				const int buf = GetFileSize(fr, NULL);
				s.ReleaseBuffer(fr.ReadLen(s.GetBuffer(buf+1), buf));
				fr.Close();
				
				// Analyser le contenu
				ParseCommandLine(s, false);
			}
			continue;
		}
		
		// Si on a un flag, alors il faut supprimer le caractre
		// situ devant le flag (tiret ou slash)
		const bool fFlag = cDebut=='-' || cDebut=='/';
		if (fFlag)
			sArg.Delete(0);
		
		ParseParam(sArg, fFlag, tok.IsLast());
	}
}

// Parseur de paramtres
void CMuzipCmdLine::ParseParam(LPCTSTR pszParam, BOOL bFlag, BOOL bLast)
{
	if (bFlag) {
		// Flag
		
		const TCHAR cPrem = (TCHAR)tolower(pszParam[0]);
		
		if (!pszParam[1]) {
			// Flags  un seul caractre
			
			int sc = FileNothing;
			
			switch (cPrem) {
				
				// Quitter
				case 'q':  m_fQuitter = true;     break;
				
				// Oprations
				case 'z':  sc = FileComp;        break;
				case 'u':  sc = FileDecomp;      break;
				case 'a':  sc = FileSendMail;    break;
				case 'e':  sc = FileScriptEdit;  break;
				case 'r':  sc = FileShortcut;    break;
				
				// Options
				case 's':  m_fDemanderOptions = false;  break;
				case 'd':  m_dwAttend |= attDest;       break;
				case 'f':  m_dwAttend |= attPlusItem;   break;
				
				// Autre
				default:
					CCommandLineInfo::ParseParam(pszParam, bFlag, bLast);
					break;
			}
			
			if (sc!=FileNothing && m_nShellCommand==FileNothing)
				SetShellCommand(sc);
			
		}else if (cPrem=='o' && pszParam[1]=='=')
			// Options: /o=options
			
			m_sOptions = pszParam+2;
		
	}else{
		// Pas de flag: paramtre normal
		
		
		// Grer les paramtres attendus
		if (m_dwAttend & attDest) {
			// Rpertoire de destination
			
			m_dwAttend &= ~attDest;
			m_sDest = pszParam;
			return;
			
		}else if (m_dwAttend & attPlusItem) {
			// Spcification CPlusItem
			
			CPlusItem *const ppi = new CPlusItem;
			ppi->SetDefautExt();
			ppi->DO_sFiltre = pszParam;
			m_arPlus.AddFin(ppi);
			return;
		}
		
		
		if (m_nShellCommand==FileShortcut && m_sShortcut.IsEmpty())
			// Nom du raccourci cible
			
			m_sShortcut = pszParam;
			
		else if (m_strFileName.IsEmpty() && m_nShellCommand!=FileComp)
			// L'archive n'a pas t spcifie
			// Elle est obligatoire sauf en mode compression
			// ou cration de raccourci
			
			m_strFileName = pszParam;
			
		else
			// Traitement par dfaut
			CCommandLineInfo::ParseParam(pszParam, bFlag, bLast);
	}
	
	if (bLast && m_nShellCommand==FileNothing && !m_strFileName.IsEmpty())
		m_nShellCommand = FileOpen;
}


// Lire une option  partir d'un tableau de valeurs
void CMuzipCmdLine::GetOption(int*& piOption, int& ri, int max)
{
	const int val = *piOption++;
	if (val>=0 && val<=max)
		ri = val;
}

// Ajoute une option  une chane
void CMuzipCmdLine::AddOption(CString& s, int i)
{
	if (!s.IsEmpty())
		s += '-';
	
	CString sVal;
	sVal.Format(_T("%d"), i);
	s += sVal;
}



//*** CMuzipDocTemplate

// Grer l'ouverture en mode invisible
void CMuzipDocTemplate::InitialUpdateFrame(CFrameWnd* pFrame, CDocument* pDoc, BOOL bMakeVisible)
{
	CMultiDocTemplate::InitialUpdateFrame(pFrame, pDoc, bMakeVisible);
	
	// En mode visible, agrandir la frame
	// En mode invisible, la masquer
	pFrame->ShowWindow((oApp.m_fOuvreVisible) ? SW_MAXIMIZE : SW_HIDE);
	oApp.m_fOuvreVisible = true;
}

// Indiquer si le modle de document correspond  un fichier
// en grant les extensions multiples
CMuzipDocTemplate::Confidence CMuzipDocTemplate::MatchDocType(
	LPCTSTR lpszPathName, CDocument*& rpDocMatch)
{
	return (m_pdt)
		? m_pdt->MatchDocType(lpszPathName, rpDocMatch)
		: CDocTemplate::MatchDocType(lpszPathName, rpDocMatch);
}



//*** CSurvDecomp

HANDLE CSurvDecomp::s_hEvent      = NULL;
HANDLE CSurvDecomp::s_hThreadTous = NULL;

// Commencer la surveillance d'un fichier dcompress
CSurvDecomp::CSurvDecomp(HANDLE hProcess, LPCTSTR pszFichier,
	CFichier* pFichier, CMuzipArc* pArc)
{
	CLstSurvDecomp &lstSurvDecomp = oApp.m_lstSurvDecomp;
	
	// Supprimer les doublons
	POS pos = lstSurvDecomp.GetPosDeb();
	while (pos) {
		const POS posSauve = pos;
		CSurvDecomp *const psd = lstSurvDecomp.GetSuiv(pos);
		if (psd->m_pFichier==pFichier) {
			lstSurvDecomp.DeleteAt(posSauve);
			break;
		}
	}
	
	// Rcuprer la date de modification du fichier
	const HANDLE hf = CreateFile(pszFichier, GENERIC_READ,
		FILE_SHARE_ALL, NULL, OPEN_ALWAYS, 0, NULL);
	if (hf==INVALID_HANDLE_VALUE) {
Erreur:
		m_pszFichier = NULL;
		delete this;
		return;
	}
	const BOOL bOK = GetFileTime(hf, NULL, NULL, &m_ftModif);
	CloseHandle(hf);
	if (!bOK)
		goto Erreur;
	
	m_dwAttr     = GetFileAttributes(pszFichier);
	
	m_hProcess   = hProcess;
	m_pszFichier = newDup(pszFichier);
	m_pFichier   = pFichier;
	m_pArc       = pArc;
	m_et         = 0;
	m_hEvent     = CreateEvent(NULL, TRUE, FALSE, NULL);
	lstSurvDecomp.AddDeb(this);
	
	// Crer et lancer le thread de surveillance
	DWORD idThread;
	m_hThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)prcThread,
		(void*)this, 0, &idThread);
}

// Destruction
CSurvDecomp::~CSurvDecomp()
{
	if (m_pszFichier) {
		m_et |= etClose;
		
		// Terminer le thread de surveillance du processus
		PulseEvent(m_hEvent);
		WaitForSingleObject(m_hThread, INFINITE);
		
		CloseHandle(m_hEvent);
		
		delete [] m_pszFichier;
	}
}

// Surveiller un fichier dcompress
void CSurvDecomp::Main()
{
	if (m_hProcess==INVALID_HANDLE_VALUE && !(m_et & etFini))
		return;
	if (m_hProcess)
		m_hProcess = INVALID_HANDLE_VALUE;
	
	if ((m_et & (etModif | etAttr)) &&
	    e_lstArc.Find(m_pArc) &&
	    AfxIsValidAddress(m_pFichier->m_pDossier, sizeof(*m_pFichier->m_pDossier))) {
		// Le document et le dossier doivent tre valides
		
		if (!m_pArc->IsLectureSeule() && (m_et & (etModif | etAttr))) {
			// On peut recompresser le fichier s'il a t modifi,
			// mais uniquement si l'archive n'est pas en lecture seule
			
			if ((m_et & etModif) || (m_pArc->GetCap() & capAttribut)) {
				// Si les attributs sont modifis,
				// il faut que le format de l'archive gre les attributs
				
				AfxFormatString1(e_sMsg, DEM_COMPMODIF,
					PathFindFileName(m_pszFichier));
				if (CDlgConfirm::GetReponse(mcfCompModif)) {
					
					if (m_et & etModif) {
						// Recompresser le fichier
						
						if (m_pFichier->RecompModif(m_pszFichier))
							m_et &= ~etModif;
						
					}else{
						// Changer uniquement les attributs
						
						m_pFichier->m_dwAttr = m_dwAttr;
						m_pArc->FautReconstruire(rmoProprietes);
					}
				}
				
				if (m_et & etModif) {
					// Le fichier n'a pas pu tre recompress
					// Mettre  jour le cache
					
					CString sFile;
					m_pFichier->GetCacheFile(sFile);
					DeleteFile(sFile);
					m_pFichier->CacheAddRef(sFile);
				}
			}
		}
		
		// Supprimer la surveillance
		if (m_hProcess==INVALID_HANDLE_VALUE)
			oApp.m_lstSurvDecomp.DeleteAt(oApp.m_lstSurvDecomp.Find(this));
		else
			m_et &= ~(etModif | etAttr);
	}
}

// Vrifier si le fichier a t modifi
void CSurvDecomp::VerifModif()
{
	FILETIME ft;
	
	// Rcuprer la date de modification du fichier
	const HANDLE hf = CreateFile(m_pszFichier, GENERIC_READ,
		FILE_SHARE_ALL, NULL, OPEN_ALWAYS, 0, NULL);
	if (hf!=INVALID_HANDLE_VALUE) {
		if (GetFileTime(hf, NULL, NULL, &ft) &&
		    CompareFileTime(&ft, &m_ftModif)) {
			m_ftModif = ft;
			m_et     |= etModif;
		}
		CloseHandle(hf);
	}
}

// Vrifier si les attributs du fichier ont t modifis
void CSurvDecomp::VerifAttribut()
{
	const DWORD dwAttr = GetFileAttributes(m_pszFichier);
	if (dwAttr!=(DWORD)-1 && dwAttr!=m_dwAttr) {
		m_et    |= etAttr;
		m_dwAttr = dwAttr;
	}
}

// Crer le thread de surveillance gnrale
void CSurvDecomp::CreateThreadTous()
{
	DWORD idThread;
	s_hThreadTous =
		::CreateThread(NULL, 0, prcThreadTous, NULL, 0, &idThread);
}

// Terminer le thread de surveillance gnrale
void CSurvDecomp::TerminateThreadTous()
{
	if (s_hThreadTous) {
		SetEvent(s_hEvent);
		WaitForSingleObject(s_hThreadTous, INFINITE);
		ResetEvent(s_hEvent);
	}
}


// Thread de surveillance gnrale
DWORD WINAPI CSurvDecomp::prcThreadTous(void*)
{
	TCHAR pszDossierTemp[MAX_PATH];
	if (e_opt.GetDossier(repTemp, pszDossierTemp)) {
		
		// 0: vnement de terminaison
		// 1: surveillance des modifications
		// 2: surveillance des attributs
		// La surveillance concernant les fichiers
		// s'effectue dans le dossier temporaire et tous ses sous-dossiers
		// On ne peut pas regrouper FILE_NOTIFY_CHANGE_LAST_WRITE
		// et FILE_NOTIFY_CHANGE_ATTRIBUTES car on veut pouvoir
		// faire la diffrence
		HANDLE aHandle[3];
		aHandle[0] = s_hEvent;
		aHandle[1] = FindFirstChangeNotification(pszDossierTemp,
			TRUE, FILE_NOTIFY_CHANGE_LAST_WRITE);
		aHandle[2] = FindFirstChangeNotification(pszDossierTemp,
			TRUE, FILE_NOTIFY_CHANGE_ATTRIBUTES);
		
		CLstSurvDecomp &lstSurvDecomp = oApp.m_lstSurvDecomp;
		POS pos;
		
		for (;;) {
			const DWORD n = WaitForMultipleObjects(3, aHandle, FALSE, INFINITE);
			if (n==WAIT_OBJECT_0)
				// vnement de terminaison
				break;
			
			pos = lstSurvDecomp.GetPosDeb();
			while (pos) {
				CSurvDecomp *const psd = lstSurvDecomp.GetSuiv(pos);
				
				if (n==WAIT_OBJECT_0+1)
					// Un fichier a t modifi
					psd->VerifModif();
				
				else
					// Les attributs d'un fichier ont t modifis
					psd->VerifAttribut();
			}
			
			FindNextChangeNotification(aHandle[n]);
		}
		
		FindCloseChangeNotification(aHandle[1]);
		FindCloseChangeNotification(aHandle[2]);
	}
	
	// Fermer le handle du thread
	const HANDLE hThreadTous = s_hThreadTous;
	s_hThreadTous = NULL;
	CloseHandle(hThreadTous);
	return 0;
}

// Thread de surveillance
DWORD WINAPI CSurvDecomp::prcThread(CSurvDecomp* psd)
{
	int nbHandle = (psd->m_hProcess) ? 2 : 1;
	const HANDLE aHandle[2] = { psd->m_hEvent, psd->m_hProcess };
	
	WaitForMultipleObjects(nbHandle, aHandle, FALSE, INFINITE);
	if (psd->m_et!=etClose)
		// Processus termin
		psd->m_et |= etFini;
	
	return 0;
}
