// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// MuzipApp.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "MuzipApp.h"
#include "Muzip2.h"
#include "MuzipDll.h"
#include "Resource.h"

#include <App.h>

#include "IOFichier.h"
#include "FileDialogEx.h"
#include "Interf.h"

#include "Options.h"
#include "MuzipDoc.h"
#include "MuzipArc.h"

#include "Z/UnZ.h"

#include "MainFrame.h"

#include "MuzipFrame.h"
#include "ArboView.h"
#include "MuzipView.h"
#include "ApercuView.h"

#include "DlgFrame.h"

#include "ScriptDoc.h"
#include "ScriptView.h"

#include "MacroDoc.h"
#include "MacroView.h"

#include "dlgAssoc.h"
#include "dlgConfirm.h"
#include "dlgNouveau.h"
#include "dlgReparer.h"
#include "dlgDecomp.h"
#include "dlgProgress.h"
#include "dlgPlusItem.h"
#include "dlgRaccourci.h"
#include "dlgOptions.h"
#include "dlgDecompMult.h"
#include "dlgMacroGerer.h"
#include "dlgAPropos.h"

#include "Reg.h"
const LPCTSTR eszGeneralVersion     = _T("Version");
const LPCTSTR eszGeneralNbAssoc     = _T("NbAssociation");
const LPCTSTR eszGeneralAssoc       = _T("Association");
const LPCTSTR kszGeneralComp        = _T("Gnral\\Compression");
const LPCTSTR eszGeneralExtract     = _T("Extraction");
const LPCTSTR kszGeneralRech        = _T("Rechercher");
const LPCTSTR eszVoirPolice         = _T("Police");

const LPCTSTR eszGeneral            = _T("Gnral");
const LPCTSTR eszFiltre             = _T("Filtre");
const LPCTSTR eszContenu            = _T("Contenu");

CWinApp   *pApp  = NULL;
CMuzipApp *e_pma = NULL;


const int tpsDelaiMAJ = 100;   // En millisecondes


const DECOMP e_decEcraseToujours =
{
	ecrToujours,
	FALSE,
	TRUE,
	FALSE,
	FALSE,
};


bool CMuzipApp::m_fErreurFatale = false;

DWORD CMuzipApp::m_aPos[2];
DWORD CMuzipApp::m_aMax[2];
int   CMuzipApp::m_aPC [2];

bool       CMuzipApp::m_fSilent;

bool       CMuzipApp::m_fMsg = true;
CLIPFORMAT CMuzipApp::m_cfMuzip;
int        CMuzipApp::m_nConfirm = 1;

CLstSurvDecomp CMuzipApp::m_lstSurvDecomp;
CLstStr1       CMuzipApp::m_lstDelDossier;

ARCHIVEPLUS_GUI CMuzipApp::m_arPlus;
CPlusItem       CMuzipApp::m_pi;
DECOMP          CMuzipApp::m_dec;
RECHOPTIONS     CMuzipApp::m_rch;

CLstDocType  CMuzipApp::m_lstDocType;
CLstAssoc    CMuzipApp::m_lstAssoc;

CDlgProgress *CMuzipApp::m_pdlgProgress;
UINT          CMuzipApp::m_ndlgProgress = 0;
CString       CMuzipApp::m_sdlgProgress;

CFont CMuzipApp::m_fntVoir;

HIMAGELIST CMuzipApp::m_himlSystem;
int CMuzipApp::m_iIconRepClosed;
int CMuzipApp::m_iIconRepOpen;

CMuzipDocTemplate *CMuzipApp::m_pmdtScript;
CMuzipDocTemplate *CMuzipApp::m_pmdtMacro;

bool CMuzipApp::m_fOuvreVisible = true;

SOURCE_ACTION CMuzipApp::m_sa = saMuzip;


struct PLUSITEM : public PLUSOPTIONS, public FILTREOPTIONS
{
};



// Constructeur
void CMuzipApp::Constructeur()
{
	m_fErreurFatale = false;
	
	pApp  = this;
	e_pma = this;
	m_hInstance = AfxGetInstanceHandle();
	
	m_cfMuzip = (CLIPFORMAT)RegisterClipboardFormat(_T("Muzip"));
	
	e_opt.m_nReparer = repPresumer;
	
	if (!m_pDocManager)
		m_pDocManager = new CDocManager;
}

// Destructeur
void CMuzipApp::Destructeur()
{
	pApp  = NULL;
	e_pma = NULL;
	
	m_lstSurvDecomp.DeleteAll();
	m_lstDelDossier.DeleteAll();
	m_lstAssoc     .DeleteAll();
	m_lstDocType   .DeleteAll();
}


// Initialisation normale
BOOL CMuzipApp::InitInstance()
{
	if (!Init(false)) {
		// Erreur fatale au chargement
		
		CSurvDecomp::TerminateThreadTous();
		AfxSetResourceHandle(AfxGetInstanceHandle());
		if (m_fErreurFatale)
			AfxMessageBox(ERR_LOAD, MB_ICONERROR);
		return false;
	}
	
	return true;
}

// Terminaison normale
int CMuzipApp::ExitInstance()
{
	if (!Exit())
		return 0;
	
	return CWinApp::ExitInstance();
}



// Initialisation ( n'appeler qu'une seule fois)
bool CMuzipApp::InitOnce()
{
	// Initialiser le gnrateur de nombres alatoires
	srand((UINT)GetTickCount() ^ (UINT)GetCurrentProcessId());
	
	
	// Charger la DLL de ressources
	if (!LoadResourcesDll()) {
		// Afficher un message d'erreur
		
		m_fErreurFatale = true;
		AfxSetResourceHandle(AfxGetInstanceHandle());
		AfxMessageBox(ERR_RESSOURCEDLL, MB_ICONERROR);
		return false;
	}
	
	
	// Initialisation
	AfxOleInit();
	AfxInitRichEdit();
	
	return true;
}


// Initialisation
bool CMuzipApp::Init(bool fSilent)
{
	m_fSilent = fSilent;
	
	// Charger la DLL des ressources
	VERIF(!m_fErreurFatale);
	if (!LoadResourcesDll()) {
		m_fErreurFatale = true;
		return false;
	}
	
	// Fichier INI
	TCHAR pszIniFile[MAX_PATH];
	GetModuleFileName(m_hInstance, pszIniFile, nbArray(pszIniFile));
	PathRemoveFileSpec(pszIniFile);
	PathAppend(pszIniFile, _T("Muzip.ini"));
	free((void*)m_pszProfileName);
	m_pszProfileName = _tcsdup(pszIniFile);
	
	
	// Initialiser la gestion de la surveillance  la dcompression
	if (!CSurvDecomp::s_hEvent)
		CSurvDecomp::s_hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	else
		ResetEvent(CSurvDecomp::s_hEvent);
	
	
#if _MFC_VER < 0x0700
	if (!m_fSilent)
#ifdef _AFXDLL
		Enable3dControls();
#else
		Enable3dControlsStatic();
#endif
#endif
	
	// Charger le titre de l'application
	free((void*)m_pszAppName);
	m_pszAppName = _tcsdup(CStringRes(AFX_IDS_APP_TITLE));
	
	
	// Rcuprer la liste d'images systme
	SHFILEINFO shfi;
	m_himlSystem = (HIMAGELIST)SHGetFileInfo(_T(""), 0, &shfi, sizeof(shfi),
		SHGFI_USEFILEATTRIBUTES | SHGFI_SYSICONINDEX | SHGFI_SMALLICON);
	
	// Rcuprer les index des icnes des dossiers
	SHGetFileInfo(_T("A"), FILE_ATTRIBUTE_DIRECTORY, &shfi, sizeof(shfi),
		SHGFI_USEFILEATTRIBUTES | SHGFI_SYSICONINDEX);
	m_iIconRepClosed = shfi.iIcon;
	SHGetFileInfo(_T("A"), FILE_ATTRIBUTE_DIRECTORY, &shfi, sizeof(shfi),
		SHGFI_USEFILEATTRIBUTES | SHGFI_SYSICONINDEX | SHGFI_OPENICON);
	m_iIconRepOpen = shfi.iIcon;
	
	
	//*** Lire les options
	
	// Si on a chang de version, dtruire la cl de Muzip
	{
		VERSION verCourant, *pverRegistre;
		
		// Rcuprer la version courante de la DLL
		GetInstanceVersion(e_em.hModule, verCourant);
		
		// Rcuprer la version enregistre dans le registre
		UINT lenVerRegistre;
		if (!GetProfileBinary(kszGeneral, eszGeneralVersion,
		    (BYTE**)&pverRegistre, &lenVerRegistre) ||
		    lenVerRegistre!=sizeof(*pverRegistre)) {
			delete pverRegistre;
			pverRegistre = NULL;
		}
		
		// Comparer les versions
		if (!pverRegistre ||
		    verCourant.verH!=pverRegistre->verH ||
		    verCourant.verL!=pverRegistre->verL) {
			// Version diffrentes:
			// - effacer entirement les paramtres
			// - enregistrer la version courante
			
			// Supprimer toutes les sous-cls de HKCU\Ryder\Muzip
			const HKEY hkeyMuzip = GetAppRegistryKey();
			TCHAR pszSubKey[256];
			while (RegEnumKey(hkeyMuzip, 0, pszSubKey, nbArray(pszSubKey)-1)==ERROR_SUCCESS)
				AfxDelRegTreeHelper(hkeyMuzip, pszSubKey);
			
			// Enregistrer la version courante
			WriteProfileBinary(kszGeneral, eszGeneralVersion,
				(BYTE*)&verCourant, sizeof(verCourant));
		}
		
		delete pverRegistre;
	}
	
	
	// Crer les types de documents et leurs associations de fichiers
	// On doit le faire vers le dbut, car on en a besoin
	// au moment du chargement de la liste des associations
	CDocType::AddDocTypes();
	
	
	UINT len;
	BYTE *pb;
	
	// Options gnrales
	e_opt.Load();
	
	// Associations
	pb = NULL;
	UINT nbAssoc = GetProfileInt(kszGeneral, eszGeneralNbAssoc, 0);
	if (nbAssoc>0 &&
	    GetProfileBinary(kszGeneral, eszGeneralAssoc, &pb, &len) &&
	    len==nbAssoc*sizeof(CAssoc)) {
		// Rcuprer les associations
		
		for (CAssoc *pAssoc=(CAssoc*)pb; nbAssoc; pAssoc++) {
			nbAssoc--;
			
			POS pos = m_lstAssoc.GetPosDeb();
			while (pos) {
				CAssoc *const pAssocCur = m_lstAssoc.GetSuiv(pos);
				
				if (!lstrcmp(pAssocCur->m_pszExt, pAssoc->m_pszExt)) {
					// Association trouve
					*pAssocCur = *pAssoc;
					break;
				}
			}
		}
	}
	delete [] pb;
	
	// Options d'ajout par dfaut
	m_arPlus.m_nSup = supRien;
	
	// Options de compression par dfaut
	if (GetProfileBinary(kszGeneralComp, eszGeneral, &pb, &len) &&
	    len==sizeof(m_pi)) {
		(PLUSOPTIONS&)  m_pi = (PLUSITEM&)*pb;
		(FILTREOPTIONS&)m_pi = (PLUSITEM&)*pb;
	}else{
		// Options par dfaut
		
		m_pi.CO_nAction         = coaPlusMAJ;
		m_pi.CO_nNiveau         = 5;
		m_pi.CO_fActionDeplacer = FALSE;
		m_pi.CO_fActionNomCourt = FALSE;
		m_pi.CO_fArchiveMoins   = FALSE;
		m_pi.CO_fArchiveQue     = FALSE;
		m_pi.CO_nArbo           = arboSous;
		m_pi.CO_fQueNouveau     = FALSE;
		
		m_pi.SetDefaut();   // Uniquement pour RECHOPTIONS
	}
	delete [] pb;
	
	m_pi.DO_sFiltre  = GetProfileString(kszGeneralComp, eszFiltre);
	m_pi.DO_sContenu = GetProfileString(kszGeneralComp, eszContenu);
	m_pi.DA_rdC.SetNow();
	m_pi.DA_rdM.SetNow();
	
	// Options d'extraction
	if (GetProfileBinary(kszGeneral, eszGeneralExtract, &pb, &len) &&
	    len==sizeof(m_dec))
		m_dec = (DECOMP&)*pb;
	else{
		// Options par dfaut
		
		m_dec.nEcrase      = ecrDemander;
		m_dec.fEcraseDate  = FALSE;
		m_dec.fArbo        = TRUE;
		m_dec.fGardeErreur = FALSE;
		m_dec.fQueTrouve   = FALSE;
	}
	delete [] pb;
	
	// Options de recherche
	if (GetProfileBinary(kszGeneralRech, eszGeneral, &pb, &len) &&
	    len==sizeof(FILTREOPTIONS))
		(FILTREOPTIONS&)m_rch = (FILTREOPTIONS&)*pb;
	else
		// Options par dfaut
		m_rch.SetDefaut();
	delete [] pb;
	
	m_rch.DO_sFiltre  = GetProfileString(kszGeneralRech, eszFiltre);
	m_rch.DO_sContenu = GetProfileString(kszGeneralRech, eszContenu);
	m_rch.OP_fSous    = TRUE;
	m_rch.OP_fCache   = TRUE;
	
	// Police du visualiseur interne
	m_fntVoir.DeleteObject();
	if (GetProfileBinary(kszVoir, eszVoirPolice, &pb, &len) &&
	    len==sizeof(LOGFONT))
		m_fntVoir.CreateFontIndirect((const LOGFONT*)pb);
	else
		m_fntVoir.CreateStockObject(SYSTEM_FONT);
	delete [] pb;
	
	
	
	//*** Types et modles de documents
	
	// Crer les modles de documents
	if (!m_fSilent) {
		POS pos = m_lstDocType.GetPosDeb();
		while (pos)
			m_lstDocType.GetSuiv(pos)->AddDocTemplate();
		
		// Ajouter manuellement le modle de document des scripts
		// et celui des macros
		
		AddDocTemplate(m_pmdtScript = new CMuzipDocTemplate(
			NULL,
			IDR_SCRIPTTYPE, IDR_SCRIPTTYPE,
			RUNTIME_CLASS(CScriptDoc),
			RUNTIME_CLASS(CDlgFrame),
			RUNTIME_CLASS(CScriptView)));
		
		AddDocTemplate(m_pmdtMacro = new CMuzipDocTemplate(
			NULL,
			IDR_MACROTYPE, IDR_MACROTYPE,
			RUNTIME_CLASS(CMacroDoc),
			RUNTIME_CLASS(CDlgFrame),
			RUNTIME_CLASS(CMacroView)));
	}
	
	
	
	//*** Divers
	
	if (!m_fSilent) {
		
		// Crer le thread de surveillance gnrale
		CSurvDecomp::CreateThreadTous();
		
		// Charger la liste des derniers fichiers ouverts
		RebuildRecentFileList();
	}
	
	
	
	//*** Fentre principale
	
	HWND hwndMainAvant = NULL;
	
	// Afficher et mettre  jour la fentre principale
	if (!m_fSilent) {
		// Mode normal: on utilise CMainFrame
		
		hwndMainAvant = SendMuzipCmd(mucIsMuzip);
		CMainFrame *const pMainFrame = new CMainFrame;
		VERIF(pMainFrame->LoadFrame(IDR_MAINFRAME));
		m_pMainWnd = pMainFrame;
		
	}else{
		// Mode silencieux: on prend CFrameWnd pour ne pas avoir
		//  charger tous les lments de l'interface (barre d'outils,
		// barre d'tat, etc.)
		// CFrameWnd est mieux que CWnd car on n'a pas besoin
		// de grer la suppression avec delete
		
		m_pMainWnd = new CFrameWnd;
		m_pMainWnd->CreateEx(0, _T("STATIC"), NULL, 0, CRect(0,0,0,0), NULL, 0);
	}
	
	
	
	//*** Nettoyage du dossier temporaire
	
	
	if (!m_fSilent && e_opt.m_fDossiersNettoyerTemp) {
		// Nettoyer le dossier Temp:
		// supprimer les dossiers/fichiers crs il y a plus de 2 jours
		
		TCHAR pszDossierTemp[MAX_PATH];
		if (e_opt.GetDossier(repTemp, pszDossierTemp)) {
			
			// Rcuprer la date d'il y a 2 jours
			CTempsDHS dtTropVieux;
			dtTropVieux.SetNow();
			dtTropVieux.AddD(-2, dtpJour);
			
			// Effectuer le nettoyage
			NettoyerTemp(pszDossierTemp, dtTropVieux);
		}
	}
	
	
	
	
	//*** Ligne de commande
	
	
	// Analyser la ligne de commande
	CMuzipCmdLine cmdInfo;
	if (!m_fSilent) {
		cmdInfo.ParseCommandLine(m_lpCmdLine);
		
		// Si on peut passer la ligne de commande  un autre Muzip.exe, le faire
		if (hwndMainAvant) {
			
			// Composer la ligne de commande
			// Les arguments sont spars pardes 'cSepPartie'
			CString sCmd;
			for (int i=1; i<__argc; i++) {
				if (i>1)
					sCmd += cSepPartie;
				sCmd += __targv[i];
			}
			
			// Essayer d'excuter la ligne de commande
			if (SendMuzipCmd(mucCmdLine, sCmd, hwndMainAvant))
				return false;
		}
	}
	
	// Crer la fentre de progression
	// On la cre en disabled et on l'active aprs,
	// sinon il y a des problmes de clignotement
	m_pdlgProgress = new CDlgProgress;
	m_pdlgProgress->Create(CDlgProgress::IDD, m_pMainWnd);
	m_pdlgProgress->EnableWindow();
	
	// Grer la ligne de commande
	if (!m_fSilent) {
		
#if 0  // CLEAN Test de performances
		const DWORD tpsDebut = GetTickCount();
#endif
		
		VERIF(ProcessShellCommand(cmdInfo));
		
#if 0  // CLEAN Test de performances
		const DWORD tpsFin = GetTickCount();
		if (tpsFin!=tpsDebut) {
			CString s;
			s.Format(_T("%lu"), tpsFin - tpsDebut);
			AfxMessageBox(s);
			if (m_pMainWnd)
				m_pMainWnd->DestroyWindow();
			return FALSE;
		}
#endif
		
		// Enregistrer Muzip
		Register();
	}
	
	
	
	//*** Affichage fentre principale
	
	// Afficher la fentre principale
	// et activer l'ouverture par glisser-dplacer
	if (!m_fSilent) {
		m_pMainWnd->ShowWindow(SW_SHOW);
		m_pMainWnd->UpdateWindow();
		m_pMainWnd->DragAcceptFiles();
	}
	
	
	
	//*** Affectation des extensions d'archives  Muzip
	
	// Demande automatique d'affectation des extensions d'archives  Muzip
	if (!fSilent && e_opt.m_fGeneralAssocVerifAuto) {
		
		// Vrifier que les extensions sont affectes  Muzip
		POS pos = m_lstAssoc.GetPosDeb();
		while (pos) {
			CAssoc *const pAssoc = m_lstAssoc.GetSuiv(pos);
			
			// Ne pas toucher  l'extension .EXE !
			// ( cause des auto-extractibles)
			if (!lstrcmpi(pAssoc->m_pszExt, _T(".EXE")))
				continue;
			
			// Ouvrir la cl correspondant  l'association
			// et lire l'association courante
			HKEY hKey;
			TCHAR pszValeur[MAX_PATH];
			const bool fMuzip = pAssoc->GetAssocCur(hKey, pszValeur);
			if (hKey)
				RegCloseKey(hKey);
			
			// Regarder si la valeur trouve est celle de Muzip
			if (!fMuzip) {
				// a ne correspond pas: Muzip n'est pas l'archiveur par dfaut
				// Demander confirmation puis la bote de dialogue
				// si Oui a t rpondu
				// Si OK a t rpondu, enregistrer la DLL d'extension du shell
				if (CDlgConfirm::GetReponseMan(e_opt.m_fGeneralAssocVerifAuto, DEM_ASSOCTOUT, cfmPlusDemander) &&
				    CDlgAssoc().DoModal()==IDOK)
					DllRegisterServer();
				break;
			}
		}
	}
	
	
	
	//*** Bote de dialogue Ouvrir au lancement
	
	// Afficher la bote de dialogue Ouvrir au lancement
	// si aucune action n'a t faire  l'ouverture
	// et si l'option le permet
	if (!m_fSilent &&
	    cmdInfo.m_nShellCommand==cmdInfo.FileNothing &&
	    e_opt.m_fGeneralOuvrirAuto)
		OnFileOpen();
	
	return true;
}



// Charger la DLL des ressources
bool CMuzipApp::LoadResourcesDll()
{
	// Charger la DLL des ressources
	TCHAR pszResourceDll[MAX_PATH];
	::GetModuleFileName(AfxGetInstanceHandle(), pszResourceDll, nbArray(pszResourceDll));
	PathRemoveFileSpec(pszResourceDll);
	PathAppend(pszResourceDll, _T("Muzip%s.dll"));
	const HINSTANCE hInstRes = AfxLoadLangResourceDLL(pszResourceDll);
	
	// L'utiliser
	if (hInstRes) {
		const HINSTANCE hResourceHandle = AfxGetModuleState()->m_hCurrentResourceHandle;
		if (hResourceHandle)
			AfxFreeLibrary(hResourceHandle);
		AfxSetResourceHandle(hInstRes);
	}
	
	return ToBool(hInstRes);
}


// Nettoyer un dossier temporaire donn:
// supprimer tous les fichiers/sous-dossiers modifis avant une certaine date
// (la date du jour moins une semaine, voir CMuzipApp::Init)
void CMuzipApp::NettoyerTemp(LPCTSTR pszDossier, FILETIME ftLimite)
{
	TCHAR pszPath[MAX_PATH];
	PathCombine(pszPath, pszDossier, _T("*.*"));
	
	for (CMyFindFile ff(pszPath); ff.Next();) {
		if (ff.IsDots())
			continue;
		
		if (CompareFileTime(&ff.m_wfd.ftCreationTime, &ftLimite)>0)
			// Le fichier/dossier est trop rcent pour tre nettoy
			continue;
		
		PathCombine(pszPath, pszDossier, ff.m_wfd.cFileName);
		
		if (ff.m_wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
			// Sous-dossier: le nettoyer
			
			// Nettoyer le contenu du sous-dossiers
			NettoyerTemp(pszPath, ftLimite);
			
			// Essayer de supprimer le sous-dossier
			RemoveDirectory(pszPath);
			
		}else
			// Fichier: le supprimer
			
			FichierSupprimer(pszPath, false);
	}
}


// Charger la liste des derniers fichiers ouverts
void CMuzipApp::RebuildRecentFileList()
{
	// Sauver et dtruire l'ancienne liste
	if (m_pRecentFileList) {
		m_pRecentFileList->WriteList();
		delete m_pRecentFileList;
		m_pRecentFileList = NULL;
	}
	
	// Crer et charger la nouvelle liste
	if (e_opt.m_nGeneralMRU > 0) {
		m_pRecentFileList = new CRecentFileList(0,
			CStringRes(IDS_RECENT1), CStringRes(IDS_RECENT2),
			e_opt.m_nGeneralMRU);
		m_pRecentFileList->ReadList();
	}
}



// Terminer l'application
bool CMuzipApp::Exit()
{
	VERIF(!m_fErreurFatale);
	
	
	POS pos;
	
	// Dtruire les fichiers  surveiller
	pos = m_lstSurvDecomp.GetPosDeb();
	while (pos) {
		CSurvDecomp *const psd = m_lstSurvDecomp.GetSuiv(pos);
		if (!psd->m_hProcess)
			DeleteFile(psd->m_pszFichier);
	}
	CSurvDecomp::TerminateThreadTous();
	
	// Dtruire les dossiers  surveiller
	pos = m_lstDelDossier.GetPosDeb();
	while (pos)
		DossierSupprimer(m_lstDelDossier.GetSuiv(pos));
	m_lstDelDossier.DeleteAll();
	
	// Dtruire la fentre de progression
	if (m_pdlgProgress) {
		m_pdlgProgress->DestroyWindow();
		delete m_pdlgProgress;
		m_pdlgProgress = NULL;
	}
	
	
	//*** Enregistrer les options
	
	// Options gnrales
	e_opt.Save();
	
	// Options de compression par dfaut
	WriteProfileBinary(kszGeneralComp, eszGeneral, (BYTE*)&m_pi, sizeof(m_pi));
	WriteProfileString(kszGeneralComp, eszFiltre,  m_pi.DO_sFiltre);
	WriteProfileString(kszGeneralComp, eszContenu, m_pi.DO_sContenu);
	
	// Associations
	{
		// Copier les associations dans un tableau
		CAssoc *const aAssoc = new CAssoc[m_lstAssoc.GetNb()];
		POS pos = m_lstAssoc.GetPosDeb();
		for (int iAssoc=0; pos; iAssoc++)
			memcpy(&aAssoc[iAssoc], m_lstAssoc.GetSuiv(pos),
				sizeof(aAssoc[iAssoc]));
		
		// crire le tout dans le registre
		WriteProfileInt   (kszGeneral, eszGeneralNbAssoc, m_lstAssoc.GetNb());
		WriteProfileBinary(kszGeneral, eszGeneralAssoc,   (BYTE*)aAssoc,
			m_lstAssoc.GetNb()*sizeof(aAssoc[0]));
		
		delete [] aAssoc;
	}
	
	// Options d'extraction
	WriteProfileBinary(kszGeneral, eszGeneralExtract, (BYTE*)&m_dec, sizeof(m_dec));
	
	// Options de recherche
	WriteProfileBinary(kszGeneralRech, eszGeneral,
		(BYTE*)(FILTREOPTIONS*)&m_rch, sizeof(FILTREOPTIONS));
	WriteProfileString(kszGeneralRech, eszFiltre,  m_rch.DO_sFiltre);
	WriteProfileString(kszGeneralRech, eszContenu, m_rch.DO_sContenu);
	
	// Police des visualiseurs
	if (m_fntVoir.m_hObject) {
		LOGFONT lf;
		m_fntVoir.GetLogFont(&lf);
		WriteProfileBinary(kszVoir, eszVoirPolice, (BYTE*)&lf, sizeof(lf));
	}
	
	// Option des types de documents
	pos = m_lstDocType.GetPosDeb();
	while (pos)
		m_lstDocType.GetSuiv(pos)->LoadSaveOptions(false);
	
	
	// Dtruire la fentre principale
	if (m_pMainWnd)
		m_pMainWnd->DestroyWindow();
	
	
	// Nettoyage final
	m_lstDocType.DeleteAll();
	
	return true;
}


// Terminer l'application ( n'appeler qu'une seule fois)
void CMuzipApp::ExitOnce()
{
	// Nettoyer
	UNZ::InflateClose();
	
	
	if (m_fErreurFatale)
		return;
	
	
	// On n'a plus besoin des ressources
	AfxFreeLibrary(AfxGetResourceHandle());
}



// A propos
void CMuzipApp::OnAppAbout()
{
	CDlgAPropos().DoModal();
}


// Excuter une ligne de commande
bool CMuzipApp::ProcessShellCommand(CMuzipCmdLine& cmdInfo, SOURCE_ACTION sa)
{
	const SOURCE_ACTION saOld = m_sa;
	m_sa = sa;
	
	// Dfinir le rpertoire courant  partir du premier fichier
	CString sRep, sRepParent;
	if (cmdInfo.m_arPlus.GetNb()) {
		
		// Dcompresser le premier fichier
		AfxExtractSubString(sRep, cmdInfo.m_arPlus.GetDeb()->DO_sFiltre,
			0, cSepPartie);
		
		// Supprimer la spcification de fichier
		PathRemoveFileSpec(sRep.GetBuffer(MAX_PATH));
		sRep.ReleaseBuffer();
		SetCurrentDirectory(sRep);
		
		// Prendre le deuxime parent
		sRepParent = sRep;
		PathRemoveFileSpec(sRepParent.GetBuffer(MAX_PATH));
		sRepParent.ReleaseBuffer();
	}
	
	
	// On devra faire le mnage aprs, il faut donc garder une trace
	// du document ou de l'archive ouverte
	CDocument *pDoc = NULL;
	CMuzipArc *pArc = NULL;
	const int nbDoc = GetOpenDocumentCount();
	
	
	switch ((int)cmdInfo.m_nShellCommand) {
		
		// Ouvrir archive ou script
		// Grer le cas de l'excution de script
		case cmdInfo.FileOpen:
			{
				// Ouvrir le document
				CDocument *pDoc = OpenDocumentFile(cmdInfo.m_strFileName);
				CScriptDoc *const pScriptDoc = DYNAMIC_DOWNCAST(CScriptDoc, pDoc);
				if (pScriptDoc) {
					// S'il s'agit d'un script, l'excuter et le fermer
					pScriptDoc->OnScriptExecuter();
					pDoc->DoFileSave();
					pDoc->OnCloseDocument();
				}
			}
			break;
		
		
		// Compresser
		case FileComp:
			{
				if (cmdInfo.m_strFileName.IsEmpty()) {
					// Demander le nom de l'archive
					
					SetCurrentDirectory(sRepParent);
					
					// Afficher la bote de dialogue ouvrir
					CFileDialogEx dlgOpen(TRUE, pszExtDefaut, NULL, fdFlagsCommon);
					if (!ShowFileOpenDialog(dlgOpen, sfodPasScript | sfodPasLectureSeule))
						break;
					cmdInfo.m_strFileName = dlgOpen.GetPathName();
					delete [] dlgOpen.m_ofn.lpstrFile;
					
					SetCurrentDirectory(sRep);
				}
				
				// Si l'archive n'existe pas dj,
				// il faudra la supprimer en cas d'erreur
				bool fSupprimer = !PathFileExists(cmdInfo.m_strFileName);
				
				if (FichierCreer(cmdInfo.m_strFileName)) {
					
					// Ouvrir l'archive
					pArc = OpenArchive(cmdInfo.m_strFileName);
					if (!pArc)
						goto FinComp;
					
					// S'occuper des options:
					// prendre les options par dfaut,
					// et afficher la bote de dialogue si cela est demand
					// dans les options de Muzip
					CPlusItem pi;
					pi.SetDefautExt();
					if (cmdInfo.m_fDemanderOptions) {
						CDlgPlusItem dlgPI(pArc, NULL, false, pArc->m_sMdp);
						(CPlusItem&)dlgPI = pi;
						if (dlgPI.DoModal()==IDCANCEL)
							goto FinComp;
						pi = dlgPI;
					}
					
					// Appliquer les options
					POS pos = cmdInfo.m_arPlus.GetPosDeb();
					while (pos)
						(PLUSOPTIONS&)*cmdInfo.m_arPlus.GetSuiv(pos) = pi;
					
					// Effectuer la compression
					fSupprimer &= !pArc->ArchivePlus(cmdInfo.m_arPlus) &&
						!pArc->Reconstruire();
				}
FinComp:
				if (fSupprimer) {
					if (pArc) {
						pArc->Release(false);
						pArc = NULL;
					}
					DeleteFile(cmdInfo.m_strFileName);
				}
			}
			break;
		
		
		// Dcompresser
		case FileDecomp:
			{
				if (cmdInfo.m_strFileName.IsEmpty())
					break;
				
				// Ouvrir l'archive
				pArc = OpenArchive(cmdInfo.m_strFileName);
				if (!pArc)
					break;
				
				// Demander les options si ncessaire
				// Utiliser le dossier spcifi par /d s'il l'a t
				CDlgDecomp dlgDecomp(false, pArc);
				if (dlgDecomp.Init(cmdInfo.m_sDest, cmdInfo.m_sOptions) ||
				    !cmdInfo.m_fDemanderOptions ||
				    dlgDecomp.DoModal()==IDOK)
					pArc->DossierDecomp(dlgDecomp.m_sDest, &dlgDecomp.m_dec);
			}
			break;
		
		
		// Envoyer mail
		case FileSendMail:
			{
				if (cmdInfo.m_strFileName.IsEmpty())
					break;
				
				// Ouvrir l'archive
				m_fOuvreVisible = false;
				pDoc = OpenDocumentFile(cmdInfo.m_strFileName);
				CMuzipDoc *const pMuzipDoc = DYNAMIC_DOWNCAST(CMuzipDoc, pDoc);
				if (pMuzipDoc)
					pMuzipDoc->OnFileSendMail();
			}
			break;
		
		
		// Crer raccourci de dcompression
		case FileShortcut:
			{
				DECOMP dec = m_dec;
				dec.FromOptions(cmdInfo.m_sOptions);
				dec.ToOptions(cmdInfo.m_sOptions);
				
				CreateArchiveShortcut(cmdInfo.m_strFileName,
					cmdInfo.m_sShortcut, cmdInfo.m_sDest, cmdInfo.m_sOptions,
					cmdInfo.m_fDemanderOptions, false);
			}
			break;
		
		
		// diter script
		// Doit tre juste avant "default:"
		case FileScriptEdit:
			cmdInfo.m_nShellCommand = cmdInfo.FileOpen;
		
		
		// Autre action
		default:
			CWinApp::ProcessShellCommand(cmdInfo);
			break;
	}
	
	// Fermer l'archive ou le document ouvert
	if (pArc)
		pArc->Release(false);
	else if (pDoc && nbDoc!=GetOpenDocumentCount()) {
		pDoc->DoFileSave();
		pDoc->OnCloseDocument();
	}
	
	
	m_sa = saOld;
	
	
	// Fermer la fentre principale si on doit quitter
	// aprs l'excution de la commande
	if (cmdInfo.m_fQuitter && m_pMainWnd)
		m_pMainWnd->DestroyWindow();
	
	
	// Arrter l l'excution si on doit quitter
	return !cmdInfo.m_fQuitter;
}


// Modifie les options de compression
// d'aprs ce qui est spcifi dans une ligne de commande
void DECOMP::FromOptions(LPCTSTR psz)
{
	int aOptions[5]; // Options
	
	// Lire les options spares par des tirets
	// Les options ignores ou invalides sont mises  -1
	CTokenize tok(psz, '-');
	while (tok.Next() && tok.GetIndex()<nbArray(aOptions)) {
		LPTSTR pszFin;
		int val = _tcstoul(tok.GetToken(), &pszFin, 10);
		if (pszFin==tok.GetToken() || val<0)
			val = -1;
		aOptions[tok.GetIndex()] = val;
	}
	
	int *piOption = aOptions;
	CMuzipCmdLine::GetOption(piOption, nEcrase, ecrMax);
	CMuzipCmdLine::GetOption(piOption, fEcraseDate);
	CMuzipCmdLine::GetOption(piOption, fArbo);
	CMuzipCmdLine::GetOption(piOption, fGardeErreur);
	fQueTrouve = FALSE;
}


// Stocke les options de compression dans une ligne de commande
void DECOMP::ToOptions(CString& s) const
{
	CMuzipCmdLine::AddOption(s, nEcrase);
	CMuzipCmdLine::AddOption(s, fEcraseDate);
	CMuzipCmdLine::AddOption(s, fArbo);
	CMuzipCmdLine::AddOption(s, fGardeErreur);
}



// Demander  l'utilisateur s'il ne faudrait pas ignorer les prochaines erreurs
bool CMuzipApp::DemanderEncoreErreur()
{
	switch (AfxMessageBox(DEM_ENCOREERREUR, MB_YESNOCANCEL)) {
		
		// Annuler: on ne fait rien
		case IDCANCEL:
			return false;
		
		// Oui: il faudra  l'avenir rpondre automatiquement aux messages d'erreur
		case IDYES:
			m_fMsg = false;
			break;
	}
	
	return true;
}


// Retourner le document correspondant  un nom de fichier
// ou NULL si le document n'est pas ouvert
CDocument* CMuzipApp::FindDocument(LPCTSTR pszFichier)
{
	// Pour chaque type de document,
	// regarder si le document est dj ouvert
	POS pos = m_lstDocType.GetPosDeb();
	while (pos) {
		CDocType &dt = *m_lstDocType.GetSuiv(pos);
		
		CDocument *pDoc;
		if (dt.MatchDocType(pszFichier, pDoc)==CDocTemplate::yesAlreadyOpen)
			// Le document est ouvert
			return pDoc;
	}
	
	return NULL;
}


// Attendre que l'application n'ait rien  faire
void CMuzipApp::DoEvents()
{
	MSG msg;
	while (::PeekMessage(&msg, NULL,0,0, PM_REMOVE)) {
		if (!oApp.PreTranslateMessage(&msg)) {
			::TranslateMessage(&msg);
			::DispatchMessage(&msg);
		}
	}
}


// Dfinir le texte de la barre d'tat
void CMuzipApp::SetMessageText(UINT nID)
{
	SetMessageText(CStringRes(nID));
}
void CMuzipApp::SetMessageText(LPCTSTR psz)
{
	((CFrameWnd*)m_pMainWnd)->SetMessageText(psz);
	SetStatusLabel(IDCLBL, psz);
}

// Mettre  jour les informations sur la progression:
// En haut:          <nID> (<iCur>/<iTotal>)
// Juste en dessous: <NomFichier>
// - Un seul fichier
void CMuzipApp::SetProgress(UINT nID, LPCTSTR pszFichier)
{
	AfxFormatString1(e_sMsg, nID, NULL);
	SetMessageText(e_sMsg);
	SetStatusFichier(pszFichier);
}
// - Un fichier parmi plusieurs
void CMuzipApp::SetProgress(UINT nID, LPCTSTR pszFichier, UINT iCur, UINT iTotal)
{
	CString s;
	s.Format(_T("%u/%u"), iCur,iTotal);
	AfxFormatString1(e_sMsg, nID, s);
	SetMessageText(e_sMsg);
	SetStatusFichier(pszFichier);
}

// Initialiser l'indicateur de progression
void CMuzipApp::InitStatusAvance(const CMuzipArc* pArc, UINT nIDSTitre)
{
	if (pArc)
		AfxFormatString1(m_sdlgProgress, nIDSTitre,
			PathFindFileName(pArc->GetPath()));
	else
		m_sdlgProgress.Empty();
}

// Afficher l'indicateur de progression
void CMuzipApp::ShowStatusAvance(DWORD nMax, bool fPeutIgnorer)
{
	m_aPos[0] = 0;
	m_aMax[0] = nMax;
	
	m_ndlgProgress++;
	
	BOOL fProgressAfficher;
	switch (m_sa) {
		case saDrop:      fProgressAfficher = e_opt.m_fProgressAfficherDrop;      break;
		case saExplorer:  fProgressAfficher = e_opt.m_fProgressAfficherExplorer;  break;
		default:          fProgressAfficher = e_opt.m_fProgressAfficherMuzip;     break;
	}
	if (!fProgressAfficher)
		return;
	
	
	if (!m_pdlgProgress->IsWindowVisible()) {
		// Masquer la deuxime barre de progression
		HideStatusAvance2();
		// Crer le timer d'attente avant l'affichage
		m_pdlgProgress->SetTimer(idTimerDlgProgress, e_opt.m_nProgressDelai, NULL);
	}
	
	// Initialiser le titre, vider la zone du fichier courant
	if (!m_sdlgProgress.IsEmpty())
		m_pdlgProgress->SetWindowText(m_sdlgProgress);
	SetStatusFichier(NULL);
	
	// Initialiser l'indicateur
	UpdateProgressPos(0);
	
	m_pdlgProgress->m_cmdIgnorer.EnableWindow(fPeutIgnorer);
	
	// Mettre  jour la fentre de progression
	m_pdlgProgress->UpdateWindow();
	
	// Attendre que l'application ait tout redessin
	DoEvents();
}

// Afficher le second indicateur de progression
void CMuzipApp::ShowStatusAvance2(DWORD nMax)
{
	m_aPos[1] = 0;
	m_aMax[1] = nMax;
	
	if (!e_opt.m_fProgressDetail)
		return;
	
	ASSERT(m_ndlgProgress>0);
	UpdateProgressPos(1);
	
	if (!m_pdlgProgress->m_f2) {
		// Afficher la seconde barre de progression
		
		m_pdlgProgress->m_f2 = true;
		
		// Afficher la seconde barre de progression et son label
		m_pdlgProgress->m_prg[1].ShowWindow(SW_SHOW);
		m_pdlgProgress->GetDlgItem(IDCLBL_2)->ShowWindow(SW_SHOW);
		
		const int yDifTotal = 70;
		const int yDif2     = yDifTotal/2;
		
		// Repositionner les boutons
		CButton* aBouton[] =
		{
			&m_pdlgProgress->m_cmdIgnorer,
			&m_pdlgProgress->m_cmdAnnuler,
		};
		for (int i=0; i<nbArray(aBouton); i++) {
			CButton &cmd = *aBouton[i];
			CRect rcBouton;
			
			cmd.GetWindowRect(rcBouton);
			rcBouton.top    += yDifTotal;
			rcBouton.bottom += yDifTotal;
			m_pdlgProgress->ScreenToClient(rcBouton);
			cmd.MoveWindow(rcBouton);
		}
		
		// Agrandir la fentre en hauteur
		CRect rc;
		m_pdlgProgress->GetWindowRect(rc);
		rc.top    -= yDif2;
		rc.bottom += yDif2;
		m_pdlgProgress->MoveWindow(rc);
	}
}


// Dfinir un label de la fentre de progression
void CMuzipApp::SetStatusLabel(UINT nIDC, UINT nIDS)
{
	SetStatusLabel(nIDC, CStringRes(nIDS));
}
void CMuzipApp::SetStatusLabel(UINT nIDC, LPCTSTR psz)
{
	m_pdlgProgress->SetDlgItemText(nIDC, psz);
}

// Dfinir le fichier affich dans la fentre de progression
void CMuzipApp::SetStatusFichier(LPCTSTR pszFichier)
{
	CWnd *const plblFichier = m_pdlgProgress->GetDlgItem(IDCLBL_FICHIER);
	
	// Rcuprer la largeur du label
	CRect rc;
	plblFichier->GetClientRect(rc);
	
	TCHAR pszTemp[MAX_PATH] = _T("");
	lstrcpy(pszTemp, pszFichier);
	
	// Compacter le nom du fichier
	CDC *const pdc = plblFichier->GetDC();
	pdc->SelectObject(plblFichier->GetFont());
	PathCompactPath(*pdc, pszTemp, rc.right);
	plblFichier->ReleaseDC(pdc);
	
	// L'affecter au label
	SetStatusLabel(IDCLBL_FICHIER, pszTemp);
}


// Masquer l'indicateur de progression
void CMuzipApp::HideStatusAvance()
{
	ASSERT(m_ndlgProgress>0);
	m_ndlgProgress--;
	
	if (!m_ndlgProgress) {
		ResetProgressStatus();
		HideStatusAvance2();
		m_pdlgProgress->KillTimer(idTimerDlgProgress);
		m_pdlgProgress->ShowWindow(SW_HIDE);
	}
	
	oApp.SetMessageText(AFX_IDS_IDLEMESSAGE);
}

// Masquer le second indicateur de progression
void CMuzipApp::HideStatusAvance2()
{
	if (m_pdlgProgress->m_f2) {
		// Masquer la deuxime barre de progression
		// en jouant sur la hauteur de la bote de dialogue
		// et la position verticale du bouton Annuler
		
		m_pdlgProgress->m_f2 = false;
		
		// Masquer la seconde barre de progression et son label
		m_pdlgProgress->m_prg[1].ShowWindow(SW_HIDE);
		m_pdlgProgress->GetDlgItem(IDCLBL_2)->ShowWindow(SW_HIDE);
		
		// Rtrcir la fentre en hauteur
		CRect rc;
		m_pdlgProgress->GetWindowRect(rc);
		const int yDifTotal = 70;
		const int yDif2     = yDifTotal/2;
		rc.top    += yDif2;
		rc.bottom -= yDif2;
		m_pdlgProgress->MoveWindow(rc);
		
		// Repositionner les boutons
		CButton* aBouton[] =
		{
			&m_pdlgProgress->m_cmdIgnorer,
			&m_pdlgProgress->m_cmdAnnuler,
		};
		for (int i=0; i<nbArray(aBouton); i++) {
			CButton &cmd = *aBouton[i];
			CRect rcBouton;
			
			cmd.GetWindowRect(rcBouton);
			rcBouton.top    -= yDifTotal;
			rcBouton.bottom -= yDifTotal;
			m_pdlgProgress->ScreenToClient(rcBouton);
			cmd.MoveWindow(rcBouton);
		}
	}
}

// Dfinir la valeur de l'indicateur de progression
void CMuzipApp::SetStatusAvance(DWORD n)
{
	m_aPos[0] = n;
	UpdateProgressPos(0);
}
void CMuzipApp::AddStatusAvance(int iType, DWORD n)
{
	ASSERT(iType==0 || iType==1);
	
	m_aPos[iType] += n;
	UpdateProgressPos(iType);
}


// Met  jour la valeur affiche par une barre de progression
// Comme cette valeur est susceptible de changer trs souvent
// et trs rapidement, on acclre sensiblement Muzip en vitant
// de modifier deux fois de suite une barre de progression
//  un intervalle trs court
// Cela fait gagner pas mal de temps  la compression !
void CMuzipApp::UpdateProgressPos(int iType)
{
	// Calculer le nouveau pourcentage
	const int nPC = GetPourcent(m_aPos[iType], m_aMax[iType]);
	if (nPC==m_aPC[iType])
		return;
	m_aPC[iType] = nPC;
	
	static DWORD s_atpsMAJ[2] = { 0,0 };
	
	// Regarder s'il s'est coul assez de temps
	// depuis la dernire mise  jour
	const DWORD tps = GetTickCount();
	if (tps > s_atpsMAJ[iType]) {
		// On peut mettre  jour
		
		s_atpsMAJ[iType] = tps + tpsDelaiMAJ;
		m_pdlgProgress->m_prg[iType].SetPos(nPC);
	}
}



// Retourner le statut courant de la progression
PRGSTATUS CMuzipApp::GetProgressStatus()
{
	return m_pdlgProgress->m_prs;
}

// Retourner le statut courant de la progression
// aprs lui avoir permis de se mettre  jour
PRGSTATUS CMuzipApp::UpdateProgressStatus()
{
	static DWORD s_tpsMAJ = 0;
	
	// Regarder s'il s'est coul assez de temps
	// depuis la dernire mise  jour
	const DWORD tps = GetTickCount();
	if (tps > s_tpsMAJ) {
		// On peut mettre  jour
		s_tpsMAJ = tps + tpsDelaiMAJ;
		DoEvents();
	}
	
	return GetProgressStatus();
}

// Remettre  zro le statut courant de progression
void CMuzipApp::ResetProgressStatus()
{
	m_pdlgProgress->m_prs = prsOK;
}


// Rien  faire
BOOL CMuzipApp::OnIdle(LONG lCount)
{
	// Surveiller des fichiers dcompresss
	POS pos = m_lstSurvDecomp.GetPosDeb();
	while (pos)
		m_lstSurvDecomp.GetSuiv(pos)->Main();
	
	return CWinApp::OnIdle(lCount);
}


// Bote de message
int CMuzipApp::DoMessageBox(LPCTSTR lpszPrompt, UINT nType, UINT nIDPrompt)
{
	if (m_fMsg)
		return CWinApp::DoMessageBox(lpszPrompt, nType, nIDPrompt);
	
	
	// Rpondre automatiquement  la question pose
	switch (nType) {
		
		case MB_ABORTRETRYIGNORE:
			return IDIGNORE;
		
		case MB_RETRYCANCEL:
			return IDCANCEL;
		
		case MB_YESNO:
		case MB_YESNOCANCEL:
			return IDYES;
		
		default:
			return IDOK;
	}
}


// Affecter l'icne Parcourir  un bouton
void CMuzipApp::SetIconeParcourir(CWnd* pWnd, UINT nID)
{
	((CButton*)pWnd->GetDlgItem(nID))->SetIcon(
		ImageList_GetIcon(m_himlSystem,
			m_iIconRepClosed, ILD_NORMAL));
}


// Nouvelle archive
void CMuzipApp::OnFileNew()
{
	CDlgNouveau dlgNouveau;
	if (dlgNouveau.DoModal()==IDCANCEL)
		return;
	const CString &sFichier = dlgNouveau.GetPathName();
	
	// Fermer l'ancien document si ncessaire
	CDocument *const pDocDeja = FindDocument(sFichier);
	if (pDocDeja)
		pDocDeja->OnCloseDocument();
	
	// Crer un fichier vide
	CFileWrite fw;
	if (!fw.Open(sFichier, FILE_SHARE_ALL))
		return;
	fw.Close();
	
	// Ouvrir le document et afficher la bote de dialogue Ajouter
	// si cela a t demand par l'utilisateur
	CMuzipDoc *const pDoc = DYNAMIC_DOWNCAST(CMuzipDoc, OpenDocumentFile(sFichier));
	if (dlgNouveau.m_fAjouter && pDoc)
		pDoc->OnArchivePlus();
}

// Nouveau script
void CMuzipApp::OnFileNewScript()
{
	m_pmdtScript->OpenDocumentFile(NULL);
}


// Ouvrir
void CMuzipApp::OnFileOpen()
{
	CFileDialogEx dlgOpen(TRUE, pszExtDefaut, NULL,
		fdFlagsOpen | OFN_ALLOWMULTISELECT, NULL, NULL);
	ShowFileOpenDialog(dlgOpen, sfodOuvrir);
}


// Rparer
void CMuzipApp::OnFileReparer()
{
	e_opt.m_nReparer = repAltere;
	
	CDlgReparer dlgReparer;
	ShowFileOpenDialog(dlgReparer, sfodOuvrir | sfodPasScript);
	
	e_opt.m_nReparer = repNon;
}


// Raccourci de dcompression
void CMuzipApp::OnFileRaccourci()
{
	CString sArchive;
	
	// Rcuprer l'archive courante
	CFrameWnd *const pFrame = ((CFrameWnd*)m_pMainWnd)->GetActiveFrame();
	if (pFrame) {
		CDocument *const pDoc = pFrame->GetActiveDocument();
		if (pDoc) {
			if (pDoc->IsKindOf(RUNTIME_CLASS(CMuzipDoc)))
				sArchive = pDoc->GetPathName();
			else // (pDoc->IsKindOf(RUNTIME_CLASS(CScriptDoc))
				((CScriptDoc*)pDoc)->EtendreSequences(sArchive, false);
		}
	}
	
	
	// Demander les options
	CDlgRaccourci dlgRaccourci((sArchive.IsEmpty()) ? 0 : -1);
	dlgRaccourci.m_dec = m_dec;
	if (dlgRaccourci.DoModal()==IDCANCEL)
		return;
	
	// Demander le fichier cible
	const CString sFilter(MAKEINTRESOURCE(IDS_FILTRERAC));
	CString sRaccourci = sArchive;
	if (!sRaccourci.IsEmpty()) {
		PathRenameExtension(sRaccourci.GetBuffer(MAX_PATH), _T(".lnk"));
		sRaccourci.ReleaseBuffer();
	}
	CFileDialogEx dlgLink(FALSE, _T("lnk"), sRaccourci,
		fdFlagsSave | OFN_NODEREFERENCELINKS,
		sFilter);
	if (dlgLink.DoModal()==IDCANCEL)
		return;
	
	// Crer le raccourci
	CString sOptions;
	dlgRaccourci.m_dec.ToOptions(sOptions);
	CreateArchiveShortcut(sArchive, dlgLink.GetPathName(),
		dlgRaccourci.m_sDest, sOptions,
		ToBool(dlgRaccourci.m_fDemanderOptions), true);
}


// Options
void CMuzipApp::OnFileOptions()
{
	CDlgOptions().DoModal();
}


// Dcompression multiple
void CMuzipApp::OnFileDecompMult()
{
	CDlgDecompMult dlgDecompMult;
	
	if (dlgDecompMult.DoModal()==IDOK) {
		// Effectuer la dcompression
		
		BOOL bConfirmErreur = TRUE;
		
		POS pos = dlgDecompMult.m_lstArchive.GetPosDeb();
		while (pos) {
			CString &sArchive = dlgDecompMult.m_lstArchive.GetSuiv(pos);
			
			// Ouvrir l'archive et grer les erreurs
			CMuzipArc *const pArc = OpenArchive(sArchive, NULL, NULL, !bConfirmErreur);
			if (!pArc && bConfirmErreur) {
				// Erreur: demander  l'utilisateur s'il veut arrter
				// et proposer d'ignorer les prochaines erreurs
				
				if (!CDlgConfirm::GetReponseMan(bConfirmErreur, DEM_CONFIRMERREUR))
					// L'utilisateur a choisi d'arrter le traitement
					break;
			}
			
			// Composer le nom du sous-dossier
			// Si l'option "" est utilise
			CString sDest = dlgDecompMult.m_sDest;
			if (dlgDecompMult.m_fDecompSousDossier) {
				
				// Rcuprer le titre de l'archive (sans extension)
				CString sTitle = PathFindFileName(sArchive);
				PathRemoveExtension(sTitle.GetBuffer(MAX_PATH));
				sTitle.ReleaseBuffer();
				
				// L'ajouter au rpertoire de destination
				PathAppend(sDest.GetBuffer(MAX_PATH), sTitle);
				sDest.ReleaseBuffer();
			}
			
			// Effectuer la dcompression
			pArc->DossierDecomp(sDest, &dlgDecompMult.m_dec);
			
			pArc->Release(false);
		}
	}
}


// Macros
void CMuzipApp::OnMacroGerer()
{
	CDlgMacroGerer().DoModal();
}



// Afficher une CFileDialog d'ouverture de fichier
// pdtExclusif restreint la liste des formats au type spcifi
// Retourne TRUE si l'utilisateur a valid
// Retourne TRUE mme si sofdOuvrir a t spcifi et si
// certains fichiers (voire tous) n'ont pas pu tre ouverts
//
// IMPORTANT
// ---------
// 
// Si la fonction retourne TRUE et si sfodOuvrir n'est pas utilis,
// l'appelant doit librer dlg.m_ofn.lpstrFile en excutant:
// delete [] dlg.m_ofn.lpstrFile
//
bool CMuzipApp::ShowFileOpenDialog(CFileDialogEx& dlg, DWORD sfod,
	CDocType* pdtExclusif)
{
	// Crer la liste des filtres
	CString sExtDefaut, sFilterTous, sFilterAutre, sDefault;
	InitFileOpenStruct(dlg.m_ofn, sExtDefaut,
		sFilterTous, sFilterAutre, sfod, pdtExclusif);
	
	// Fin du formattage du filtre
	CString sFilter;
	AfxFormatString2(sFilter,
		(pdtExclusif)
			? IDS_FILTREEXCLUSIF
			: ((sfod & sfodPasScript) ? IDS_FILTREARCHIVE : IDS_FILTREOPEN),
		sFilterTous, sFilterAutre);
	dlg.m_ofn.nMaxCustFilter++;
	sFilter.Replace('|', 0);
	
	//*** Initialiser la bote de dialogue
	
	// Dossier initial
	TCHAR pszRep[MAX_PATH];
	GetCurrentDirectory(nbArray(pszRep), pszRep);
	
	// Tampon pour les fichiers
	const int bufFichiers = 4<<10;
	const LPTSTR pszFichiers = new TCHAR[bufFichiers];
	lstrcpy(pszFichiers, dlg.m_ofn.lpstrFile);
	
	// Autres paramtres
	dlg.m_ofn.lpstrFilter     = sFilter;
	dlg.m_ofn.nFilterIndex    = 1;
	dlg.m_ofn.lpstrFile       = pszFichiers;
	dlg.m_ofn.nMaxFile        = bufFichiers;
	dlg.m_ofn.lpstrInitialDir = pszRep;
	
	
	// Afficher la bote de dialogue
	if (dlg.DoModal()!=IDOK) {
		// Erreur: on libre la mmoire
		
		delete [] pszFichiers;
		dlg.m_ofn.lpstrFile = NULL;
		return false;
	}
	
	
	if (sfod & sfodOuvrir) {
		// Ouvrir les documents choisis
		
		POSITION pos = dlg.GetStartPosition();
		while (pos)
			oApp.OpenDocumentFile(dlg.GetNextPathName(pos));
		
		// Librer le tampon
		delete [] pszFichiers;
		dlg.m_ofn.lpstrFile = NULL;
	}
	
	return true;
}


// Rcuprer le filtre (au format de CFileDialog, avec des '|')
// pour tous les types d'archives
// sExtDefaut ne sert qu'au stockage de l'extension par dfaut
void CMuzipApp::InitFileOpenStruct(OPENFILENAME& ofn, CString& sExtDefaut,
	CString& sTous, CString& sAutre, DWORD sfod, CDocType* pdtExclusif)
{
	bool fPrem = true;
	POS pos = m_lstDocType.GetPosDeb();
	while (pos) {
		CDocType &dt = *m_lstDocType.GetSuiv(pos);
		
		// Si demand, ignorer les formats en lecture seule
		if ((sfod & sfodPasLectureSeule) && dt.GetIdType()==IDR_SFXTYPE)
			continue;
		
		// Grer l'exclusivit du format (un seul format possible)
		if (pdtExclusif && dt.GetIdType()!=pdtExclusif->GetIdType())
			continue;
		
		
		CString sFilterExt, sFilterName;
		dt.GetDocString(sFilterExt,  CDocTemplate::filterExt);
		dt.GetDocString(sFilterName, CDocTemplate::filterName);
		
		// Rcuprer l'extension par dfaut
		CString sExtDefautCur;
		dt.GetExtDefaut(sExtDefautCur);
		
		if (fPrem && !ofn.lpstrDefExt)
			// Dfinir l'extension par dfaut (sans le point)
			ofn.lpstrDefExt = sExtDefaut = (LPCTSTR)sExtDefautCur + 1;
		
		// Dfinir le filtre
		// Mettre l'extension par dfaut en premier,
		// car lpstrDefExt ne sert que pour le filtre "*.*"
		// et l'extension par dfaut utilise sinon est la premire du filtre
		sAutre += sFilterName;
		sAutre += '|';
		CString sPlus = '*';
		sPlus += sExtDefautCur;
		CTokenize tok(sFilterExt, ';');
		bool fPremExt = true;
		while (tok.Next()) {
			if (fPremExt)
				fPremExt = false;
			else
				sPlus += ';';
			
			if (lstrcmpi(tok.GetToken(), sExtDefautCur)) {
				// N'ajouter l'extension que si ce n'est pas l'extension par dfaut
				// qui est ajoute manuellement au dbut
				sPlus += '*';
				sPlus += tok.GetToken();
			}
		}
		if (!fPrem)
			sTous += ';';
		sTous  += sPlus;
		sAutre += sPlus;
		sAutre += '|';
		ofn.nMaxCustFilter++;
		
		fPrem = false;
	}
	
	// Ajouter le filtre universel
	sAutre += CStringRes(AFX_IDS_ALLFILTER);
	sAutre += _T("|*.*|");
}


// Dfinir la police du visualiseur interne
void CMuzipApp::SetFontVoir(const LOGFONT* plf)
{
	// Recrer la police
	m_fntVoir.DeleteObject();
	m_fntVoir.CreateFontIndirect(plf);
	
	// Mettre  jour les contrles RTF de tous les visualiseurs
	POS pos = e_lstArc.GetPosDeb();
	while (pos) {
		CMuzipDoc *const pDoc = e_lstArc.GetSuiv(pos)->m_pi->GetDoc();
		if (pDoc && pDoc->m_papr)
			pDoc->m_papr->m_pApercu->OnSetFont();
	}
}



// Calculer un pourcentage avec un maximum de prcision
int CMuzipApp::GetPourcent(DWORD dwCur, DWORD dwTot)
{
	if (!dwTot)
		return 100;
	
	// 1<<7 = 128 plus petite puissance de 2 suprieure  100
	// Donc 1<<(32-7) - 1 = (DWORD)-1)>>7 est le plus grand
	// nombre que l'on peut multiplier par 100 sans sortir d'un DWORD
	// ( une puissance de 2 prs)
	const DWORD minValeur = ((DWORD)-1)>>7;
	if (dwCur>=minValeur && dwTot>=minValeur) {
		dwCur >>= 7;
		dwTot >>= 7;
	}
	
	const int nPC = (int)((dwCur*100) / dwTot);
	return (nPC > 100) ? 100 : nPC;
}


CString CMuzipApp::GetProfileString(LPCTSTR lpszSection, LPCTSTR lpszEntry, LPCTSTR lpszDefault)
{
	ASSERT(lpszSection && lpszEntry && m_pszProfileName);
	
	if (!lpszDefault)
		lpszDefault = _T("");
	
	CString s;
	for (DWORD buf = 0x1000; buf <= 0x100000; buf *= 2) {
		const DWORD len = ::GetPrivateProfileString(lpszSection, lpszEntry,
			lpszDefault, s.GetBuffer(buf), buf, m_pszProfileName);
		s.ReleaseBuffer(len);
		if (len < buf - 1)
			return s;
	}
	return lpszDefault;
}

BOOL CMuzipApp::GetProfileBinary(LPCTSTR lpszSection, LPCTSTR lpszEntry,
	BYTE** ppData, UINT* pBytes)
{
	ASSERT(lpszSection && lpszEntry && ppData && pBytes && m_pszProfileName);
	*ppData = NULL;
	*pBytes = 0;

	CString str = GetProfileString(lpszSection, lpszEntry, NULL);
	VERIF(!str.IsEmpty() && !(str.GetLength() & 1));
	INT_PTR nLen = str.GetLength();
	*pBytes = UINT(nLen)/2;
	*ppData = new BYTE[*pBytes];
	for (int i=0 ; i <nLen; i += 2)
		(*ppData)[i/2] = (BYTE)(((str[i+1] - 'A') << 4) + (str[i] - 'A'));
	
	return TRUE;
}




// Rcuprer le document courant
CDocument* GetActiveDocument()
{
	if (oApp.m_pMainWnd) {
		CFrameWnd *const pFrame =
			((CMainFrame*)oApp.m_pMainWnd)->GetActiveFrame();
		if (pFrame) {
			CDocument *const pDoc = pFrame->GetActiveDocument();
			if (pDoc)
				return pDoc;
		}
	}
	
	return NULL;
}

// Dfinir le document actif
void SetActiveDocument(CDocument* pDocument)
{
	if (oApp.m_pMainWnd) {
		POSITION pos = pDocument->GetFirstViewPosition();
		if (pos)
			pDocument->GetNextView(pos)->GetParentFrame()->ActivateFrame();
	}
}


// Pareil que DDX_Text  l'exception que le texte "infini"
// est considr comme ayant la valeur numrique zro
// et ne provoque donc pas d'erreur de validation
void DDX_TextInfini(CDataExchange* pDX, int nIDC, int& value)
{
	CString sInfini(MAKEINTRESOURCE(IDS_INFINI));
	
	if (pDX->m_bSaveAndValidate) {
		// Lecture: remplacer "infini" par "0"
		
		CString s;
		pDX->m_pDlgWnd->GetDlgItemText(nIDC, s);
		if (s==sInfini)
			pDX->m_pDlgWnd->SetDlgItemText(nIDC, _T("0"));
		
	}else{
		// criture: crire "infini"  la place de "0"
		
		if (!value) {
			pDX->m_pDlgWnd->SetDlgItemText(nIDC, sInfini);
			return;
		}
	}
	
	DDX_Text(pDX, nIDC, value);
}


// Mettre "infini" dans une zone de texte
// si elle contient le nombre zro
void InfiniRemplaceZero(CWnd* pwndParent, UINT nIDC)
{
	CEdit *const ptxt = (CEdit*)pwndParent->GetDlgItem(nIDC);
	
	BOOL bOK;
	if (pwndParent->GetDlgItemInt(nIDC, &bOK, FALSE)==0 &&
	    bOK && ptxt->GetSafeHwnd()) {
		// Afficher 'infini' au lieu de '0' (zro)
		
		ptxt->SetWindowText(CStringRes(IDS_INFINI));
		ptxt->SetSel(0,-1);
	}
}


// Redimensionner la bote de dialogue d'aprs le texte du label IDCLBL
// d'une bote de dialogue
// La fonction retourne le dcalage  appliquer aux contrles situs en-dessous
// (dcalage  appliquer avec MoveControlY)
int DlgRedimSelonLabel(CDialog& dlg, const CString& sMsg)
{
	CWnd *const plbl = dlg.GetDlgItem(IDCLBL);
	
	plbl->SetWindowText(sMsg);
	
	// Rcuprer la position du label du message
	// Cela servira  positionner tous les autres contrles
	CRect rcMsg;
	plbl->GetWindowRect(rcMsg);
	dlg.ScreenToClient(rcMsg);
	const int cyMsgOrig = rcMsg.Height();
	
	CRect rcTemp;
	
	// Rcuprer la hauteur du texte
	CDC *const pdc = plbl->GetDC();
	pdc->SelectObject(plbl->GetFont());
	plbl->GetClientRect(rcTemp);
	pdc->DrawText(sMsg, rcTemp,
		DT_CALCRECT | DT_NOPREFIX | DT_WORDBREAK);
	plbl->ReleaseDC(pdc);
	
	// La hauteur minimum est celle du label dans la bote de dialogue originale
	const int yDiff = (rcTemp.bottom < cyMsgOrig)
		? 0 : rcTemp.bottom - cyMsgOrig;
	
	// Redimensionner le label du message
	rcMsg.bottom += yDiff;
	plbl->MoveWindow(rcMsg);
	
	// Redimensionner la bote de dialogue
	// On n'a pas besoin de recalculer les coordonnes pour centrer,
	// le centrage est effectu aprs WM_INITDIALOG
	dlg.GetWindowRect(rcTemp);
	dlg.SetWindowPos(NULL,
		0, 0,
		rcTemp.Width(), rcTemp.Height() + yDiff,
		SWP_NOZORDER | SWP_NOMOVE);
	
	return yDiff;
}


// Dcaler verticalement un contrle
void MoveControlY(CDialog& dlg, UINT nIDC, int yDiff)
{
	CWnd *const pwnd = dlg.GetDlgItem(nIDC);
	
	// Rcuprer le rectangle du contrle,
	// le dcaler vers le bas,
	// puis repositionner le contrle avec
	CRect rc;
	pwnd->GetWindowRect(rc);
	dlg.ScreenToClient(rc);
	rc.OffsetRect(0, yDiff);
	pwnd->MoveWindow(rc);
}
