// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// MuzipExt.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "Muzip.h"
#include "MuzipDll.h"
#include "Muzip2.h"
#include "MuzipArc.h"

#include "MainFrame.h"
#include "lstMuzip.h"

#include "MuzipExt.h"

#include "Resource.h"
#include <afxRes.h>
#include "../Franais/Resource.h"

#include "../Reg.h"
#include "../IOFichier.h"


#include <afxPriv.h>
#include <Pidl.h>


ULONG e_nbTotRef        = 0;
HMENU e_hMenuContextuel = NULL;


class COptionsExt
{
public:  // Fonctions
	
	COptionsExt();
	
	
public:  // Donnes
	
	DWORD      m_dwShell;
	MCT_OPTION m_aMCT[mctTotal];
};


struct ADDSTATE
{
	COptionsExt  opt;
	
	MENUITEMINFO mii;
	HMENU        hMenu;
	UINT         indexMenu;
	UINT         idCmdFirst;
	
	int          nIndex;      // Index dans m_aid et m_afidAuto
};


#define INITGUID
#include <InitGuid.h>

// {06712C80-5BDC-11D4-B8EE-B5DE61CA987B}
DEFINE_GUID(CLSID_MuzipExt, 0x6712C80, 0x5BDC, 0x11D4, 0xB8, 0xEE, 0xB5, 0xDE, 0x61, 0xCA, 0x98, 0x7B);


const TCHAR   pszDescription[] = _T("Extension Shell de Muzip");
const LPCTSTR kszMuzip         = _T("Software\\Ryder\\Muzip");
const LPCTSTR kszMuzipGeneral  = _T("Software\\Ryder\\Muzip\\Gnral");



#define pszZipExtA  ".zip"
#ifdef UNICODE
const LPCTSTR pszZipExt = _T(pszZipExtA);
#else
#define pszZipExt  pszZipExtA
#endif


#define ResultFromShort(i)   ResultFromScode(MAKE_SCODE(SEVERITY_SUCCESS, 0, (WORD)(i)))
#define HRVERIF(exp)         { if (FAILED(exp))  return E_FAIL; }
#define REGERR(f)            ((f)!=ERROR_SUCCESS)


static HICON s_hIcon = NULL;


static bool GetCLSID(CString& sCLSID, CString& sKey);

static void PathAddZipExtension(LPTSTR pszPath);

static void CALLBACK TimerFreeUnusedLibraries(HWND hWnd, UINT uMsg, UINT idEvent, DWORD dwTime);


// Les MFC exigent que l'objet CWinApp soit dtruit
// par le thread qui l'a cr.
//
// Cela pose un problme avec le point d'entre de la DLL:
// le thread appel avec DLL_PROCESS_DETACH n'est pas forcment
// celui qui a t appel avec DLL_PROCESS_ATTACH, cela  cause
// de COM: un objet A est cr, puis un objet B. Si A est dtruit,
// et ensuite seulement B, alors le thread de B recevra DLL_PROCESS_DETACH
// et non le thread de A.
//
// Conclusion: on cre un thread spcialement charg de cr
// et dtruire l'objet CWinApp, comme a il n'y a pas de problme.
// Il faut simplement synchroniser les threads avec force vnements
// et fonctions d'attente.
//
// Mais cela ne suffit pas: Windows force la destruction des threads
// AVANT d'appeler DLL_PROCESS_DETACH, donc on ne peut pas terminer
// ThreadApp proprement dans le point d'entre. Une astuce consiste
//  utiliser DllCanUnloadNow(), qui est SYSTMATIQUEMENT appele
// avant que la DLL ne soit dcharge. Donc ds que DllCanUnloadNow()
// est appele et qu'on retourne "oui", on dcharge l'application
//  CE MOMENT. On est certain que c'est propre car:
// 1) notre thread est le seul qui reste, car tous les autres
//    auraient rfrenc un objet et DllCanUnloadNow()
//    n'aurait pas t appel; mme si ce n'tait pas le cas,
//    on est certain que l'objet CWinApp n'est pas utilis
// 2) Windows n'appellera DLL_PROCESS_DETACH qu'une fois l'application
//    dcharge, donc on se moque de savoir avec quel thread
//    le point d'entre sera excut


static HANDLE s_hEventAppCreated = NULL;
static void  CreateApp();
static DWORD WINAPI ThreadApp(void* pvParam);


// Charger l'application (objet CWinApp)
void CreateApp()
{
	// Si l'application est dj cre, on a rien besoin de faire
	if (e_pma)
		return;
	
	if (s_hEventAppCreated) {
		// L'application est en train de se crer: attendre et quitter
		WaitForSingleObject(s_hEventAppCreated, INFINITE);
		return;
	}
	
	s_hEventAppCreated = CreateEvent(NULL, FALSE, FALSE, NULL);
	
	TRACE0("CreateApp - En cours...\n");
	
	// Lancer un thread crateur d'application
	// On lui passe un vnement pour attendre la cration effective
	// de l'objet CWinApp et pouvoir continuer
	DWORD dwThreadId;
	e_hThreadApp = CreateThread(NULL, 0,
		ThreadApp, NULL, 0, &dwThreadId);
	
	// Attendre que l'application soit cre
	WaitForSingleObject(s_hEventAppCreated, INFINITE);
	const HANDLE hEventOld = s_hEventAppCreated;
	s_hEventAppCreated = NULL;
	CloseHandle(hEventOld);
	
	TRACE0("CreateApp - OK\n");
}


// Thread crateur et destructeur de l'unique objet CApp
DWORD WINAPI ThreadApp(void*)
{
	if (e_pma)
		// L'objet CApp est dj cr
		return 0;
	
	// Crer l'application et avertir que cela est fait
	(new CApp)->Init();
	SetEvent(s_hEventAppCreated);
	
	// Crer l'venement d'arrt de la boucle de l'application
	e_hEventAppClose = CreateEvent(NULL, FALSE, FALSE, NULL);
	
	// Boucle de messages, mais d'un type spcial:
	// on attend l'vnement de fermeture ou un message
	// Si l'vnement d'arrt arrive, on sort de la boucle
	// MsgWaitForMultipleObjects() vite d'utiliser un timer
	// ou tout autre moyen artisanal pour que GetMessage()
	// ne bloque pas trop longtemps le test de l'vnement de fermeture
	// QS_ALLINPUT permet de ragir  tous les vnements possibles:
	// - SendMessage et PostMessage
	// - messages spciaux (WM_TIMER, WM_PAINT, WM_KEYDOWN, etc.)
	// On met un timer de scurit d'une seconde pour viter de figer
	// le shell en cas de bug...
	MSG msg;
	while (MsgWaitForMultipleObjects(1, &e_hEventAppClose,
			FALSE, 1000, QS_ALLINPUT)!=WAIT_OBJECT_0) {
		
		// Traiter le message s'il y en a un
		if (PeekMessage(&msg, NULL,0,0, PM_REMOVE) &&
		    !oApp.PreTranslateMessage(&msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	
	
	// Dtruire l'application
	if (e_pma && e_pma->m_fSilent) {
		e_pma->Exit();
		delete e_pma;
	}
	
	
	//*** Fermer les handles de manire totalement propre
	
	HANDLE hSauve;
	
	hSauve = e_hEventAppClose;
	e_hEventAppClose = NULL;
	CloseHandle(hSauve);
	
	hSauve = e_hThreadApp;
	e_hThreadApp = NULL;
	CloseHandle(e_hThreadApp);
	
	return 0;
}



//*** COptionsExt

// Constructeur
COptionsExt::COptionsExt()
{
	m_dwShell = pApp->GetProfileInt(kszGeneral, eszGeneralShell, gsDefaut);
	
	MCT_OPTION *const aMCT = MCT_OPTION::GetArray();
	memcpy(m_aMCT, aMCT, sizeof(m_aMCT));
	delete [] (BYTE*)aMCT;
}





// ConvertirsKey en chane le CLSID de l'extension du shell
bool GetCLSID(CString& sCLSID, CString& sKey)
{
	// Convertir le CLSID en chane
	LPWSTR wszCLSID;
	if (FAILED(StringFromCLSID(CLSID_MuzipExt, &wszCLSID)))
		return false;
	
	// Copier puis librer la chane Unicode
	sCLSID = wszCLSID;
	IMalloc *pMalloc;
	if (SUCCEEDED(SHGetMalloc(&pMalloc))) {
		pMalloc->Free(wszCLSID);
		pMalloc->Release();
	}
	
	// Formatter la cl du CLSID
	sKey.Format(_T("CLSID\\%s"), (LPCTSTR)sCLSID);
	
	return true;
}


// Ajouter l'extension ".zip"  un chemin d'accs
void PathAddZipExtension(LPTSTR pszPath)
{
	const int len = lstrlen(pszPath);
	lstrcpyn(pszPath + len, pszZipExt, MAX_PATH - len - 1);
}


// Demande de dchargement
// On dcharge l'application AVANT de retourner S_OK
// pour viter que le thread crateur de l'application soit kill
// par Windows (car en attente infinie) et ne puisse plus
// dtruire proprement l'application
STDAPI DllCanUnloadNow()
{
	if (e_nbTotRef > 0)
		return S_FALSE;
	
	// On peut dcharger l'application
	TRACE0("DllCanUnloadNow() - Fermeture application\n");
	SetEvent(e_hEventAppClose);
	WaitForSingleObject(e_hThreadApp, INFINITE);
	
	return S_OK;
}

// Essayer de librer la DLL
void CALLBACK TimerFreeUnusedLibraries(HWND, UINT, UINT idEvent, DWORD)
{
	KillTimer(NULL, idEvent);
	
	CoFreeUnusedLibraries();
}


// Demande d'interface
STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, void** ppvOut)
{
	// Charger l'interface de manipulation des archives
	// si cela n'a pas t fait
	CreateApp();
	
	// Grer la cration de CMuzipExtClassFactory
	*ppvOut = NULL;
	if (IsEqualIID(rclsid, CLSID_MuzipExt))
		return (new CMuzipExtClassFactory)->QueryInterface(riid, ppvOut);
	
	return CLASS_E_CLASSNOTAVAILABLE;
}


const LPCTSTR pszCM       = _T("ContextMenuHandlers\\Muzip");
const LPCTSTR pszApproved = _T("Software\\Microsoft\\Windows\\CurrentVersion\\Shell Extensions\\Approved");

struct
{
	LPCTSTR ksz1;
	LPCTSTR ksz2;
}
static const s_akszShellExt[] =
{
	{ _T("*"),             pszCM },
	{ _T("Directory"),     pszCM },
	{ _T("Muzip.Archive"), _T("DropHandler") },
};
const LPCTSTR pszShellExtFormat = _T("%s\\shellex\\%s");


// Enregistrement
STDAPI DllRegisterServer()
{
	int i;
	
	// Rcuprer le CLSID et sa cl
	CString sCLSID, sKey;
	if (!GetCLSID(sCLSID, sKey))
		return E_OUTOFMEMORY;
	
	HRESULT hr = SELFREG_E_CLASS;
	
	TCHAR pszDLL[MAX_PATH];
	HKEY hKey = NULL;
	const TCHAR pszApartment[] = _T("Apartment");
	
	GetModuleFileName(e_em.hModule, pszDLL, nbArray(pszDLL));
	
	// CLSID\{CLSID} = description
	if (REGERR(RegCreateKeyEx(HKEY_CLASSES_ROOT, sKey, 0, NULL, 0, KEY_WRITE, NULL, &hKey, NULL)))
		goto Erreur;
	if (REGERR(RegSetValueEx(hKey, NULL, 0, REG_SZ,
	   (const BYTE*)pszDescription, sizeof(pszDescription))))
		goto Erreur;
	
	// CLSID\{CLSID}\InProcServer32                 = <Chemin DLL>
	// CLSID\{CLSID}\InProcServer32, ThreadingModel = "Apartment"
	sKey += _T("\\InprocServer32");
	RegCloseKey(hKey);
	hKey = NULL;
	if (REGERR(RegCreateKeyEx(HKEY_CLASSES_ROOT, sKey, 0, NULL, 0, KEY_WRITE, NULL, &hKey, NULL)))
		goto Erreur;
	if (REGERR(RegSetValueEx(hKey, NULL, 0, REG_SZ,
	   (const BYTE*)pszDLL, (lstrlen(pszDLL)+1)*sizeof(pszDLL[0]))))
		goto Erreur;
	if (REGERR(RegSetValueEx(hKey, _T("ThreadingModel"), 0, REG_SZ,
	   (const BYTE*)pszApartment, nbArray(pszApartment)*sizeof(pszApartment[0]))))
		goto Erreur;
	
	// HKEY_CLASSES_ROOT\*\shellex\ContextMenuHandlers\Muzip = <CLSID>
	// HKEY_CLASSES_ROOT\Directory\shellex\ContextMenuHandlers\Muzip = <CLSID>
	// HKEY_CLASSES_ROOT\Muzip.Archive\shellex\DragDropHandlers\Muzip = <CLSID>
	for (i=0; i<nbArray(s_akszShellExt); i++) {
		sKey.Format(pszShellExtFormat,
			s_akszShellExt[i].ksz1, s_akszShellExt[i].ksz2);
		if (REGERR(RegSetValue(HKEY_CLASSES_ROOT, sKey,
				REG_SZ, sCLSID, sCLSID.GetLength()*sizeof(TCHAR))))
			goto Erreur;
	}
	
	// HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\Shell Extensions\Approved
	//   <CLSID> = <Description>
	if (REGERR(RegCreateKeyEx(HKEY_LOCAL_MACHINE, pszApproved,
	    0, NULL, 0, KEY_WRITE, NULL, &hKey, NULL)))
		goto Erreur;
	if (REGERR(RegSetValueEx(hKey, sCLSID, 0, REG_SZ,
	   (const BYTE*)pszDescription, nbArray(pszDescription))))
		goto Erreur;
	
	hr = S_OK;
	
Erreur:
	
	if (hKey)
		RegCloseKey(hKey);
	
	return hr;
}

// Dsenregistrement
STDAPI DllUnregisterServer()
{
	int i;
	
	// Rcuprer le CLSID et sa cl
	CString sCLSID, sKey;
	if (!GetCLSID(sCLSID, sKey))
		return E_OUTOFMEMORY;
	
	
	// Effacer:
	// HKEY_CLASSES_ROOT\CLSID\{<CLSID>}
	// HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\Shell Extensions\Approved, <CLSID>
	AfxDelRegTreeHelper(HKEY_CLASSES_ROOT, sKey);
	HKEY hKey;
	if (!REGERR(RegOpenKeyEx(HKEY_LOCAL_MACHINE, pszApproved, 0, KEY_SET_VALUE, &hKey))) {
		RegDeleteValue(hKey, sCLSID);
		RegCloseKey(hKey);
	}
	// HKEY_CLASSES_ROOT\*\shellex\ContextMenuHandlers\Muzip
	// HKEY_CLASSES_ROOT\Directory\shellex\ContextMenuHandlers\Muzip
	// HKEY_CLASSES_ROOT\Muzip.Archive\shellex\DragDropHandlers\Muzip
	for (i=0; i<nbArray(s_akszShellExt); i++) {
		sKey.Format(pszShellExtFormat,
			s_akszShellExt[i].ksz1, s_akszShellExt[i].ksz2);
		AfxDelRegTreeHelper(HKEY_CLASSES_ROOT, sKey);
	}
	
	return S_OK;
}



//*** CRefMgr

// Ajouter une instance
ULONG CRefMgr::AddRef()
{
	e_nbTotRef++;
	
	// Charger l'interface de manipulation des archives
	// si cela n'a pas t fait
	CreateApp();
	
	return ++m_nbRef;
}

// Librer une instance
ULONG CRefMgr::Release()
{
	if (!--e_nbTotRef)
		// Essayer de librer la DLL
		// On doit passer par un timer
		// pour que la fonction appelante se termine
		SetTimer(NULL, 0, 0, TimerFreeUnusedLibraries);
	
	return --m_nbRef;
}



//*** IUnknown

// Demande d'interface
STDMETHODIMP CMuzipExtClassFactory::QueryInterface(REFIID riid, void** ppvOut)
{
	AddRef();
	
	if (IsEqualIID(riid, IID_IUnknown) ||
	    IsEqualIID(riid, IID_IClassFactory))
		*ppvOut = (IClassFactory*)this;
	else{
		*ppvOut = NULL;
		Release();
		return E_NOINTERFACE;
	}
	
	return S_OK;
}



//*** IClassFactory

// Cration
STDMETHODIMP CMuzipExtClassFactory::CreateInstance(
	IUnknown* pUnkOuter, REFIID riid, void** ppvObj)
{
	*ppvObj = NULL;
	
	if (pUnkOuter)
		return CLASS_E_NOAGGREGATION;
	
	// Crer l'objet principal
	return (new CMuzipExt)->QueryInterface(riid, ppvObj);
}

// Blocage
STDMETHODIMP CMuzipExtClassFactory::LockServer(BOOL fLock)
{
	if (fLock)
		AddRef();
	else
		Release();
	
	return S_OK;
}



//*** CMuzipExt

// Demande d'interface
STDMETHODIMP CMuzipExt::QueryInterface(REFIID riid, void** ppvOut)
{
	AddRef();
	
	if (IsEqualIID(riid, IID_IUnknown) ||
	    IsEqualIID(riid, IID_IShellExtInit))
		*ppvOut = (IShellExtInit*)this;
#define GERER_INTERFACE(nom) \
	else if (IsEqualIID(riid, IID_##nom)) \
		*ppvOut = (##nom*)this;
	GERER_INTERFACE(IShellExtInit)
	GERER_INTERFACE(IContextMenu)
	GERER_INTERFACE(IContextMenu2)
	GERER_INTERFACE(IContextMenu3)
	GERER_INTERFACE(IPersistFile)
	GERER_INTERFACE(IQueryInfo)
	GERER_INTERFACE(IShellPropSheetExt)
#undef GERER_INTERFACE
	else if (IsEqualIID(riid, IID_IDropTarget)) {
		// IDropTarget: vrifier que l'option "glisser-dplacer" est coche
		
		if (!(COptionsExt().m_dwShell & gsDrop))
			goto NoInterface;
		*ppvOut = (IDropTarget*)this;
		
	}else{
NoInterface:
		*ppvOut = NULL;
		Release();
		return E_NOINTERFACE;
	}
	
	return S_OK;
}


// Initialiser la slection courante d'aprs un HDROP
HRESULT CMuzipExt::InitDataObject(IDataObject* pdo)
{
	m_lstFichier.RemoveAll();
	
	HRESULT hr = E_FAIL;
	
	if (pdo) {
		
		// Rcuprer le HDROP dans fe.hGlobal
		STGMEDIUM stmDrop;
		FORMATETC fe;
		fe.cfFormat = CF_HDROP;
		fe.ptd      = NULL;
		fe.dwAspect = DVASPECT_CONTENT;
		fe.lindex   = -1;
		fe.tymed    = TYMED_HGLOBAL;
		HRVERIF(pdo->GetData(&fe, &stmDrop));
		
		// Il doit au moins y avoir un fichier
		const UINT nbDrop = DragQueryFile((HDROP)stmDrop.hGlobal, (UINT)-1, NULL, 0);
		if (nbDrop) {
			
			// Rcuprer tous les fichiers
			for (UINT i=0; i<nbDrop; i++) {
				TCHAR pszFichier[MAX_PATH];
				DragQueryFile((HDROP)stmDrop.hGlobal, i, pszFichier, nbArray(pszFichier));
				PathUnquoteSpaces(pszFichier);
				m_lstFichier.AddFin(pszFichier);
			}
			
			InitFichier();
			
			hr = S_OK;
		}
		
		ReleaseStgMedium(&stmDrop);
	}
	
	return hr;
}


// Le fichier  manipuler est dfini
void CMuzipExt::InitFichier()
{
	const TCHAR apszExt[][4] =
	{
		_T("zip"),
		_T("gz"),_T("z"),_T("tgz"), _T("taz"),
		_T("tar"),
		_T("uue"), _T("uu"), _T("xxe"),
	};
	
	// Regarder si le fichier est une archive,
	// et lui ajouter l'extension Zip si ce n'en est pas une
	m_mct = (GetFileAttributes(m_lstFichier.GetDeb()) & FILE_ATTRIBUTE_DIRECTORY)
		? mctDossier : mctFichier;
	
	lstrcpy(m_pszArchive, m_lstFichier.GetDeb());
	if (PathIsDirectory(m_pszArchive))
		PathAddZipExtension(m_pszArchive);
	else{
		LPTSTR pszExt = PathFindExtension(m_pszArchive);
		if (m_mct==mctFichier && pszExt && *pszExt=='.') {
			pszExt++;   // Sauter le point
			for (int i=0;; i++) {
				if (i==nbArray(apszExt)) {
					lstrcpy(pszExt, pszZipExt+1);   // +1 pour sauter le point
					break;
				}
				if (!lstrcmpi(pszExt, apszExt[i])) {
					m_mct = mctArchive;
					break;
				}
			}
		}else
			lstrcpy(pszExt, pszZipExt);
	}
	
	// Utiliser le rpertoire du fichier courant
	TCHAR pszDir[MAX_PATH];
	lstrcpy(pszDir, m_pszArchive);
	PathRemoveFileSpec(pszDir);
	SetCurrentDirectory(pszDir);
	
	m_fInitialized = true;
}



//*** IShellExtInit

// Initialiser
STDMETHODIMP CMuzipExt::Initialize(LPCITEMIDLIST, IDataObject* pdo, HKEY)
{
	// Parfois Windows appelle Initialize() avec un pdo corrompu,
	// il faut donc tre prudent
	try
	{
		pdo->AddRef();
		InitDataObject(pdo);
		pdo->Release();
	}
	catch (...)
	{
	}
	
	return S_OK;
}


//*** IPersistFile

STDMETHODIMP CMuzipExt::Load(LPCOLESTR pszFichier, DWORD)
{
	if (lstrlenW(pszFichier)>=MAX_PATH)
		return E_FAIL;
	
	m_lstFichier.RemoveAll();
	m_lstFichier.AddFin(pszFichier);
	InitFichier();
	return S_OK;
}


//*** IDropTarget

// Glisser-dplacer - Dbut de survol
STDMETHODIMP CMuzipExt::DragEnter(
	IDataObject* pdo, DWORD, POINTL, DWORD* pdwEffect)
{
	if (!(*pdwEffect & DROPEFFECT_COPY))
		return E_INVALIDARG;
	
	// Vrifier que les donnes comportent un objet de type HDROP
	// et changer en consquence la forme du curseur (copie ou interdit)
	FORMATETC fe;
	fe.cfFormat = CF_HDROP;
	fe.ptd      = NULL;
	fe.dwAspect = DVASPECT_CONTENT;
	fe.lindex   = -1;
	fe.tymed    = TYMED_HGLOBAL;
	*pdwEffect = (!pdo || pdo->QueryGetData(&fe)!=S_OK)
		? DROPEFFECT_NONE : DROPEFFECT_COPY;
	
	*pdwEffect = DROPEFFECT_COPY;
	return S_OK;
}

// Glisser-dplacer - Survol
STDMETHODIMP CMuzipExt::DragOver(DWORD, POINTL, DWORD* pdwEffect)
{
	if (!(*pdwEffect & DROPEFFECT_COPY))
		return E_INVALIDARG;
	
	*pdwEffect = DROPEFFECT_COPY;
	return S_OK;
}

// Glisser-dplacer - Fin de survol
STDMETHODIMP CMuzipExt::DragLeave()
{
	return S_OK;
}

// Glisser-dplacer - Lcher
STDMETHODIMP CMuzipExt::Drop(
	IDataObject* pdo, DWORD, POINTL, DWORD* pdwEffect)
{
	// Rcuprer les fichiers qu'on a fait glisser
	// On garde m_pszArchive car c'tait l'archive de destination
	const CString sArchive = m_pszArchive;
	HRVERIF(InitDataObject(pdo));
	lstrcpy(m_pszArchive, sArchive);
	
	cmdCompresser(CMD_COMPRESSERAUTO,
		ToBool(oApp.GetProfileInt(GetKSZ(Comp), GetESZ(Comp,f,OptionsDropArchive), TRUE)));
	
	*pdwEffect = DROPEFFECT_COPY;
	return S_OK;
}



//*** IContextMenu

// Demande d'ajout d'items de menu
STDMETHODIMP CMuzipExt::QueryContextMenu(
	HMENU hMenu, UINT indexMenu, UINT idCmdFirst, UINT, UINT uFlags)
{
	TRACE1("################ CMuzipExt::QueryContextMenu: uFlags = %X\n", uFlags);
	
	if (!m_fInitialized || hMenu == e_hMenuContextuel ||
	   (uFlags & CMF_DEFAULTONLY) || !(uFlags & CMF_CANRENAME))
		return ResultFromShort(1);
	
	
	ADDSTATE ads;
	
	// Il faut que le menu contextuel soit actif
	// et qu'il y ait au moins un item  insrer
	if (!(ads.opt.m_dwShell & gsMenu) ||
	    ! ads.opt.m_aMCT[m_mct].uVisible)
		return S_OK;
	
	
	// Rcuprer la version du shell
	bool fShell4 = false;
	const HINSTANCE hShell32 = AfxLoadLibrary(_T("Shell32.dll"));
	if (hShell32) {
		
		// Rcuprer la fonction DllGetVersion()
		const DLLGETVERSIONPROC pfnDllGetVersion =
			(DLLGETVERSIONPROC)GetProcAddress(hShell32, "DllGetVersion");
		
		// Appeler la fonction
		// Si elle est introuvable, ce n'est pas la version 4
		if (pfnDllGetVersion) {
			DLLVERSIONINFO dvi;
			dvi.cbSize = sizeof(dvi);
			if (SUCCEEDED(pfnDllGetVersion(&dvi)))
				fShell4 = (dvi.dwMajorVersion>=4);
		}else
			fShell4 = false;
		
		AfxFreeLibrary(hShell32);
	}
	
	
	const bool fIcone     = ToBool(ads.opt.m_dwShell & gsMenuIcone);
	const bool fOwnerDraw = (fShell4 && fIcone);
	
	ads.hMenu      = hMenu;
	ads.indexMenu  = indexMenu;
	ads.idCmdFirst = idCmdFirst;
	ads.nIndex     = 0;
	
	const UINT nType = (fOwnerDraw)
		? MFT_STRING | MFT_OWNERDRAW
		: MFT_STRING;
	
	ads.mii.cbSize = sizeof(ads.mii);
	ads.mii.wID    = ads.idCmdFirst;
	if (!fOwnerDraw || fIcone) {
		// Dfinir les bitmaps de coche
		ads.mii.hbmpChecked   = NULL;
		ads.mii.hbmpUnchecked = (fIcone)
			? (HBITMAP)LoadImage(AfxGetResourceHandle(),
				MAKEINTRESOURCE(IDB_APP), IMAGE_BITMAP, 0,0,
				LR_DEFAULTCOLOR | LR_DEFAULTSIZE | LR_LOADMAP3DCOLORS)
			: NULL;
	}
	
	if (ads.opt.m_dwShell & gsMenuSous) {
		// Insrer un sous-menu
		
		// Crer le sous-menu
		ads.mii.hSubMenu = CreatePopupMenu();
		
		// Insrer le sous-menu dans le menu donn par Windows
		ads.mii.fMask      = (fIcone)
			? MIIM_TYPE | MIIM_SUBMENU | MIIM_CHECKMARKS
			: MIIM_TYPE | MIIM_SUBMENU;
		ads.mii.fType      = MFT_STRING;
		ads.mii.dwTypeData = (LPTSTR)pApp->m_pszAppName;
		InsertMenuItem(ads.hMenu, ads.indexMenu, TRUE, &ads.mii);
		
		// Utiliser le sous menu comme menu parent,
		// et repartir au premier item
		ads.hMenu     = ads.mii.hSubMenu;
		ads.indexMenu = 0;
	}
	
	// Prparer l'insertion d'items
	ads.mii.fMask = (fOwnerDraw)
		? MIIM_ID | MIIM_TYPE | MIIM_DATA
		: MIIM_ID | MIIM_TYPE | MIIM_DATA | MIIM_CHECKMARKS;
	ads.mii.fType = nType;
	
	
	// Ajouter les items demands
	for (UINT id=cmdDeb; id<=cmdFin; id++)
		if (ads.opt.m_aMCT[m_mct].uVisible & (1<<(id-cmdDeb)))
			AddMenuItem(ads, id);
	
	if (!(ads.opt.m_dwShell & gsMenuSous)) {
		// Insrer le sparateur final
		ads.mii.fMask = MIIM_TYPE;
		ads.mii.fType = MFT_SEPARATOR;
		InsertMenuItem(ads.hMenu, ads.indexMenu++, TRUE, &ads.mii);
		ads.idCmdFirst--;
	}
	
	
	// Retourner le nombre d'items ajouts
	return ResultFromShort(ads.mii.wID - ads.idCmdFirst + 1);
}

// Commande invoque
STDMETHODIMP CMuzipExt::InvokeCommand(CMINVOKECOMMANDINFO* pcmi)
{
	// Quelques tests d'erreur
	if (!m_lstFichier.GetNb())
		return E_FAIL;
	if (HIWORD(pcmi->lpVerb))
		return E_INVALIDARG;
	
	
	// Rcuprer l'ID de la commande
	const UINT cmd = m_aid[LOWORD(pcmi->lpVerb)];
	
	// Rcuprer les options de la commande
	MCT_OPTION *const aMCT = MCT_OPTION::GetArray();
	bool fDemander = ToBool(aMCT[m_mct].uDemande & (1<<(cmd-cmdDeb)));
	
	switch (cmd) {
		
		// Ouverture d'archive
		case CMD_OUVRIR:
			cmdOuvrir();
			break;
		
		// Extraction
		case CMD_DECOMP:
			fDemander = true;
		case CMD_DECOMPAUTO:
		case CMD_DECOMPICI:
			cmdDecomp(cmd, fDemander);
			break;
		
		// Compression
		case CMD_COMPRESSER:
			fDemander = true;
		case CMD_COMPRESSERAUTO:
			cmdCompresser(cmd, fDemander);
			break;
		
		// Envoi de mail
		case CMD_MAIL:
			cmdMail();
			break;
	}
	
	delete [] (BYTE*)aMCT;
	
	return S_OK;
}

// Texte d'aide demand
STDMETHODIMP CMuzipExt::GetCommandString(
	UINT idCmd, UINT uFlags, UINT*, LPSTR pszName, UINT cchMax)
{
	switch (uFlags) {
		
		// Texte d'aide
		case GCS_HELPTEXTA:
		case GCS_HELPTEXTW:
			{
				const UINT id = m_aid[idCmd-1];
				
				// Prendre la fin de la chane (la description dtaille)
				CString sDesc(MAKEINTRESOURCE(id));
				sDesc = (LPCTSTR)sDesc + sDesc.Find('\n');
				
				// Rajouter le nom du premier fichier slectionn si ncessaire
				if (id==CMD_COMPRESSERAUTO || id==CMD_DECOMPAUTO) {
					TCHAR pszPlus[MAX_PATH];
					lstrcpy(pszPlus, PathFindFileName(m_pszArchive));
					
					if (id==CMD_DECOMPAUTO)
						// Extraction dans un dossier: enlever l'extension
						PathRemoveExtension(pszPlus);
					
					// Ajouter la chane  la description
					PathQuoteSpaces(pszPlus);
					sDesc += pszPlus;
				}
				
				// Stocker la chane dans le tampon
				if (uFlags == GCS_HELPTEXTA)
					strtoA(pszName, sDesc, cchMax);
				else
					strtoW((LPWSTR)pszName, sDesc, cchMax);
			}
			break;
		
		// Verbe (ID textuel)
		case GCS_VERBA:
			_snprintf(pszName,cchMax-1, "%u", idCmd);
			break;
		case GCS_VERBW:
			_snwprintf((LPWSTR)pszName,cchMax-1, L"%u", idCmd);
			break;
	}
	
	return S_OK;
}


//*** IContextMenu2, IContextMenu3

// Dessin personnalis
STDMETHODIMP CMuzipExt::HandleMenuMsg(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	return HandleMenuMsg2(uMsg, wParam, lParam, NULL);
}
STDMETHODIMP CMuzipExt::HandleMenuMsg2(UINT uMsg, WPARAM, LPARAM lParam, LRESULT*)
{
	switch (uMsg) {
		
		// Mensurations
		case WM_MEASUREITEM:
			{
				#define pmis  ((MEASUREITEMSTRUCT*)lParam)
				
				// Rcuprer un DC de l'cran avec la police des menus
				// et effectuer les mesures
				
				CDC dc;
				dc.Attach(::GetDC(NULL));
				
				dc.SelectStockObject(DEFAULT_GUI_FONT);
				pmis->itemWidth  = dc.GetTextExtent((LPCTSTR)pmis->itemData).cx +
					GetSystemMetrics(SM_CYMENU);
				pmis->itemHeight = GetSystemMetrics(SM_CYMENU);
				
				::ReleaseDC(NULL, dc.Detach());
				
				#undef pmis
			}
			break;
		
		// Dessin
		case WM_DRAWITEM:
			{
				#define pdis  ((DRAWITEMSTRUCT*)lParam)
				
				CDC dc;
				dc.Attach(pdis->hDC);
				
				// Dessiner le fond
				::FillRect(dc, &pdis->rcItem, GetSysColorBrush(
					(pdis->itemState & ODS_SELECTED)
						? COLOR_HIGHLIGHT : COLOR_MENU));
				
				// Dessiner l'icne
				if (!s_hIcon)
					s_hIcon = (HICON)LoadImage(AfxGetResourceHandle(),
						MAKEINTRESOURCE(IDR_MAINFRAME), IMAGE_ICON, 16,16, 0);
				DrawIconEx(dc,
					pdis->rcItem.left,
					(pdis->rcItem.top+pdis->rcItem.bottom-16)/2, s_hIcon,
					16,16, 0, NULL, DI_NORMAL);
				
				// Dessiner le texte
				dc.SetTextColor(GetSysColor(
					(pdis->itemState & ODS_SELECTED)
						? COLOR_HIGHLIGHTTEXT : COLOR_MENUTEXT));
				dc.SetBkMode(TRANSPARENT);
				pdis->rcItem.left += GetSystemMetrics(SM_CYMENU)+2;
				dc.DrawText((LPCTSTR)pdis->itemData,
					&pdis->rcItem, DT_SINGLELINE | DT_LEFT | DT_VCENTER);
				
				dc.Detach();
				
				#undef pdis
			}
			break;
		
		// Raccourci clavier
		case WM_MENUCHAR:
			return E_FAIL;
	}
	
	return S_OK;
}


// Ouvrir l'archive
void CMuzipExt::cmdOuvrir()
{
	// Lancer Muzip
	// Ne pas oublier d'ajouter des guillemets autour du nom de l'archive
	::ShellExecute(NULL, NULL, pszMuzipEXE,
		MyPathQuoteSpacesCopy(m_pszArchive), NULL, SW_SHOWDEFAULT);
}


// Dcompresser l'archive dans un dossier
void CMuzipExt::cmdDecomp(UINT cmd, bool fDemander)
{
	CMuzipCmdLine cmdInfo;
	cmdInfo.SetShellCommand(FileDecomp);
	cmdInfo.m_fDemanderOptions = fDemander;
	cmdInfo.m_strFileName      = m_pszArchive;
	
	if (cmd!=CMD_DECOMP) {
		// S'occuper du dossier d'extraction
		// Mode auto: prendre le nom du fichier cliqu sans l'extension
		// Mode ici:  prendre le dossier du fichier cliqu
		
		cmdInfo.m_sDest = m_lstFichier.GetDeb();
		cmdInfo.m_sDest.GetBuffer(MAX_PATH);
		if (cmd==CMD_DECOMPAUTO)  PathRemoveExtension((STRING)cmdInfo.m_sDest);
		else                      PathRemoveFileSpec ((STRING)cmdInfo.m_sDest);
		cmdInfo.m_sDest.ReleaseBuffer();
	}
	
	e_pma->ProcessShellCommand(cmdInfo, saExplorer);
}


// Compresser les fichiers dans une archive
void CMuzipExt::cmdCompresser(UINT cmd, bool fDemander)
{
	CMuzipCmdLine cmdInfo;
	cmdInfo.SetShellCommand(FileComp);
	cmdInfo.m_fDemanderOptions = fDemander;
	if (cmd==CMD_COMPRESSERAUTO)
		cmdInfo.m_strFileName = m_pszArchive;
	
	CPlusItem *const ppi = new CPlusItem;
	cmdInfo.m_arPlus.AddFin(ppi);
	ppi->SetDefautExt();
	
	CString &sFiltre = ppi->DO_sFiltre;
	
	if (m_lstFichier.GetNb()>1) {
		// Ajouter la slection  la liste des fichiers/dossiers  compresser
		
		POS pos = m_lstFichier.GetPosDeb();
		while (pos) {
			TCHAR pszFichier[MAX_PATH];
			lstrcpy(pszFichier, m_lstFichier.GetSuiv(pos));
			PathQuoteSpaces(pszFichier);
			
			if (!sFiltre.IsEmpty())
				sFiltre += cSepPartie;
			sFiltre += pszFichier;
		}
		
	}else{
		// Un seul fichier: grer le cas des dossiers
		
		TCHAR pszFichier[MAX_PATH];
		lstrcpy(pszFichier, m_lstFichier.GetDeb());
		if (GetFileAttributes(pszFichier) & FILE_ATTRIBUTE_DIRECTORY)
			// C'est bien un rpertoire, et il est tout seul:
			// lui rajouter \*.*
			PathAppend(pszFichier, _T("*.*"));
		
		PathQuoteSpaces(pszFichier);
		sFiltre = pszFichier;
	}
	
	e_pma->ProcessShellCommand(cmdInfo, saExplorer);
}

// Attacher l'archive  un nouveau mail
// On doit utiliser ShellExecute() car l'attachement d'un mail
// demande un CDocument
// Ligne de commande excute: Muzip.exe /a /q <Archive>
void CMuzipExt::cmdMail()
{
	CString sArgs = _T("/a /q ");
	sArgs += m_pszArchive;
	::ShellExecute(NULL, NULL, pszMuzipEXE, sArgs, NULL, SW_HIDE);
}


// Ajouter un item au menu contextuel
void CMuzipExt::AddMenuItem(ADDSTATE& ads, UINT id)
{
	// Charger le nom de l'item
	m_aid[ads.nIndex] = id;
	if (!ads.nIndex)
		m_idFirst = id;
	CString &sItem = m_as[ads.nIndex];
	sItem.LoadString(id);
	sItem.ReleaseBuffer(sItem.Find('\n'));
	
	
	// Rcuprer le nom du premier fichier slectionn,
	// sans chemin d'accs ni extension
	// (pour les items  intitul dynamique)
	TCHAR pszFichier[MAX_PATH];
	lstrcpy(pszFichier, PathFindFileName(m_lstFichier.GetDeb()));
	
	switch (id) {
		
		// Dcompresser dans
		case CMD_DECOMPAUTO:
			PathRemoveExtension(pszFichier);
			break;
		
		// Compresser dans
		case CMD_COMPRESSERAUTO:
			if (PathIsDirectory(pszFichier))
				PathAddZipExtension(pszFichier);
			else
				PathRenameExtension(pszFichier, pszZipExt);
			break;
		
		// Autres items
		default:
			*pszFichier = 0;
			break;
	}
	
	// Supprimer ce qu'il y a aprs le '\n' (le message d'aide)
	// et ajouter  la chane finale l'ventuel nom du fichier
	if (*pszFichier)
		PathQuoteSpaces(pszFichier);
	sItem += pszFichier;
	
	// Ajouter l'item
	ads.mii.dwItemData = (DWORD)(ads.mii.dwTypeData = (STRING)sItem);
	InsertMenuItem(ads.hMenu, ads.indexMenu++, TRUE, &ads.mii);
	ads.mii.wID++;
	ads.nIndex++;
}



//*** IQueryInfo

// Demande de l'info-bulle
STDMETHODIMP CMuzipExt::GetInfoTip(DWORD, LPWSTR* poszTip)
{
	COptionsExt opt;
	if (!(opt.m_dwShell & gsInfoBulle) || !m_lstFichier.GetNb())
		return E_NOTIMPL;
	
	IMalloc *pMalloc;
	if (FAILED(SHGetMalloc(&pMalloc)))
		return E_NOTIMPL;
	
	HRESULT hrRes = E_FAIL;
	
	// Ouvrir l'archive
	CMuzipArc *const pArc = OpenArchive(m_pszArchive, NULL, NULL, true);
	if (pArc) {
		
		// Passer l'archive en lecture seule
		pArc->SetLectureSeule();
		
		
		//*** Rcuprer les valeurs (taille totale, nombre de fichiers)
		
		CLstFichier lstFichier;
		DWORD       dwTailleC = 0;
		DWORD       dwTailleN = 0;
		
		// Rcuprer la liste des fichiers
		pArc->ListerFichier(lstFichier);
		
		// Calculer la taille totale de l'archive non-compresse
		POS pos = lstFichier.GetPosDeb();
		while (pos) {
			CFichier *const pFichier = lstFichier.GetSuiv(pos);
			dwTailleC += pFichier->m_dwTailleC;
			dwTailleN += pFichier->m_dwTailleN;
		}
		
		
		//*** Formatter les valeurs
		
		// Formatter le nombre de fichiers et la taille dcompresse totale
		WCHAR oszTailleN[32];
		StrFormatByteSizeW(dwTailleN, oszTailleN, nbArray(oszTailleN));
		
		// Formatter le message complet
		CString sTout;
		sTout.Format(CStringRes(IDS_INFOBULLE), lstFichier.GetNb(), oszTailleN);
		const int bufTout = sTout.GetLength()+1;
		
		// Allouer un tampon et y copier la chane
		*poszTip = (LPWSTR)pMalloc->Alloc(bufTout*sizeof(*poszTip[0]));
		if (!*poszTip) {
			hrRes = E_OUTOFMEMORY;
			goto Suite;
		}
		
		// Tout est OK
		strtoW(*poszTip, sTout, bufTout);
		hrRes = S_OK;
		
Suite:
		pArc->Release(false);
	}
	
	pMalloc->Release();
	
	return hrRes;
}
