// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// MuzipExt.h


#ifndef _MUZIPEXT_H_
#define _MUZIPEXT_H_

#include "Options.h"


extern "C" AFX_EXT_CLASS ULONG e_nbTotRef;
extern                   HMENU e_hMenuContextuel;


class CRefMgr
{
public:  // Fonctions
	
	CRefMgr() : m_nbRef(0) {}
	
	ULONG GetCount() const { return m_nbRef; }
	
	ULONG AddRef();
	ULONG Release();
	
	
protected:  // Donnes
	
	ULONG m_nbRef;      // Compteur de rfrence
};


#define REFMGR_DECLARE(classe) \
protected: \
	CRefMgr m_refMgr; \
public: \
	\
	STDMETHODIMP_(ULONG) AddRef() \
		{ \
			return m_refMgr.AddRef(); \
		} \
	\
	STDMETHODIMP_(ULONG) Release() \
		{ \
			TRACE1(#classe _T(" - Release (%d)\n"), m_refMgr.GetCount()-1); \
			\
			if (!m_refMgr.Release()) { \
				delete this; \
				return 0; \
			} \
			return m_refMgr.GetCount(); \
		}



// Menus contextuels pour Windows 95
class CMuzipExtClassFactory : public IClassFactory
{
	REFMGR_DECLARE(CMuzipExtClassFactory)
	
public:  // Hritage
	
	// IUnknown
	STDMETHODIMP QueryInterface(REFIID riid, void** ppv);
	
	// IClassFactory
	STDMETHODIMP CreateInstance(IUnknown* pUnkOuter, REFIID riid, void** ppvObj);
	STDMETHODIMP LockServer(BOOL fLock);
};


struct ADDSTATE;


// Classe de l'application
class CMuzipExt : public
	IShellExtInit,
	IDropTarget,
	IContextMenu3,
	IPersistFile,
	IQueryInfo
{
	REFMGR_DECLARE(CMuzipExt)
	
public:  // Hritage
	
	HRESULT InitDataObject(IDataObject* pdo);
	void InitFichier();
	
	
	// IUnknown
	STDMETHODIMP QueryInterface(REFIID riid, void** ppv);
	
	// IShellExtInit
	STDMETHODIMP Initialize(LPCITEMIDLIST pidlFolder, IDataObject* pdo, HKEY hKeyID);
	
	// IDropTarget
	STDMETHODIMP DragEnter(
		IDataObject* pdo, DWORD grfKeyState, POINTL pt, DWORD* pdwEffect);
	STDMETHODIMP DragOver(
		DWORD grfKeyState, POINTL pt, DWORD* pdwEffect);
	STDMETHODIMP DragLeave();
	STDMETHODIMP Drop(
		IDataObject* pdo, DWORD grfKeyState, POINTL pt, DWORD* pdwEffect);
	
	STDMETHODIMP HandleMenuMsg (UINT uMsg, WPARAM wParam, LPARAM lParam);
	STDMETHODIMP HandleMenuMsg2(UINT uMsg, WPARAM wParam, LPARAM lParam, LRESULT* plResult);
	
	// IContextMenu
	STDMETHODIMP QueryContextMenu(
		HMENU hMenu, UINT indexMenu, UINT idCmdFirst, UINT idCmdLast, UINT uFlags);
	STDMETHODIMP InvokeCommand(CMINVOKECOMMANDINFO* pcmi);
	STDMETHODIMP GetCommandString(
		UINT idCmd, UINT uFlags, UINT*, LPSTR pszName, UINT cchMax);
	
	// IPersistFile
	STDMETHODIMP GetClassID(CLSID*)             { return E_FAIL; }
	STDMETHODIMP IsDirty()                      { return S_FALSE; }
	STDMETHODIMP Load         (LPCOLESTR, DWORD);
	STDMETHODIMP Save         (LPCOLESTR, BOOL) { return E_FAIL; }
	STDMETHODIMP SaveCompleted(LPCOLESTR)       { return E_FAIL; }
	STDMETHODIMP GetCurFile   (LPOLESTR*)       { return E_FAIL; }
	
	// IQueryInfo
	STDMETHODIMP GetInfoFlags(DWORD*) { return E_NOTIMPL; }
	STDMETHODIMP GetInfoTip(DWORD dwFlags, LPWSTR* poszTip);
	
	
public:  // Fonctions
	
	
	CMuzipExt() : m_fInitialized(false) {}
	
	enum DEST
	{
		destDemander,
		destAuto,
		destIci,
	};
	
	LPCTSTR GetArchive() const { return m_pszArchive; }
	
	void cmdOuvrir();
	void cmdDecomp    (UINT cmd, bool fDemander);
	void cmdCompresser(UINT cmd, bool fDemander);
	void cmdMail();
	
	
protected:  // Interne
	
	void AddMenuItem(ADDSTATE& ads, UINT id);
	
	
	bool m_fInitialized;
	
	// Page de proprits
	static UINT    CALLBACK PropPageCallback(HWND hDlg, UINT uMsg, PROPSHEETPAGE* ppsp);
	static BOOL    CALLBACK PropPageDlgProc (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
	
	// Version de IContextMenu demande
	int m_iContextMenuVersion;
	
	// Chemin d'accs du fichier cliqu et de l'archive correspondante
	TCHAR        m_pszArchive[MAX_PATH];
	int          m_mct;          // Type de slection
	
	// Donnes du menu contextuel
	CLstCString2 m_lstFichier;
	enum{ maxItem = 5 };
	
	UINT    m_aid[maxItem];   // ID des items insrs dans le menu (CMD_*)
	CString m_as[maxItem];    // Intitul des items
	UINT    m_idFirst;        // ID rel (dans le menu) du premier item ajout
};


#endif
