// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// Options.cpp


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "Muzip.h"
#include "Options.h"

#include "IOFichier.h"

#include "MuzipArc.h"
#include "Interf.h"
#include "MuzipDoc.h"
#include "MuzipView.h"
#include "ApercuView.h"

#include "dlgResRech.h"
#include "lstMuzip.h"

#include "Reg.h"

#include <Pidl.h>


const LPCTSTR eszGeneralDLL = _T("DLL");


const UINT auShellMenuMax[mctTotal] =
{
	// Archive
	mccOuvrir | mccDecomp | mccDecompAuto | mccDecompIci | mccMail,
	
	// Fichier
	mccCompresser | mccCompresserAuto,
	
	// Dossier
	mccCompresser | mccCompresserAuto,
};



COptions e_opt;


// Charger les options
void COptions::Load()
{
	// Enregistrer la DLL d'extension si ncessaire
	if (!oApp.GetProfileInt(kszGeneral, eszGeneralDLL, FALSE)) {
		DllRegisterServer();
		oApp.WriteProfileInt(kszGeneral, eszGeneralDLL, TRUE);
	}
	
	
	// Charger toutes les valeurs
	#define LoadValeur(page, type, prefixe, nom, defaut) \
		GetMembre(page, prefixe, nom) = oApp.GetProfileInt(GetKSZ(page), \
			GetESZ(page, prefixe, nom), defaut);
	#define LoadChaine(page, type, prefixe, nom, defaut) \
		GetMembre(page, prefixe, nom) = oApp.GetProfileString(GetKSZ(page), \
			GetESZ(page, prefixe, nom), defaut);
	ManipulerValeurComplet(fnManipulerMembres1, LoadValeur) \
	ManipulerChaineComplet(fnManipulerMembres2, LoadChaine)
	#undef LoadValeur
	#undef LoadChaine
	
	
	// Options des dossiers
	for (int i=0; i<repTotal; i++) {
		DOSSIER_OPTION &doo = m_aDossier[i];
		
		// Lire les options
		CString esz;
		esz.Format(eszDossierType, i);
		doo.repo   = oApp.GetProfileInt   (GetKSZ(Dossiers), esz, repoArchive);
		esz.Format(eszDossierPerso, i);
		doo.sPerso = oApp.GetProfileString(GetKSZ(Dossiers), esz);
		
		// Empcher les options impossibles
		if (doo.repo<0 || doo.repo>=repoTotal)
			doo.repo = repoArchive;
		if ((doo.repo==repoArchive || doo.repo==repoDernier) &&
		   !(i==repAjout || i==repDecomp))
			doo.repo = repoAuto;
	}
	
	
	BYTE *pb;
	UINT len;
	
	// Options de la reconstruction manuelle
	bool fOK = false;
	if (oApp.GetProfileBinary(GetKSZ(Comp), GetESZ(Comp, a,RmOption), &pb, &len) &&
	    len==sizeof(m_aRmOption)) {
		// Charger les options et les valider
		
		memcpy(m_aRmOption, pb, sizeof(m_aRmOption));
		fOK = true;
		
		// Vrifier qu'on a bien une et une seule fois chaque option
		bool afDeja[nbRmOption];
		ZeroMemory(afDeja, sizeof(afDeja));
		for (int i=0; i<nbRmOption; i++) {
			if (m_aRmOption[i].rmo<0 ||
			    m_aRmOption[i].rmo>=nbRmOption ||
			    afDeja[i]) {
				fOK = false;
				break;
			}
			afDeja[i] = true;
		}
	}
	delete [] pb;
	
	if (!fOK) {
		// Options par dfaut
		
		for (int i=0; i<nbRmOption; i++) {
			m_aRmOption[i].rmo     = (RM_OPTION_AC)i;
			m_aRmOption[i].fActive = true;
		}
	}
	
	
	MAJAffichageStyleEx();
	MAJPasCompExt();
}


// Sauver les options
void COptions::Save()
{
	// Enregistrer toutes les valeurs
	#define SaveValeur(page, type, prefixe, nom, defaut) \
		pApp->WriteProfileInt(GetKSZ(page), GetESZ(page, prefixe, nom), \
			GetMembre(page, prefixe, nom));
	#define SaveChaine(page, type, prefixe, nom, defaut) \
		pApp->WriteProfileString(GetKSZ(page), GetESZ(page, prefixe, nom), \
			GetMembre(page, prefixe, nom));
	ManipulerValeurComplet(fnManipulerMembres1, SaveValeur) \
	ManipulerChaineComplet(fnManipulerMembres2, SaveChaine)
	#undef SaveValeur
	#undef SaveChaine
	
	
	// Options des dossiers
	for (int i=0; i<repTotal; i++) {
		const DOSSIER_OPTION &doo = m_aDossier[i];
		
		CString esz;
		esz.Format(eszDossierType, i);
		oApp.WriteProfileInt   (GetKSZ(Dossiers), esz, doo.repo);
		esz.Format(eszDossierPerso, i);
		oApp.WriteProfileString(GetKSZ(Dossiers), esz, doo.sPerso);
	}
	
	
	// Options de la reconstruction manuelle
	oApp.WriteProfileBinary(GetKSZ(Comp), GetESZ(Comp, a,RmOption),
		(BYTE*)m_aRmOption, sizeof(m_aRmOption));
}


// Appliquer les options
void COptions::Appliquer()
{
	// Mettre  jour le style des listes
	// et les aperus
	POS pos = e_lstArc.GetPosDeb();
	while (pos) {
		CMuzipDoc *const pDoc = e_lstArc.GetSuiv(pos)->m_pi->GetDoc();
		if (!pDoc)
			continue;
		
		pDoc->m_plst->m_plstMuzip->OnOptionsChange();
		if (pDoc->m_papr && pDoc->m_papr->m_pApercu)
			pDoc->m_papr->m_pApercu->MAJ();
	}
	
	// Ne pas oublier la liste des rsultats de la recherche !
	if (e_pdlgResRech)
		e_pdlgResRech->m_lst.OnOptionsChange();
	
	
	// Recrer le thread de surveillance gnrale
	// car il utilise GetDossierTemp()
	CSurvDecomp::TerminateThreadTous();
	CSurvDecomp::CreateThreadTous();
}

// Mettre  jour le style des listes d'aprs les paramtres
void COptions::MAJAffichageStyleEx()
{
	int afs  = afsClicDeux;
	int afss = afssJamais;
	
	SHELLFLAGSTATE sfs;
	sfs.fWin95Classic = TRUE;
	
	// Excuter SHGetSettings
	const HINSTANCE hShell32 = AfxLoadLibrary(_T("Shell32.dll"));
	if (hShell32) {
		typedef void (WINAPI *PFNSHGETSETTINGSPROC)(SHELLFLAGSTATE* psfs, DWORD dwMask);
		const PFNSHGETSETTINGSPROC pfnSHGetSettings =
			(PFNSHGETSETTINGSPROC)GetProcAddress(hShell32, "SHGetSettings");
		
		if (pfnSHGetSettings)
			pfnSHGetSettings(&sfs, SSF_DOUBLECLICKINWEBVIEW | SSF_WIN95CLASSIC);
		
		AfxFreeLibrary(hShell32);
	}
	
	if (!sfs.fWin95Classic && !sfs.fDoubleClickInWebView) {
		// Pas de clic pour slectionner
		// Rcuprer la valeur IconUnderline, premier octet:
		// 03 - toujours soulign (comme navigateur)
		// 02 - soulign sous pointeur
		
		afs  = afsClicUn;
		afss = afssPointe;
		
		HKEY hKey;
		if (RegOpenKeyEx(HKEY_CURRENT_USER, _T("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer"), 0, KEY_ALL_ACCESS, &hKey)==ERROR_SUCCESS) {
			DWORD len;
			if (RegQueryValueEx(hKey, _T("IconUnderline"), 0, NULL, NULL, &len)==ERROR_SUCCESS && len>=1) {
				// La valeur existe, on a rcuprer sa longueur
				BYTE *const pb = new BYTE[len];
				if (RegQueryValueEx(hKey, _T("IconUnderline"), 0, NULL, pb, &len)==ERROR_SUCCESS &&
				    pb[0]==0x03)
					afss = afssToujours;
				delete [] pb;
			}
			RegCloseKey(hKey);
		}
	}
	
	// craser les options de Windows si ncessaire
	if (m_nAffichageSouris!=afsWindows) {
		afs = m_nAffichageSouris;
		if (m_nAffichageSouris==afsClicUn)
			// Simple clic
			afss = m_nAffichageSouris1;
	}
	
	// Option par dfaut pour le simple clic
	if (m_nAffichageSouris!=afsClicUn)
		m_nAffichageSouris1 = afss;
	
	if (afs==afsClicUn) {
		// Simple clic
		
		if      (afss==afssJamais)
			m_nAffichageStyleEx = LVS_EX_ONECLICKACTIVATE | LVS_EX_TRACKSELECT;
		else if (afss==afssPointe)
			m_nAffichageStyleEx = LVS_EX_ONECLICKACTIVATE | LVS_EX_TRACKSELECT | LVS_EX_ONECLICKACTIVATE | LVS_EX_UNDERLINEHOT;
		else // (afss==afssToujours)
			m_nAffichageStyleEx = LVS_EX_ONECLICKACTIVATE | LVS_EX_TRACKSELECT | LVS_EX_ONECLICKACTIVATE | LVS_EX_UNDERLINECOLD;
		
	}else // (afs==afsClicDeux)
		// Double clic
		m_nAffichageStyleEx = 0;
	
	if (m_fAffichageGrille)    m_nAffichageStyleEx |= LVS_EX_GRIDLINES;
	if (m_fAffichageSelLigne)  m_nAffichageStyleEx |= LVS_EX_FULLROWSELECT;
}

// Mettre  jour la liste des extensions  ne pas compresser
void COptions::MAJPasCompExt()
{
	m_lstPasCompExt.RemoveAll();
	
	CTokenize tok(m_sCompPasCompExt, ';');
	while (tok.Next()) {
		if (tok.GetToken().IsEmpty())
			continue;
		
		// Mettre l'extension en minuscules dans la liste
		tok.GetToken().MakeLower();
		m_lstPasCompExt.AddFin(tok.GetToken());
	}
}


// Rcuprer le nom d'un dossier
bool COptions::GetDossier(int rep, LPTSTR pszDossier, LPCTSTR pszArchive) const
{
	const DOSSIER_OPTION &doo = m_aDossier[rep];
	
	bool fOK = true;
	
	switch (doo.repo) {
		
		// Rpertoire de l'archive
		case repoArchive:
			if (pszArchive) {
				lstrcpy(pszDossier, pszArchive);
				PathRemoveFileSpec(pszDossier);
			}else
				*pszDossier = 0;
			break;
		
		// Automatique
		case repoAuto:
			switch (rep) {
				
				// Ajout/extraction: rpertoire du bureau
				case repAjout:
				case repDecomp:
					{
						LPITEMIDLIST pidl;
						VERIF(SUCCEEDED(SHGetSpecialFolderLocation(NULL,
							CSIDL_DESKTOP, &pidl)));
						fOK = ToBool(SHGetPathFromIDList(pidl, pszDossier));
						
						e_pidlManager.Free(pidl);
					}
					break;
				
				// Dossier temporaire
				// On prend le dossier temporaire de Windows
				// et on y ajoute un nom de sous-dossier
				case repTemp:
					if (GetTempPath(MAX_PATH, pszDossier))
						PathAppend(pszDossier, oApp.m_pszAppName);
					else
						fOK = false;
					break;
			}
			break;
		
		// Dernier rpertoire
		// Rpertoire personnalis
		case repoDernier:
		case repoPerso:
			lstrcpy(pszDossier, doo.sPerso);
			break;
	}
	
	
	// S'assurer de l'existence du dossier
	if (fOK)
		fOK = DossierCreer(pszDossier);
	else
		*pszDossier = 0;
	
	return fOK;
}


// Mmoriser le dernier dossier de type donn
// s'il est en mode "dernier"
void COptions::SetDossierDernier(int rep, LPCTSTR pszDossier)
{
	if (m_aDossier[rep].repo==repoDernier)
		m_aDossier[rep].sPerso = pszDossier;
}



//*** MCT_OPTION

// Lire les options dans le registre
// Attention ! Le pointeur retourn doit tre libr par:
// delete [] (BYTE*)pointeur_ici;
MCT_OPTION* MCT_OPTION::GetArray()
{
	// Lire les options
	MCT_OPTION *aMCT;
	UINT len;
	if (!oApp.GetProfileBinary(kszGeneral, eszGeneralMCT, (BYTE**)&aMCT, &len) ||
	    len!=bufMCT) {
		// Pas d'options: prendre des options par dfaut
		delete [] aMCT;
		aMCT = (MCT_OPTION*)new BYTE[bufMCT];
		
		for (int mct = 0; mct < mctTotal; mct++) {
			aMCT[mct].uVisible = auShellMenuMax[mct];
			aMCT[mct].uDemande = mccCompresser;
		}
	}
	
	return aMCT;
}
