// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// FileBuf.h
//
// CFileBuf   - classe abstraite de base
// CFileRead  - lecture de fichier
// CFileWrite - criture dans un fichier
//
// CFileRead et CFileWrite sont deux classes permettant respectivement
// de lire et d'crire des fichiers en mode bufferis avec une gestion
// complte des erreurs
// Ces classes sont  prfrer  celles de la bibliothque standard,
// et ce pour deux raisons:
// 1) Leur code est relativement simple et reste trs proche de Win32,
//    donc trs simple  dbugger
// 2) La gestion des erreurs est automatique (pas de errno)
//
//
// SYMBOLES
//
// - FILEBUF_VIRTUALREAD
//   Dclare CFileRead::ReadProtect() en virtuel
//   Cela permet d'intercepter dans une classe drive
//   toutes les oprations de lecture
// 
// - FILEBUF_VIRTUALWRITE
//   Dclare CFileWrite::WriteProtect() en virtuel
//   Cela permet d'intercepter dans une classe drive
//   toutes les oprations d'criture
//
//
// REQUIERT
// - FileBuf.cpp
// - Application Windows
// - Fonction AfxMessageBox() dclare implmente
// - Dans StdAfx.h:
//   #pragma include_alias("AppResource.h", "../RpertoireProjet/Resource.h")
//
// Guillaume Ryder, gryder@netcourrier.com, http://utilfr42.free.fr/


#ifndef _FILEBUF_H_
#define _FILEBUF_H_
#if _MSC_VER>1000
#pragma once
#endif


class CFileBuf
{
protected:  // Interne
	
	// Constructeur
	CFileBuf::CFileBuf()
		: m_hf(INVALID_HANDLE_VALUE),
#ifdef _DEBUG
		m_fAttach(false),
#endif
		m_fVerbeux(true), m_fPeutFermer(true)
		{}
	
	// Destructeur
#if defined(_DEBUG) && defined(ASSERT)
	~CFileBuf()
		{
			ASSERT(m_fAttach || m_hf==INVALID_HANDLE_VALUE);
		}
#endif
	
	
public:  // Fonctions
	
	enum{ lenFileBuf = 0x8000 };
	
	bool GetVerbeux() const { return m_fVerbeux; }
	void SetVerbeux(bool fVerbeux=true) { m_fVerbeux = fVerbeux; }
	bool GetPeutFermer() const { return m_fPeutFermer; }
	void SetPeutFermer(bool fPeutFermer=true) { m_fPeutFermer = fPeutFermer; }
	
	LPCTSTR GetPath() const { return m_pszFichier; }
	
	
	void Close();
	
	void   Attach(HANDLE hf);
	HANDLE Detach();
	
	
	operator HANDLE() const { return m_hf; }
	
	
protected:  // Interne
	
	void CloseErreur();
	
	int Erreur(UINT nID, UINT nFlags);
	
	
	TCHAR  m_pszFichier[MAX_PATH];
	HANDLE m_hf;                  // Handle du fichier
	size_t m_len;                 // Nombre d'octets lus dans le tampon
	BYTE   m_abBuf[lenFileBuf];   // Tampon + 4 octets de rab
#ifdef _DEBUG
	bool   m_fAttach;             // TRUE Attach() a t utilis
#endif
	bool   m_fVerbeux;            // TRUE en mode verbeux
	bool   m_fPeutFermer;         // TRUE si le fichier peut tre ferm si erreur
	
	static DWORD s_dwES;          // Tampon DWORD pour WriteFile() et ReadFile()
};



#ifndef NO_FILEREAD

class CFileRead : public CFileBuf
{
	friend class CFileWrite;
	
public:  // Fonctions
	
	bool Open(LPCTSTR pszFichier,
		DWORD dwShare=0, DWORD dwCreate=OPEN_EXISTING,
		DWORD dwFlags=FILE_ATTRIBUTE_NORMAL);
	
	void   Attach(HANDLE hf);
	HANDLE Detach();
	
	bool  Read   (void* pBuf, DWORD len);
	DWORD ReadLen(void* pBuf, DWORD len);
	bool  Read(BYTE&  b);
	bool  Read(WORD&  w);
	bool  Read(DWORD& dw) { return Read(&dw, sizeof(dw)); }
	bool  RAZ();
	
	DWORD Tell() const;
	bool  Seek   (long lMove, DWORD dwMethode);
	DWORD SeekPos(long lMove, DWORD dwMethode);
	
	bool IsEOF() const
		{
#ifdef ASSERT
			ASSERT(m_hf!=INVALID_HANDLE_VALUE);
#endif
			return m_fEOF;
		}
	
	
protected:  // Interne
	
#ifdef FILEBUF_VIRTUALREAD
	virtual
#endif
	bool ReadProtect(BYTE* pbBuf, DWORD len, DWORD lenFaut);
	
	
	size_t m_posCur;   // Position courante de lecture
	bool   m_fEOF;     // TRUE si fin du fichier atteinte
};

#endif


#ifndef NO_FILEWRITE

class CFileWrite : public CFileBuf
{
public:  // Fonctions
	
	bool Open(LPCTSTR pszFichier,
		DWORD dwShare=0, DWORD dwCreate=CREATE_ALWAYS,
		DWORD dwFlags=FILE_ATTRIBUTE_NORMAL);
	void Close();
	
	HANDLE Detach();
	
	bool  Write(const void* pBuf, DWORD len);
	bool  Write(LPCSTR  psz);
	bool  Write(LPCWSTR psz);
	bool  Write(BYTE    b);
	bool  Write(WORD    w);
	bool  Write(DWORD   dw);
	bool  Flush();
	
	DWORD Tell() const;
	bool  Seek   (long lMove, DWORD dwMethode);
	DWORD SeekPos(long lMove, DWORD dwMethode);
	
	bool WriteFrom(CFileRead& fr, DWORD len);
	
	
protected:  // Interne
	
#ifdef FILEBUF_VIRTUALWRITE
	virtual
#endif
	bool WriteProtect(const BYTE* pbBuf, DWORD len);
};

#endif


#endif
