// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// GUnicode.h
//
// Macros pour la gestion des conversions entre ASCII et Unicode,
// dans les deux sens
//
// toBSTR(psz)   - convertit une chane "normale" en BSTR
// fromBSTR(psz) - convertit une BSTR en chane "normale"
//
// REQUIERT
// - Dans StdAfx.h:
//   #include <Windows.h>
//   Pour toBSTR uniquement:
//   #include <OLEAuto.h>
// - Pour fromBSTR uniquement:
//   Application MFC
//
// Guillaume Ryder, gryder@netcourrier.com, http://utilfr42.free.fr/


#ifndef _GUNICODE_H_
#define _GUNICODE_H_
#if _MSC_VER>1000
#pragma once
#endif


// Macros pour toutes les conversions possibles

#define strAtoA(pszA1, pszA2, buf)   strcpy(pszA1, pszA2)
#define strWtoW(pszW1, pszW2, buf)   wcscpy(pszW1, pszW2)

#define strAtoW(pszW, pszA, buf) \
	MultiByteToWideChar(CP_ACP,0, pszA,-1, pszW,buf)
#define strWtoA(pszA, pszW, buf) \
	WideCharToMultiByte(CP_ACP,0, pszW,-1, pszA,buf, NULL,NULL)

#define initAtoA( nom, pszA, buf)  const LPSTR   nom = pszA
#define initAtoAC(nom, pszA, buf)  const LPCSTR  nom = pszA
#define initWtoW( nom, pszW, buf)  const LPWSTR  nom = pszW
#define initWtoWC(nom, pszW, buf)  const LPCWSTR nom = pszW

#define initAtoW(nom, pszA, buf) \
	WCHAR nom[buf]; \
	strAtoW(nom, pszA, buf)
#define initAtoWC  initAtoW

#define initWtoA(nom, pszW, buf) \
	char nom[buf]; \
	strWtoA(nom, pszW, buf)
#define initWtoAC  initWtoA



#ifdef    UNICODE
// Mode Unicode

#define strtoA     strWtoA
#define strtoW     strWtoW
#define strfromA   strAtoW
#define strfromW   strWtoW

#define initA     initAtoW
#define initAC    initAtoWC
#define initW     initWtoW
#define initWC    initWtoWC

#define inittoA   initWtoA
#define inittoAC  initWtoAC
#define inittoW   initWtoW
#define inittoWC  initWtoWC


#else // !UNICODE
// Mode ASCII

#define strtoA    strAtoA
#define strtoW    strAtoW
#define strfromA  strAtoA
#define strfromW  strWtoA

#define initA     initAtoA
#define initAC    initAtoAC
#define initW     initWtoA
#define initWC    initWtoAC

#define inittoA   initAtoA
#define inittoAC  initAtoAC
#define inittoW   initAtoW
#define inittoWC  initAtoWC


#endif // UNICODE

#ifdef _OLEAUTO_H_

#ifdef MFCOK
#include <afxPriv.h>
#endif


class toBSTR
{
public:  // Fonctions
	
	toBSTR(LPCSTR asz)
		{
			const size_t len = strlen(asz);
			if (len) {
				m_bsz = ::SysAllocStringLen(NULL, len);
				strAtoW(m_bsz, asz, len+1);
			}else
				m_bsz = NULL;
		}
	
	toBSTR(LPCWSTR wsz)
		{
			m_bsz = ::SysAllocString(wsz);
		}
	
#ifdef MFCOK
	toBSTR(UINT nID)
		{
			const int lenMax = 512;
			TCHAR psz[lenMax];
			const int len = AfxLoadString(nID, psz, lenMax);
			if (len) {
#ifdef UNICODE
				m_bsz = ::SysAllocStringLen(psz, len);
#else
				m_bsz = ::SysAllocStringLen(NULL, len);
				strtoW(m_bsz, psz, len+1);
#endif
			}else
				m_bsz = NULL;
		}
#endif
	
	toBSTR(HINSTANCE hInst, UINT nID)
		{
			const int len = 512;
			TCHAR psz[len];
			if (::LoadString(hInst, nID, psz, len)) {
#ifdef UNICODE
				m_bsz = ::SysAllocString(psz);
#else
				WCHAR wsz[len];
				strtoW(wsz, psz, len);
				m_bsz = ::SysAllocString(wsz);
#endif
			}else
				m_bsz = NULL;
		}
	
	~toBSTR()
		{
			::SysFreeString(m_bsz);
		}
	
	operator const BSTR() const { return m_bsz; }
	
	
protected:  // Donnes
	
	BSTR m_bsz;
};

#endif // _OLEAUTO_H_


#ifdef    MFCOK

#ifdef    UNICODE
#define fromBSTR(bsz)          bsz
#else // !UNICODE
#define fromBSTR(bsz)  CString(bsz)
#endif // UNICODE

#endif // !MFCOK


#endif
