// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// ControlBarEx.cpp
//
// ControlBarEx - classe de base pour les barres d'outils avec effets 3D
//
// REQUIERT
// - Application MFC
//
// Guillaume Ryder, gryder@netcourrier.com, http://utilfr42.free.fr/


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include <App.h>
#include <ControlBarEx.h>


#ifdef TOOLBAREX_LISTE

CListeBaseBarEx e_lstBaseBarEx;


// Ajouter
void CListeBaseBarEx::Add(const CControlBar* pBar)
{
	const CControlBar* *const aNew = new const CControlBar*[m_nb+1];
	memcpy(aNew, m_a, m_nb*sizeof(const CControlBar*));
	delete [] m_a;
	m_a = aNew;
	m_a[m_nb++] = pBar;
}

// Supprimer
void CListeBaseBarEx::Del(const CControlBar* pBar)
{
	for (int i=0; i<m_nb; i++) {
		if (m_a[i]==pBar) {
			m_a[i] = m_a[--m_nb];
			break;
		}
	}
}

// Rechercher
bool CListeBaseBarEx::IsBaseBarEx(const CControlBar* pBar) const
{
	for (int i=0; i<m_nb; i++)
		if (m_a[i]==pBar)  return true;
	return false;
}

#endif


#ifdef TOOLBAREX_REG

const LPTSTR eszBarreOutilsVisible  = _T("Visible");
const LPTSTR eszBarreOutilsFlottant = _T("Flottant");
const LPTSTR eszBarreOutilsPosition = _T("Position");
const LPTSTR eszBarreOutilsStyle    = _T("Style");


// Enregistrer les donnes dcrivant la barre d'outils
// dans le registre,  appeler dans OnClose() et pas dans OnDestroy()
void ToReg(const CControlBar* pBar, LPCTSTR pszKey)
{
	pApp->WriteProfileInt(pszKey, eszBarreOutilsVisible,  pBar->IsVisible());
	pApp->WriteProfileInt(pszKey, eszBarreOutilsFlottant, pBar->IsFloating());
	
	// Rcuprer la position cran de la barre d'outils
	CRect rc;
	pBar->GetWindowRect(rc);
	
	if (!pBar->IsFloating()) {
		// Pour les barres d'outils dockes, il faut enregistrer
		// leur style (haut, bas, gauche, droite) et relativiser
		// leur position par rapport  la fentre
		pBar->GetParentFrame()->ScreenToClient(rc);
		pApp->WriteProfileInt(pszKey, eszBarreOutilsStyle, ((CControlBar*)pBar)->GetBarStyle());
	}
	
	// Enregistrer la position cran de la barre d'outils
	pApp->WriteProfileBinary(pszKey, eszBarreOutilsPosition, (BYTE*)&rc, sizeof(RECT));
}


// Paramtrer la barre d'outils suivant les donnes places
// dans une clef du registre
bool FromReg(CControlBar* pBar, LPCTSTR pszKey)
{
	CFrameWnd *const pParent = pBar->GetParentFrame();
	
	// Barre d'outils visible ?
	pParent->ShowControlBar(pBar, pApp->GetProfileInt(pszKey, eszBarreOutilsVisible, TRUE), TRUE);
	
	// Barre d'outils flottante ?
	// Si la valeur n'est pas dfinie,
	// passer  la barre d'outils suivante
	const BOOL fFloat = pApp->GetProfileInt(pszKey, eszBarreOutilsFlottant, -1);
	VERIF(fFloat!=-1);
	
	// Rcuprer la position et les dimensions
	// de la barre d'outils
	CRect *prc;
	UINT len;
	VERIF(pApp->GetProfileBinary(pszKey, eszBarreOutilsPosition, (BYTE**)&prc, &len));
	
	// Le chargement est russi
	// Vrifier que la taille est la bonne (on ne sait jamais!)
	if (len!=sizeof(CRect)) {
		// Erreur: taille diffrente
		delete prc;
		return false;
	}
	if (prc) {
		if (fFloat)
			// Barre d'outils flottante
			// La dplacer sans se compliquer
			pParent->FloatControlBar(pBar, prc->TopLeft());
		
		else{
			// Barre d'outils accroche
			// Plus complexe, car il faut avoir:
			// - sa position en coordonnes
			// - sa position par rapport  la fentre (haut, bas...)
			
			// La barre d'outils doit tre en coordonnes cran
			// Elle a t enregistres en coordonnes fentre
			// pour garder le positionnement par rapport  la fentre
			pParent->ClientToScreen(prc);
			
			// Rcuprer le positionnement
			DWORD dwStyle = pApp->GetProfileInt(pszKey, eszBarreOutilsStyle, CBRS_ALIGN_TOP);
			
			// Le "convertir" en AFX_IDW_DOCKBAR_*
			if      (dwStyle & CBRS_ALIGN_TOP)     dwStyle = AFX_IDW_DOCKBAR_TOP;
			else if (dwStyle & CBRS_ALIGN_BOTTOM)  dwStyle = AFX_IDW_DOCKBAR_BOTTOM;
			else if (dwStyle & CBRS_ALIGN_LEFT)    dwStyle = AFX_IDW_DOCKBAR_LEFT;
			else if (dwStyle & CBRS_ALIGN_RIGHT)   dwStyle = AFX_IDW_DOCKBAR_RIGHT;
			
			// "Docker" la barre d'outils
			pParent->DockControlBar(pBar, dwStyle, prc);
		}
		
		// Dtruire le rectangle lu dans le registre
		delete prc;
	}
	
	return true;
}

#endif


#ifdef TOOLBAREX_MENU

#include "AppResource.h"


// Afficher le menu contextuel d'affichage des barres d'outils
void OnToolBarMenu(CFrameWnd* pFrame, CToolBar* ptlb)
{
	// Composer le menu
	CMenu mnu;
	mnu.CreatePopupMenu();
	
	
	if (ptlb) {
		// Ajouter la commande "Personnaliser"
		
		mnu.AppendMenu(MF_STRING, 1, CString(MAKEINTRESOURCE(IDS_CUSTOMIZE)));
		mnu.AppendMenu(MF_SEPARATOR);
	}
	
	
	struct BARREOUTILS
	{
		CControlBar *pBar;
		UINT         nID;
	};
	int nb = 0;
	BARREOUTILS *const abo = new BARREOUTILS[pFrame->m_listControlBars.GetCount()+1];
	BARREOUTILS *pbo = abo;
	UINT nID = 10;
	
	POSITION pos = pFrame->m_listControlBars.GetHeadPosition();
	while (pos) {
		CControlBar *const pBar = (CControlBar*)pFrame->m_listControlBars.GetNext(pos);
		if (ISKINDOF_BASEBAREX(pBar)) {
			
			pbo  ->pBar = pBar;
			pbo++->nID  = ++nID;
			
			CString s;
			pBar->GetWindowText(s);
			UINT nFlags = MF_BYPOSITION;
			if (pBar->IsWindowVisible())  nFlags |= MF_CHECKED;
			nb++;
			mnu.InsertMenu((UINT)-1, nFlags, nID, s);
		}
	}
	if (nb>1) {
		// Afficher le menu
		pbo->nID = 0;
		
		CPoint pt;
		GetCursorPos(&pt);
		const UINT nCmd = mnu.TrackPopupMenu(
			TPM_NONOTIFY | TPM_RETURNCMD | TPM_LEFTBUTTON | TPM_LEFTALIGN,
			pt.x,pt.y, pFrame);
		mnu.DestroyMenu();
		
		if (nCmd==1 && ptlb)
			// Personnaliser la barre d'outils
			
			ptlb->GetToolBarCtrl().Customize();
			
		else{
			// Afficher ou masquer la barre d'outils
			
			pbo = abo;
			while (pbo->nID) {
				if (pbo->nID==nCmd) {
					pFrame->ShowControlBar(pbo->pBar, !pbo->pBar->IsWindowVisible(), FALSE);
					break;
				}
				pbo++;
			}
		}
	}
	
	delete [] abo;
}

#endif
