// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// ShellMenu.cpp
//
// CShellMenu - permet de crer des menus ayant le mme contenu
// que le menu contextuel affich par le shell lorsqu'on clique
// droit sur un/des fichier(s) ou un dossier
//
// REQUIERT
// - Application MFC
//
// Guillaume Ryder, gryder@netcourrier.com, http://utilfr42.free.fr/


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "ShellMenu.h"

#include <ShlObj.h>


const UINT idVerbeMin = 0x1000;
const UINT idVerbeMax = 0x2000;


// Nettoyer les interfaces
void CShellMenu::Nettoyer()
{
	if (m_pcm) {
		m_pcm->Release();
		m_pcm = NULL;
	}
}


// Initialiser le menu sur des fichiers
void CShellMenu::InitFiles(IShellFolder* psf, int nb, const LPCITEMIDLIST* apidl)
{
	Nettoyer();
	
	// Rcuprer la meilleure interface IContextMenu possible
	// et s'en servir pour remplir le menu
	// On conserve son pointeur dans un membre de la classe
	// car on en aura besoin pour InvokeCommand()
	if (SUCCEEDED(psf->GetUIObjectOf(*m_pWnd,
			nb, (LPCITEMIDLIST*)apidl,
			IID_IContextMenu, NULL, (void**)&m_pcm))) {
		
		IContextMenu3 *pcm;
		if (SUCCEEDED(m_pcm->QueryInterface(IID_IContextMenu3, (void**)&pcm))) {
			m_pcm->Release();
			m_pcm = pcm;
			m_ncm = 3;
		}else if (SUCCEEDED(m_pcm->QueryInterface(IID_IContextMenu2, (void**)&pcm))) {
			m_pcm->Release();
			m_pcm = pcm;
			m_ncm = 2;
		}else
			m_ncm = 1;
		
		// Construire le menu
		m_pcm->QueryContextMenu(*m_pMenu, 0,
			idVerbeMin,idVerbeMax, CMF_NORMAL | CMF_CANRENAME | CMF_INCLUDESTATIC);
	}
}


// Messages
bool CShellMenu::WindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam, LRESULT* plResult)
{
	// Grer les messages des menus du shell
	// toutes versions:   WM_COMMAND
	// version 2 minimum: WM_INITMENUPOPUP, WM_DRAWITEM, WM_MEASUREITEM
	// version 3 minimum: WM_MENUCHAR
	
	if (m_pcm) {
		
		if (uMsg==WM_COMMAND && LOWORD(wParam)>=idVerbeMin && LOWORD(wParam)<=idVerbeMax) {
			CMINVOKECOMMANDINFO cmi;
			ZeroMemory(&cmi, sizeof(cmi));
			cmi.cbSize = sizeof(cmi);
			cmi.hwnd   = *m_pWnd;
			cmi.lpVerb = (LPCSTR)MAKEINTRESOURCE(LOWORD(wParam)-idVerbeMin);
			cmi.nShow  = SW_SHOWNORMAL;
			m_pcm->InvokeCommand(&cmi);
			*plResult = 0;
			return true;
		}
		
		
		if (m_ncm>=2) {
			
			switch (uMsg) {
				
				// Initialisation
				case WM_INITMENUPOPUP:
					{
						const UINT id = GetMenuItemID((HMENU)wParam, 0);
						if (id>=idVerbeMin && id<=idVerbeMax)  goto HandleMenuMsg;
					}
					break;
				
				// Le dbut des structures DRAWITEMSTRUCT et MEASUREITEMSTRUCT
				// est identique (du moins pour CtlType et itemID)
				case WM_DRAWITEM:
				case WM_MEASUREITEM:
					#define pdis  ((DRAWITEMSTRUCT*)lParam)
					if (pdis->CtlType==ODT_MENU &&
					    pdis->itemID>=idVerbeMin && pdis->itemID<=idVerbeMax) {
HandleMenuMsg:
						m_pcm->HandleMenuMsg(uMsg, wParam, lParam);
						*plResult = TRUE;
						return true;
					}
					break;
					#undef pdis
				
				// Touche de raccourci
				case WM_MENUCHAR:
					if (m_ncm>=3)
						return (HMENU)lParam==*m_pMenu &&
							SUCCEEDED(m_pcm->HandleMenuMsg2(uMsg, wParam, lParam, plResult));
					break;
			}
		}
	}
	
	return false;
}
