// Muzip 4.03
// 11 fvrier 2005
//
//
// Copyright (C) 2005 Guillaume Ryder
// gryder@club-internet.fr
// http://utilfr42.free.fr/muzip/
//
// Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou
// le modifier conformment aux dispositions de la Licence Publique Gnrale
// GNU, telle que publie par la Free Software Foundation ; version 2 de la
// licence, ou encore ( votre choix) toute version ultrieure.
//
// Ce programme est distribu dans l'espoir qu'il sera utile, mais
// SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION
// ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail,
// voir la Licence Publique Gnrale GNU.
//
// Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU
// en mme temps que ce programme ; si ce n'est pas le cas, crivez  la
// Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.


// Temps.cpp
//
// Gestion du temps  l'aide de classes
// CTemps     - temps en nombre de secondes depuis le 01/01/1970
// CTempsD    - date uniquement
// CTempsH    - heure uniquement
// CTempsDH   - date et heure
// CTempsHS   - heure avec secondes
// CTempsDHS  - date, heure et secondes
// CTempsSpan - diffrence entre deux temps
//
// Les classes CTempsD,H,DH,HS,DHS maintiennent le temps en stockant
// chaque valeur sparment (jour, mois, ...)
//
// REQUIERT
// - Application Windows
// - Application MFC (facultatif)
// - Dans StdAfx.h:
//   #pragma include_alias("AppResource.h", "../RpertoireProjet/Resource.h")
//
// Guillaume Ryder, gryder@netcourrier.com, http://utilfr42.free.fr/


#include "StdAfx.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "AppResource.h"
#include "Temps.h"

#include <math.h>
#include <tchar.h>

const int bufTempsDlg    = 32;
const int bufTempsFormat = 64;


const UINT e_aNbJourMois[12] =
{
	31, 28, 31, 30, 31, 30,
	31, 31, 30, 31, 30, 31,
};


#pragma warning (disable: 4710 4238)


const int secMinute =   60;                 // Secondes par minute
const int secHeure  = (  60 * secMinute);   // Secondes par heure
const int secJour   = (  24 * secHeure);    // Secondes par jour
const int secAn     = ( 365 * secJour);     // Secondes par anne de 365 jours
const int secAnB    = ( 366 * secJour);     // Secondes par anne de 366 jours
const int secAn4    = (1461 * secJour);     // Secondes pour 4 ans


// Nombre de jours entre le dbut d'une anne et un mois
static const USHORT usJours[] =
{
	0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365
};

// Nombre de jours entre le dbut d'une anne et la fin d'un mois
static const USHORT usJoursB[] =
{
	0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366
};


// Retourne un TEMPS  partir de la dfinition d'une date en JJ/MM/AAAA  HH:MM:SS
// Le temps sera en nombre de secondes sparant le 01/01/1970
// de la date/heure fournie
TEMPS MakeTemps(UINT uJour, UINT uMois, UINT uAn, UINT uHeure, UINT uMinute, UINT uSeconde)
{
	UINT nbJ;       // Jours depuis le 01/01/1970
	UINT nbJ_Jour;  // Jours depuis le 01/01 de l'anne
	UINT nbJ_An;    // Jours du 01/01/1970 au 01/01 de l'anne
	
	// Vrifier la validit des donnes
	if (
			uAn      < minAn                   ||
			uAn      > maxAn                   ||
			uMois    < 1                       ||
			uMois    > 12                      ||
			uJour    < 1                       ||
			uJour    > NbJoursMois(uMois, uAn) ||
			uHeure   > 23                      ||
			uMinute  > 59                      ||
			uSeconde > 59)
		return 0;
	
	
	// Jours couls dans l'anne donne jusqu'au mois donn
	// Rajouter un jour si on est aprs fvrier d'une anne bisextille
	nbJ_Jour = usJours[uMois-1];
	if (!((uAn-1900)&3) && uMois>2)  nbJ_Jour++;
	
	// Jours couls entre le 01/01/1970 et l'anne de la date
	// 365 jours par anne depuis 1970 + 1 jour par anne bisextille
	// 17 est le nombre d'annes bisextilles entre 1900 et 1970
	nbJ_An = (uAn - minAn) * 365 + ((uAn-1901) >> 2) - 17 + nbJ_Jour;
	
	// Jours couls jusqu' la date courante
	// Dpassement possible
	nbJ = nbJ_An + (nbJ_Jour = uJour-1);
	
	return (TEMPS)((nbJ * 24 + uHeure) * 60 + uMinute) * 60 + uSeconde;
}

// Remplir une structure TEMPSD  partir d'un TEMPS
void MakeTempsD(TEMPS tps, TEMPSD* ptpsd)
{
	// Vrifier la validit du temps
	if (!tps)  return;
	
	// Anne bisextille ?
	bool fBisext = false;
	
	// Dterminer les annes depus 1970
	
	// D'abord, identifier l'intervalle de 4 ans pour rendre la gestion
	// des annes bisextilles plus facile
	UINT uTemp = tps / secAn4;
	tps -= uTemp * secAn4;
	
	// Ensuite dterminer l'anne de l'intervalle
	uTemp = (uTemp * 4) + 70;   // 1970, 1974, 1978  etc.
	
	if (tps>=secAn) {
		uTemp++;                  // 1971, 1975, 1979  etc.
		tps -= secAn;
		
		if (tps>=secAn) {
			uTemp++;                // 1972, 1976, 1980  etc.
			tps -= secAn;
			
			if (tps>=secAnB) {
				uTemp++;              // 1973, 1977, 1981  etc.
				tps -= secAnB;
			}else
				// Il s'agit d'une anne bisextille
				fBisext = true;
		}
	}
	
	// Maintenant uTemp contient l'anne
	// et tps le nombre de secondes depuis le dbut de l'anne
	ptpsd->uAn = 1900 + uTemp;
	
	// Mettre dans uTemp le nombre de secondes depuis le dbut du jour
	UINT uJAn = tps  / secJour;
	tps      -= uJAn * secJour;
	
	// Dterminer le mois et le jour du mois
	const USHORT *pusJours = (fBisext) ? usJoursB : usJours;
	for (uTemp=1; pusJours[uTemp]<=uJAn; uTemp++);
	ptpsd->uMois = uTemp--;
	ptpsd->uJour = uJAn - pusJours[uTemp] + 1;
	
	// Dterminer heure, minute et seconde
	ptpsd->uHeure = tps  / secHeure;
	tps -= ptpsd->uHeure * secHeure;
	
	ptpsd->uMinute  = tps / secMinute;
	ptpsd->uSeconde = tps - ptpsd->uMinute * secMinute;
}


// Renvoyer TRUE si une date est valide, FALSE sinon
bool IsDateValide(UINT uJour, UINT uMois, UINT uAn)
{
	return
		uAn   >= minAn && uAn   <= maxAn  &&
		uMois >= 1     && uMois <= 12     &&
		uJour >= 1     && uJour <= NbJoursMois(uMois,uAn);
}

// Renvoyer TRUE si une date est valide, FALSE sinon
bool IsDateValide(SYSTEMTIME& st)
{
	return
		st.wYear  >= minAn && st.wYear  <= maxAn  &&
		st.wMonth >= 1     && st.wMonth <= 12     &&
		st.wDay   >= 1     && st.wDay   <= NbJoursMois(st.wMonth,st.wYear);
}

#ifdef _DEBUG
// Remplir une structure SYSTEMTIME avec J/M/A
void FillDate(int iJour, int iMois, int iAn, SYSTEMTIME& st)
{
	st.wDay          = (WORD)iJour;
	st.wMonth        = (WORD)iMois;
	st.wYear         = (WORD)iAn;
	st.wHour         = 0;
	st.wMinute       = 0;
	st.wSecond       = 0;
	st.wMilliseconds = 0;
}
#endif // _DEBUG



//***
//
// Classe CTemps
//
//***


// Jour de la date
UINT CTemps::GetJour() const
{
	TEMPSD tpsd;
	MakeTempsD(m_tps, &tpsd);
	return tpsd.uJour;
}

// Mois de la date
UINT CTemps::GetMois() const
{
	TEMPSD tpsd;
	MakeTempsD(m_tps, &tpsd);
	return tpsd.uMois;
}

// Anne de la date
UINT CTemps::GetAn() const
{
	TEMPSD tpsd;
	MakeTempsD(m_tps, &tpsd);
	return tpsd.uAn;
}


// Placer la date et l'heure actuelles dans la classe
void CTemps::SetNow()
{
	SYSTEMTIME st;
	GetLocalTime(&st);
	*this = st;
}


// CTemps& CTemps::operator = (const tm& otm)
CTemps& CTemps::operator = (const tm& otm)
{
	MakeTemps(otm.tm_mday, otm.tm_mon+1, otm.tm_yday+1900, otm.tm_hour, otm.tm_min, otm.tm_sec);
	return *this;
}

// CTemps& CTemps::operator = (const SYSTEMTIME& st)
CTemps& CTemps::operator = (const SYSTEMTIME& st)
{
	MakeTemps(st.wDay, st.wMonth, st.wYear, st.wHour, st.wMinute, st.wSecond);
	return *this;
}

// CTemps& CTemps::operator = (const FILETIME& ft)
CTemps& CTemps::operator = (const FILETIME& ft)
{
	FILETIME ftLocal;
	SYSTEMTIME st;
	FileTimeToLocalFileTime(&ft, &ftLocal);
	FileTimeToSystemTime(&ftLocal, &st);
	MakeTemps(st.wDay, st.wMonth, st.wYear, st.wHour, st.wMinute, st.wSecond);
	return *this;
}


// Retourner un TEMPSD
CTemps::operator TEMPSD() const
{
	TEMPSD tpsd;
	MakeTempsD(m_tps, &tpsd);
	return tpsd;
}




//***
//
// Classes CTempsD  CTempsH  CTempsDH  CTempsHS  CTempsDHS
//
//***


// CTempsX::CTempsX(const TEMPSD& tps)
#define dclD(type)   CTemps##type::CTemps##type(const TEMPSD& tpsd) { \
	Set##type(
#define dclF         ); }
dclD(D  )  tpsd.uJour,  tpsd.uMois,  tpsd.uAn                                             dclF
dclD( H )                                       tpsd.uHeure, tpsd.uMinute                 dclF
dclD(DH )  tpsd.uJour,  tpsd.uMois,  tpsd.uAn,  tpsd.uHeure, tpsd.uMinute                 dclF
dclD( HS)                                       tpsd.uHeure, tpsd.uMinute, tpsd.uSeconde  dclF
dclD(DHS)  tpsd.uJour,  tpsd.uMois,  tpsd.uAn,  tpsd.uHeure, tpsd.uMinute, tpsd.uSeconde  dclF
#undef dclD
#undef dclF

// CTempsX::CTempsX(const TEMPS& tps)
#define dclD(type)   CTemps##type::CTemps##type(const TEMPS& tps) { \
	TEMPSD tpsd; \
	MakeTempsD(tps, &tpsd); \
	Set##type(
#define dclF         ); }
dclD(D  )  tpsd.uJour,  tpsd.uMois,  tpsd.uAn                                             dclF
dclD( H )                                       tpsd.uHeure, tpsd.uMinute                 dclF
dclD(DH )  tpsd.uJour,  tpsd.uMois,  tpsd.uAn,  tpsd.uHeure, tpsd.uMinute                 dclF
dclD( HS)                                       tpsd.uHeure, tpsd.uMinute, tpsd.uSeconde  dclF
dclD(DHS)  tpsd.uJour,  tpsd.uMois,  tpsd.uAn,  tpsd.uHeure, tpsd.uMinute, tpsd.uSeconde  dclF
#undef dclD
#undef dclF

// CTempsX::CTempsX(const CTemps& tps)
#define dclD(type)   CTemps##type::CTemps##type(const CTemps& tps) { \
	TEMPSD tpsd = (TEMPSD)tps; \
	Set##type(
#define dclF         ); }
dclD(D  )  tpsd.uJour,  tpsd.uMois,  tpsd.uAn                                             dclF
dclD( H )                                       tpsd.uHeure, tpsd.uMinute                 dclF
dclD(DH )  tpsd.uJour,  tpsd.uMois,  tpsd.uAn,  tpsd.uHeure, tpsd.uMinute                 dclF
dclD( HS)                                       tpsd.uHeure, tpsd.uMinute, tpsd.uSeconde  dclF
dclD(DHS)  tpsd.uJour,  tpsd.uMois,  tpsd.uAn,  tpsd.uHeure, tpsd.uMinute, tpsd.uSeconde  dclF
#undef dclD
#undef dclF


// CTempsX::CTempsX(const SYSTEMTIME& st)
#define dclD(type)   CTemps##type::CTemps##type(const SYSTEMTIME& st) { \
	Set##type(
#define dclF         ); }
dclD(D  )  st.wDay,  st.wMonth,  st.wYear                                     dclF
dclD( H )                                   st.wHour, st.wMinute              dclF
dclD(DH )  st.wDay,  st.wMonth,  st.wYear,  st.wHour, st.wMinute              dclF
dclD( HS)                                   st.wHour, st.wMinute, st.wSecond  dclF
dclD(DHS)  st.wDay,  st.wMonth,  st.wYear,  st.wHour, st.wMinute, st.wSecond  dclF
#undef dclD


// CTempsX::CTempsX(const FILETIME& ft)
#define dclD(type)   CTemps##type::CTemps##type(const FILETIME& ft) { \
	FILETIME ftLocal; FileTimeToLocalFileTime(&ft, &ftLocal); \
	SYSTEMTIME st;    FileTimeToSystemTime(&ftLocal, &st); \
	Set##type(
#define dclF         ); }
dclD(D  )  st.wDay,  st.wMonth,  st.wYear                                     dclF
dclD( H )                                   st.wHour, st.wMinute              dclF
dclD(DH )  st.wDay,  st.wMonth,  st.wYear,  st.wHour, st.wMinute              dclF
dclD( HS)                                   st.wHour, st.wMinute, st.wSecond  dclF
dclD(DHS)  st.wDay,  st.wMonth,  st.wYear,  st.wHour, st.wMinute, st.wSecond  dclF
#undef dclD
#undef dclF


// CTempsX:CTemps(const time_t& t)
#define dclD(type)   CTemps##type::CTemps##type(const time_t& t) { \
	tm *const ptm = localtime(&t); \
	if (!ptm)  return; \
	Set##type(
#define dclF         ); }
dclD(D  )  ptm->tm_mday, ptm->tm_mon+1, ptm->tm_year+1900                                          dclF
dclD( H )                                                  ptm->tm_hour, ptm->tm_min               dclF
dclD(DH )  ptm->tm_mday, ptm->tm_mon+1, ptm->tm_year+1900, ptm->tm_hour, ptm->tm_min               dclF
dclD( HS)                                                  ptm->tm_hour, ptm->tm_min, ptm->tm_sec  dclF
dclD(DHS)  ptm->tm_mday, ptm->tm_mon+1, ptm->tm_year+1900, ptm->tm_hour, ptm->tm_min, ptm->tm_sec  dclF
#undef dclD
#undef dclF


// Remplir la classe avec la date et l'heure courantes
//
// void CTempsX::SetNow()
#define dclD(type)   void CTemps##type::SetNow() { \
	SYSTEMTIME st; \
	GetLocalTime(&st); \
	Set##type(
#define dclF         ); }
dclD(D  )  st.wDay,  st.wMonth,  st.wYear                                     dclF
dclD( H )                                   st.wHour, st.wMinute              dclF
dclD(DH )  st.wDay,  st.wMonth,  st.wYear,  st.wHour, st.wMinute              dclF
dclD( HS)                                   st.wHour, st.wMinute, st.wSecond  dclF
dclD(DHS)  st.wDay,  st.wMonth,  st.wYear,  st.wHour, st.wMinute, st.wSecond  dclF
#undef dclD
#undef dclF


// Remplir la classe avec les donnes venant d'une bote de dialogue
// Ordre des ID: J-M-A-H-M-S
//
// void CTempsX::FromDlg(HWND hDlg, int nIDC)
#define dclD(type)   void CTemps##type::FromDlg##type(HWND hDlg, int nIDC) { \
	Set##type(
#define dclF         ); }
#define get(n)   GetDlgItemInt(hDlg, nIDC+n, NULL, FALSE)
dclD(D  )  get(0), get(1), get(2)                          dclF
dclD( H )  get(0), get(1)                                  dclF
dclD(DH )  get(0), get(1), get(2), get(3), get(4)          dclF
dclD( HS)  get(0), get(1), get(2)                          dclF
dclD(DHS)  get(0), get(1), get(2), get(3), get(4), get(5)  dclF
#undef get
#undef dclD
#undef dclF


// Remplir une bote de dialogue les donnes de la classe
// Ordre des ID: J-M-A-H-M-S
//
// void CTempsX::ToDlg(HWND hDlg, int nIDC) const
#define dclD(type)   void CTemps##type::ToDlg##type(HWND hDlg, int nIDC) const {
#define dclM         TCHAR psz[bufTempsDlg];
#define dclF         ; }
#define set(n, len, val)  _stprintf(psz, _T("%0")_T(#len)_T("u"), (UINT)m_u##val);  SetDlgItemText(hDlg, nIDC+n, psz);
dclD(D  )  if (!m_uJour || !m_uMois || !m_uAn)  return;
dclM       set(0, 2,Jour)   set(1, 2,Mois)  set(2, 4,An)                                                        dclF
dclD( H )
dclM       set(0, 2,Heure)  set(1, 2,Minute)                                                                    dclF
dclD(DH )  if (!m_uJour || !m_uMois || !m_uAn)  return;
dclM       set(0, 2,Jour)   set(1, 2,Mois)  set(2, 4,An)  set(3, 2,Heure)  set(4, 2,Minute)                     dclF
dclD( HS)
dclM       set(0, 2,Heure)  set(1, 2,Minute)  set(2, 2,Seconde)                                                 dclF
dclD(DHS)  if (!m_uJour || !m_uMois || !m_uAn)  return;
dclM       set(0, 2,Jour)   set(1, 2,Mois)  set(2, 4,An)  set(3, 2,Heure)  set(4, 2,Minute)  set(5, 2,Seconde)  dclF
#undef set
#undef dclD
#undef dclM
#undef dclF


#ifdef MFCEXTOK

// Remplir la classe avec les donnes venant d'une bote de dialogue
// Ordre des ID: J-M-A-H-M-S
//
// void CTempsX::FromDlgX(CWnd* pWnd, int nIDC)
#define dclD(type)   void CTemps##type::FromDlg##type(CWnd* pWnd, int nIDC) { \
	Set##type(
#define dclF         ); }
#define get(n)   pWnd->GetDlgItemInt(nIDC+n)
dclD(D  )  get(0), get(1), get(2)                          dclF
dclD( H )  get(0), get(1)                                  dclF
dclD(DH )  get(0), get(1), get(2), get(3), get(4)          dclF
dclD( HS)  get(0), get(1), get(2)                          dclF
dclD(DHS)  get(0), get(1), get(2), get(3), get(4), get(5)  dclF
#undef get
#undef dclD
#undef dclF


// Remplir une bote de dialogue les donnes de la classe
// Ordre des ID: J-M-A-H-M-S
//
// void CTempsX::ToDlgX(CWnd* pWnd, int nIDC) const
#define dclD(type)   void CTemps##type::ToDlg##type(CWnd* pWnd, int nIDC) const {
#define dclM         TCHAR psz[bufTempsDlg];
#define dclF         ; }
#define set(n, len, val)  _stprintf(psz, _T("%0")_T(#len)_T("u"), (UINT)m_u##val);  pWnd->SetDlgItemText(nIDC+n, psz);
dclD(D  )  if (!m_uJour || !m_uMois || !m_uAn)  return;
dclM       set(0, 2,Jour)   set(1, 2,Mois)  set(2, 4,An)                                                        dclF
dclD( H )
dclM       set(0, 2,Heure)  set(1, 2,Minute)                                                                    dclF
dclD(DH )  if (!m_uJour || !m_uMois || !m_uAn)  return;
dclM       set(0, 2,Jour)   set(1, 2,Mois)  set(2, 4,An)  set(3, 2,Heure)  set(4, 2,Minute)                     dclF
dclD( HS)
dclM       set(0, 2,Heure)  set(1, 2,Minute)  set(2, 2,Seconde)                                                 dclF
dclD(DHS)  if (!m_uJour || !m_uMois || !m_uAn)  return;
dclM       set(0, 2,Jour)   set(1, 2,Mois)  set(2, 4,An)  set(3, 2,Heure)  set(4, 2,Minute)  set(5, 2,Seconde)  dclF
#undef set
#undef dclD
#undef dclM
#undef dclF

#endif // MFCEXTOK



// void CTempsX::Format(LPTSTR psz, int len, DWORD dwFlags) const
// void CTempsX::Format(LPTSTR psz, int len, LPCTSTR pszFormat) const
#define dcl(type, fonction) \
	void CTemps##type::Format##type(LPTSTR psz, int len, DWORD dwFlags) const { \
		fonction(LOCALE_USER_DEFAULT, dwFlags, &(SYSTEMTIME)*this, NULL, psz, len); } \
	void CTemps##type::Format##type(LPTSTR psz, int len, LPCTSTR pszFormat) const { \
		fonction(NULL, 0, &(SYSTEMTIME)*this, pszFormat, psz, len); }
dcl(D,  GetDateFormat)
dcl( H, GetTimeFormat)
dcl( HS,GetTimeFormat)
#undef dcl


#ifdef MFCOK

// CString CTempsX::Format(DWORD dwFlags) const
// CString CTempsX::Format(LPCTSTR pszFormat) const
#define dcl(type, fonction) \
	CString CTemps##type::Format##type(DWORD dwFlags) const { \
		TCHAR psz[bufTempsFormat]; \
		fonction(LOCALE_USER_DEFAULT, dwFlags, &(SYSTEMTIME)*this, NULL, psz, bufTempsFormat); \
		return psz; } \
	CString CTemps##type::Format##type(LPCTSTR pszFormat) const { \
		TCHAR psz[bufTempsFormat]; \
		fonction(NULL, 0, &(SYSTEMTIME)*this, pszFormat, psz, bufTempsFormat); \
		return psz; }
dcl(D,  GetDateFormat)
dcl( H, GetTimeFormat)
dcl( HS,GetTimeFormat)
#undef dcl

#endif  // MFCOK


// Renvoyer la partie de date invalide, dtpOK si la date est valide
DATEPART CTempsD::PasValide() const
{
	if (m_uJour<1   || m_uJour>NbJoursMois(m_uMois,m_uAn))  return dtpJour;
	if (m_uMois<1   || m_uMois>12)                          return dtpMois;
	if (m_uAn<minAn || m_uAn>maxAn)                         return dtpAn;
	                                                        return dtpOK;
}
// Renvoyer la partie de date invalide, dtpOK si la date est valide
DATEPART CTempsH::PasValide() const
{
	if (m_uHeure >59)  return dtpHeure;
	if (m_uMinute>59)  return dtpMinute;
	return dtpOK;
}
// Renvoyer la partie de date invalide, dtpOK si la date est valide
DATEPART CTempsDH::PasValide() const
{
	if (m_uJour<1   || m_uJour>NbJoursMois(m_uMois,m_uAn))  return dtpJour;
	if (m_uMois<1   || m_uMois>12)                          return dtpMois;
	if (m_uAn<minAn || m_uAn>maxAn)                         return dtpAn;
	if (m_uHeure >59)                                       return dtpHeure;
	if (m_uMinute>59)                                       return dtpMinute;
	                                                        return dtpOK;
}
// Renvoyer la partie de date invalide, dtpOK si la date est valide
DATEPART CTempsHS::PasValide() const
{
	if (m_uHeure  >59)  return dtpHeure;
	if (m_uMinute >59)  return dtpMinute;
	if (m_uSeconde>59)  return dtpSeconde;
	                    return dtpOK;
}
// Renvoyer la partie de date invalide, dtpOK si la date est valide
DATEPART CTempsDHS::PasValide() const
{
	if (m_uJour<1   || m_uJour>NbJoursMois(m_uMois,m_uAn))  return dtpJour;
	if (m_uMois<1   || m_uMois>12)                          return dtpMois;
	if (m_uAn<minAn || m_uAn>maxAn)                         return dtpAn;
	if (m_uHeure  >59)                                      return dtpHeure;
	if (m_uMinute >59)                                      return dtpMinute;
	if (m_uSeconde>59)                                      return dtpSeconde;
	                                                        return dtpOK;
}


void CTempsD::AddD(long n, DATEPART dtp)
{
	int iJour = m_uJour;
	int iMois = m_uMois;
	int iAn   = m_uAn;
	
	switch (dtp) {
		case dtpJour:
			{
				iJour += n;
				int nbJours;
				while (nbJours = NbJoursMois(iMois, iAn),
				       iJour<1 || iJour>nbJours) {
					if (iJour<1) {
						iMois--;
						while (iMois<1) {
							iMois += 12;
							iAn--;
						}
						iJour += NbJoursMois(iMois, iAn);
					}else{
						iJour -= nbJours;
						iMois++;
						while (iMois>12) {
							iMois -= 12;
							iAn++;
						}
					}
				}
			}
			n = 0;
		
		case dtpMois:
			iMois += n;
			while (iMois<1) {
				iMois += 12;
				iAn--;
			}
			while (iMois>12) {
				iMois -= 12;
				iAn++;
			}
			n = 0;
		
		case dtpAn:
			iAn += n;
			if (iAn<minAn) {
				iAn   = minAn;
				iMois =
				iJour = 1;
			}
			if (iAn>maxAn) {
				iAn   = maxAn;
				iMois = 12;
				iJour = 31;
			}
	}
	m_uJour = (UINT)iJour;
	m_uMois = (UINT)iMois;
	m_uAn   = (UINT)iAn;
}

void CTempsH::AddH(long n, DATEPART dtp)
{
	int iHeure  = m_uHeure;
	int iMinute = m_uMinute;
	
	switch (dtp) {
		case dtpMinute:
			iMinute += n;
			while (iMinute<0) {
				iMinute += 60;
				iHeure--;
			}
			while (iMinute>59) {
				iMinute -= 60;
				iHeure++;
			}
			n = 0;
		
		case dtpHeure:
			iHeure += n;
			if (iHeure<0)   iHeure = iMinute = 0;
			if (iHeure>59)  iHeure = iMinute = 59;
	}
	m_uHeure  = (UINT)iHeure;
	m_uMinute = (UINT)iMinute;
}
void CTempsDH::AddDH(long n, DATEPART dtp)
{
	int iJour   = m_uJour;
	int iMois   = m_uMois;
	int iAn     = m_uAn;
	int iHeure  = m_uHeure;
	int iMinute = m_uMinute;
	
	switch (dtp) {
		case dtpMinute:
			iMinute += n;
			while (iMinute<0) {
				iMinute += 60;
				iHeure--;
			}
			while (iMinute>59) {
				iMinute -= 60;
				iHeure++;
			}
			n = 0;
		
		case dtpHeure:
			iHeure += n;
			if (iHeure<0)   iHeure = iMinute = 0;
			if (iHeure>59)  iHeure = iMinute = 59;
			n = 0;
		
		case dtpJour:
			{
				iJour += n;
				int nbJours;
				while (nbJours = NbJoursMois(iMois, iAn),
				       iJour<1 || iJour>nbJours) {
					if (iJour<1) {
						iMois--;
						while (iMois<1) {
							iMois += 12;
							iAn--;
						}
						iJour += NbJoursMois(iMois, iAn);
					}else{
						iJour -= nbJours;
						iMois++;
						while (iMois>12) {
							iMois -= 12;
							iAn++;
						}
					}
				}
			}
			n = 0;
		
		case dtpMois:
			iMois += n;
			while (iMois<1) {
				iMois += 12;
				iAn--;
			}
			while (iMois>12) {
				iMois -= 12;
				iAn++;
			}
			n = 0;
		
		case dtpAn:
			iAn += n;
			if (iAn<minAn) {
				iAn   = minAn;
				iMois =
				iJour = 1;
			}
			if (iAn>maxAn) {
				iAn = maxAn;
				iMois = 12;
				iJour = 31;
			}
	}
	m_uJour   = (UINT)iJour;
	m_uMois   = (UINT)iMois;
	m_uAn     = (UINT)iAn;
	m_uHeure  = (UINT)iHeure;
	m_uMinute = (UINT)iMinute;
}

void CTempsHS::AddHS(long n, DATEPART dtp)
{
	int iHeure   = m_uHeure;
	int iMinute  = m_uMinute;
	int iSeconde = m_uSeconde;
	
	switch (dtp) {
		case dtpSeconde:
			iSeconde += n;
			while (iSeconde<0) {
				iSeconde += 60;
				iMinute--;
			}
			while (iSeconde>59) {
				iSeconde -= 60;
				iMinute++;
			}
			n = 0;
		
		case dtpMinute:
			iMinute += n;
			while (iMinute<0) {
				iMinute += 60;
				iHeure--;
			}
			while (iMinute>59) {
				iMinute -= 60;
				iHeure++;
			}
			n = 0;
		
		case dtpHeure:
			iHeure += n;
			if (iHeure<0)   iHeure = iMinute = 0;
			if (iHeure>59)  iHeure = iMinute = 59;
			n = 0;
	}
	m_uHeure   = (UINT)iHeure;
	m_uMinute  = (UINT)iMinute;
	m_uSeconde = (UINT)iSeconde;
}

void CTempsDHS::AddDHS(long n, DATEPART dtp)
{
	int iJour    = m_uJour;
	int iMois    = m_uMois;
	int iAn      = m_uAn;
	int iHeure   = m_uHeure;
	int iMinute  = m_uMinute;
	int iSeconde = m_uSeconde;
	
	switch (dtp) {
		case dtpSeconde:
			iSeconde += n;
			while (iSeconde<0) {
				iSeconde += 60;
				iMinute--;
			}
			while (iSeconde>59) {
				iSeconde -= 60;
				iMinute++;
			}
			n = 0;
		
		case dtpMinute:
			iMinute += n;
			while (iMinute<0) {
				iMinute += 60;
				iHeure--;
			}
			while (iMinute>59) {
				iMinute -= 60;
				iHeure++;
			}
			n = 0;
		
		case dtpHeure:
			iHeure += n;
			if (iHeure<0)   iHeure = iMinute = 0;
			if (iHeure>59)  iHeure = iMinute = 59;
		
		case dtpJour:
			{
				iJour += n;
				int nbJours;
				while (nbJours = NbJoursMois(iMois, iAn),
				       iJour<1 || iJour>nbJours) {
					if (iJour<1) {
						iMois--;
						while (iMois<1) {
							iMois += 12;
							iAn--;
						}
						iJour += NbJoursMois(iMois, iAn);
					}else{
						iJour -= nbJours;
						iMois++;
						while (iMois>12) {
							iMois -= 12;
							iAn++;
						}
					}
				}
			}
			n = 0;
		
		case dtpMois:
			iMois += n;
			while (iMois<1) {
				iMois += 12;
				iAn--;
			}
			while (iMois>12) {
				iMois -= 12;
				iAn++;
			}
			n = 0;
		
		case dtpAn:
			iAn += n;
			if (iAn<minAn) {
				iAn   = minAn;
				iMois =
				iJour = 1;
			}
			if (iAn>maxAn) {
				iAn = maxAn;
				iMois = 12;
				iJour = 31;
			}
	}
	m_uJour    = (UINT)iJour;
	m_uMois    = (UINT)iMois;
	m_uAn      = (UINT)iAn;
	m_uHeure   = (UINT)iHeure;
	m_uMinute  = (UINT)iMinute;
	m_uSeconde = (UINT)iSeconde;
}


// CTempsX& CTempsX::operator = (const TEMPSD& tps)
#define dclD(type)   CTemps##type& CTemps##type::operator = (const TEMPSD& tpsd) { \
	Set##type(
#define dclF         ); return *this; }
dclD(D  )  tpsd.uJour,  tpsd.uMois,  tpsd.uAn                                             dclF
dclD( H )                                       tpsd.uHeure, tpsd.uMinute                 dclF
dclD(DH )  tpsd.uJour,  tpsd.uMois,  tpsd.uAn,  tpsd.uHeure, tpsd.uMinute                 dclF
dclD( HS)                                       tpsd.uHeure, tpsd.uMinute, tpsd.uSeconde  dclF
dclD(DHS)  tpsd.uJour,  tpsd.uMois,  tpsd.uAn,  tpsd.uHeure, tpsd.uMinute, tpsd.uSeconde  dclF
#undef dclD
#undef dclF


// CTempsX& CTempsX::operator = (const TEMPS& tps)
#define dclD(type)   CTemps##type& CTemps##type::operator = (const TEMPS& tps) { \
	TEMPSD tpsd; \
	MakeTempsD(tps, &tpsd); \
	Set##type(
#define dclF         ); return *this; }
dclD(D  )  tpsd.uJour,  tpsd.uMois,  tpsd.uAn                                             dclF
dclD( H )                                       tpsd.uHeure, tpsd.uMinute                 dclF
dclD(DH )  tpsd.uJour,  tpsd.uMois,  tpsd.uAn,  tpsd.uHeure, tpsd.uMinute                 dclF
dclD( HS)                                       tpsd.uHeure, tpsd.uMinute, tpsd.uSeconde  dclF
dclD(DHS)  tpsd.uJour,  tpsd.uMois,  tpsd.uAn,  tpsd.uHeure, tpsd.uMinute, tpsd.uSeconde  dclF
#undef dclD
#undef dclF


// CTempsX& CTempsX::operator = (const CTemps& tps)
#define dclD(type)   CTemps##type& CTemps##type::operator = (const CTemps& tps) { \
	TEMPSD tpsd = (TEMPSD)tps; \
	Set##type(
#define dclF         ); return *this; }
dclD(D  )  tpsd.uJour,  tpsd.uMois,  tpsd.uAn                                             dclF
dclD( H )                                       tpsd.uHeure, tpsd.uMinute                 dclF
dclD(DH )  tpsd.uJour,  tpsd.uMois,  tpsd.uAn,  tpsd.uHeure, tpsd.uMinute                 dclF
dclD( HS)                                       tpsd.uHeure, tpsd.uMinute, tpsd.uSeconde  dclF
dclD(DHS)  tpsd.uJour,  tpsd.uMois,  tpsd.uAn,  tpsd.uHeure, tpsd.uMinute, tpsd.uSeconde  dclF
#undef dclD
#undef dclF


// CTempsX& CTempsX::operator = (const CTempsX& dt)
#define dcl(type)   CTemps##type& CTemps##type::operator = (const CTemps##type& dt) { \
	return *this = (tm)dt; }
dcl(D  )
dcl( H )
dcl(DH )
dcl( HS)
dcl(DHS)
#undef dcl


// CTemps& CTempsX::operator = (const tm& otm);
#define dclD(type)   CTemps##type& CTemps##type::operator = (const tm& otm) { \
	Set##type(
#define dclF         ); return *this; }
#define get(champ)   (UINT)otm.tm_##champ
dclD(D  )  get(mday), get(mon)+1, get(year)+1900                                 dclF
dclD( H )                                         get(hour), get(min)            dclF
dclD(DH )  get(mday), get(mon)+1, get(year)+1900, get(hour), get(min)            dclF
dclD( HS)                                         get(hour), get(min), get(sec)  dclF
dclD(DHS)  get(mday), get(mon)+1, get(year)+1900, get(hour), get(min), get(sec)  dclF
#undef get
#undef dclD
#undef dclF


// CTemps& CTempsX::operator = (const time_t& t)
#define dclD(type)   CTemps##type& CTemps##type::operator = (const time_t& t) { \
	tm *const ptm = localtime(&t); \
	if (!ptm)  return *this; \
	Set##type(
#define dclF         ); return *this; }
#define get(champ)   (UINT)ptm->tm_##champ
dclD(D  )  get(mday), get(mon)+1, get(year)+1900                                 dclF
dclD( H )                                         get(hour), get(min)            dclF
dclD(DH )  get(mday), get(mon)+1, get(year)+1900, get(hour), get(min)            dclF
dclD( HS)                                         get(hour), get(min), get(sec)  dclF
dclD(DHS)  get(mday), get(mon)+1, get(year)+1900, get(hour), get(min), get(sec)  dclF
#undef get
#undef dclD
#undef dclF


#ifdef MFCOK

// CTemps& CTempsX::operator = (const CTime& dt)
#define dclD(type)   CTemps##type& CTemps##type::operator = (const CTime& dt) { \
	Set##type(
#define dclF         ); return *this; }
#define get(champ)   (UINT)dt.Get##champ()
dclD(D  )  get(Day), get(Month), get(Year)                                                  dclF
dclD( H )                                              get(Hour), get(Minute)               dclF
dclD(DH )  get(Day), get(Month), get(Year), get(Hour), get(Minute)                          dclF
dclD( HS)                                              get(Hour), get(Minute), get(Second)  dclF
dclD(DHS)  get(Day), get(Month), get(Year), get(Hour), get(Minute), get(Second)             dclF
#undef get
#undef dclD
#undef dclF

#endif // MFCOK


// CTemps& CTempsX::operator = (const SYSTEMTIME& st)
#define dclD(type)   CTemps##type& CTemps##type::operator = (const SYSTEMTIME& st) { \
	Set##type(
#define dclF         ); return *this; }
dclD(D  )  st.wDay,  st.wMonth,  st.wYear                                     dclF
dclD( H )  st.wHour, st.wMinute                                               dclF
dclD(DH )  st.wDay,  st.wMonth,  st.wYear,  st.wHour, st.wMinute              dclF
dclD( HS)  st.wHour, st.wMinute, st.wSecond                                   dclF
dclD(DHS)  st.wDay,  st.wMonth,  st.wYear,  st.wHour, st.wMinute, st.wSecond  dclF
#undef dclD
#undef dclF


// CTemps& CTempsX::operator = (const FILETIME& ft)
#define dcl(type)   CTemps##type& CTemps##type::operator = (const FILETIME& ft) { \
	FILETIME ftLocal; \
	SYSTEMTIME st; \
	FileTimeToLocalFileTime(&ft, &ftLocal); \
	FileTimeToSystemTime(&ftLocal, &st); \
	return *this = st; }
dcl(D  )
dcl( H )
dcl(DH )
dcl( HS)
dcl(DHS)
#undef dcl


// CTempsX CTempsX::operator + (const CTempsSpan& ts) const
#define dcl(type)   CTemps##type CTemps##type::operator + (const CTempsSpan& ts) const { \
	return CTemps##type((time_t)*this + (time_t)ts.GetTotalSeconde()); }
dcl(D  )
dcl( H )
dcl(DH )
dcl( HS)
dcl(DHS)
#undef dcl

// CTempsX CTempsX::operator - (const CTempsSpan& ts) const
#define dcl(type)   CTemps##type CTemps##type::operator - (const CTempsSpan& ts) const { \
	return CTemps##type((time_t)*this - (time_t)ts.GetTotalSeconde()); }
dcl(D  )
dcl( H )
dcl(DH )
dcl( HS)
dcl(DHS)
#undef dcl

// CTempsX& CTempsX::operator += (const CTempsSpan& ts)
#define dcl(type)   CTemps##type& CTemps##type::operator += (const CTempsSpan& ts) { \
	return *this = CTemps##type((time_t)*this + (time_t)ts.GetTotalSeconde()); }
dcl(D  )
dcl( H )
dcl(DH )
dcl( HS)
dcl(DHS)
#undef dcl

// CTempsX& CTempsX::operator -= (const CTempsSpan& ts)
#define dcl(type)   CTemps##type& CTemps##type::operator -= (const CTempsSpan& ts) { \
	return *this = CTemps##type((time_t)*this - (time_t)ts.GetTotalSeconde()); }
dcl(D  )
dcl( H )
dcl(DH )
dcl( HS)
dcl(DHS)
#undef dcl


// CTempsSpan CTempsX::operator - (const CTempsX& dt) const
#define dcl(type)   CTempsSpan CTemps##type::operator - (const CTemps##type& dt) const { \
	return CTempsSpan((TEMPS)*this - (TEMPS)dt); }
dcl(D  )
dcl( H )
dcl(DH )
dcl( HS)
dcl(DHS)
#undef dcl


// bool CTempsX::operator == (const CTemps& dt) const
#define dclD(type)   bool CTemps##type::operator == (const CTemps##type& dt) const { return
#define dclF         ; }
#define egl(champ) (m_u##champ==dt.m_u##champ)
dclD(D  )  egl(Jour)  && egl(Mois)   && egl(An)                                                    dclF
dclD( H )                                               egl(Heure) && egl(Minute)                  dclF
dclD(DH )  egl(Jour)  && egl(Mois)   && egl(An)      && egl(Heure) && egl(Minute)                  dclF
dclD( HS)                                               egl(Heure) && egl(Minute) && egl(Seconde)  dclF
dclD(DHS)  egl(Jour)  && egl(Mois)   && egl(An)      && egl(Heure) && egl(Minute) && egl(Seconde)  dclF
#undef egl
#undef dclD
#undef dclF


// bool CTempsX::operator != (const CTemps& dt) const
#define dclD(type)   bool CTemps##type::operator != (const CTemps##type& dt) const { return
#define dclF         ; }
#define dif(champ) (m_u##champ!=dt.m_u##champ)
dclD(D  )  dif(Jour)  || dif(Mois)   || dif(An)                                                    dclF
dclD( H )                                               dif(Heure) || dif(Minute)                  dclF
dclD(DH )  dif(Jour)  || dif(Mois)   || dif(An)      || dif(Heure) || dif(Minute)                  dclF
dclD( HS)                                               dif(Heure) || dif(Minute) || dif(Seconde)  dclF
dclD(DHS)  dif(Jour)  || dif(Mois)   || dif(An)      || dif(Heure) || dif(Minute) || dif(Seconde)  dclF
#undef dif
#undef dclD
#undef dclF


#define dclD(type)   bool CTemps##type::operator egal (const CTemps##type& dt) const {
#define dclF         }
#define cmp0(champ)       if (m_u##champ sens0 dt.m_u##champ)  return true;  else if (m_u##champ sens1 dt.m_u##champ)  return false;
#define cmp1(champ)  else if (m_u##champ sens0 dt.m_u##champ)  return true;  else if (m_u##champ sens1 dt.m_u##champ)  return false;
#define cmp2(champ)  else if (m_u##champ egal  dt.m_u##champ)  return true;  else                                      return false;
#define Comparaison \
	dclD(D  )  cmp0(An)     cmp1(Mois)   cmp2(Jour)                                            dclF \
	dclD( H )                                        cmp0(Heure)  cmp2(Minute)                 dclF \
	dclD(DH )  cmp0(An)     cmp1(Mois)   cmp1(Jour)  cmp1(Heure)  cmp2(Minute)                 dclF \
	dclD( HS)                                        cmp0(Heure)  cmp1(Minute)  cmp2(Seconde)  dclF \
	dclD(DHS)  cmp0(An)     cmp1(Mois)   cmp1(Jour)  cmp1(Heure)  cmp1(Minute)  cmp2(Seconde)  dclF \

// bool CTempsX::operator < (const CTempsX& dt) const
#define sens0 <
#define sens1 >
#define egal  <
Comparaison

// bool CTempsX::operator <= (const CTempsX& dt) const
#undef  egal
#define egal  <=
Comparaison
#undef  sens0
#undef  sens1

// bool CTempsX::operator > (const CTempsX& dt) const
#define sens0 >
#define sens1 <
#undef  egal
#define egal  >
Comparaison

// bool CTempsX::operator >= (const CTempsX& dt) const
#undef  egal
#define egal  >=
Comparaison
#undef  egal
#undef  sens0
#undef  sens1
#undef Comparaison
#undef dclD
#undef dclF


// CTempsX::operator TEMPSD() const
#define dclD(type)   CTemps##type::operator TEMPSD() const { \
	TEMPSD tpsd = {
#define dclF         }; return tpsd; }
dclD(D  )  m_uJour, m_uMois,  m_uAn,        0,         0,          0  dclF
dclD( H )        1,       1,   2000, m_uHeure, m_uMinute,          0  dclF
dclD(DH )  m_uJour, m_uMois,  m_uAn, m_uHeure, m_uMinute,          0  dclF
dclD( HS)        1,       1,   2000, m_uHeure, m_uMinute, m_uSeconde  dclF
dclD(DHS)  m_uJour, m_uMois,  m_uAn, m_uHeure, m_uMinute, m_uSeconde  dclF
#undef dclD
#undef dclF


// CTempsX::operator TEMPS() const
#define dclD(type)   CTemps##type::operator TEMPS() const { \
	return MakeTemps(
#define dclF         ); }
dclD(D  )  m_uJour, m_uMois,  m_uAn,        0,         0,          0  dclF
dclD( H )        1,       1,   2000, m_uHeure, m_uMinute,          0  dclF
dclD(DH )  m_uJour, m_uMois,  m_uAn, m_uHeure, m_uMinute,          0  dclF
dclD( HS)        1,       1,   2000, m_uHeure, m_uMinute, m_uSeconde  dclF
dclD(DHS)  m_uJour, m_uMois,  m_uAn, m_uHeure, m_uMinute, m_uSeconde  dclF
#undef dclD
#undef dclF


// CTempsX::operator CTemps() const
#define dclD(type)   CTemps##type::operator CTemps() const { \
	return CTemps(
#define dclF         ); }
dclD(D  )  m_uJour, m_uMois,  m_uAn,        0,         0,          0  dclF
dclD( H )        1,       1,   2000, m_uHeure, m_uMinute,          0  dclF
dclD(DH )  m_uJour, m_uMois,  m_uAn, m_uHeure, m_uMinute,          0  dclF
dclD( HS)        1,       1,   2000, m_uHeure, m_uMinute, m_uSeconde  dclF
dclD(DHS)  m_uJour, m_uMois,  m_uAn, m_uHeure, m_uMinute, m_uSeconde  dclF
#undef dclD
#undef dclF


// CTempsX::operator SYSTEMTIME() const
#define dclD(type)   CTemps##type::operator SYSTEMTIME() const {
#define dclF         ; }
#define ret(uJour, uMois, uAn, uHeure, uMinute, uSeconde) \
	SYSTEMTIME st = { (WORD)uAn, (WORD)uMois, 0, (WORD)uJour, (WORD)uHeure, (WORD)uMinute, (WORD)uSeconde, 0 };  return st
dclD(D  )  ret(m_uJour, m_uMois,  m_uAn,        0,         0,          0)  dclF
dclD( H )  ret(      1,       1,   2000, m_uHeure, m_uMinute,          0)  dclF
dclD(DH )  ret(m_uJour, m_uMois,  m_uAn, m_uHeure, m_uMinute,          0)  dclF
dclD( HS)  ret(      1,       1,   2000, m_uHeure, m_uMinute, m_uSeconde)  dclF
dclD(DHS)  ret(m_uJour, m_uMois,  m_uAn, m_uHeure, m_uMinute, m_uSeconde)  dclF
#undef ret
#undef dclD
#undef dclF


// CTempsX::operator FILETIME() const
#define dclD(type)   CTemps##type::operator FILETIME() const {
#define dclF         ; }
#define ret \
	FILETIME ft; \
	const LONGLONG llLocal = Int32x32To64((time_t)*this, 10000000) + 116444736000000000; \
	LocalFileTimeToFileTime((FILETIME*)&llLocal, &ft); \
	return ft
dclD(D  ) ret  dclF
dclD( H ) ret  dclF
dclD(DH ) ret  dclF
dclD( HS) ret  dclF
dclD(DHS) ret  dclF
#undef ret
#undef dclD
#undef dclF


// CTempsX::operator tm() const
#define dclD(type)   CTemps##type::operator tm() const {
#define dclF         ; }
#define ret(uJour, uMois, uAn, uHeure, uMinute, uSeconde) \
	tm otm = { (int)uSeconde, (int)uMinute, (int)uHeure, (int)uJour, (int)uMois-1, (int)uAn-1900, 0, 0, -1 };  return otm
dclD(D  ) ret(m_uJour, m_uMois,  m_uAn,          0,         0,          0)  dclF
dclD( H ) ret(      1,       1,   2000, m_uHeure+1, m_uMinute,          0)  dclF
dclD(DH ) ret(m_uJour, m_uMois,  m_uAn, m_uHeure+1, m_uMinute,          0)  dclF
dclD( HS) ret(      1,       1,   2000, m_uHeure+1, m_uMinute, m_uSeconde)  dclF
dclD(DHS) ret(m_uJour, m_uMois,  m_uAn, m_uHeure+1, m_uMinute, m_uSeconde)  dclF
#undef ret
#undef dclD
#undef dclF


// CTempsX::operator time_t() const
#define dcl(type)   CTemps##type::operator time_t() const { \
	return mktime(&(tm)*this); }
dcl(D  )
dcl( H )
dcl(DH )
dcl( HS)
dcl(DHS)
#undef dcl


// Dfinir la dure
void CTempsSpan::Set(long lJour, int nHeure, int nMinute, int nSeconde)
{
	m_lSpan = lJour*secJour + nHeure*secHeure + nMinute*secMinute + nSeconde;
}


// Formatter H:MM
void CTempsSpan::FormatHM(LPTSTR psz) const
{
	if (m_lSpan<0)  _stprintf(psz, _T("-%d:%02d"), -GetTotalHeure(), -GetMinute());
	else            _stprintf(psz, _T("%d:%02d"),   GetTotalHeure(),  GetMinute());
}

// Formatter H:MM:SS
void CTempsSpan::FormatHMS(LPTSTR psz) const
{
	if (m_lSpan<0)  _stprintf(psz, _T("-%d:%02d:%02d"), -GetTotalHeure(), -GetMinute(), -GetSeconde());
	else            _stprintf(psz, _T("%d:%02d:%02d"),   GetTotalHeure(),  GetMinute(),  GetSeconde());
}


#ifdef MFCOK

// Formatter H:MM
CString CTempsSpan::FormatHM() const
{
	CString s;
	if (m_lSpan<0)  s.Format(_T("%d:%02d"), -GetTotalHeure(), -GetMinute());
	else            s.Format(_T("%d:%02d"),  GetTotalHeure(),  GetMinute());
	return s;
}

// Formatter H:MM:SS
CString CTempsSpan::FormatHMS() const
{
	CString s;
	if (m_lSpan<0)  s.Format(_T("-%d:%02d:%02d"), -GetTotalHeure(), -GetMinute(), -GetSeconde());
	else            s.Format(_T("%d:%02d:%02d"),   GetTotalHeure(),  GetMinute(),  GetSeconde());
	return s;
}

#endif  // MFCOK


// Rcuprer d'une bote de dialogue
bool CTempsSpan::FromDlgHM(HWND hDlg, int nIDC)
{
	const UINT u1 = GetDlgItemInt(hDlg, nIDC+1, NULL, FALSE);
	if (u1>59)  return false;
	
	Set(0, GetDlgItemInt(hDlg, nIDC, NULL, FALSE), u1);
	return true;
}

// Rcuprer d'une bote de dialogue
bool CTempsSpan::FromDlgHMS(HWND hDlg, int nIDC)
{
	const UINT u1 = GetDlgItemInt(hDlg, nIDC+1, NULL, FALSE);
	if (u1>59)  return false;
	const UINT u2 = GetDlgItemInt(hDlg, nIDC+2, NULL, FALSE);
	if (u2>59)  return false;
	
	Set(0, GetDlgItemInt(hDlg, nIDC, NULL, FALSE), u1, u2);
	return true;
}


// Formatter dans une bote de dialogue
void CTempsSpan::ToDlgHM(HWND hDlg, int nIDC) const
{
	TCHAR psz[bufTempsDlg];
	_stprintf(psz, _T("%1d"),  abs(GetTotalHeure()));  SetDlgItemText(hDlg, nIDC+0, psz);
	_stprintf(psz, _T("%02d"), abs(GetMinute()));      SetDlgItemText(hDlg, nIDC+1, psz);
}

// Formatter dans une bote de dialogue
void CTempsSpan::ToDlgHMS(HWND hDlg, int nIDC) const
{
	TCHAR psz[bufTempsDlg];
	_stprintf(psz, _T("%1d"),  abs(GetTotalHeure()));  SetDlgItemText(hDlg, nIDC+0, psz);
	_stprintf(psz, _T("%02u"), abs(GetMinute()));      SetDlgItemText(hDlg, nIDC+1, psz);
	_stprintf(psz, _T("%02u"), abs(GetSeconde()));     SetDlgItemText(hDlg, nIDC+2, psz);
}


#ifdef MFCEXTOK

// Rcuprer d'une bote de dialogue
bool CTempsSpan::FromDlgHM(CWnd* pWnd, int nIDC)
{
	UINT u1 = pWnd->GetDlgItemInt(nIDC+1, NULL, FALSE);
	if (u1>59)  return false;
	
	Set(0, pWnd->GetDlgItemInt(nIDC, NULL, FALSE), u1);
	return true;
}

// Rcuprer d'une bote de dialogue
bool CTempsSpan::FromDlgHMS(CWnd* pWnd, int nIDC)
{
	UINT u1 = pWnd->GetDlgItemInt(nIDC+1, NULL, FALSE);
	if (u1>59)  return false;
	UINT u2 = pWnd->GetDlgItemInt(nIDC+2, NULL, FALSE);
	if (u2>59)  return false;
	
	Set(0, pWnd->GetDlgItemInt(nIDC, NULL, FALSE), u1, u2);
	return true;
}


// Formatter dans une bote de dialogue
void CTempsSpan::ToDlgHM(CWnd* pWnd, int nIDC) const
{
	TCHAR psz[bufTempsDlg];
	_stprintf(psz, _T("%1u"),  abs(GetTotalHeure()));  pWnd->SetDlgItemText(nIDC+0, psz);
	_stprintf(psz, _T("%02u"), abs(GetMinute()));      pWnd->SetDlgItemText(nIDC+1, psz);
}

// Formatter dans une bote de dialogue
void CTempsSpan::ToDlgHMS(CWnd* pWnd, int nIDC) const
{
	TCHAR psz[bufTempsDlg];
	_stprintf(psz, _T("%1u"),  abs(GetTotalHeure()));  pWnd->SetDlgItemText(nIDC+0, psz);
	_stprintf(psz, _T("%02u"), abs(GetMinute()));      pWnd->SetDlgItemText(nIDC+1, psz);
	_stprintf(psz, _T("%02u"), abs(GetSeconde()));     pWnd->SetDlgItemText(nIDC+2, psz);
}

#endif // MFCEXTOK


// CDumpContext& AFXAPI operator << (CDumpContext& dc, CTempsX dt)
#ifdef MFCOK_DEBUG
#define dcl(type)  CDumpContext& AFXAPI operator << (CDumpContext& dc, CTemps##type dt) { \
	return dc << dt.Format##type(); }
dcl(D  )
dcl( H )
dcl( HS)
#undef dcl
#define dcl(type, type1, type2)  CDumpContext& AFXAPI operator << (CDumpContext& dc, CTemps##type dt) { \
	return dc << dt.Format##type1() << _T(" ") << dt.Format##type2(); }
dcl(DH , D, H )
dcl(DHS, D, HS)
#undef dcl
#endif


// CDumpContext& AFXAPI operator << (CDumpContext& dc, const CTempsSpan ts)
#ifdef MFCOK_DEBUG
CDumpContext& AFXAPI operator << (CDumpContext& dc, const CTempsSpan ts)
{
	return dc << ts.GetJour() << _T(" ") << ts.FormatHMS();
}
#endif


#define cmp(type) \
int Cmp##type(const CTemps##type& dt1, const CTemps##type& dt2) { \
	if      (dt1==dt2)  return  0; \
	else if (dt1<dt2)   return -1; \
	else                return  1; \
}
// Comparer deux dates (0 si 1=2, -1 si 1<2, 1 si 1>2)
cmp(D  )
// Comparer deux heures (0 si dt1=dt2, -1 si dt1<dt2, 1 si dt1>dt2)
cmp( H )
// Comparer deux dates-heures (0 si dt1=dt2, -1 si dt1<dt2, 1 si dt1>dt2)
cmp(DH )
// Comparer deux heures (0 si dt1=dt2, -1 si dt1<dt2, 1 si dt1>dt2)
cmp( HS)
// Comparer deux dates-heures (0 si dt1=dt2, -1 si dt1<dt2, 1 si dt1>dt2)
cmp(DHS)
#undef cmp


#ifdef MFCEXTOK

// Entres/sorties avec une bote de dialogue
void DDX_TempsD(CDataExchange* pDX, int nIDC, CTempsD& dt)
{
	if (pDX->m_bSaveAndValidate)
		dt.FromDlgD(pDX->m_pDlgWnd, nIDC);
	else
		dt.ToDlgD  (pDX->m_pDlgWnd, nIDC);
}
void DDX_TempsDHS(CDataExchange* pDX, int nIDC, CTempsDHS& dt)
{
	if (pDX->m_bSaveAndValidate)
		dt.FromDlgDHS(pDX->m_pDlgWnd, nIDC);
	else
		dt.ToDlgDHS  (pDX->m_pDlgWnd, nIDC);
}

// Vrifier la validit d'une date entre dans une bote de dialogue
void DDV_TempsD(CDataExchange* pDX, int nIDC, CTempsD& dt, bool fPeutNul)
{
	if (pDX->m_bSaveAndValidate && dt.PasValide()!=dtpOK) {
		if (!fPeutNul || dt.m_uJour+dt.m_uMois+dt.m_uAn) {
			AfxMessageBox(ERR_DATEINVALIDE, MB_ICONEXCLAMATION);
#if _MFC_VER >= 0x0700
			pDX->m_idLastControl = nIDC;
#else
			pDX->m_hWndLastControl = pDX->m_pDlgWnd->GetDlgItem(nIDC)->m_hWnd;
#endif
			pDX->Fail();
		}
	}
}
void DDV_TempsDHS(CDataExchange* pDX, int nIDC, CTempsDHS& dt, bool fPeutNul)
{
	if (pDX->m_bSaveAndValidate && dt.PasValide()!=dtpOK) {
		if (!fPeutNul || dt.m_uJour+dt.m_uMois+dt.m_uAn+dt.m_uHeure+dt.m_uMinute+dt.m_uSeconde) {
			AfxMessageBox(ERR_DATEINVALIDE, MB_ICONEXCLAMATION);
#if _MFC_VER >= 0x0700
			pDX->m_idLastControl = nIDC;
#else
			pDX->m_hWndLastControl = pDX->m_pDlgWnd->GetDlgItem(nIDC)->m_hWnd;
#endif
			pDX->Fail();
		}
	}
}
#endif  // MFCEXTOK

#pragma warning (default: 4710 4238)
